/* Mesquite module.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite;

import java.awt.*;
import java.awt.image.*;
import java.net.*;
import java.util.*;
import java.io.*;
import java.awt.event.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;
import mesquite.lib.characters.*;
import mesquite.trunk.*;
import com.apple.mrj.*;
import com.apple.eawt.*;


/* The root module of the tree of modules.  This contains the main method which uses MesquiteModuleLoader to compile information
about available modules.  Also hires FileCoordinator modules (which in turn hire others to do analyses).   */

public class Mesquite extends MesquiteTrunk
{
	public static Image general, calculations, display, left, right;
	static boolean mesquiteExiting = false;
	public boolean openFilesNowUsed = false;
	public boolean ready=false;
	public int numDirectories = 0;
	private boolean preferencesSet = false;
	private String storedManualString = null;
	private boolean showLogWindow = true;
	private boolean showAbout = true;
	private boolean consoleMode = false;
	private Vector splashURLInfo = new Vector(3);
	private ListableVector splashNames = new ListableVector(3);
	public static AboutWindow about;
	private String supportFilesPath;
	private File supportFilesDirectory;
	ProjectTreeWindow projectsWindow;
	private int numUses =0;
	String configFile = "all";
	HPanel browser;
	ListableVector configurations;
	private  FileOpener fileHandler;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		return true;
  	 }
  	 public void dispose(){
  	 	splashNames.dispose();
  	 	configurations.dispose();
  	 	super.dispose();
  	 }
	/*.................................................................................................................*/
   	 public boolean isSubstantive(){
   	 	return false;
   	 }
	/*.................................................................................................................*/
   	 public boolean isPrerelease(){
   	 	return false;
   	 }
	/*.................................................................................................................*/
	public void substantivePrereleasesFound(){
   		if (!substantivePrereleasesExist){
	   		substantivePrereleasesExist = true;
	   		if (about!=null && about.aboutPanel!=null)
	   			about.aboutPanel.repaint();
   		}
	}
	/*.................................................................................................................*/
	public void init()
	{
		if (isApplet())
			mesquiteTrunk = this;
		incrementMenuResetSuppression();
		boolean makeNewPrefsDirectory = false;
		configurations = new ListableVector();
		packages = new ListableVector();
/* 
(1) Look for Mesquite.pref file in .Mesquite_Prefs folder of user.home.  If exists, read it 
(2) Try to find mesquite manual.  If not found, query user for manual.  Remember that it hadn't been found, and later store its place
*/

		String sep = MesquiteFile.fileSeparator;
		supportFilesPath = System.getProperty("user.home") + sep + "Mesquite_Support_Files";
		
		//finding mesquite directory
		File classpath = new File(System.getProperty("java.class.path"));
		StringTokenizer st = new StringTokenizer(System.getProperty("java.class.path"), ":");
		while (st.hasMoreElements()){
			String token = st.nextToken();
			File mesquiteClass = new File(token + MesquiteFile.fileSeparator + "mesquite" + MesquiteFile.fileSeparator + "Mesquite.class");
			if (mesquiteClass.exists()) {
				mesquiteDirectory = new File(token);
				if (!mesquiteDirectory.exists())
					mesquiteDirectory = null;
			}
		}
		if (mesquiteDirectory ==null){
			File mesquiteClass = new File(System.getProperty("user.dir") + MesquiteFile.fileSeparator + "mesquite" + MesquiteFile.fileSeparator + "Mesquite.class");
			if (mesquiteClass.exists()) {
				mesquiteDirectory = new File(System.getProperty("user.dir"));
				if (!mesquiteDirectory.exists())
					mesquiteDirectory = null;
			}
		}

		/* EMBEDDED delete these if embedded */
		userDirectory = new File(System.getProperty("user.home")+sep);  //used to be user.dir, but caused permission problems in linux

		supportFilesDirectory = new File(supportFilesPath);
		if (!supportFilesDirectory.exists())
			supportFilesDirectory.mkdir();
			
			
		MesquiteModule.prefsDirectory = new File(supportFilesPath + sep + "Mesquite_Prefs"); //checking both possible names
		File prefsFile = new File(prefsDirectory.toString() + sep + "Mesquite.pref");
		if (!MesquiteModule.prefsDirectory.exists() || !MesquiteModule.prefsDirectory.isDirectory()) {
			makeNewPrefsDirectory = true;
			findMesquiteDirectory();
		}
		else if (prefsFile.exists()) {
			loadPreferences();
			if (!preferencesSet || MesquiteModule.mesquiteDirectory==null) {
				findMesquiteDirectory();
			}
		}
		else {
			findMesquiteDirectory();
		}
		
 		String logPath = supportFilesPath + sep + MesquiteTrunk.logFileName; 
		File logFile = new File(logPath);
		if (logFile.exists()) {
			File prevLogFile, prevLogFileNewName;
			for (int i = MesquiteTrunk.numPrevLogs-1; i>0; i--) {
				prevLogFile = new File(supportFilesPath  + sep + MesquiteTrunk.logFileName + "_(Previous#" + i + ")");
				if (prevLogFile.exists()){
					prevLogFileNewName = new File(supportFilesPath + sep + MesquiteTrunk.logFileName + "_(Previous#" + (i+1) + ")");
					prevLogFile.renameTo(prevLogFileNewName);
				}
			}
			prevLogFileNewName = new File(supportFilesPath + sep + MesquiteTrunk.logFileName + "_(Previous#1)");
			logFile.renameTo(prevLogFileNewName);
		}
      		
		/**/
/*
(3) Try to find the logo for the about window
*/
		logWindow = new LogWindow();
		mesquiteTrunk.logo = MesquiteImage.getImage(MesquiteModule.getRootPath() + "images/mesquite.gif");
		MediaTracker mt = new MediaTracker(logWindow);
		mt.addImage(logo, 0);
		try {
			mt.waitForAll();
		} catch (Exception e) {
			MesquiteMessage.warnProgrammer("logo image exception------------");
			e.printStackTrace();
		}
		if (about !=null)
			about.setImage(logo);

		
		makeEmployeeVector();
	
	

		/*EMBEDDED include following if embedded *
		setLayout( new BorderLayout() );
		embeddedPanel = new Panel();
		add("Center", embeddedPanel);
		embeddedPanel.setBackground(Color.cyan);
		embeddedPanel.setVisible(true);
		embeddedPanel.setLayout(null);
		embeddedPanel.repaint();
		/*EMBEDDED include above if embedded */
		
		
		if (isApplet()) { 
			mesquiteTrunk = this;
			prepareMesquite();
		}
		

		logWindow.setConsoleMode(consoleMode);
		logln("Mesquite version " + getMesquiteVersion() + getBuildVersion());
		logln("Copyright (c) 1997-2004 W. Maddison and D. Maddison");
		String notice = "The basic Mesquite package (class library and basic modules) is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License. "
			+ "\nMesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.  For details on license and "
			+ "lack of warranty see the GNU Lesser General Public License by selecting \"Display License\" from the Window menu or at www.gnu.org\n";


		logln(notice);
		startupTime = System.currentTimeMillis();
		Date dnow = new Date(startupTime);
		logln( dnow.toString());
		String mrj = System.getProperty("mrj.version");
		if (mrj == null)
			mrj = "";
		else
			mrj = "; MRJ version " + mrj;
		logln("Running under Java " + System.getProperty("java.version") +"; virtual machine by " + System.getProperty("java.vendor") + mrj + " on " + System.getProperty("os.name") + " " + System.getProperty("os.version"));
		logln("User: " + System.getProperty("user.name") );
		logln(" ");
		logWindow.setWindowSize(400,350);
		logWindow.setLocation(320, 4);
		if (showLogWindow)
			logWindow.setVisible(true);
		/* EMBEDDED add following if embedded *
		logWindow.setVisible(false);
		 /* */
		
		Random rNG = new Random();


		MesquiteWindow.centerWindow(about);
		
		
		if (!preferencesSet){
			String s = "Welcome to Mesquite.  If this is the first time you've run Mesquite, you might want to try the example files to learn about how to use it.  You might also check the menu items in the Help menu for links to documentation.";
			s += "\n\nThe default is for Mesquite to load all available modules.  If you want it to load a subset, and thus reduce startup time, use the Activate/Deactivate Packages submenu after Mesquite has started.";
			new WelcomeDialog(containerOfModule(), "Welcome to Mesquite", s);
			if (makeNewPrefsDirectory)
				MesquiteModule.prefsDirectory.mkdir();
			storePreferences();
		}
		
		if (about !=null) {
			if (isApplet())
				about.setVisible(false);
			else {
				MesquiteWindow.centerWindow(about);
				about.setVisible(true);
			}
			about.setBackground(ColorDistribution.light[0]);
			about.repaint();
		}
		
		/*----*/

		suggestedDirectory = mesquiteDirectoryPath + "examples";
		if (mesquiteDirectory!=null)
			logln("Mesquite directory: " + mesquiteDirectoryPath);
		else
			logln("Mesquite directory null ");
		if (userDirectory!=null)
			logln("Log file located in: " + supportFilesPath);
		if (prefsDirectory!=null)
			logln("Prefs directory: " + prefsDirectory.getAbsolutePath());
		else
			logln("Prefs directory null ");

		logln(" ");
		
		/**/
		
		
		
		/* loading in modules (MesquiteModules) */
		MesquiteModule.mesquiteTrunk = this;
		
		setupColors();
		
		ModuleLoader mBL = new ModuleLoader(this);
		
		mBL.init(configFile, configurations, logWindow.keyDown());
		int count = 0;
		if (isPrerelease())
			count++;
		for (int i= 0; i<mesquiteModulesInfoVector.size(); i++){
			MesquiteModuleInfo mmi = (MesquiteModuleInfo)mesquiteModulesInfoVector.elementAt(i);
			if (mmi.isSubstantive() && mmi.isPrerelease()) {
				count++;
			}
		}
		
	
	
		if (count > 0){
			logln(" ");
			logln(Integer.toString(count) + " prerelease substantive modules installed.");
		}
		logln(" ");

		String ackn = "Mesquite makes use of BrowserLauncher by Eric Albert and corejava.Format by Horstmann & Cornell.";
		ackn += "  Some modules make use of JAMA by The MathWorks and NIST, and JSci by Mark Hale, Jaco van Kooten and others (see Mesquite source code for details).";
		ackn += "  The PAL library by Drummond and Strimmer is used by the GTR substitution model for DNA sequence simulations.";
		logln(ackn);
		
		logln(" ");
		logln("------------------------------------");
		logln(" ");
		makeMenu("Mesquite"); //just in case employees have no where else to put
		
		openExternalSMS = new MesquiteSubmenuSpec(null, "Open External", this);
		openExternalSMS.setCommand(makeCommand("openGeneral", this));
		openExternalSMS.setList(GeneralFileMaker.class);
		
		addMenuItem(helpMenu, "Mesquite Manual", makeCommand("showHomePage",  this));
		addMenuItem(helpMenu, "Support and Advice", makeCommand("showSupport",  this));
		addMenuItem(helpMenu, "Modules Loaded", makeCommand("showModules",  this));
		addMenuItem(helpMenu, "Keyword Search...", makeCommand("keywordSearch",  this));
		addMenuItem(helpMenu, "Scripting Commands", makeCommand("showCommands",  this));
		addMenuItem(helpMenu, "-", null);
		for (int i=0; i<splashNames.size(); i++) {
			MesquiteString ms = (MesquiteString)splashNames.elementAt(i);
			MesquiteCommand mc = makeCommand("showPackageInfo", this);
			MesquiteModuleInfo mmi = (MesquiteModuleInfo)splashURLInfo.elementAt(i);
			mc.setDefaultArguments(ParseUtil.tokenize(mmi.getSplashURL()));
			addMenuItem(helpMenu, ms.getValue(), mc);
		}
		addMenuItem(helpMenu, "-", null);
		addMenuItem(helpMenu, "Active Module Tree", makeCommand("showEmployeeTree",  this));
		addMenuItem(helpMenu, "List active modules", makeCommand("dumpEmployeeTree",  this));
		addMenuItem(helpMenu, "List prerelease modules", makeCommand("dumpSubstantivePrerelease",  this));
		addMenuItem(helpMenu, "Show Developer Statistics", makeCommand("showStatistics",  this));
		
		MesquiteSubmenuSpec mms = addSubmenu(fileMenu, "Activate/Deactivate Packages");
		addItemToSubmenu(fileMenu, mms, "Use All Installed Packages", makeCommand("setConfigAll",  this));
		addItemToSubmenu(fileMenu, mms, "Choose Configuration...", makeCommand("setConfig",  this));
		addItemToSubmenu(fileMenu, mms, "Delete Configuration...", makeCommand("deleteConfig",  this));
		addItemToSubmenu(fileMenu, mms, "-", null);
		addItemToSubmenu(fileMenu, mms, "Define Configuration...", makeCommand("defineConfig",  this));
		MesquiteSubmenuSpec macroSM = addSubmenu(fileMenu, "Macros");
		addItemToSubmenu(fileMenu, macroSM, "Show Macro List...", makeCommand("showMacros",  this));
		addItemToSubmenu(fileMenu, macroSM, "Edit Macro Information...", makeCommand("editMacroInfo",  this));
		addItemToSubmenu(fileMenu, macroSM, "Delete Auto-saved Macros...", makeCommand("deleteAutoMacros",  this));
		addMenuItem(fileMenu, "Rename Log File...", makeCommand("renameLog",  this));
		addMenuItem(fileMenu, "-", null);
		
		new MesquiteColorTable(); //initialize default charstate colors

		resetContainingMenuBar();

		/* hire all inits */
		hireAllEmployees(CommandRecord.nonscriptingRecord, MesquiteInit.class);
		
		
		
		if (!isApplet()){
			BrowseHierarchy projectHierarchyTask= (BrowseHierarchy)hireEmployee(CommandRecord.nonscriptingRecord, BrowseHierarchy.class, "Hierarchy browser");
			projectsWindow = new ProjectTreeWindow(this, projectHierarchyTask);
			setModuleWindow(projectsWindow);
			projectsWindow.setVisible(true);
		}
		if (about !=null) {
			about.setQuitIfGoAway(false);
			about.hide();
		}
		decrementMenuResetSuppression();

		if (logWindow!=null) {
			if (logWindow.isConsoleMode()){
				logln("Command-line Mode On.  Type \"help\" for some console commands.  Note: command-line mode is experimental.  Currently it is not properly protected against simultaneous calculations, e.g. doing different modifications simultaneously of the same tree or data.");
				logWindow.showPrompt();
			}
		}
		
		/* */
		ClockWatcherThread cwt = new ClockWatcherThread(this);
		cwt.start();
		/**/
		if (!MesquiteTrunk.consoleListenSuppressed) {
			ConsoleThread cot = new ConsoleThread(this, this, true);
			cot.start();
		}
		/* */
	} 
	int projectBrowserSuppression = 0;
	boolean projectBrowserRefreshPending = false;

	/*.................................................................................................................*/
	public void refreshBrowser(Class c){
		if ((c == MesquiteProject.class || c == MesquiteFile.class || c == FileElement.class)) {
			if (projectBrowserSuppression>0) {
				projectBrowserRefreshPending = true;
				//projectsWindow.disposeReferences();
			}
			else if (projectsWindow!=null)
				projectsWindow.renew();
		}
		else
			super.refreshBrowser(c);
	}
	/*.................................................................................................................*/
	/** Increments suppression level of hierarchy views (project window); if 0 then can be repainted. */
	public void incrementProjectBrowserRefreshSuppression(){
		projectBrowserSuppression++;
	}
	/*.................................................................................................................*/
	/** Decrements suppression level of hierarchy views (project window); if 0 then can be repainted. */
	public void decrementProjectBrowserRefreshSuppression(){
		if (projectBrowserSuppression ==0) {
			//MesquiteMessage.warnProgrammer("decrementBrowserResetSuppression when already zero");
			return;
		}
		projectBrowserSuppression--;
		if (projectBrowserSuppression<0)
			projectBrowserSuppression =0;
		if (projectBrowserSuppression ==0){  //menu suppression just removed and requests pending; reset menus
			if (projectBrowserRefreshPending)
				refreshBrowser(FileElement.class);
			projectBrowserRefreshPending = false;
		}
	}
	/*.................................................................................................................*/
	/** Returns whether suppression level of project window hierarchy is greater than 0.*/
	public boolean getProjectBrowserRefreshSuppression(){
		return projectBrowserSuppression>0;
	}
	/*.................................................................................................................*/
	/** Put a message in the lower left corner of the About window; integers are for thermometer use  */
	public  void mesquiteMessage (String s, long total, long current) {
		
		if (about!=null) {
			ThermoPanel mp = about.aboutMessagePanel;
			if (mp!=null){
				mp.setVisible(true);
				if (s!=null)
					mp.setText(s);
				mp.setTime(total, current);
				mp.repaint();
				//mp.select(0,0);
			}
			
		}
		
	}
	/*.................................................................................................................*/
	/** add a package splash banner to Mesquite's list of currently active packages */
	public void addSplash(MesquiteModuleInfo mmi){
		if (mmi.hasSplash() || mmi.getSplashURL()!=null) {
			if (about !=null && mmi.hasSplash()) 
				about.addSplash(mmi);
			if (!StringUtil.blank(mmi.getSplashURL())) {
				splashURLInfo.addElement(mmi);
				if (StringUtil.blank(mmi.getPackageName()))
					splashNames.addElement(new MesquiteString(mmi.getName()), false);
				else
					splashNames.addElement(new MesquiteString(mmi.getPackageName()), false);
			}
		}
	}
	/*.................................................................................................................EMBEDDED delete this if embedded */
	private void findMesquiteDirectory(){
 		String sep = "" + MesquiteFile.fileSeparator;
		if (mesquiteDirectory!=null) {
      	 		String manualString = mesquiteDirectory + sep + "docs/mesquite" + sep + "manual.html";
      	 		File manualM = new File(manualString);
      	 		if (manualM.exists()) {
				storedManualString = manualString;
			}
		}
		else {
	      	 	String manualString = System.getProperty("user.dir")  + sep + "docs/mesquite" + sep + "manual.html";
			File manual = new File(manualString);
			if (!manual.exists()) {
				if (!StringUtil.blank(storedManualString))
					manual = new File(storedManualString);
				if (manual.exists()) {
					MesquiteModule.mesquiteDirectory = new File(StringUtil.getAllButLastItem(StringUtil.getAllButLastItem(StringUtil.getAllButLastItem(storedManualString, sep),sep),sep) + sep);
				}
		        	else {
			        	manualString = MesquiteFile.checkFilePath(System.getProperty("user.dir") + sep + "docs/mesquite" + sep + "manual.html", "Please find the Mesquite manual file named \"manual.html\", which should be in the \"docs/mesquite\" directory.");
					if (manualString == null || !manualString.endsWith("manual.html")) {
						MesquiteTrunk.mesquiteTrunk.alert("Sorry, the manual was not found correctly.  Mesquite will not function properly.");
					}
					else {
						storedManualString = manualString;
					       MesquiteModule.mesquiteDirectory = new File(StringUtil.getAllButLastItem(StringUtil.getAllButLastItem(StringUtil.getAllButLastItem(manualString, sep),sep),sep)+sep);
					}
				}
			}
			else {
				MesquiteModule.mesquiteDirectory = new File(System.getProperty("user.dir") + sep);
				storedManualString = manualString;
			}
		}
		
		if (mesquiteDirectory!=null){
			mesquiteDirectoryPath = mesquiteDirectory.getAbsolutePath();
			if (mesquiteDirectoryPath.lastIndexOf(sep)!= mesquiteDirectoryPath.length()-1)
				mesquiteDirectoryPath+= sep;
		}
	}
	/*.................................................................................................................*/
	public void processPreferencesFromFile (String[] prefs) {
		if (prefs!=null && prefs.length>0) {
			preferencesSet = true; //done to see that prefs file found; if not ask for registration
			//showLogWindow=prefs[0].charAt(0) == 'L';
			if (prefs.length<2)
				return;
			storedManualString = prefs[1];
			if (prefs.length<3)
				return;

			int iq = MesquiteInteger.fromString(prefs[2]);
			if (MesquiteInteger.isCombinable(iq))
				MesquiteModule.textEdgeCompensationHeight = iq;

			if (prefs.length<4)
				return;			
			iq = MesquiteInteger.fromString(prefs[3]);
			if (MesquiteInteger.isCombinable(iq))
				MesquiteModule.textEdgeCompensationWidth = iq;
			
			if (prefs.length<6)
				return;
			/* EMBEDDED disable if embedded */
			findMesquiteDirectory();
			/**/
			if (prefs.length<7)
				return;
			iq = MesquiteInteger.fromString(prefs[6]);
			if (prefs.length<8)
				return;
			numUses = MesquiteInteger.fromString(prefs[7]);
			if (!MesquiteInteger.isCombinable(numUses))
				numUses = 0;
			else
				numUses++;
			if (prefs.length<9)
				return;
			browserString = prefs[8];
			if (prefs.length<10)
				return;
			if (prefs[9].length()>0)
				defaultHideMesquiteWindow=prefs[9].charAt(0) == 'H';
			if (prefs.length<11)
				return;
			if (prefs[10].length()>0)
				numDirectories =MesquiteInteger.fromString(prefs[10], false);
			if (prefs.length<12)
				return;
			if (prefs[11].length()>0)
				configFile = prefs[11];
			if (prefs.length<13)
				return;
			if (prefs[12]!=null)
				consoleMode = "console".equals(prefs[12]);
    	 		if (prefs.length>13 && prefs[13] !=null) {
				int numLogs = MesquiteInteger.fromString(prefs[13]);
				if (MesquiteInteger.isCombinable(numLogs))
	 				MesquiteTrunk.numPrevLogs = numLogs;
					
			}
		}
	}
	/*.................................................................................................................*/
	public String[] preparePreferencesForFile () {
		String[] prefs;
		prefs = new String[14];
		
		if (logWindow != null && logWindow.isVisible())
			prefs[0] = "L";
		else
			prefs[0]= "N";
		prefs[0] += "A02Y8O*=#";
		prefs[1]= "" + storedManualString;
		prefs[2]= Integer.toString(MesquiteModule.textEdgeCompensationHeight);
		prefs[3]= Integer.toString(MesquiteModule.textEdgeCompensationWidth);
		prefs[4]= "mq";
		prefs[5]= "0";
		prefs[6] = "1"; 
		prefs[7]= Integer.toString(numUses);
		if (browserString != null)
			prefs[8] = browserString;
		else
			prefs[8] = "";
		if (showAbout)
			prefs[9] = "H";
		else
			prefs[9] = "S";
		prefs[10] = Integer.toString(numDirectories);
		prefs[11] = configFile;
		if (logWindow != null && logWindow.isConsoleMode())
			prefs[12] = "console";
		else
			prefs[12] = "noconsole";
		prefs[13] = Integer.toString(MesquiteTrunk.numPrevLogs);
		return prefs;
	}
	/*.................................................................................................................*/
	void setupColors(){
		for (int i=0; i<ColorDistribution.numColorSchemes; i++){
			String[] colorStrings = MesquiteFile.getFileContentsAsStrings(MesquiteModule.getRootPath() + "images/colors/" + i);
			if (colorStrings !=null && colorStrings.length>=5){
				stringPos.setValue(0);
				float r = (float)MesquiteDouble.fromString(colorStrings[0], stringPos);
				float g = (float)MesquiteDouble.fromString(colorStrings[0], stringPos);
				float b = (float)MesquiteDouble.fromString(colorStrings[0], stringPos);
				ColorDistribution.pale[i] = new Color(r, g, b);
				stringPos.setValue(0);
				r = (float)MesquiteDouble.fromString(colorStrings[1], stringPos);
				g = (float)MesquiteDouble.fromString(colorStrings[1], stringPos);
				b = (float)MesquiteDouble.fromString(colorStrings[1], stringPos);
				ColorDistribution.light[i] = new Color(r, g, b);
				stringPos.setValue(0);
				r = (float)MesquiteDouble.fromString(colorStrings[2], stringPos);
				g = (float)MesquiteDouble.fromString(colorStrings[2], stringPos);
				b = (float)MesquiteDouble.fromString(colorStrings[2], stringPos);
				ColorDistribution.medium[i] = new Color(r, g, b);
				stringPos.setValue(0);
				r = (float)MesquiteDouble.fromString(colorStrings[3], stringPos);
				g = (float)MesquiteDouble.fromString(colorStrings[3], stringPos);
				b = (float)MesquiteDouble.fromString(colorStrings[3], stringPos);
				ColorDistribution.dark[i] = new Color(r, g, b);
				stringPos.setValue(0);
				r = (float)MesquiteDouble.fromString(colorStrings[4], stringPos);
				g = (float)MesquiteDouble.fromString(colorStrings[4], stringPos);
				b = (float)MesquiteDouble.fromString(colorStrings[4], stringPos);
				ColorDistribution.project[i] = new Color(r, g, b);
			}
			else {
				ColorDistribution.pale[i] = ColorDistribution.bisque;
				ColorDistribution.light[i] = ColorDistribution.paleGoldenRod;
				ColorDistribution.medium[i] = ColorDistribution.burlyWood;
				ColorDistribution.dark[i] = ColorDistribution.sienna;
				ColorDistribution.project[i] = ColorDistribution.burlyWood;
			}
			InfoBar.baseImage[i]=  MesquiteImage.getImage(MesquiteModule.getRootPath() +"images/colors/" + i + "infoBarBase.gif");  
			InfoBar.graphicsTab[i]=  MesquiteImage.getImage(MesquiteModule.getRootPath() +"images/colors/" + i + "graphicsTab.gif");  
			InfoBar.textTab[i]=  MesquiteImage.getImage(MesquiteModule.getRootPath() +"images/colors/" + i + "textTab.gif");  
			InfoBar.parametersTab[i]=  MesquiteImage.getImage(MesquiteModule.getRootPath() +"images/colors/" + i + "parametersTab.gif");  
			InfoBar.modulesTab[i]=  MesquiteImage.getImage(MesquiteModule.getRootPath() +"images/colors/" + i + "modulesTab.gif");  
			InfoBar.citationsTab[i]=  MesquiteImage.getImage(MesquiteModule.getRootPath() +"images/colors/" + i + "citationsTab.gif");  
		 	MesquiteButton.offImage[i]=  MesquiteImage.getImage(MesquiteModule.getRootPath() + "images/colors/" + i + "blank-off.gif");  
		 	MesquiteButton.onImage[i]=  MesquiteImage.getImage(MesquiteModule.getRootPath() + "images/colors/" + i + "blank-on.gif");  
		}
		InfoBar.triangleImage=  MesquiteImage.getImage(MesquiteModule.getRootPath() +"images/infoBarTriangle.gif");  
		InfoBar.triangleImageDown=  MesquiteImage.getImage(MesquiteModule.getRootPath() +"images/triangleDown.gif");  
		InfoBar.releaseImage=  MesquiteImage.getImage(MesquiteModule.getRootPath() +"images/release.gif");  
		InfoBar.prereleaseImage=  MesquiteImage.getImage(MesquiteModule.getRootPath() +"images/prerelease.gif");  
		ExplanationArea.plusImage=  MesquiteImage.getImage(MesquiteModule.getRootPath() +"images/explanationPlus.gif");  
		ExplanationArea.minusImage=  MesquiteImage.getImage(MesquiteModule.getRootPath() +"images/explanationMinus.gif");  
		ExplanationArea.minusOffImage=  MesquiteImage.getImage(MesquiteModule.getRootPath() +"images/explanationMinusOff.gif");  
	}
	/*.................................................................................................................*/
	public String getName() {
		return "Mesquite";
	}
	/*.................................................................................................................*/
	public String getDutyName() {
		return "Root module for Mesquite system";
	}
	/*.................................................................................................................*/
	public String getExplanation() {
		return "This is the central module for the Mesquite system. " +
		"In it is the main method that starts up the Mesquite application.  This module" +
		"loads information for all of the other modules, and hires FileCoordinator " +
		"modules as needed to deal with open files.  Thus, in the tree of employees" +
		"that active modules make, this module is at its root.";
	}
	/*.................................................................................................................*/
	public String getParameters() {
		return "";
	}
	/*.................................................................................................................*/
	public String getCitation() {
 		return "Maddison, W.P. & D.R. Maddison. 2004.  Mesquite: A modular system for evolutionary analysis.  Version 1.01.  http://mesquiteproject.org";
	}
	/*.................................................................................................................*/
	public String getVersion() {
		return "1.01";
	}
	/*.................................................................................................................*/
	public String getAuthors() {
		return "Wayne P. Maddison and David R. Maddison";
	}

	/*.................................................................................................................*/
	public String getDateReleased() {
		return "January 2004";
	}


	/*.................................................................................................................*/
	private void searchKeyword(String s){
		String results = "";
		for (int i=0; i<mesquiteModulesInfoVector.size(); i++){
			MesquiteModuleInfo mbi = (MesquiteModuleInfo)mesquiteModulesInfoVector.elementAt(i);
			if (stringFound(mbi.getName(), s) || stringFound(mbi.getExplanation(), s)){ // || stringFound(mbi.getCitation())
				results += keywordInfo(mbi, s); 
			}
		}
		if (StringUtil.blank(results))
			alert("No module with keywords found");
		else {
			results = "<h2>Modules whose name or explanation contains the string \"" + s + "\"</h2><ul>" + results + "</ul>";
			MesquiteFile.putFileContents(prefsDirectory + MesquiteFile.fileSeparator + "tempKeywordSearch.html", results, true);
 			showWebPage(prefsDirectory + MesquiteFile.fileSeparator + "tempKeywordSearch.html", true);
		}
	}
	/*.................................................................................................................*/
	private boolean stringFound(String s, String target){
		if (s == null || target == null)
			return false;
		String lcS = s.toLowerCase();
		String lcT = target.toLowerCase();
		return (lcS.indexOf(lcT)>=0);
	}
	/*.................................................................................................................*/
	private String keywordInfo(MesquiteModuleInfo mbi, String target){
			String nameString = "<li>";
			String gcp = MesquiteFile.massageFilePathToURL(MesquiteModule.prefsDirectory + MesquiteFile.fileSeparator + "commands" + MesquiteFile.fileSeparator + mbi.getShortClassName() + ".html");
			if (StringUtil.blank(gcp))
				nameString += mbi.getName();
			else
				nameString += "<a href =\"" + gcp + "\">" + mbi.getName() + "</a>";
		return nameString + ":  " + mbi.getExplanation() + StringUtil.lineEnding() + StringUtil.lineEnding(); 
	}
	/*.................................................................................................................*/
	private void exitNow() {
		mesquiteExiting = true;
		System.exit(0);
	}
	/*.................................................................................................................*/
	public void start() {
		incrementMenuResetSuppression();
		ready = true;
		if (fileHandler !=null && fileHandler.isWaiting())
			fileHandler.openFilesNow();
		/*
		else if (isApplet()) { //EMBEDDED
			String urlString = getParameter("URL");
			if (urlString!=null) {
				openURLString(urlString, CommandRecord.nonscriptingRecord);
			}
		}
		*/
		decrementMenuResetSuppression();
	}
    
	/*.................................................................................................................*/
	public MesquiteProject openGeneral(String readerString, CommandRecord commandRec){
		return newProject(readerString, 3, commandRec);
	}
	/*.................................................................................................................*/
	public MesquiteProject openURLString(String urlString, CommandRecord commandRec){
		return newProject(urlString, 2, commandRec);
	}
	/*.................................................................................................................*/
	/* give alternative method which takes full file and passes it to FileCoordinator,
	which then uses alternative methods in MesquiteProject reader that parses this string
	instead of input stream*/
	public MesquiteProject openFile(String pathname, CommandRecord commandRec){
		return newProject(pathname, 1, commandRec);
	}
	/*.................................................................................................................*/
	/* makes and returns a new project, in process making taxa block.*/
	public MesquiteProject newFileWithTaxa(String arguments, CommandRecord commandRec){
		return newProject(arguments, 0, commandRec);
	}
	/*.................................................................................................................*/
	/* makes and returns a new project.*/
	public MesquiteProject newFile(String arguments, CommandRecord commandRec){
		return newProject(arguments, -1, commandRec);
	}
	/*.................................................................................................................*/
	/* makes and returns a new project.*/
	public MesquiteProject newProject(String arguments, int code, CommandRecord commandRec){
		if (commandRec.scripting()) {
			ObjectContainer projCont = new ObjectContainer();
			ProjectRead pr = new ProjectRead(arguments, commandRec, code, mesquiteTrunk, projCont);
			pr.run();
			MesquiteProject p = (MesquiteProject)projCont.getObject();
			projCont.setObject(null); //This is done because Threads not being finalized, and need to remove references to projects
			return p;
		}
		else {
			ProjectRead pr = new ProjectRead(arguments, commandRec, code, mesquiteTrunk, null);
			MesquiteThread pt = new MesquiteThread(pr);
			pr.setThread(pt);
			pt.start();
			return null;
		}
	}

	/*.................................................................................................................*/
	/** Requests a window to close.  In the process, subclasses of MesquiteWindow might close down their owning MesquiteModules etc.*/
 	public void windowGoAway(MesquiteWindow whichWindow) {
			whichWindow.hide();
	}
	boolean closeAllProjects(CommandRecord commandRec){
 		Projects projs = getProjectList();
 		MesquiteProject[] ps = new MesquiteProject[projs.getNumProjects()];
 		for (int i= 0; i<ps.length; i++) {
			ps[i]= projs.getProject(i);
 		}
 		for (int i= 0; i<ps.length; i++) {
 			MesquiteProject p = ps[i];
 			if (p!=null) {
 				FileCoordinator fc = p.getCoordinatorModule();
 				if (fc!=null) {
 					if (!fc.closeFile(p.getHomeFile(), commandRec)) {
 						logln("Sorry, unable to close " + p.getName());
 						return false;
 					}
 				}
 			}
 		}
 		return true;
	}
	void saveAllProjects(CommandRecord commandRec){
 		Projects projs = getProjectList();
 		MesquiteProject[] ps = new MesquiteProject[projs.getNumProjects()];
 		for (int i= 0; i<ps.length; i++) {
			ps[i]= projs.getProject(i);
 		}
 		for (int i= 0; i<ps.length; i++) {
 			MesquiteProject p = ps[i];
 			if (p!=null) {
 				FileCoordinator fc = p.getCoordinatorModule();
 				if (fc!=null) 
 					fc.saveAllFiles();
 			}
 		}
	}
	public void fileDirtiedByCommand(MesquiteCommand command){
		if (command !=null && command.getOwner() == this && "quit".equalsIgnoreCase(command.getName()))
			return;
		super.fileDirtiedByCommand(command);
	}
	/*.................................................................................................................*/
	private ListableVector getMacroRecords(boolean autoOnly){
		ListableVector macros = new ListableVector();
		for (int i = 0; i<mesquiteModulesInfoVector.size(); i++){
			MesquiteModuleInfo mmi = (MesquiteModuleInfo)mesquiteModulesInfoVector.elementAt(i);
			if (mmi.getMacros()!=null && mmi.getMacros().size()>0){
				Vector mForM = mmi.getMacros();
				for (int j = 0; j<mForM.size(); j++){
					MesquiteMacro mmr = (MesquiteMacro)mForM.elementAt(j);
					if (!autoOnly || mmr.isAutoSave())
						macros.addElement(mmr, false);
				}
			}
		}
		if (macros.size() == 0)
			return null;
		return macros;
	}
	public void reportMemory(){
		if (!checkMemory) {
			logln("Memory status cannot be reported because checkMemory is not set");
			return;
		}
					logln("MesquiteProject.totalFinalized " +  MesquiteProject.totalFinalized + " MesquiteProject.totalDisposed " + MesquiteProject.totalDisposed + " MesquiteProject.totalCreated " +  MesquiteProject.totalCreated);
					logln("MesquiteFile.totalFinalized " +  MesquiteFile.totalFinalized + " MesquiteFile.totalDisposed " + MesquiteFile.totalDisposed + " MesquiteFile.totalCreated " +  MesquiteFile.totalCreated);
					logln("ProjectRead.totalFinalized " +  ProjectRead.totalFinalized + " ProjectRead.totalCreated " +  ProjectRead.totalCreated);
					logln("FileElement.totalFinalized " +  FileElement.totalFinalized + " FileElement.totalDisposed " + FileElement.totalDisposed + " FileElement.totalCreated " +  FileElement.totalCreated);
					Vector cre, fin, ct, ctd;
					cre = FileElement.classesCreated;
					fin = FileElement.classesFinalized;
					ct = FileElement.countsOfClasses;
					ctd = FileElement.countsOfClassesDisposed;
					if (cre.size()>fin.size()){
						logln("File element classes created but not finalized    " );
						for (int i=0; i<cre.size(); i++){
							if (fin.indexOf(cre.elementAt(i))<0)
								logln("    " + cre.elementAt(i) + "  count: " + ct.elementAt(i));
						}
					}
					logln("File element classes with fewer disposals than creations    " );
					for (int i=0; i<ct.size() && i<ctd.size(); i++){
						MesquiteInteger mi = (MesquiteInteger)ct.elementAt(i);
						MesquiteInteger mid = (MesquiteInteger)ctd.elementAt(i);
						if (mi.getValue()!=mid.getValue())
							logln("    " + cre.elementAt(i) + "  created: " + mi + " disposed: " + mid);
					}
					
					logln("NexusBlock.totalFinalized " +  NexusBlock.totalFinalized + " NexusBlock.totalDisposed " + NexusBlock.totalDisposed + " NexusBlock.totalCreated " +  NexusBlock.totalCreated);
					cre = NexusBlock.classesCreated;
					fin = NexusBlock.classesFinalized;
					ct = NexusBlock.countsOfClasses;
					ctd = NexusBlock.countsOfClassesFinalized;
					logln("NexusBlock class instantiations    " );
					for (int i=0; i<cre.size(); i++){
						logln("    " + cre.elementAt(i) + "  count: " + ct.elementAt(i) + "  ever finalized? " + (fin.indexOf(cre.elementAt(i))>=0) + "  num finalizations " + ctd.elementAt(i));
					}
					
					logln("ListableVector.totalFinalized " +  ListableVector.totalFinalized + " ListableVector.totalDisposed " + ListableVector.totalDisposed + " ListableVector.totalCreated " +  ListableVector.totalCreated);
					logln("MesquiteCommand.totalFinalized " +  MesquiteCommand.totalFinalized + " MesquiteCommand.totalDisposed " + MesquiteCommand.totalDisposed + " MesquiteCommand.totalCreated " +  MesquiteCommand.totalCreated);

					cre = MesquiteCommand.classesLinked;
					fin = MesquiteCommand.classesUnlinked;
					ct = MesquiteCommand.countsOfClasses;
					if (cre.size()>fin.size()){
						logln("MesquiteCommand objects linked but not unlinked    " );
						for (int i=0; i<cre.size(); i++){
							if (fin.indexOf(cre.elementAt(i))<0)
								logln("    " + cre.elementAt(i) + "  count: " + ct.elementAt(i));
						}
					}
					
					logln(" MesquiteMenuItem.totalFinalized " +  MesquiteMenuItem.totalFinalized + " MesquiteMenuItem.totalDisposed " + MesquiteMenuItem.totalDisposed + " MesquiteMenuItem.totalCreated " +  MesquiteMenuItem.totalCreated);
					logln(" MesquiteMenuItemSpec.totalFinalized " +  MesquiteMenuItemSpec.totalFinalized + " MesquiteMenuItemSpec.totalDisposed " + MesquiteMenuItemSpec.totalDisposed + " MesquiteMenuItemSpec.totalCreated " +  MesquiteMenuItemSpec.totalCreated);
					logln(" MesquiteModule.totalFinalized " +  MesquiteModule.totalFinalized + " EmployerEmployee.totalDisposed " +  EmployerEmployee.totalDisposed + " EmployerEmployee.totalCreated " +  EmployerEmployee.totalCreated);
					logln("Projects.projectsAdded " + Projects.projectsAdded + " Projects.projectsRemoved " +  Projects.projectsRemoved);
					logln("FileCoordinator.totalFinalized " + FileCoordinator.totalFinalized + " FileCoordinator.totalCreated " +  FileCoordinator.totalCreated);
					logln("MesquiteTree.totalDisposed " + MesquiteTree.totalDisposed + " MesquiteTree.totalCreated " +  MesquiteTree.totalCreated);
					logln("CharacterData.totalDisposed " + CharacterData.totalDisposed + " CharacterData.totalCreated " +  CharacterData.totalCreated);
					logln("TreeVector.totalDisposed " + TreeVector.totalDisposed + " TreeVector.totalCreated " +  TreeVector.totalCreated);
					logln("Listened.listenersRemaining " + Listened.listenersRemaining);
					logln("Listened.listenersReport\n" + Listened.reportListeners());
					logln("Threads");
					for (int i=0; i<MesquiteThread.threads.size(); i++){
							logln("    " + MesquiteThread.threads.elementAt(i));
					}
	}
	/*.................................................................................................................*/
	MesquiteInteger pos = new MesquiteInteger();
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Sets the packages of modules loaded at startup, using a configuration file", null, commandName, "setConfig")) {
	    	 	//need Module Activation menu item with submenu items: Use All Installed Modules; Choose Module set
	    	 	//use MesquiteTrunks.configs, a vector of configurations found (each stores name, explanation)
    	 		Listable[] list = new Listable[configurations.size()];
    	 		for (int i=0; i<configurations.size(); i++){
	    	 		Object o = configurations.elementAt(i);
	    	 		ConfigFileRecord cfr = (ConfigFileRecord)o;
	    	 		list[i]= cfr;
    	 		}
    	 		String c = configuration;
    	 		if (c == null)
    	 			c = "All Installed Modules";
    	 		Listable chosen = ListDialog.queryList(containerOfModule(), "Choose configuration", "Choose the configuration file that indicates what packages of modules are to be loaded at startup.  Current configuration is \"" + c + "\".", MesquiteString.helpString, list, 0);
    	 		if (chosen != null){
    	 			configFile = ((ConfigFileRecord)chosen).getPath();
    	 			storePreferences();
    	 			alert("You will need to restart Mesquite to load the selected configuration");
    	 		}
    	 		
    	 	}
    	 	else if (checker.compare(this.getClass(), "Defines a set of  packages to be loaded at startup, using a configuration file", null, commandName, "defineConfig")) {
			boolean[] selected = new boolean[packages.size()];
			for (int i= 0; i<packages.size(); i++){
				MesquitePackageRecord mpr = (MesquitePackageRecord)packages.elementAt(i);
				selected[i] =mpr.getLoaded();
				
			}
    	 		String c = getConfiguration();
    	 		if (c == null)
    	 			c = "All Installed Modules";
			Listable[] list = ChecklistDialog.queryListMultiple(containerOfModule(), "Define Configuration", "Shown are packages of modules available in this installation of Mesquite.  The currently loaded packages are shown in boldface.  Select and deselect to define a new configuration of packages to be loaded.  Current configuration is \"" + c + "\".", packages, selected);
			if (list!=null && list.length>0) {
				String[] contents = new String[list.length+1];
				
				MesquiteBoolean answer = new MesquiteBoolean(false);
				MesquiteString name = new MesquiteString("User-defined module configuration");
				MesquiteString explanation = new MesquiteString("");
				MesquiteString.queryTwoStrings(containerOfModule(), "Specify saved configuration", "Name of configuration of module packages", "Explanation of configuration of module packages",  answer, name, explanation,true);
				if (!answer.getValue())
					return null;
				contents[0] = ParseUtil.tokenize(name.getValue()) + "   " +  ParseUtil.tokenize(explanation.getValue());
				for (int i=0; i<list.length; i++) {
					MesquitePackageRecord mpr = (MesquitePackageRecord)list[i];
					contents[i+1] = mpr.getPackageName();
				}
				String base = MesquiteModule.prefsDirectory+ MesquiteFile.fileSeparator +"configs";
				if (!MesquiteFile.fileExists(base)) {
					File f = new File(base);
					f.mkdir();
				}
				String candidate = base + "/test1.config";
				int count = 2;
				while (MesquiteFile.fileExists(candidate)){
					candidate = base + "/test" + (count++) + ".config";
				}
				MesquiteFile.putFileContents(candidate, contents, true); 
    	 			configFile = candidate;
    	 			storePreferences();
    	 			alert("You will need to restart Mesquite to load the defined configuration");
				ConfigFileRecord cfr = new ConfigFileRecord(true);
				cfr.cStored[0] = candidate;
				cfr.cStored[1] = name.getValue();
				cfr.cStored[2]= explanation.getValue();
				configurations.addElement(cfr, false);
			}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Brings up a dialog box by which user-defined configuration files can be deleted", null, commandName, "deleteConfig")) {
			
			//first get list of user-defined configs; must filter configurations vector to include only user-defined.  Then delete corresponding files!
			ListableVector ud = new ListableVector();
			for (int i= 0; i<configurations.size(); i++){
				ConfigFileRecord config = (ConfigFileRecord)configurations.elementAt(i);
				if (config.isUserDefined())
					ud.addElement(config, false);
			}
    	 		String c = configuration;
    	 		if (c == null)
    	 			c = "All Installed Modules";
			Listable[] list = ListDialog.queryListMultiple(containerOfModule(), "Delete Configuration", "Select the configurations to be deleted and hit \"OK\" to delete them.  Current configuration is \"" + c + "\".", MesquiteString.helpString, "Delete", false, ud, null);
			if (list!=null && list.length>0) {
				if (AlertDialog.query(containerOfModule(), "Delete?", "Are you sure you want to delete the configuration(s)?")){
					for (int i=0; i<list.length; i++) {
						ConfigFileRecord config = (ConfigFileRecord)list[i];
						File f = new File(config.getPath());
						f.delete();
						configurations.removeElement(config, false);
					}
				}
			}
    	 	}
      	 	else if (checker.compare(this.getClass(), "Brings up a dialog box to change name and explanation of user-saved macros", null, commandName, "editMacroInfo")) {
      	 		MesquiteMacro macro = MesquiteMacro.findMacro(parser.getFirstToken(arguments));
      	 		if (macro==null)
      	 			return null;
      	 		String[] oldFile = MesquiteFile.getFileContentsAsStrings(macro.getPath());
      	 		if (oldFile == null || oldFile.length ==0)
      	 			return null;
			MesquiteBoolean answer = new MesquiteBoolean(false);
			MesquiteString name = new MesquiteString(macro.getName());
			MesquiteString explanation = new MesquiteString(macro.getRawExplanation());
			MesquiteString.queryTwoStrings(containerOfModule(), "Edit Macro Information", "Name of macro for window (to appear as menu item)", "Explanation of macro",  answer, name, explanation, true);
			if (!answer.getValue())
				return null;
			macro.setName(name.getValue());
			macro.setExplanation(explanation.getValue());
			//SAVE FILE
			oldFile[0] = "telling " +   macro.getModuleInfo().getClassName()  + "  " + ParseUtil.tokenize(name.getValue()) + "   " +  ParseUtil.tokenize(explanation.getValue()) + ";" + StringUtil.lineEnding();
			MesquiteFile.putFileContents(macro.getPath(), oldFile, true);
			resetAllMenuBars();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Brings up a dialog box to examine macro files", null, commandName, "showMacros")) {
			
			//first get list of user-defined configs; must filter configurations vector to include only user-defined.  Then delete corresponding files!
			ListableVector ud = getMacroRecords(false);
			
			Listable[] list = ListDialog.queryListMultiple(containerOfModule(), "Macro Information", "These are the installed macros.  Touch on their names to see an explanation.", MesquiteString.helpString, ud, null);
    	 	}
   	 	else if (checker.compare(this.getClass(), "Brings up a dialog box by which automatically-defined macros files can be deleted", null, commandName, "deleteAutoMacros")) {
			
			//first get list of user-defined configs; must filter configurations vector to include only user-defined.  Then delete corresponding files!
			ListableVector ud = getMacroRecords(true);
			
			Listable[] list = ListDialog.queryListMultiple(containerOfModule(), "Delete Macro", "Select the macros to be deleted and hit \"OK\" to delete them", MesquiteString.helpString, "Delete", false, ud, null);
			
			if (list!=null && list.length>0) {
				if (AlertDialog.query(containerOfModule(), "Delete?", "Are you sure you want to delete the macro(s)?")){
					for (int i=0; i<list.length; i++) {
						MesquiteMacro macro = (MesquiteMacro)list[i];
						MesquiteModuleInfo mmi = macro.getModuleInfo();
						File f = new File(macro.getPath());
						f.delete();
						if (mmi !=null)
							mmi.removeMacro(macro);
					}
					resetAllMenuBars();
				}
			}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Redirects log saving to the given filename", "[filename]", commandName, "redirectLog")) {
      	 		String name = parser.getFirstToken(arguments);
    	 		if (StringUtil.blank(name))
    	 			name = MesquiteString.queryString(containerOfModule(), "Redirect Log", "Indicate name of log file to which subsequent log output is to be directored.  It will be saved in the directory " + supportFilesPath, MesquiteTrunk.logFileName);
    	 		if (StringUtil.blank(name))
    	 			return null;
   	 		logln("Log redirected to \"" + name + "\"");
   	 		MesquiteFile.closeLog();
      	 		MesquiteTrunk.logFileName = name;
    	 	}
    	 	else if (checker.compare(this.getClass(), "Changes the current log file's name", "[filename]", commandName, "renameLog")) {
    	 		String name = parser.getFirstToken(arguments);
    	 		if (StringUtil.blank(name))
    	 			name = MesquiteString.queryString(containerOfModule(), "Rename Log", "Indicate new name of log file.  It will be saved in the directory " + supportFilesPath, MesquiteTrunk.logFileName);
    	 		if (StringUtil.blank(name))
    	 			return null;
    	 		MesquiteFile.closeLog();
    	 		logln("Log renamed to \"" + name + "\"");
			File current = new File(supportFilesPath  + MesquiteFile.fileSeparator + MesquiteTrunk.logFileName);
    	 		MesquiteTrunk.logFileName = name;
			if (current.exists()){
				File newFile = new File(supportFilesPath + MesquiteFile.fileSeparator + MesquiteTrunk.logFileName);
				current.renameTo(newFile);
			}
			
    	 	}
    	 	else if (checker.compare(this.getClass(), "Indicates that all installed modules are to be loaded at startup", null, commandName, "setConfigAll")) {
    	 		configFile = "all";
    	 		alert("You will need to restart Mesquite to load all of the modules");
    	 	}
    	 	else if (checker.compare(this.getClass(), "Opens file on disk.  The file will be opened as a separate project (i.e. not sharing information) from any other files currently open.", "[name and path of file] - if parameter absent then presents user with dialog box to choose file", commandName, "openFile")) {
    	 		arguments = ParseUtil.getFirstToken(arguments, stringPos);
    	 		MesquiteProject f;
    	 		CommandRecord cr;
    	 		if (commandRec == CommandRecord.nonscriptingRecord || commandRec.getID()<2) {
    	 			cr =new CommandRecord((CommandThread)null, false);
    	 			cr.setFromCommandLine(commandRec.isFromCommandLine());
    	 		}
    	 		else
    	 			cr = commandRec;
    	 		if (StringUtil.blank(arguments)) {
    	 			f = openFile(null, cr); 
    	 		}
    	 		else {
    	 			String baseN = null;
    	 			if (commandRec.scripting()) {
    	 				MesquiteFile base = null;
    	 				base = commandRec.getScriptingFile();
    	 				if (base !=null)
    	 					baseN = base.getDirectoryName();
    	 			}
    	 			if (!MesquiteFile.fileExists(arguments) && (!StringUtil.blank(baseN) && MesquiteFile.fileExists(baseN+arguments)))
    	 				arguments = baseN+arguments;
    	 			f= openFile(arguments, cr);
			}
			return f;
    	 	}
    	 	else if (checker.compare(this.getClass(), "Opens file on web server.  The file will be opened as a separate project (i.e. not sharing information) from any other files currently open.", "[URL of file] - if parameter absent then presents user with dialog box to enter URL", commandName, "openURL")){
    	 		arguments = ParseUtil.getFirstToken(arguments, stringPos);
			return openURLString(arguments, commandRec);
    	 	}
    	 	else if (checker.compare(this.getClass(), "Opens module for project reading or making.", "[module]", commandName, "openGeneral")){
    	 		//arguments = ParseUtil.getFirstToken(arguments, stringPos);
			return openGeneral(arguments, commandRec);
    	 	}
   	 	else if (checker.compare(this.getClass(), "Closes file and any other files linked to it (i.e. closes the project)", "[ID number of project]", commandName, "closeProjectByID")) {
    	 		int num = MesquiteInteger.fromString(arguments);
    	 		if (MesquiteInteger.isCombinable(num)) {
				MesquiteProject proj = MesquiteTrunk.mesquiteTrunk.projects.getProjectByID(num);
				if (proj==null)
					return null;
				FileCoordinator coord = proj.getCoordinatorModule();
				if (coord==null)
					return null;
				coord.closeFile(proj.getHomeFile(), commandRec);
			}
	       }
	       else if (checker.compare(this.getClass(), "Makes a new file (as a new project)", null, commandName, "newProject")) {
			return newFileWithTaxa(arguments, commandRec);
	       	}
	       else if (checker.compare(this.getClass(), "Makes a new file (as a new project)", null, commandName, "newEmptyProject")) {
			return newFile(arguments, commandRec);
	       	}
	       else if (checker.compare(this.getClass(), "Forces a reset of the menus.", null, commandName, "resetMenus")) {
	       		zeroMenuResetSuppression();
	       		resetAllMenuBars();
	       	}
	       else if (checker.compare(this.getClass(), "Shows the GNU Lesser General Public License.", null, commandName, "showLicense")) {
			TextDisplayer displayer = (TextDisplayer)hireEmployee(commandRec, TextDisplayer.class, null);
			if (displayer!=null)
				displayer.showFile(getRootPath()  + "lesser.txt", 100000, true); 
			return displayer;
	       	}
	       else if (checker.compare(this.getClass(), "Shows the currently executing command and offers to stop it.", null, commandName, "currentCommand")) {
	       		if (MainThread.getCurrentlyExecuting()!=null) {
		       		MainThread.mainThread.suspend();
	       			if (!AlertDialog.query(containerOfModule(), "Current Command", "The current command is\n" + MainThread.getCurrentlyExecuting().getListName(), "Continue", "STOP COMMAND")) {
		       			MainThread.getCurrentlyExecuting().interrupted = true;
		       			MainThread.mainThread.interrupt();
		       			MainThread.mainThread.stop();
					alert("Stopped " + MainThread.getCurrentlyExecuting().getListName()); 
					MainThread.mainThread = new MainThread();
					MainThread.mainThread.start();
				}
		       		else 
		       			MainThread.mainThread.resume();
	       		}
	       		else
	       			alert("There is no command currently executing");
	       	}
	       else if (checker.compare(this.getClass(), "Shows the list of pending commands and offers some to be suspended.", null, commandName, "pendingCommands")) {
	       		if (MesquiteCommand.anyQueuedCommands(true)) {
	       			String message = "This is a list of pending commands.  If you want to suspend any commands, select them and hit STOP COMMAND.  Otherwise, hit CANCEL.  The currently executing command is NOT listed.";
	       			Listable[] commandsToKill = ListDialog.queryListMultiple(containerOfModule(), "Commands pending", message, MesquiteString.helpString, "STOP COMMAND", false, MainThread.listPendingCommands(), null);
	       			if (commandsToKill !=null && commandsToKill.length>0){
	       				for (int i=0; i<commandsToKill.length;  i++)
	       					MainThread.pendingCommands.removeElement(commandsToKill[i], false);
	       			}
	       		}
	       		else
	       			alert("There are no commands pending");
	       	}
	       	else if (checker.compare(this.getClass(), "Reports memory status, for debugging of memory leaks.  Static boolean checkMemory needs to have been set first.", null, commandName, "reportMemory")) {
	       		reportMemory();
	       	}
	       	else if (checker.compare(this.getClass(), "Quits Mesquite", null, commandName, "quit")) {
			//CommandRecord.checkThread = false; //suppress thread checking
	       		PendingCommand pc = MainThread.getCurrentlyExecuting();
	       		
	       		if (pc!=null && pc.getCommandRecord() != commandRec) {
	       			if (AlertDialog.query(containerOfModule(), "Command executing", "Are you sure you want to quit?  There is a command currently executing (" + MainThread.getCurrentlyExecuting().getListName() + ").", "Continue", "Quit")){
					//CommandRecord.checkThread = true;
	       				return null;
	       			}
	       		}
				incrementMenuResetSuppression();
				if (!closeAllProjects(commandRec)) {
					decrementMenuResetSuppression();
					//CommandRecord.checkThread = true;
					return null;
				}
				/**/
				logln("==vvvvvvvvvvvvvvvvvvvv==");
				logln("The following is used to to check efficiencies.");
				logln("MesquiteWindow.totalCheckDoomedCount " + MesquiteWindow.totalCheckDoomedCount);
				logln("--Classes painted\n" + MesquiteWindow.componentsPainted.recordsToString());
				logln("Listened.notifications " + Listened.notifications);
				logln("--Classes notifying\n" + Listened.classes.recordsToString());
				logln("--Classes notified\n" + Listened.classesNotified.recordsToString());
				if (MesquiteTrunk.checkMemory){
					logln("==vvvvvvvvvvvvvvvvvvvv==");
					logln("The following is used to trace memory leaks");
					reportMemory();
					logln("==^^^^^^^^^^^^^^^^^^^^==");
				}
				/*
				if (Listened.listenersRemaining>0) {
					for (int i = 0; i<Listened.allListeners.size(); i++){
						Object r = Listened.allListeners.elementAt(i);
						MesquiteMessage.println("listener remaining " + r);
					}
					
				}
				*/
				//	logln("Mesquite is quitting with some listeners remaining.  This is a symptom that some objects were not properly disposed, and it may relate to inflation of memory use.  Please report this problem.");
				/**/
				
				
				/*
				logln("total time to startup modules "  + EmployerEmployee.startupTime.getAccumulatedTime() );
				logln("total time to make window "  + MesquiteWindow.startingTime.getAccumulatedTime() );
				logln("total time to show window "  + MesquiteWindow.windowShowTime.getAccumulatedTime() );
				logln("total time to compose menus "  + MenuOwner.menuCompositionTime.getAccumulatedTime() );
				logln("total time to reset menus "  + MesquiteWindow.resetMenuTime.getAccumulatedTime() );
				logln("total time to make tables  "  + MesquiteTable.tableTime.getAccumulatedTime() );
				logln("Modules: total started: " + EmployerEmployee.totalCreated + "; total ended: " + EmployerEmployee.totalDisposed);
				logln("Trees: total created: " + MesquiteTree.totalCreated + "; total disposed: " + MesquiteTree.totalDisposed);
				*/
				doomed = true;
				showAbout = (about!= null && !about.isShowing());
				logln("trunk module quitting  " );
			
		       		iQuit();
				logln("disposing remaining windows  " );
		       		while (windowVector.size()>0) {
			 		Enumeration e = windowVector.elements();
			 		while (e.hasMoreElements()) {
						Object obj = e.nextElement();
						MesquiteWindow mw = (MesquiteWindow)obj;
				 		mw.setVisible(false);
				 		mw.dispose();
			 		}
		 		}
				logln("exiting " );
				MesquiteFile.closeLog();
		 		exitNow();
	       	}
	       	else if (checker.compare(this.getClass(), "Force Quit Mesquite (data may be lost) via click go-away", null, commandName, "goAwayForceQuit")) {
			incrementMenuResetSuppression();
			doomed = true;
	       		while (windowVector.size()>0) {
		 		Enumeration e = windowVector.elements();
		 		while (e.hasMoreElements()) {
					Object obj = e.nextElement();
					MesquiteWindow mw = (MesquiteWindow)obj;
			 		mw.setVisible(false);
			 		mw.dispose();
		 		}
	 		}
			showAbout = false;
			exitNow();
	       	}
	       	else if (checker.compare(this.getClass(), "Force Quit Mesquite (data may be lost)", null, commandName, "forceQuit")) {
			exitNow();
	       	}
	       	else if (checker.compare(this.getClass(), "Brings all windows to front", null, commandName, "showAllWindows")) {
	 		Enumeration e = windowVector.elements();
	 		while (e.hasMoreElements()) {
				Object obj = e.nextElement();
				MesquiteWindow mw = (MesquiteWindow)obj;
		 		if (mw.isVisible())
		 			mw.setVisible(true);
	 		}
	       	}
	       	else if (checker.compare(this.getClass(), "Closes all current projects and files", null, commandName, "closeAllProjects")) {
	 		closeAllProjects(commandRec);
	       	}
	       	else if (checker.compare(this.getClass(), "Saves all current projects and files", null, commandName, "saveAllProjects")) {
	       		int numFiles = 0;
	 		Projects projs = getProjectList();
	 		for (int i= 0; i<projs.getNumProjects(); i++) {
				numFiles += projs.getProject(i).getNumberLinkedFiles();
	 		}
	       		String s= "Do you want to save all of the open files?  There are currently " + numFiles + " files open";
	       		if (numFiles == getProjectList().getNumProjects())
	       			s += ".";
	       		else
	       			s += " in " + getProjectList().getNumProjects() + " separate projects." ;
	 		if (commandRec.scripting() || AlertDialog.query(containerOfModule(), "Save all?", s, "Save", "Cancel", 1))
	 			saveAllProjects(commandRec);
	       	}
	       	else if (checker.compare(this.getClass(), "Performs keyword search to find module with particular function.", null, commandName, "keywordSearch")) {
	 		//system could log employers used for various modules to give examples of use
			String s = MesquiteString.queryString(containerOfModule(), "Keyword", "Enter a search string.  Mesquite will search through module information for a match (currently, the string as a whole must match).", "");
			if (s!=null) {
				searchKeyword(s);
			}
	       	}
	       	else if (checker.compare(this.getClass(), "Shows web page of installed and loaded modules", null, commandName, "showModules")) {
	 		if (MesquiteTrunk.isApplet()) 
	 			return null;
			if (!CommandChecker.documentationComposed) {
				CommandChecker cchecker = new CommandChecker();
				cchecker.composeDocumentation();
			}	
			String moduleListPath= MesquiteModule.prefsDirectory + MesquiteFile.fileSeparator + "modules.html";
			File testing = new File(moduleListPath);
			if (testing.exists()) {
			 	showWebPage(moduleListPath, true);
	 		}
	       	}
	       	else if (checker.compare(this.getClass(), "Shows web page introducing a package of installed modules", null, commandName, "showPackageInfo")) {
	 		if (MesquiteTrunk.isApplet()) 
	 			return null;
			String packageURL= parser.getFirstToken(arguments); //MesquiteModule.prefsDirectory + MesquiteFile.fileSeparator + "modules.html";
			File testing = new File(packageURL);
			if (testing.exists()) {
			 	showWebPage(packageURL, false);
	 		}
	       	}
	       	else if (checker.compare(this.getClass(), "Shows web page of available commands for scripting", null, commandName, "showCommands")) {
	 		if (MesquiteTrunk.isApplet()) 
	 			return null;
			if (!CommandChecker.documentationComposed) {
				CommandChecker cchecker = new CommandChecker();
				cchecker.composeDocumentation();
			}	
			String commandListPath= MesquiteModule.prefsDirectory + MesquiteFile.fileSeparator + "commands.html";
			File testing = new File(commandListPath);
			if (testing.exists()) {
			 	showWebPage(commandListPath, true);
	 		}
	       	}
	       	else if (checker.compare(this.getClass(), "Shows home page of Mesquite manual", null, commandName, "showHomePage")) {
	 		if (MesquiteTrunk.isApplet()) 
	 			return null;
			if (!CommandChecker.documentationComposed) {
				CommandChecker cchecker = new CommandChecker();
				cchecker.composeDocumentation();
			}	
	 		String manualPath = mesquiteDirectoryPath + "docs/mesquite" + MesquiteFile.fileSeparator + "manual.html" ; 
			File testing = new File(manualPath);
			if (testing.exists()) {
			 	showWebPage(manualPath, false);
			}
	       	}
	       	else if (checker.compare(this.getClass(), "Shows Support page of Mesquite manual", null, commandName, "showSupport")) {
	 		if (MesquiteTrunk.isApplet()) 
	 			return null;
			if (!CommandChecker.documentationComposed) {
				CommandChecker cchecker = new CommandChecker();
				cchecker.composeDocumentation();
			}	
	 		String manualPath = mesquiteDirectoryPath + "docs/mesquite" + MesquiteFile.fileSeparator + "support.html" ; 
			File testing = new File(manualPath);
			if (testing.exists()) {
			 	showWebPage(manualPath, false);
			}
	       	}
    	 	else if (checker.compare(this.getClass(), "Sets whether the Mesquite window appears automatically when no windows are showing", "[on or off]", commandName, "windowAutoShow")){
    	 		MesquiteWindow.autoShow = MesquiteBoolean.fromOffOnString(ParseUtil.getFirstToken(arguments, stringPos));
    	 	}
    	 	else if (checker.compare(this.getClass(), "Returns string (\"on\", \"off\") whether the Mesquite window appears automatically when no windows are showing", null, commandName, "getWindowAutoShow")){
    	 		return MesquiteBoolean.toOffOnString(MesquiteWindow.autoShow);
    	 	}
	       	else if (checker.compare(this.getClass(), "Dumps the entire employee tree to the log", null, commandName, "dumpEmployeeTree")) {
	       		showLogWindow();
			logln("======= Module Employee Tree =======\n" + listEmployees("  ") + "\n================================");
	       	}
	       	else if (checker.compare(this.getClass(), "Shows some statistics that might be of interest to developers", null, commandName, "showStatistics")) {
	       		showLogWindow();
	       		dumpFileList(getRootPath() + "mesquite", "mesquite", " ");
			logln("MesquiteWindow.totalCheckDoomedCount " + MesquiteWindow.totalCheckDoomedCount);
			logln("--Classes painted\n" + MesquiteWindow.componentsPainted.recordsToString());
			logln("Listened.notifications " + Listened.notifications);
			logln("--Classes notifying\n" + Listened.classes.recordsToString());
			logln("--Classes notified\n" + Listened.classesNotified.recordsToString());
	       	}
	       	else if (checker.compare(this.getClass(), "Dumps to the log a list of those modules that are substantive and (prerelease or without citations)", null, commandName, "dumpSubstantivePrerelease")) {
				showLogWindow();
				logln("\nModules that are marked as substantive and prerelease:\n");
				int count = 0;
				if (isPrerelease()) {
					count++;
					logln(getClass().getName());
				}
				for (int i= 0; i<mesquiteModulesInfoVector.size(); i++){
					MesquiteModuleInfo mmi = (MesquiteModuleInfo)mesquiteModulesInfoVector.elementAt(i);
					if (mmi.isSubstantive() && mmi.isPrerelease()) {
						logln(mmi.getClassName());
						count++;
					}
				}
				if (count == 0)
					logln("   There are no modules marked as substantive and prerelease.\n");
				else
					logln("\n[number of substantive and prerelease: " + count + "]\n");
				if (false){ //these are useful to list to find those who should be asking to have their citations shown!
					logln("======= Substantive Modules that don't ask to have their citations shown =======");
					count = 0;
					for (int i= 0; i<mesquiteModulesInfoVector.size(); i++){
						MesquiteModuleInfo mmi = (MesquiteModuleInfo)mesquiteModulesInfoVector.elementAt(i);
						if (mmi.isSubstantive() && !mmi.showCitation()) {
							logln(mmi.getClassName());
							count++;
						}
					}
					logln("============ number substantive but without citation " + count + " ==============");
				}
	       	}	
	     
	       	else
			return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
   	 }
   	 
	/*.................................................................................................................*/
   	 private void dumpFileList(String path, String name, String spacer){
		File f = new File(path);  //
		if (!f.exists())
			return;
		if (name != null){
		 	if (f.isDirectory())
				MesquiteMessage.println(spacer + name + "--------------"); 
			else if (name.indexOf(".class")>=0)
				MesquiteMessage.println(spacer + name + "    " + (new Date(f.lastModified()))); //class files only
		}
	
		if (f.isDirectory()){  // is a directory; hence look inside at each item
			String[] filesList = f.list();
			for (int i=0; i<filesList.length; i++){
				dumpFileList(path+ MesquiteFile.fileSeparator + filesList[i], filesList[i], spacer + "   ");
			}
		}
   	 }
	/*.................................................................................................................*/
	public static void prepareMesquite(){
		if (!mesquiteTrunk.isApplet() && System.getProperty("java.version").startsWith("1.1")){
			MRJOSType textType = new MRJOSType("TEXT");
			MRJFileUtils.setDefaultFileType(textType);
		}
		windowVector = new ListableVector();
		dialogVector = new ListableVector();
		fileMenu=new MesquiteMenuSpec(null, "File", mesquiteTrunk);
		editMenu=new MesquiteMenuSpec(null, "Edit", mesquiteTrunk);
		windowsMenu=new MesquiteMenuSpec(null, "Window", mesquiteTrunk);
		charactersMenu = new MesquiteMenuSpec(null, "Characters", mesquiteTrunk);
		treesMenu = new MesquiteMenuSpec(null, "Taxa&Trees", mesquiteTrunk);
		analysisMenu = new MesquiteMenuSpec(null, "Analysis", mesquiteTrunk);
		helpMenu = new MesquiteMenuSpec(null, "Help", mesquiteTrunk);
		projects = new Projects();
		mesquiteTrunk.newFileCommand = makeCommand("newProject",  mesquiteTrunk);
		mesquiteTrunk.openFileCommand = makeCommand("openFile",  mesquiteTrunk);
		mesquiteTrunk.openURLCommand = makeCommand("openURL",  mesquiteTrunk);
		mesquiteTrunk.currentCommandCommand = makeCommand("currentCommand",  mesquiteTrunk);
		mesquiteTrunk.currentCommandCommand.setQueueBypass(true);
		mesquiteTrunk.pendingCommandsCommand = makeCommand("pendingCommands",  mesquiteTrunk);
		mesquiteTrunk.pendingCommandsCommand.setQueueBypass(true);
		mesquiteTrunk.resetMenusCommand = makeCommand("resetMenus",  mesquiteTrunk);
		mesquiteTrunk.showLicenseCommand = makeCommand("showLicense",  mesquiteTrunk);
		mesquiteTrunk.quitCommand = makeCommand("quit",  mesquiteTrunk);
		mesquiteTrunk.quitCommand.setQueueBypass(true);
		mesquiteTrunk.forceQuitCommand = makeCommand("forceQuit",  mesquiteTrunk);
		mesquiteTrunk.forceQuitCommand.setQueueBypass(true);
		mesquiteTrunk.showAllCommand = makeCommand("showAllWindows", mesquiteTrunk);
		mesquiteTrunk.closeAllCommand = makeCommand("closeAllProjects", mesquiteTrunk);
		mesquiteTrunk.saveAllCommand = makeCommand("saveAllProjects", mesquiteTrunk);

		about =new AboutWindow(mesquiteTrunk);
		about.setResizable(false);
		//about.setWindowSize(AboutWindow.totalWidth, AboutWindow.aboutHeight); 
		about.setQuitIfGoAway(true);
		
		MesquiteWindow.centerWindow(about);
		/*EMBEDDED	if embedded delete the following    */
		//about.addToWindow(mesquiteTrunk);
		/**/
		CommandChecker.registerClass(Mesquite.class, MesquiteModule.class);
		CommandChecker.registerClass(AboutWindow.class, MesquiteWindow.class);
		CommandChecker.registerClass(MesquitePanel.class, MesquitePanel.class);
		CommandChecker.registerClass(System.class, System.class);
	}
	
	private void registerMacHandlers(){
		if (MesquiteTrunk.isMacOSX() && MesquiteTrunk.getJavaVersionAsDouble()>1.39) {
			fileHandler = new EAWTHandler(this);
			((EAWTHandler)fileHandler).register();
		}
		else {
		    	fileHandler = new MRJHandler(this);
			try {
				MRJApplicationUtils.registerOpenDocumentHandler((MRJHandler)fileHandler);
				MRJApplicationUtils.registerQuitHandler((MRJHandler)fileHandler);
				MRJApplicationUtils.registerAboutHandler((MRJHandler)fileHandler);
				
			}
			catch (SecurityException e) {}
			catch (Exception e){
			}
			catch (Error e){
			}
		}
	}
	/*.................................................................................................................*/
	public static void main(String args[])
	{
		if (args !=null){
			for (int i=0; i<args.length; i++)
				if (args[i]!=null){
					if (args[i].equals("-w"))
						MesquiteWindow.suppressAllWindows = true;
					else if (args[i].equals("-b"))
						MesquiteTrunk.consoleListenSuppressed = true;
				}
		}
		MesquiteWindow.componentsPainted = new ClassVector();//for detecting efficiency problems
		Listened.classes = new ClassVector();//for detecting efficiency problems
		Listened.classesNotified = new ClassVector();//for detecting efficiency problems
		try { //code testing for GUI from Kevin Herrboldt via java-dev@lists.apple.com
			Frame foo = new Frame();
			
			if (foo==null)
				MesquiteWindow.GUIavailable = false;// no GUI available, need to be non-gui
			else {
				foo.pack();
				foo.getToolkit();       // will throw exception if unable to open GUI
			}
		} catch (Throwable t) {
			MesquiteWindow.GUIavailable = false;// no GUI available, need to be non-gui
		}
		try { 
			String mrj = System.getProperty("mrj.version");
			if (!StringUtil.blank(mrj) && (mrj.startsWith("3.0") || mrj.startsWith("3.1"))) //Java2D turned off for OS X prior to 10.2 because of bugs
				MesquiteWindow.Java2Davailable = false;
			else {
				BasicStroke bs = new BasicStroke();
				MesquiteWindow.Java2Davailable = bs !=null;
			}
		} catch (Throwable t) {
			MesquiteWindow.Java2Davailable = false;
		}
		try {
			incrementMenuResetSuppression();
			isApplication = true;
		    	
		      // create a new instance of this applet 
		      	mesquiteTrunk = new Mesquite();
		    	Mesquite mesq = (Mesquite)mesquiteTrunk;  //for easy of reference below
		      	mesq.registerMacHandlers();
			MainThread.mainThread = new MainThread();
			MainThread.mainThread.start();
		      	prepareMesquite();

			// initialize the applet
			mesquiteTrunk.init();
			//EMBEDDED: include this  
			((Mesquite)mesquiteTrunk).start(); 
			
			// open the files requested at startup
			if (args.length>0) {
				// report arguments
				String s = "Arguments: ";
				for ( int i = 0; i < args.length; i++ ) {
						s += " [ " + args[i] + " ]";
				}
				MesquiteTrunk.mesquiteTrunk.logln(s);

				for ( int i = 0; i < args.length; i++ ) {
					if (args[i]!=null && !args[i].startsWith("-"))
						mesquiteTrunk.openFile(args[i], new CommandRecord((CommandThread)null, false));
				}
			}
			
			mesquiteTrunk.resetAllMenuBars();
			decrementMenuResetSuppression();
		}
		catch (Exception e){
			if (!Mesquite.mesquiteExiting){
				e.printStackTrace();
				MesquiteMessage.warnProgrammer("Exception caught in Mesquite: " + e.getMessage());
				MesquiteModule.showLogWindow();
				PrintWriter pw = MesquiteFile.getLogWriter();
				if (pw!=null)
					e.printStackTrace(pw);
			}
		}
		catch (Error e){
			if (!Mesquite.mesquiteExiting){
				e.printStackTrace();
				MesquiteMessage.warnProgrammer("Error caught in Mesquite: " + e.getMessage());
				MesquiteModule.showLogWindow();
				PrintWriter pw = MesquiteFile.getLogWriter();
				if (pw!=null)
					e.printStackTrace(pw);
				throw e;
			}
			
		}
		if (about !=null && mesquiteTrunk.getProjectList()!=null && mesquiteTrunk.getProjectList().getNumProjects()>0 && defaultHideMesquiteWindow)
			about.hide();
	}

   	 
}
