/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.ancstates.AncestralStatesAll;

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;

/* ======================================================================== */
public class AncestralStatesAll extends CharsStatesForNodes {
	CharStatesForNodes assignTask;
	MCharactersHistory resultStates;
	MesquiteString assignTaskName;
	MesquiteCommand atC;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
 		assignTask = (CharStatesForNodes)hireEmployee(commandRec, CharStatesForNodes.class, "Reconstruction method");
		atC = makeCommand("setMethod",  this);
		assignTask.setHiringCommand(atC);

		assignTaskName = new MesquiteString(assignTask.getName());
		if (numModulesAvailable(CharStatesForNodes.class)>1){
			MesquiteSubmenuSpec mss = addSubmenu(null, "Reconstruction Method", atC, CharStatesForNodes.class);
			mss.setSelected(assignTaskName);
		}
 		if (assignTask == null){
 			return sorry(commandRec, getName() + " couldn't start because no reconstruction module was obtained");
 		}
 		return true;
 	}
 	
  	 public void employeeQuit(MesquiteModule m){
  	 	iQuit();
  	 }
	  	 public Snapshot getSnapshot(MesquiteFile file) {
   	 	Snapshot temp = new Snapshot();
  	 	temp.addLine("setMethod ",assignTask);
  	 	return temp;
  	 }
	public boolean allowsStateWeightChoice(){
		return false;
	}
/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Sets module used to reconstruct ancestral states", "[name of module]", commandName, "setMethod")) {
    	 		CharStatesForNodes temp=  (CharStatesForNodes)replaceEmployee(commandRec, CharStatesForNodes.class, arguments, "Reconstruction method", assignTask);
 			if (temp!=null) {
 				assignTask= temp;
				assignTask.setHiringCommand(atC);
				assignTaskName.setValue(assignTask.getName());
				parametersChanged(null, commandRec);
 			}
 			return assignTask;
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
   	 }
	/*...............................................................................................................*/
	public MCharactersHistory calculateStates(Tree tree, MCharactersDistribution observedStates, MCharactersHistory resultStates, MesquiteString resultString, CommandRecord commandRec){
		if (observedStates==null || tree == null)
			return null;
		resultStates=observedStates.adjustHistorySize(tree, resultStates);
	     	CharacterDistribution oneObserved = observedStates.getCharacterDistribution(0);
		CharacterHistory oneReconstructed = oneObserved.adjustHistorySize(tree, null);
		if (assignTask!=null) {
     	 		for (int ic=0; ic<observedStates.getNumChars(); ic++) {
	     	 		oneObserved = observedStates.getCharacterDistribution(ic);
				assignTask.calculateStates(tree, oneObserved, oneReconstructed, null, commandRec);
				resultStates.transferFrom(ic, oneReconstructed);
 			}
 		}
 		return resultStates;
	}
   
	/*.................................................................................................................*/
    	 public String getName() {
		return "Ancestral States All";
   	 }
	/*.................................................................................................................*/
   	 public boolean isSubstantive(){
   	 	return true;
   	 }
	/*.................................................................................................................*/
   	 public boolean isPrerelease(){
   	 	return false;
   	 }
	/*.................................................................................................................*/
   	 public boolean showCitation(){
   	 	return true;
   	 }
	/*.................................................................................................................*/
    	 public String getNameAndParameters() {
		if (assignTask == null)
			return getName();
		return "Reconstructed Ancestral States (" + assignTask.getName() + ")";
   	 }
	/*.................................................................................................................*/
   	 
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Reconstructs ancestral states at nodes for all characters supplied by character source." ;
   	 }
	/*.................................................................................................................*/
 	/** returns current parameters, for logging etc..*/
 	public String getParameters() {
 		if (assignTask == null)
 			return "";
		return "Reconstructed Ancestral States (" + assignTask.getName() + ")";
   	 }
   	 
}

