/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.ancstates.LabelStatesOnTree;

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;

/* ======================================================================== */
public class LabelStatesOnTree extends DisplayStatesAtNodes {
	TreeDisplay treeDisplay;
 	Vector labellers;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
 		labellers = new Vector();
 		return true;
	}
	/*.................................................................................................................*/
	public   TreeDecorator createTreeDecorator(TreeDisplay treeDisplay, TreeDisplayExtra ownerExtra) {
		LabelStatesDecorator newLabeller = new LabelStatesDecorator(this, treeDisplay, ownerExtra);
		labellers.addElement(newLabeller);
		return newLabeller;
	}
 	
	/*.................................................................................................................*/
	/** returns whether this module is requesting to appear as a primary choice */
   	public boolean requestPrimaryChoice(){
   		return true;  
   	}
	/*.................................................................................................................*/
 	public void endJob() {
		Enumeration e = labellers.elements();
		while (e.hasMoreElements()) {
			Object obj = e.nextElement();
			if (obj instanceof LabelStatesDecorator) {
				LabelStatesDecorator tCO = (LabelStatesDecorator)obj;
	 			tCO.turnOff();
	 		}
		}
 		super.endJob();
   	 }
	/*.................................................................................................................*/
   	 public boolean isSubstantive(){
   	 	return false;
   	 }
	/*.................................................................................................................*/
    	 public String getName() {
		return "Label states";
   	 }
   	 
	/*.................................................................................................................*/
   	 
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "A module that displays character states on tree using labels.  This is a display-only module,"
 		+ " and would be hired by another module that organizes assigning states to the nodes." ;
   	 }
	/*.................................................................................................................*/
   	 
}

/* ======================================================================== */
class LabelStatesDecorator extends TreeDecorator {
	LabelsAtNodes labelsAtNodes;
 	LabelStatesOnTree ownerModule;

	public LabelStatesDecorator (LabelStatesOnTree ownerModule, TreeDisplay treeDisplay, TreeDisplayExtra ownerExtra) {
		super(treeDisplay, ownerExtra);
		this.ownerModule=ownerModule;
	}
	/*.................................................................................................................*/
	public   void labelNode(int N,  Tree tree, CharacterHistory statesAtNodes, CharacterDistribution observedStates, Graphics g) {
		for (int d = tree.firstDaughterOfNode(N); tree.nodeExists(d); d = tree.nextSisterOfNode(d))
				labelNode(d, tree, statesAtNodes, observedStates, g);
		int nodeX = treeDisplay.getTreeDrawing().x[N];
		int nodeY = treeDisplay.getTreeDrawing().y[N];
		if (labelsAtNodes==null)
			return;
		MesquiteLabel c = (MesquiteLabel)labelsAtNodes.getPanel(N);
		if (c==null)
			return;
		if (statesAtNodes == null)
			c.setText("");
		else
			c.setText(statesAtNodes.toString(N, "\n")); 
		c.setLocation(nodeX, nodeY);
		if (!c.isVisible())
			c.setVisible(true);
		c.repaint();
		g.setColor(Color.black);
		/*Note: this does not yet show the observed states separately.  In light of the likely limited use of this compared to ShadeStates, it doesn't
		seem worth enhancing this too much.
		if (tree.nodeIsTerminal(N)) {
			if (observedStates !=null) {
				if (!observedStates.isUnassigned(tree.taxonNumberOfNode(N))&&!(observedStates.isInapplicable(tree.taxonNumberOfNode(N))))
					;
			}
			else
				;
		}
		*/
	}
	/*.................................................................................................................*/
	public   void drawOnTree(Tree tree, int drawnRoot, Object obj, Object obj2, Object obj3, Graphics g) {
		if (!(obj instanceof CharacterHistory))
			return;
		CharacterHistory statesAtNodes = (CharacterHistory)obj;
		CharacterDistribution observedStates = (CharacterDistribution)obj2;
		if (treeDisplay!=null && tree!=null && statesAtNodes!=null) {
				if (observedStates == null)
					observedStates = statesAtNodes.getObservedStates();
	 			if (labelsAtNodes==null) {
	 				labelsAtNodes = new LabelsAtNodes(ownerModule, tree.getNumNodeSpaces(), treeDisplay);
	 				labelsAtNodes.setTreeDisplay(treeDisplay);
	 			}
	 			else if (labelsAtNodes.getNumNodes()!=tree.getNumNodeSpaces() ) {
	 				labelsAtNodes.resetNumNodes(tree.getNumNodeSpaces());
	 			}
			
			statesAtNodes.prepareColors(tree, drawnRoot);
			labelsAtNodes.locatePanels(tree, drawnRoot);
			labelsAtNodes.showPanels(tree, drawnRoot);
			labelNode(drawnRoot, tree, statesAtNodes,observedStates, g);

		}
		else
			MesquiteMessage.warnProgrammer("Error: draw on tree label states//null");
	}
	
	/*.................................................................................................................*/
	public void turnOff() {
		if (labelsAtNodes!=null)
			labelsAtNodes.dispose();
		labelsAtNodes = null;
	}
}
