/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.assoc.StoredAssociations;
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;
import mesquite.assoc.lib.*;

/* ======================================================================== */
public class StoredAssociations extends AssociationSource implements MesquiteListener {
	int currentAssociationIndex=MesquiteInteger.unassigned;
	AssociationsManager manager;
	TaxaAssociation currentAssociation = null;
	Taxa currentTaxa = null;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		manager = (AssociationsManager)findElementManager(TaxaAssociation.class);
		if (manager==null) {
			return sorry(commandRec, getName() + " couldn't start because no associations manager was found.");
		}
		if (manager.getNumberOfAssociations(commandRec)==0) {
			return sorry(commandRec, "No stored associations between taxa are available.");
		}
		addMenuItem("Choose Taxa Association...", makeCommand("chooseAssociation", this));
		return true;
  	 }
   	 
   	 public boolean isPrerelease(){
   	 	return false;
   	 }
	/*.................................................................................................................*/
	public void changed(Object caller, Object obj, Notification notification, CommandRecord commandRec){
		if (Notification.appearsCosmetic(notification))
			return;
		parametersChanged(notification, commandRec);
	}
	/*.................................................................................................................*/
	public void disposing(Object obj) {
		if (obj == currentAssociation || obj == currentTaxa) {
			parametersChanged(null, CommandRecord.getRecNSIfNull());
		}
	}
	/*.................................................................................................................*/
	public boolean okToDispose(Object obj, int i) {
		//?? what to do here?
		return true;
	}
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) { 
   	 	if (!MesquiteInteger.isCombinable(currentAssociationIndex))
   	 		return null;
   	 	Snapshot temp = new Snapshot();
  	 	temp.addLine("setCurrentAssociation " + currentAssociationIndex); 
  	 	return temp;
  	 }
	MesquiteInteger pos = new MesquiteInteger();
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	 if (checker.compare(this.getClass(), "Sets the current association number", "[number of current association]", commandName, "setCurrentAssociation")) {
			int c= MesquiteInteger.fromFirstToken(arguments, pos);
			if (MesquiteInteger.isCombinable(c)) {
				currentAssociationIndex = c;
				parametersChanged(null, commandRec);
    	 		}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Choose the current association", null, commandName, "chooseAssociation")) {
			if (currentTaxa !=null) {
				TaxaAssociation t = chooseAssociation(containerOfModule(), currentTaxa, commandRec);
				if (t!=null)
					parametersChanged(null, commandRec);
				return t;
			}
			else
				discreetAlert(commandRec, "Sorry, a taxa association cannot be chosen unless a set of taxa has been previously indicated.  The fact that you get this message probably means that there has been a programming error.");
    	 	}
    	 	else {
 			return super.doCommand(commandName, arguments, commandRec, checker);
 		}
		return null;
   	 }
	/*.................................................................................................................*/
	/*.................................................................................................................*/
   	public TaxaAssociation getAssociation(Taxa taxa, int ic, CommandRecord commandRec) {
   		currentAssociationIndex=ic;
   		currentTaxa = taxa;
   		return getCurrentAssociation(taxa, commandRec);
   	}
	/*.................................................................................................................*/
   	public TaxaAssociation getCurrentAssociation(Taxa taxa, CommandRecord commandRec) { 
   		TaxaAssociation oldAssociation = currentAssociation;
   		currentTaxa = taxa;
    		if (!commandRec.scripting() && !MesquiteInteger.isCombinable(currentAssociationIndex) && getNumberOfAssociations(taxa, commandRec)>1)
   			currentAssociation  =  chooseAssociation(containerOfModule(), taxa, commandRec);
   		else {
   			if (getNumberOfAssociations(taxa, commandRec) ==1)
   				currentAssociationIndex = 0;
  	 		else if (currentAssociationIndex>=getNumberOfAssociations(taxa, commandRec) || currentAssociationIndex<0)
   				return  null;
	     		currentAssociation = manager.getAssociation(taxa, currentAssociationIndex, commandRec);
 		}
   		if (currentAssociation!=null && currentAssociation!=oldAssociation){
   			if (oldAssociation!=null)
   				oldAssociation.removeListener(this);
   			currentAssociation.addListener(this);
   		}
   		return currentAssociation;
   	}
 	/*.................................................................................................................*/
	public TaxaAssociation chooseAssociation(Frame frame, Taxa taxa, CommandRecord commandRec){
 		int numAssoc = getNumberOfAssociations(taxa, commandRec);
 		String[] assocNames = new String[numAssoc];
   		
   		currentAssociation = manager.getAssociation(taxa, currentAssociationIndex, commandRec);
 		for (int i=0; i<numAssoc;i++) 
 			assocNames[i] = manager.getAssociation(taxa, i, commandRec).getName();
		int whichAssoc = ListDialog.queryList(frame, "Select Association", "Select an association between two taxa blocks", MesquiteString.helpString, assocNames, -1);
		if (!MesquiteInteger.isCombinable(whichAssoc)) 
			return null;
		if (whichAssoc>=0) { 
			currentAssociationIndex = whichAssoc;
			return  manager.getAssociation(taxa, whichAssoc, commandRec);
		}
		else
			return manager.getAssociation(taxa, 0, commandRec);
	}
	/*.................................................................................................................*/
   	public int getNumberOfAssociations(Taxa taxa, CommandRecord commandRec) {
   		currentTaxa = taxa;
		return manager.getNumberOfAssociations(taxa, commandRec);
   	}
   
	/*.................................................................................................................*/
   	public String getAssociationNameString(Taxa taxa, int index, CommandRecord commandRec) {
   		currentTaxa = taxa;
		return manager.getAssociation(taxa, index, commandRec).getName();
   	}
	/*.................................................................................................................*/
    	 public String getName() {
		return "Stored Taxa Associations";
   	 }
   	 
	/*.................................................................................................................*/
  	 public String getParameters() {
  	 	if (currentAssociation !=null)
  	 		return "Current association: " + currentAssociation.getName();
		return null;
   	 }
	/*.................................................................................................................*/
  	 public String getExplanation() {
		return "Supplies associations between taxa that are stored, for instance in a file.";
   	 }
}
