/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.assoc.lib;

import java.awt.*;
import java.util.*;
import mesquite.lib.*;


/* ======================================================================== */
class TaxonWithTaxa{
	Taxon t = null;
	Taxa taxaB;
	Taxon[] associates;
	int numA = 0;
	public TaxonWithTaxa(Taxon t, Taxa taxaB){
		this.t = t;
		associates= new Taxon[taxaB.getNumTaxa()];
		this.taxaB = taxaB;
	}
	Taxon getAssociate(int i){
		if (i<0 || i>=associates.length)
			return null;
		return associates[i];
	}
	void addAssociate(Taxon tB){
		if (tB==null)
			return;
		for (int i=0; i<associates.length; i++){
			if (tB == associates[i])
				return;
			if (associates[i] == null) {
				associates[i] = tB;
				return;
			}
		}
	}
	void deleteDefunct(){
		for (int where = associates.length-1; where>=0; where--){
			Taxon tD = associates[where];
			if (tD!=null && taxaB.whichTaxonNumber(tD)<0)
				deleteAssociate(tD);
		}
	}
	void deleteAssociate(Taxon tB){
		if (tB==null)
			return;
		int where = findAssociate(tB);
		int last = findLastAssociate();
		if (where>-1) {
			associates[where] = null;
			if (last>-1) {
				associates[where] = associates[last];
				associates[last] = null;
			}
		}
	}

	void deleteAllAssociates(){
		for (int i=0; i<associates.length; i++)
			associates[i] = null;
	}
	
	int findAssociate(Taxon tB){
		for (int i=0; i<associates.length; i++){
			if (tB == associates[i])
				return i;
		}
		return -1;
	}
	int findLastAssociate(){
		for (int i=0; i<associates.length; i++){
			if (associates[i] == null) {
				return i-1;
			}
		}
		return -1;
	}
	int length(){
		return associates.length;
	}
	public String toString(){
		String s = "With " + t.getName() + ":";
		for (int i=0; i<associates.length; i++){
			if (associates[i] != null) {
				s += " \"" + associates[i].getName() + "\"";
			}
		}
		return s;
	}
}
/** .*/
public class TaxaAssociation extends FileElement  {
	//TODO: needs to listen to changes in taxa and respond to change numbers etc.
	Taxa taxaA, taxaB;
	TaxonWithTaxa[] associations;
	public TaxaAssociation(){
	}
 	/*.................................................................................................................*/
	public void setTaxa(Taxa taxa, int index){
		if (index==0)
			taxaA = taxa;
		else if (index==1)
			taxaB = taxa;
		else
			MesquiteMessage.warnProgrammer("Error: Taxa association allows only two Taxa objects");
		if (taxaA!=null && taxaB!=null) {
			associations= new TaxonWithTaxa[taxaA.getNumTaxa()];
			for (int i=0; i<getAssociatesLength(); i++) {
				associations[i] = new TaxonWithTaxa(taxaA.getTaxon(i), taxaB);
				for (int j = 0; j<getAssociatesLength0(); j++){
					setAssociated(i, j, false);
				}
			}
		}
		else
			associations = null;
	}
 	/*.................................................................................................................*/
	private boolean areAssociated(int a, int b){
		if (a >=0 && a < taxaA.getNumTaxa() && b >= 0 && b < taxaB.getNumTaxa()){
			return associations[a].findAssociate(taxaB.getTaxon(b))>=0;
		}
		return false;
	}
 	/*.................................................................................................................*/
	private void setAssociated(int a, int b, boolean assoc){
		if (a >=0 && a < taxaA.getNumTaxa() && b >= 0 && b < taxaB.getNumTaxa()) {
			if (assoc){
				associations[a].addAssociate(taxaB.getTaxon(b));
			}
			else {
				associations[a].deleteAssociate(taxaB.getTaxon(b));
			}
		}
	}
 	/*.................................................................................................................*/
	public void zeroAllAssociations(Taxon taxon){
		if (taxaA==null || taxaB == null)
			return;
			
		int a = taxaA.whichTaxonNumber(taxon);
		if (a>-1) { //taxon belongs to taxaA
			associations[a].deleteAllAssociates();
		}
		else {
			//taxon belongs to taxaB; remove all references to it
			for (int i = 0; i<taxaA.getNumTaxa(); i++)
				associations[i].deleteAssociate(taxon);
		}
	}
 	/*.................................................................................................................*/
	private int getAssociatesLength(){
		return taxaA.getNumTaxa();
	}
 	/*.................................................................................................................*/
	private int getAssociatesLength0(){
		return taxaB.getNumTaxa();
	}
 	/*.................................................................................................................*/
	public void resetTaxaAfterChange(){
		//go through deleting defunct taxa
		for (int a = 0; a < associations.length; a++) {
			if (taxaA.whichTaxonNumber(associations[a].t)<0)
				associations[a] = null;
			else {
				associations[a].deleteDefunct();
			}	
		}
		//change size of arrays as needed
		TaxonWithTaxa[] newAssociations= new TaxonWithTaxa[taxaA.getNumTaxa()];
		for (int i=0; i<taxaA.getNumTaxa(); i++)
			newAssociations[i] = new TaxonWithTaxa(taxaA.getTaxon(i), taxaB);
		
		int count = 0;
		for (int i=0; i<associations.length; i++) {
			if (associations[i]!=null){
				for (int j = 0; j<associations[i].length(); j++){
					newAssociations[count].addAssociate(associations[i].getAssociate(j));
				}
				count++;
			}
		}
		associations = newAssociations;
	}
	/*-----------------------------------------*/
	public String toString(){
		String names = "";
		if (taxaA!=null && taxaB!=null)
			names =  " involving " + taxaA.getName() + " and " + taxaB.getName();
		String s = "Taxa Association " + getName() + names+  "\n";
		for (int i=0; i<associations.length; i++) {
			if (associations[i]!=null){
				s += "  " + associations[i] + "\n";
			}
		}
		return s;
		
	}
	/*-----------------------------------------*/
	/** Returns explanation (what taxa involved)*/
	public String getExplanation(){
		String s ="This stores the association between taxa of ";
		if (taxaA==null || taxaB == null)
			s+= "two blocks";
		else 
			s+=  "the block \"" + taxaA.getName() + "\", having " + taxaA.getNumTaxa() + " taxa, with taxa of the block \"" + taxaB.getName() + "\", having " + taxaB.getNumTaxa() + " taxa";
		return s;
	}
	/*-----------------------------------------*/
	/** For HNode interface.  todo: this should be renamed getHNodeShow*/
	public boolean getHShow(){
		return true;
	}
	
	/*-----------------------------------------*/
	/** For HNode interface.*/
	public String getTypeName(){
		return "Taxa Association";
	}
 	/*.................................................................................................................*/
	public Taxa getTaxa(int index){
		if (index==0)
			return taxaA;
		else if (index==1)
			return taxaB;
		else
			MesquiteMessage.warnProgrammer("Error: Taxa association allows only two Taxa objects *");
		return null;
	}
 	/*.................................................................................................................*/
	public Taxa getOtherTaxa(Taxa taxa){
		if (taxa==taxaB)
			return taxaA;
		else if (taxa == taxaA)
			return taxaB;
		else
			MesquiteMessage.warnProgrammer("Error: taxa passed to getOtherTaxa not present *");
		return null;
	}
 	/*.................................................................................................................*/
	public void setAssociation(Taxon taxonA, Taxon taxonB, boolean associated){
		if (taxaA==null || taxaB == null)
			return;
		int a = taxaA.whichTaxonNumber(taxonA);
		int b;
	//enlarge matrices!
		if (a==-1) { //either illegal or second taxon belongs to first Taxa
			a = taxaB.whichTaxonNumber(taxonA);
			b = taxaA.whichTaxonNumber(taxonB);
			if (a!=-1 && b!=-1)
				setAssociated(b,a, associated);
		}
		else  {
			b = taxaB.whichTaxonNumber(taxonB);
			if (b!=-1)
				setAssociated(a, b, associated);
		}
	}
	
 	/*.................................................................................................................*/
	public boolean getAssociation(Taxon taxonA, Taxon taxonB){
		if (!includedTaxon(taxonA) || !includedTaxon(taxonA))
			return false;
		int a = taxaA.whichTaxonNumber(taxonA);
		int b;
		if (a==-1) { //either illegal or second taxon belongs to first Taxa
			a = taxaB.whichTaxonNumber(taxonA);
			b = taxaA.whichTaxonNumber(taxonB);
			if (a!=-1 && b!=-1)
				return areAssociated(b,a);
		}
		else  {
			b = taxaB.whichTaxonNumber(taxonB);
			if (b!=-1)
				return areAssociated(a, b);
		}
		return false;
	}
 	/*.................................................................................................................*/
	public int getNumAssociates(Taxon taxon){
		if (!includedTaxon(taxon))
			return 0;
		int which = taxaA.whichTaxonNumber(taxon);
		if (which==-1) { //taxon belongs to B; need to look down matrix
			which = taxaB.whichTaxonNumber(taxon);
			if (which==-1)
				return 0;
			int num = 0;
			for (int i=0; i<getAssociatesLength(); i++) //counting how many associates
				if (areAssociated(i, which))
					num++;
			return num;
		}
		else   { //taxon belongs to A; need to look across matrix
			int num = 0;
			for (int i=0; i<getAssociatesLength0(); i++) //counting how many associates
				if (areAssociated(which,i))
					num++;
			return num;
		}
	}
  	/*.................................................................................................................*/
  	/* this version avoids instantiation of Taxon array*/
	public Taxon[] getAssociates(Taxon taxon, Taxon[] result){
		
		if (!includedTaxon(taxon))
			return null;
		int which = taxaA.whichTaxonNumber(taxon);
		if (which==-1) { //taxon belongs to B; need to look down matrix
			which = taxaB.whichTaxonNumber(taxon);
			if (which==-1)
				return null;
			int num = 0;
			for (int i=0; i<getAssociatesLength(); i++) //counting how many associates
				if (areAssociated(i, which))
					num++;
			if (num==0)
				return null;
			if (result==null || result.length < num)
				result = new Taxon[num];
			for (int i=0; i<result.length; i++)
				result[i] = null;
			int count=0;
			for (int i=0; i<getAssociatesLength(); i++) //accumulating associates
				if (areAssociated(i, which)) {
					if (count<result.length)
						result[count] = taxaA.getTaxon(i);
					count++;
				}
			return result;
		}
		else   { //taxon belongs to A; need to look across matrix
			int num = 0;
			for (int i=0; i<getAssociatesLength0(); i++) //counting how many associates
				if (areAssociated(which,i))
					num++;
			if (num==0)
				return null;
			if (result==null || result.length < num)
				result = new Taxon[num];
			for (int i=0; i<result.length; i++)
				result[i] = null;
			int count=0;
			for (int i=0; i<getAssociatesLength0(); i++)  //accumulating associates
				if (areAssociated(which, i)) {
					if (count<result.length)
						result[count] = taxaB.getTaxon(i);
					count++;
				}
			return result;
		}
	}
	/*.................................................................................................................*/
	public Taxon[] getAssociates(Taxon taxon){
		if (!includedTaxon(taxon))
			return null;
		int which = taxaA.whichTaxonNumber(taxon);
		if (which==-1) { //taxon belongs to B; need to look down matrix
			which = taxaB.whichTaxonNumber(taxon);
			if (which==-1)
				return null;
			int num = 0;
			for (int i=0; i<getAssociatesLength(); i++) //counting how many associates
				if (areAssociated(i, which))
					num++;
			if (num==0)
				return null;
			Taxon[] result = new Taxon[num];
			int count=0;
			for (int i=0; i<getAssociatesLength(); i++) //accumulating associates
				if (areAssociated(i, which))
					result[count++] = taxaA.getTaxon(i);
			return result;
		}
		else   { //taxon belongs to A; need to look across matrix
			int num = 0;
			for (int i=0; i<getAssociatesLength0(); i++) //counting how many associates
				if (areAssociated(which,i))
					num++;
			if (num==0)
				return null;
			Taxon[] result = new Taxon[num];
			int count=0;
			for (int i=0; i<getAssociatesLength0(); i++)  //accumulating associates
				if (areAssociated(which,i))
					result[count++] = taxaB.getTaxon(i);
			return result;
		}
	}
	public boolean includedTaxon(Taxon taxon){
		if (taxaA==null || taxaB == null)
			return false;
		int which = taxaA.whichTaxonNumber(taxon);
		if (which==-1) { //taxon belongs to B; need to look down matrix
			which = taxaB.whichTaxonNumber(taxon);
			if (which==-1 || which >= getAssociatesLength0())
				return false;
		}
		else   { //taxon belongs to A; need to look across matrix
			if (which >= getAssociatesLength())
				return false;
		}
		return true;
	}
}
