/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.basic.ManageNotesBlock;
/*~~  */

import java.util.*;
import java.awt.*;
import java.io.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;

/** Manages the NOTES block of a NEXUS file */
public class ManageNotesBlock extends FileInit {
	Vector blocks;
	int numBlocks =0;
	public Class getDutyClass(){
		return ManageNotesBlock.class;
	}
	/*.................................................................................................................*/
	public NexusBlockTest getNexusBlockTest(){ return new NotesBlockTest();}
	/*.................................................................................................................*/

	/*.................................................................................................................*/
   	 public boolean isSubstantive(){
   	 	return false;
   	 }
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		return true;
  	 }
	/*.................................................................................................................*/
 	/** A method called immediately after the file has been read in or completely set up (if a new file).*/
 	public void fileReadIn(MesquiteFile f) {
 		NexusBlock[] bs = getProject().getNexusBlocks(NotesBlock.class, f); //added Dec 01
		if (bs == null || bs.length ==0){ 
			NotesBlock ab = new NotesBlock(f, this);
			numBlocks++;
			addNEXUSBlock(ab);
		}
		
	}

	private NotesBlock makeBlock(MesquiteFile file, String contents) {
		NotesBlock b = new NotesBlock(file, this);
		b.setName("Notes Block");
		numBlocks++;
		if (contents !=null)
			b.setText(contents);
		if (blocks==null)
			blocks = new Vector();
		blocks.addElement(b);
		return b;
	}
/*
Begin Notes;
	Text [Taxon=taxon-set] [Character=character-set]
			[State=state-set] [Tree=tree-set]
			Source={Inline|File|Resource} 
			Text=text or source descriptor;
	Picture [Taxon=taxon-set] [CHARACTER=character-set]
			[State=state-set] [Tree=tree-set] 
			[Format={PICT|TIFF|EPS|JPEG|GIF}] 
			[Encode={None|UUEncode|BinHex}]
			Source={Inline|File|Resource} 
			Picture=picture-or-source-descriptor;
End;
	/*.................................................................................................................*/

	public NexusBlock readNexusBlock(MesquiteFile file, String name, FileBlock block, StringBuffer blockComments, String fileReadingArguments, CommandRecord commandRec){
		NotesBlock b = makeBlock(file, block.toString());
		int c = 0;
		String s;
		MesquiteString comment = new MesquiteString();
		while (!StringUtil.blank(s=block.getCommand(c++, comment))) {
			String commandName = parser.getFirstToken(s);
			if (!commandName.equalsIgnoreCase("BEGIN") && !commandName.equalsIgnoreCase("END") && !commandName.equalsIgnoreCase("ENDBLOCK"))
				readUnrecognizedCommand(file, b, "NOTES", block, commandName, s, blockComments, comment);
		}
		return b;
	}
	/*.................................................................................................................*/
	public String employeesGetCommands(MesquiteModule module, MesquiteFile mf) {
		Enumeration enum=module.getEmployeeVector().elements();
		MesquiteModule employee;
		String commands="";
		while (enum.hasMoreElements()){
			Object obj = enum.nextElement();
			employee = (MesquiteModule)obj;
			String command =employee.getNexusCommands(mf, "NOTES");
			if (!StringUtil.blank(command))
				commands+=command + StringUtil.lineEnding();
			commands+=employeesGetCommands(employee, mf);
		}
		return commands;
	}
	/*.................................................................................................................*/
    	 public String getName() {
		return "Manage NOTES blocks";
   	 }
   	 
	/*.................................................................................................................*/
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Manages footnotes and pictures attached to taxa, characters and data points (including read/write NOTES block in NEXUS file)." ;
   	 }
}
	
class NotesBlock extends NexusBlock {
	String blockContents = "";
	public NotesBlock(MesquiteFile f, MesquiteModule mb){
		super(f, mb);
	}
	public void written() {
		MesquiteMessage.warnProgrammer("written Notes");
	}
	public boolean mustBeAfter(NexusBlock block){
		if (block==null)
			return false;
		return (block.getBlockName().equalsIgnoreCase("TAXA") ||  block.getBlockName().equalsIgnoreCase("TREES") ||block.getBlockName().equalsIgnoreCase("CHARACTERS"));
		
	}
	public String getBlockName(){
		return "NOTES";
	}
	public boolean contains(FileElement e) {
		return false;
	}
	public void setText(String contents) {
		blockContents = contents;
	}
	public String getText() {
		return blockContents;
	}
	public String getName(){
		return "Notes block";
	}
	public String getNEXUSBlock(){
		String contents = ((ManageNotesBlock)getManager()).employeesGetCommands(getManager().getFileCoordinator(), getFile());
		String unrec =getUnrecognizedCommands();
		if (StringUtil.blank(contents) && StringUtil.blank(unrec))
			return null;
		String blocks="BEGIN NOTES;" + StringUtil.lineEnding()+ contents + StringUtil.lineEnding()+ unrec + StringUtil.lineEnding() + "END;" + StringUtil.lineEnding();
		return blocks;
	}
	//remember to store unrecognixed commands
}
/* ======================================================================== */
class NotesBlockTest extends NexusBlockTest  {
	public NotesBlockTest () {
	}
	public  boolean readsWritesBlock(String blockName, FileBlock block){ //returns whether or not can deal with block
		return blockName.equalsIgnoreCase("NOTES");
	}
}

