/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.basic.ManageTaxaPartitions;
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;

/** Manages specifications of partitions of taxa, including reading and writing from NEXUS file. */
public class ManageTaxaPartitions extends SpecsSetManager {
	TaxaGroupVector groups; //TODO: dealing with taxa groups should probably be a separate module to allow it to be recognized as element manager
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		groups = new TaxaGroupVector();
 		return true;
	}
	
	
	/*.................................................................................................................*/
	public MesquiteModule showElement(FileElement e, CommandRecord commandRec){
		//TODO:
		if (e != null)
			alert("Sorry, the " + e.getTypeName() + "  cannot be shown by this means yet.");
		return null;
	}
	public NexusBlock elementAdded(FileElement e){
		if (e instanceof TaxaGroup){
			if (groups.indexOf(e)<0) {
				groups.addElement(e, false);
			}
			e.setManager(this);
			return null;
		}
		else if (e!=null && e instanceof Taxa){
			e.prepareSpecsSetVector(TaxaPartition.class);
		}
		return null;
	}
	public void elementDisposed(FileElement e){
		if (groups !=null)
			groups.removeElement(e, false);
	}
	public Class getElementClass(){
		return TaxaPartition.class;
	}
	/*.................................................................................................................*/
 	/** A method called immediately after the file has been read in.*/
 	public void projectEstablished() {
		MesquiteSubmenuSpec mmis = getFileCoordinator().addSubmenu(MesquiteTrunk.treesMenu,"List of Taxa Partitions", makeCommand("showPartitions",  this), (ListableVector)getProject().taxas);
		mmis.setOwnerModuleID(getID());
		mmis.setBehaviorIfNoChoice(MesquiteSubmenuSpec.ONEMENUITEM_ZERODISABLE);
		groups.addToFile(getProject().getHomeFile(), getProject(), this);
		super.projectEstablished();
 	}
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) { 
   	 	Snapshot temp = new Snapshot();
		for (int i = 0; i<getNumberOfEmployees(); i++) {
			MesquiteModule e=(MesquiteModule)getEmployeeVector().elementAt(i);
			if (e instanceof ManagerAssistant && (e.getModuleWindow()!=null) && e.getModuleWindow().isVisible() && e.getName().equals("List of Taxa Partitions")) {
				Object o = e.doCommand("getTaxa", null, CommandRecord.getRecNSIfNull(), CommandChecker.defaultChecker);
				
				if (o !=null && o instanceof Taxa) {
					//int wh =getProject().getTaxaReference((Taxa)o);
  	 				temp.addLine("showPartitions " + getProject().getTaxaReference((Taxa)o), e); 
  	 			}
  	 			else
  	 				temp.addLine("showPartitions ", e); 
  	 		}
		}
  	 	return temp;
  	 }
  	 
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Shows lists of the taxa partitions", null, commandName, "showPartitions")) {
    	 		if (StringUtil.blank(arguments)) {
	    	 		for (int i = 0; i< getProject().getNumberTaxas(checker.getFile()); i++) {
	    	 			showSpecsSets(getProject().getTaxa(checker.getFile(), i), "List of Taxa Partitions", commandRec);
				}
    	 		}
    	 		else {
    	 			Taxa t = getProject().getTaxa(checker.getFile(), parser.getFirstToken(arguments));
 	 			if (t!=null ) {
 	 				return showSpecsSets(t, "List of Taxa Partitions", commandRec);
 	 			}
    	 		}
//    	 			alert("Sorry, there are no taxa partitions");
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
   	 }
   
	private TaxaGroup findGroup(String token){ 
		if (token ==null)
			return null;
		for (int i = 0; i< groups.size(); i++){
			TaxaGroup cg = (TaxaGroup)groups.elementAt(i);
			if (token.equalsIgnoreCase(cg.getName())){
				return cg;
			}
		}
		return null;
	}
	
	private TaxaGroup makeGroup(String name, Parser subcommands, MesquiteFile file){
			TaxaGroup group = findGroup(name);
			if (group==null) {
				group = new TaxaGroup();
				group.setName(name);
				group.addToFile(file, getProject(), this);
				if (groups.indexOf(group)<0) 
					groups.addElement(group, false);
			}
			if (subcommands !=null){ //this should be passed into group to handle?
				String token = null;
				while ((token = subcommands.getNextToken())!=null){
					if (token.equalsIgnoreCase("COLOR")){
						token = subcommands.getNextToken(); //=
						token = subcommands.getNextToken(); // (
						token = subcommands.getNextToken(); // (
							if (token!=null && token.equalsIgnoreCase("RGB")) {
								double red = MesquiteDouble.fromString(subcommands.getNextToken()); //Red
								double green = MesquiteDouble.fromString(subcommands.getNextToken()); //green
								double blue = MesquiteDouble.fromString(subcommands.getNextToken()); //blue
								if (MesquiteDouble.isCombinable(red) && MesquiteDouble.isCombinable(green) && MesquiteDouble.isCombinable(blue)){
									Color c = new Color((float)red, (float)green, (float)blue);
									group.setColor(c);
								}
							}
					}
				}
			}
			return group;
	}
	/*.................................................................................................................*/
	String nexusStringForSpecsSet(TaxaPartition taxaPartition, Taxa taxa, MesquiteFile file, boolean isCurrent){
		String s= "";
		if (taxaPartition !=null && (taxaPartition.getFile()==file || (taxaPartition.getFile()==null && taxa.getFile()==file))) {
			String sT = " ";
			TaxaGroup[] parts = taxaPartition.getGroups();
			boolean firstTime =true;
			
			if (parts!=null)
				for (int i=0; i<parts.length; i++) {
					String q = ListableVector.getListOfMatches((Listable[])taxaPartition.getProperties(), parts[i], 1);
					if (q != null) {
						if (!firstTime)
							sT += ", ";
						firstTime = false;
						sT += StringUtil.tokenize(parts[i].getName()) + " : " + q ;
					}
				}
			
			if (!StringUtil.blank(sT)) {
				s+= "TAXPARTITION " ;
				if (isCurrent)
					s += "* ";
				s+= StringUtil.tokenize(taxaPartition.getName()) + " ";
				if (file.getProject().getNumberTaxas()>1)
					s+= " (TAXA = " +  StringUtil.tokenize(taxa.getName()) + ")";
				s+= " = "+  sT + ";" + StringUtil.lineEnding();
			}
		}
		return s;
   	}
	/*.................................................................................................................*/
	public String getNexusCommands(MesquiteFile file, String blockName){ 
		if (blockName.equalsIgnoreCase("LABELS")) {
			String s = "";
			for (int i = 0; i< groups.size(); i++){
				TaxaGroup cg = (TaxaGroup)groups.elementAt(i);
				if (cg.getFile() == file){
					s += "TAXAGROUPLABEL " + ParseUtil.tokenize(cg.getName());
					if (cg.colorSet()){
						Color c = cg.getColor();
						s += " COLOR = (RGB " + MesquiteDouble.toString(c.getRed()/255.0) + " " + MesquiteDouble.toString(c.getGreen()/255.0) + " " + MesquiteDouble.toString(c.getBlue()/255.0) + ") ";
					}
					s += ";" + StringUtil.lineEnding();
				}
			}
			if (StringUtil.blank(s))
				return null;
			else
				return s;
		}
		else  if (blockName.equalsIgnoreCase("SETS")) {
			String s= "";
	 		for (int ids = 0; ids<file.getProject().getNumberTaxas(); ids++) {

				Taxa taxa =  file.getProject().getTaxa(ids);
				int numSets = taxa.getNumSpecsSets(TaxaPartition.class);
				SpecsSetVector ssv = taxa.getSpecSetsVector(TaxaPartition.class);
				if (ssv!=null){
						TaxaPartition ms = (TaxaPartition)taxa.getCurrentSpecsSet(TaxaPartition.class);
						if (ms!=null && (ms.getNexusBlockStored()==null || blockName.equalsIgnoreCase(ms.getNexusBlockStored()))) {
							ms.setNexusBlockStored(blockName);
							ms.setName("UNTITLED");
							s += nexusStringForSpecsSet(ms, taxa, file, true);
						}
					
						
					for (int ims = 0; ims<numSets; ims++) {
						s += nexusStringForSpecsSet((TaxaPartition)taxa.getSpecsSet(ims, TaxaPartition.class), taxa, file, false);
					}
				}

			}
			return s;
		}
		return null;
	}
	/*.................................................................................................................*/
	public boolean readNexusCommand(MesquiteFile file, String blockName, String command, MesquiteString comment){ 
		if (blockName.equalsIgnoreCase("LABELS")) {
			String commandName = parser.getFirstToken(command);
			if ("TAXAGROUPLABEL".equalsIgnoreCase(commandName)) {
				String name = parser.getNextToken();
				makeGroup(name, parser, file); //pass whole command
				return true;
			}
			return false;
		}
		else if (blockName.equalsIgnoreCase("SETS") || blockName.equalsIgnoreCase("ASSUMPTIONS")) {
			MesquiteInteger startCharT = new MesquiteInteger(0);

			String commandName = ParseUtil.getToken(command, startCharT);
			if (commandName.equalsIgnoreCase("TAXAPARTITION") || commandName.equalsIgnoreCase("TAXPARTITION")) {
				String token = ParseUtil.getToken(command, startCharT);
				boolean isDefault = false;
				if ("*".equals(token)) {
					token = ParseUtil.getToken(command, startCharT);
					isDefault = true;
				}
				String nameOfTypeset = token; // name of typeset 
				token = ParseUtil.getToken(command, startCharT);
				String paradigmString = null;
				Taxa taxa = null;
				if (token.equalsIgnoreCase("(")) {
					token = ParseUtil.getToken(command, startCharT); //TAXA  //TODO: check to see what parameter is being set!
					token = ParseUtil.getToken(command, startCharT); //=
					token = (ParseUtil.getToken(command, startCharT)); // name of data
					taxa = file.getProject().getTaxa(token); //taxa is named; seek without restriction to current file
					token = ParseUtil.getToken(command, startCharT); //)
					token = ParseUtil.getToken(command, startCharT);  //=
				}
				else if (file.getProject().getNumberTaxas(file)>0)//taxa not named; seek with restriction to current file
					taxa = file.getProject().getTaxa(file, 0);
				else  
					taxa = file.getProject().getTaxa(0);

				if (taxa == null)
					return false;
				
				if (token.equals("="))
					token = ParseUtil.getToken(command, startCharT); 

				TaxaGroup defaultProperty =  null;
	
				TaxaGroup lastCM =makeGroup(token, null, file);
				//lastCM.setGroupNumber(MesquiteInteger.fromString(token));
		 		TaxaPartition taxaPartition= new TaxaPartition(nameOfTypeset, taxa.getNumTaxa(), defaultProperty, taxa);
		 		taxaPartition.setNexusBlockStored(blockName);

				int lastChar = -1;
				boolean join = false;
				boolean nextIsPart = false;
				while (!token.equals(";") && token.length()>0) {
					if (token.equals("-")) {
						if (lastChar!=-1)
							join = true;
					}
					else if (token!=null) {
					
						if (token.startsWith("-")) {
							if (lastChar!=-1)
								join = true;
							token = token.substring(1, token.length());
						}
						if (token.equals(":")) {
							nextIsPart = true;
						}
						else if (token.equals(","))
							nextIsPart=false;
						else if (nextIsPart) {
							//MesquiteInteger cm = new MesquiteInteger(MesquiteInteger.fromString(token));
							int whichTaxon = Taxon.toInternal(MesquiteInteger.fromString(token, false));
							if (MesquiteInteger.isCombinable(whichTaxon)) {
								if (join) {
									for (int j = lastChar; j<=whichTaxon; j++) {
										taxaPartition.setProperty(lastCM,j);
									}
									join = false;
									lastChar = -1;
								}
								else {
									lastChar = whichTaxon;
									taxaPartition.setProperty(lastCM,whichTaxon);
								}
							}
						}
						else {
							lastCM = makeGroup(token, null, file);
							/*int whichGroup = MesquiteInteger.fromString(token, false);
							if (!MesquiteInteger.isCombinable(whichGroup))
								whichGroup = ColorDistribution.getStandardColorNumber(token);
							lastCM.setGroupNumber(whichGroup);
							*/
							nextIsPart = true;
						}
					}
					token = ParseUtil.getToken(command, startCharT); 
				}
				
				if (isDefault) { //todo: OR IS FIRST
					if (!"UNTITLED".equals(taxaPartition.getName())) {
			 			taxa.storeSpecsSet(taxaPartition, TaxaPartition.class);
			 		}
			 		taxaPartition.addToFile(file, getProject(), this);
			 		SpecsSet ss = taxaPartition.cloneSpecsSet();
			 		ss.addToFile(file, getProject(), this);
					taxa.setCurrentSpecsSet(ss, TaxaPartition.class);
				}
				else {
		 			taxa.storeSpecsSet(taxaPartition, TaxaPartition.class);
			 		taxaPartition.addToFile(file, getProject(), this);
			 	}
			 	return true;
			}
		}
		return false;
	}
	/*.................................................................................................................*/
	public NexusCommandTest getNexusCommandTest(){ 
		return new TPartNexusCommandTest();
	}
	/*.................................................................................................................*/
    	 public String getName() {
		return "Manage taxa partititions";
   	 }
   	 
	/*.................................................................................................................*/
   	 
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Manages (including NEXUS read/write) taxa partitions." ;
   	 }
	/*.................................................................................................................*/
   	 
}

class TPartNexusCommandTest  extends NexusCommandTest{
	public boolean readsWritesCommand(String blockName, String commandName, String command){  //returns whether or not can deal with command
		if ((blockName.equalsIgnoreCase("SETS") || blockName.equalsIgnoreCase("ASSUMPTIONS"))  && (commandName.equalsIgnoreCase("TAXAPARTITION") ||commandName.equalsIgnoreCase("TAXPARTITION")))
			return true;
		if ((blockName.equalsIgnoreCase("LABELS")) && commandName.equalsIgnoreCase("TAXAGROUPLABEL"))
			return true;
		return false;
	}
}

