/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.categ.ManageCategoricalChars;

import java.util.*;
import java.awt.*;
import java.io.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import mesquite.categ.lib.*;

/* ======================================================================== 
Manages matrices of categorical characters (excluding molecular sequences)*/
public class ManageCategoricalChars extends CharMatrixManager {
	
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName){
		return true;
	}
	
	/*.................................................................................................................*/
	public boolean isPrerelease(){
		return false;
	}
	public Class getDataClass(){
		return CategoricalData.class;
	}
	/*.................................................................................................................*/
	public String getDataClassName(){
		return "Standard Categorical Data";
	}
	/*.................................................................................................................*/
	public  CharacterData getNewData(Taxa taxa, int numChars){
		return new CategoricalData(this, taxa.getNumTaxa(), numChars, taxa);
	}
	/*.................................................................................................................*/
	public boolean readsWritesDataType(Class dataClass){
		return (dataClass == CategoricalData.class);
	}
	/*.................................................................................................................*/
	public boolean readsWritesDataType(String dataType){
		return dataType.equalsIgnoreCase("Standard");
	}
	/*.................................................................................................................*/
	public CharacterData processFormat(MesquiteFile file, Taxa taxa, String dataType, String formatCommand, MesquiteInteger stringPos, int numChars) {
		MesquiteProject proj=null;
		if (file!=null)
			proj = file.getProject();
		CategoricalData data= new CategoricalData(this, taxa.getNumTaxa(), numChars, taxa);
		String tok = ParseUtil.getToken(formatCommand, stringPos);
		while (!tok.equals(";")) {
			if (tok.equalsIgnoreCase("TRANSPOSE")) {
				alert("Sorry, Transposed matrices of categorical characters can't yet be read");
				return null;
			}
			else if (tok.equalsIgnoreCase("interleave")) {
				int sp = stringPos.getValue();
				String e = ParseUtil.getToken(formatCommand, stringPos); //eating up = ?
				if ("=".equals(e)){
					String y = ParseUtil.getToken(formatCommand, stringPos); //yes or no ?
					data.interleaved = ("yes".equalsIgnoreCase(y));
						
				}
				else {
					stringPos.setValue(sp);
					data.interleaved = true;
				}
			}
			else if (tok.equalsIgnoreCase("MISSING")) { 
				ParseUtil.getToken(formatCommand, stringPos); //eating up =
				String t = ParseUtil.getToken(formatCommand, stringPos);
				if (t!=null && t.length()==1)
					data.setUnassignedSymbol(t.charAt(0));
			}
			else if (tok.equalsIgnoreCase("DATATYPE")) { 
				alert("Error in NEXUS file format: DATATYPE subcommand must appear as the first item in the FORMAT command");
				return null;
			}
			else if (tok.equalsIgnoreCase("GAP")) { 
				ParseUtil.getToken(formatCommand, stringPos); //eating up =
				String t = ParseUtil.getToken(formatCommand, stringPos);
				if (t!=null && t.length()==1)
					data.setInapplicableSymbol(t.charAt(0));
			}
			else if (tok.equalsIgnoreCase("MATCHCHAR")) { 
				ParseUtil.getToken(formatCommand, stringPos); //eating up =
				String t = ParseUtil.getToken(formatCommand, stringPos);
				if (t!=null && t.length()==1)
					data.setMatchChar(t.charAt(0));
			}
			else if (tok.equalsIgnoreCase("SYMBOLS")) { 
				ParseUtil.getToken(formatCommand, stringPos); //eating up =
				String t = ParseUtil.getToken(formatCommand, stringPos); //eating up "
				if (t!=null && t.equalsIgnoreCase("\"")){
					t = ParseUtil.getToken(formatCommand, stringPos); //getting next token
					int count = 0;
					while (t!=null && !t.equals("\"") &&  !t.equals(";")){ 
						for (int i = 0; i<t.length(); i++){ //process entire string
							char c = t.charAt(i);
							if (!StringUtil.whitespace(c, null)) {
								if (count<=CategoricalState.maxCategoricalState)
									data.setSymbol(count, c);
								count++;
							}
						}
						t = ParseUtil.getToken(formatCommand, stringPos); //eating up "
					}
				}
			}
			else {
				alert("Unrecognized token (\"" + tok+ "\") in FORMAT statement of categorical matrix; matrix will be stored as foreign, and not processed.");
				return null;
			}
			tok = ParseUtil.getToken(formatCommand, stringPos);
		}
		return data;
	}
	/*.................................................................................................................*/
	public boolean processCommand(CharacterData data, String commandName, String commandString) {
		if (commandName.equalsIgnoreCase("CHARSTATELABELS")){
			String cN = parser.getFirstToken(commandString); //eating up command name
			cN = parser.getNextToken();
			String charName;
			String stateName;
			while (!StringUtil.blank(cN) && !cN.equals(";") ) {
				int charNumber =MesquiteInteger.fromString(cN);
				charName = parser.getNextToken();  
				if (!charName.equals(",") && !charName.equals(";")) {
					if (!charName.equals("/")) {   // there is a character name
						data.setCharacterName(charNumber-1, charName);
						stateName = parser.getNextToken(); // eating up slash
					}
					else   // there is no character name, only a slash
						stateName = "";
					int stateNumber = 0;
					while (stateName!=null && !stateName.equals(",") && !stateName.equals(";")) {
						stateName = parser.getNextToken();
						if (!stateName.equals(",") && !stateName.equals(";") && data instanceof CategoricalData) 
							((CategoricalData)data).setStateName(charNumber-1, stateNumber, stateName);
						stateNumber++;
					}
				}
				if (charName.equals(";"))    // we've encountered the semicolon
					cN = ";";
				else
					cN = parser.getNextToken();
			}
			return true;
		}
		else if (commandName.equalsIgnoreCase("STATELABELS")){
			String cN = parser.getFirstToken(commandString); //eating up command name
			cN = parser.getNextToken();
			while (!StringUtil.blank(cN) && !cN.equals(";") ) {
				int charNumber =MesquiteInteger.fromString(cN);

				String stateName;
				int stateNumber = 0;
				while ((stateName = parser.getNextToken())!=null && !stateName.equals(",") && !stateName.equals(";")) {
					if (!stateName.equals(",")&&!stateName.equals(";") && data instanceof CategoricalData) 
						((CategoricalData)data).setStateName(charNumber-1, stateNumber, stateName);
					stateNumber++;
				}
				
				cN = parser.getNextToken();
			}
			return true;
		}
		return false;
	}
	/*.................................................................................................................*/
	public String getCharStateLabels(CharacterData data) {
		String csl = "\tCHARSTATELABELS " + StringUtil.lineEnding();
		boolean found = false;
		String end = " "; //StringUtil.lineEnding()
		CategoricalData dData = (CategoricalData)data;
		for (int i = 0; i<data.getNumChars(); i++) {
			String cslC="";
			if (i>0 && found)
				cslC += "," + end;
			else if (i ==0)
				cslC += "\t\t";
			cslC += Integer.toString(i+1) + " ";    //i+1 to convert to 1 based
			boolean foundInCharacter = false;
			if (data.characterHasName(i)) {
				foundInCharacter = true;
				cslC += StringUtil.tokenize(data.getCharacterName(i));
			}
			if (dData.hasStateNames(i)) {
				cslC +=   " / ";
				for (int s = 0; s<CategoricalState.maxCategoricalState; s++) {
					if (dData.hasStateName(i,s)) {
						String sn = dData.getStateName(i, s);
						if (sn!=null) {
							foundInCharacter = true;
							cslC += " " + StringUtil.tokenize(sn);
						}
					}
				}
			}
			if (foundInCharacter) {
				csl += cslC;
				found = true;
			}
		}
		if (found)
			return csl + " ; " + StringUtil.lineEnding();
		else
			return "";
	}
	/*.................................................................................................................*/
	public void writeCharactersBlock(CharacterData data, CharactersBlock cB, MesquiteFile file, ProgressIndicator progIndicator){
		String endLine =";" + StringUtil.lineEnding();
		StringBuffer blocks = new StringBuffer(2*data.getNumChars());
		if (data.getFile()!=null && data.getFile().useDataBlock)
			blocks.append("BEGIN DATA");
		else
			blocks.append("BEGIN CHARACTERS");
		blocks.append(endLine);
		if (data.getName()!=null &&  (getProject().getNumberCharMatrices()>1 || !NexusBlock.suppressTITLE)){
			blocks.append("\tTITLE  ");
			blocks.append( StringUtil.tokenize(data.getName()));
			blocks.append(endLine);
		}
		//if (data.getTaxa().getName()!=null  && getProject().getNumberTaxas(cB.getFile())>1){ //before 13 Dec 01 had been this
		if (data.getTaxa().getName()!=null  && getProject().getNumberTaxas()>1){ // should have an isUntitled method??
			blocks.append("\tLINK TAXA = ");
			blocks.append(StringUtil.tokenize(data.getTaxa().getName()));
			blocks.append(endLine);
		}
		if (data.getAnnotation()!=null) {
			blocks.append("[!" + data.getAnnotation() + "]");
			blocks.append(StringUtil.lineEnding());
		}
		blocks.append("\tDIMENSIONS NCHAR=" + data.getNumChars());
		blocks.append(endLine);
		blocks.append("\tFORMAT");
		blocks.append(" DATATYPE = STANDARD");
		blocks.append(" GAP = " + data.getInapplicableSymbol() + " MISSING = " + data.getUnassignedSymbol());
		blocks.append(" SYMBOLS = \" ");
		int maxS = ((CategoricalData)data).getMaxSymbolDefined();
		if (maxS < 1)
			maxS = 1;
		for (int i=0; i<=maxS; i++)
			blocks.append(" " + ((CategoricalData)data).getSymbol(i));
		blocks.append("\"");
		blocks.append(endLine);
		if (data.isLinked()){
			blocks.append("\tOPTIONS ");
			Vector ds = data.getDataLinkages();
			for (int i = 0; i<ds.size(); i++) {
				blocks.append(" LINKCHARACTERS = ");
				blocks.append(StringUtil.tokenize(((CharacterData)ds.elementAt(i)).getName()));
			}
			blocks.append(endLine);
		}
		progIndicator.setText("Writing character & state names");
		blocks.append(getCharStateLabels(data));
		progIndicator.setText("Writing character matrix");
		blocks.append("\tMATRIX" + StringUtil.lineEnding());
		int numTotal = data.getNumTaxa() * data.getNumChars();
		//MesquiteModule.mesquiteTrunk.mesquiteMessage("Composing categorical matrix ", numTotal, 0);
		int numTaxa = data.getTaxa().getNumTaxa();
		int tot = 0;
		String taxonName="";
		int maxNameLength = data.getTaxa().getLongestTaxonNameLength();
		for (int it=0; it<numTaxa; it++) {
			taxonName = data.getTaxa().getTaxon(it).getName();
			if (taxonName!=null) {
				blocks.append("\t"+StringUtil.tokenize(taxonName));
				for (int i = 0; i<(maxNameLength-taxonName.length()+2); i++)
					blocks.append(" ");
			}
			int totInTax = 0;
			
			progIndicator.setText("Writing data for taxon " + taxonName);
			//USE SYMBOLS
			for (int ic=0;  ic<data.getNumChars(); ic++) {
				data.statesIntoNEXUSStringBuffer(ic, it, blocks);
				tot++;
				totInTax++;
				if (tot % 5000 == 0)
					logln("Composing categorical matrix: " + tot + " of " + numTotal);
				if (totInTax % 1000 == 0)
					blocks.append(StringUtil.lineEnding()); //this is here becuase of current problem (at least on mrj 2.2) of long line slow writing
				
			}
			blocks.append(StringUtil.lineEnding());
			file.writeLine(blocks.toString());
			blocks.setLength(0);
		}
		blocks.append(StringUtil.lineEnding());
		blocks.append(endLine);
		blocks.append(cB.getUnrecognizedCommands() + StringUtil.lineEnding());
		blocks.append("END;" + StringUtil.lineEnding());
		progIndicator.setText("Finished writing matrix");
		//MesquiteModule.mesquiteTrunk.mesquiteMessage("Categorical matrix composed", 1, 0);

		file.writeLine( blocks.toString());
	}

	/*.................................................................................................................*/
	public NexusCommandTest getNexusCommandTest(){ 
		return new CategNexusCommandTest();
	}
	/*...................................................................................................................*/
	public boolean readNexusCommand(MesquiteFile file, String blockName, String command, MesquiteString comment){ 
		if (blockName.equalsIgnoreCase("NOTES")) {
			MesquiteProject project = file.getProject();
			String commandName = parser.getFirstToken(command);
			int code = 0;
			if  (commandName.equalsIgnoreCase("TEXT")) 
				code = 1;
			if (code >0 ) {
				int integer = MesquiteInteger.unassigned;
				String name = null;
				stringPos.setValue(parser.getPosition());
				String[][] subcommands  = ParseUtil.getSubcommands(command, stringPos);
				if (subcommands == null || subcommands.length == 0 || subcommands[0] == null || subcommands[0].length == 0)
					return false;
				int whichState = MesquiteInteger.unassigned;
				int whichCharacter = MesquiteInteger.unassigned;
				String text = null;
				Taxa taxa = getProject().getTaxa(0);
				CharacterData data = getProject().getCharacterMatrix(taxa, 0);
				
				for (int i=0; i<subcommands[0].length; i++){
					String subC = subcommands[0][i];
					if ("TAXA".equalsIgnoreCase(subC)) {
						String token = subcommands[1][i];
						Taxa t = getProject().getTaxa(token);
						if (t==null){
							int wt = MesquiteInteger.fromString(token);
							if (!MesquiteInteger.isCombinable(wt))
								t = getProject().getTaxa(wt-1);
						}
						if (t!=null)
							taxa = t;
						else
							return false;
					}
					else if ("CHARACTERS".equalsIgnoreCase(subC)) {
						String token = subcommands[1][i];
						CharacterData t = getProject().getCharacterData(token);
						if (t==null){
							int wt = MesquiteInteger.fromString(token);
							if (!MesquiteInteger.isCombinable(wt))
								t = getProject().getCharacterMatrix(taxa, wt-1);
						}
						if (t!=null)
							data = t;
						else
							return false;
					}
					else if ("TEXT".equalsIgnoreCase(subC)) {
						text = subcommands[1][i];
					}
					else if ("STATE".equalsIgnoreCase(subC)) {
						String token = subcommands[1][i];
						whichState = MesquiteInteger.fromString(token);
						if (!MesquiteInteger.isCombinable(whichState))
							return false;
					}
					else if ("INTEGER".equalsIgnoreCase(subC)) {
						integer = MesquiteInteger.fromString(subcommands[1][i]);
					}
					else if ("CHARACTER".equalsIgnoreCase(subC)) {
						String token = subcommands[1][i];
						whichCharacter = MesquiteInteger.fromString(token);
						if (!MesquiteInteger.isCombinable(whichCharacter))
							return false;
						whichCharacter = CharacterStates.toInternal(whichCharacter);
					}
				}
				if (!MesquiteInteger.isCombinable(whichCharacter))
					return false;
				if (code ==1){
					if (taxa !=null && text !=null && data !=null) {
						if (MesquiteInteger.isCombinable(whichState) && (data instanceof CategoricalData)) {
							((CategoricalData)data).setStateNote(whichCharacter, whichState, text);
							return true;
						}
					}
				}
			}
		}
		return false;
	}
	/*.................................................................................................................*/
	public String getNexusCommands(MesquiteFile file, String blockName){ 
		if (blockName.equalsIgnoreCase("NOTES")) {
			String s ="";
			boolean found = false;
			MesquiteProject project = file.getProject();
			for (int i=0; i<project.getNumberCharMatrices(); i++){
				CharacterData d = getProject().getCharacterMatrix(i);
				if (d instanceof CategoricalData){
					CategoricalData data = (CategoricalData)d;
					String dataSpec = "";
					if (project.getNumberTaxas()>1)
						dataSpec = " TAXA = " +  StringUtil.tokenize(data.getTaxa().getName()) + "";
					if (project.getNumberCharMatrices()>1)
						dataSpec += " CHARACTERS = " +  StringUtil.tokenize(data.getName()) + "";
					for (int ic = 0; ic<data.getNumChars(); ic++){
						for (int is = 0; is<CategoricalState.maxCategoricalState; is++){
							String obj = data.getStateNote(ic, is);
							if (!StringUtil.blank(obj)){
								s += "\tTEXT  " + dataSpec + " CHARACTER = " + CharacterStates.toExternal(ic) + " STATE = " + is+ " TEXT = " + StringUtil.tokenize(obj) + ";" + StringUtil.lineEnding();
								found = true;
							}
						}
					}
				}
			}
			if (found)
				return s;
			else
				return null;
		}
		return null;
	}

	/*.................................................................................................................*/
    	 public String getName() {
		return "Manage categorical character matrices";
   	 }
   	 
	/*.................................................................................................................*/
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Manages categorical data matrices (including read/write in NEXUS file)." ;
   	 }
}
	
/* ======================================================================== */
/** An object of this kind can be returned by getNexusCommandTest that will be stored in the modulesinfo vector and used
to search for modules that can read a particular command in a particular block.  (Much as the NexusBlockObject.)*/
class CategNexusCommandTest extends NexusCommandTest  {
	MesquiteInteger pos = new MesquiteInteger();
	/**returns whether or not the module can deal with command*/
	public boolean readsWritesCommand(String blockName, String commandName, String command){
		boolean b = (blockName.equalsIgnoreCase("NOTES")  && (commandName.equalsIgnoreCase("TEXT")));
		if (b){
			pos.setValue(0);
			String firstToken = ParseUtil.getFirstToken(command,  pos);
			
			String[][] subcommands  = ParseUtil.getSubcommands(command, pos);
			if (subcommands == null)
				return false;
			if (StringArray.indexOfIgnoreCase(subcommands, 0, "CHARACTER")<0)
				return false;
			if (StringArray.indexOfIgnoreCase(subcommands, 0, "STATE")<0)
				return false;
			return true;
		}
		return false;
	} 
}
