/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.categ.StateNamesEditor;

import java.util.*;
import java.awt.*;
import java.awt.image.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import mesquite.lib.table.*;
import mesquite.categ.lib.*;


/* ======================================================================== */
public class StateNamesEditor extends CategDataEditorInit {
	CategoricalData data;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		getEmployer().addMenuItem("Edit State Names", makeCommand("makeWindow", this));
		return true;
	}
   	/** Called to alter data in all cells*/
   	public void setTableAndData(MesquiteTable table, CharacterData data, CommandRecord commandRec){
			if (!(data instanceof CategoricalData))
				return;
			//TODO: query here if state names editing allowed
			this.data = (CategoricalData)data;
   	}
	/*.................................................................................................................*/
	public boolean isSubstantive(){
		return false;
	}
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) { 
   	 	if (getModuleWindow()==null || !getModuleWindow().isVisible())
   	 		return null;
   	 	Snapshot temp = new Snapshot();
  	 	temp.addLine("makeWindow"); 
  	 	temp.addLine("tell It"); 
  		Snapshot sn = getModuleWindow().getSnapshot(file);
  		temp.incorporate(sn, true);
  	 	temp.addLine("endTell"); 
  	 	temp.addLine("showWindow"); 
  	 	return temp;
  	 }
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Makes the state names editor window (but doesn't display it)", null, commandName, "makeWindow")) {
   	 		if (data!=null ){
				if (getModuleWindow()==null) {
					setModuleWindow(new StateNamesWindow(this, data));
			 		resetContainingMenuBar();
					resetAllWindowsMenus();
				}
				if (!commandRec.scripting())
					getModuleWindow().setVisible(true);
				return getModuleWindow();
				//set up a listener
   	 		}
    	 	}

    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
	return null;
   	 }
	/*.................................................................................................................*/
    	 public String getName() {
		return "Edit State Names";
   	 }
	/*.................................................................................................................*/
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Edits state names of a categorical data matrix." ;
   	 }
   	 
}

/* ======================================================================== */
class StateNamesWindow extends TableWindow {
	protected StateNamesTable table;
	private int windowWidth=360;
	private int windowHeight=400;
	StateNamesEditor owner;
	ToolPalette palette = null;
	TableTool arrowTool, ibeamTool, fillTool, dropperTool;
	CategoricalData data;
	String fillString;
	boolean neverFilled = true;
	public StateNamesWindow (StateNamesEditor ownerModule, CategoricalData data) {
		super(ownerModule, true); //INFOBAR
		setWindowSize(windowWidth, windowHeight);
		ownerModule.setModuleWindow(this);
		owner = ownerModule;
		this.data = data;
		table = new StateNamesTable (data, windowWidth, windowHeight, 130, this);
		setShowAnnotation(true);

		String selectExplanation = "This tool selects items in the matrix.  By holding down shift while clicking, the selection will be extended from the first to the last touched cell. ";
		selectExplanation += " A block of cells can be selected either by using shift-click to extend a previous selection, or by clicking on a cell and dragging with the mouse button still down";
		selectExplanation += " Discontinous selections are allowed, and can be obtained by a \"meta\"-click (right mouse button click, or command-click on a MacOS system). ";
		arrowTool = new TableTool(this, "arrow", MesquiteModule.getRootImageDirectoryPath(), "arrow.gif", 4,2,"Select", selectExplanation, MesquiteModule.makeCommand("arrowTouchCell",  this) , MesquiteModule.makeCommand("arrowDragCell",  this), MesquiteModule.makeCommand("arrowDropCell",  this));
		arrowTool.setIsArrowTool(true);
		arrowTool.setUseTableTouchRules(true);
		addTool(arrowTool);
		getPalette().setFirstToolHeight(26);
  	 	setCurrentTool(arrowTool);
 		arrowTool.setInUse(true);
		ibeamTool = new TableTool(this, "ibeam", MesquiteModule.getRootImageDirectoryPath(), "ibeam.gif", 7,7,"Edit", "This tool can be used to edit the contents of cells in the matrix.", MesquiteModule.makeCommand("editCell",  (Commandable)table) , null, null);
		ibeamTool.setWorksOnRowNames(true);
		addTool(ibeamTool);

		fillTool = new TableTool(this, "fill", MesquiteModule.getRootImageDirectoryPath(), "bucket.gif", 13,14,"Fill with unassigned", "This tool fills selected cells with text.  The text to be used can be determined by using the dropper tool.", MesquiteModule.makeCommand("fillTouchCell",  this) , null, null);
		fillTool.setOptionsCommand(MesquiteModule.makeCommand("fillOptions",  this));
		addTool(fillTool);
		dropperTool = new TableTool(this, "dropper", MesquiteModule.getRootImageDirectoryPath(), "dropper.gif", 1,14,"Copy states", "This tool fills the paint bucket with the text in the cell touched on", MesquiteModule.makeCommand("dropperTouchCell",  this) , null, null);
		addTool(dropperTool);

		table.setColumnWidthsUniform(100);
		//table.setUserAdjust(MesquiteTable.NOADJUST, MesquiteTable.RESIZE);
 		addToWindow(table);
 		table.setVisible(true);
		table.setLocation(0,0);
		MesquiteWindow.addKeyListener(this, table);
		setWindowSize(windowWidth, windowHeight);
		resetTitle();
	}
	
	/*.................................................................................................................*/
	/** When called the getModuleWindow() will determine its own title.  MesquiteWindows need
	to be self-titling so that when things change (names of files, tree blocks, etc.)
	they can reset their titles properly*/
	public void resetTitle(){
		setTitle("State names");
	}
	
	/*.................................................................................................................*/
	String fillName;
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Hides state names editor window", null, commandName, "hideWindow")) {
   	 		hide();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Selects a row in the editor", "[row number]", commandName, "selectRow")) { //TODO: should this use internal or external character numbers???
    	 		selectRow(MesquiteInteger.fromString(arguments));
    	 	}
    	 	else if (checker.compare(this.getClass(), "Selects a column in the editor", "[column number]", commandName, "selectColumn")) {
    	 		selectColumn(MesquiteInteger.fromString(arguments));
    	 	}
    	 	else if (checker.compare(this.getClass(), "Fills the touched cell with current paint", "[column][row]", commandName, "fillTouchCell")) {
   	 		if (table!=null && data !=null){
 	   	 		MesquiteInteger io = new MesquiteInteger(0);
	   			int column= MesquiteInteger.fromString(arguments, io);
	   			int row= MesquiteInteger.fromString(arguments, io);
 	   			if (!table.rowLegal(row)|| !table.columnLegal(column))
	   				return null;
	   			if (table.anyCellSelected()) {
		   			if (table.isCellSelected(column, row)) {
						for (int i=0; i<table.getNumColumns(); i++)
							for (int j=0; j<table.getNumRows(); j++)
								if (table.isCellSelected(i,j))
									data.setStateName(j,i, fillName);
		 	   			table.repaintAll();
					}
				}
				else if (table.anyRowSelected()) {
		   			if (table.isRowSelected(row)) {
						for (int j=0; j<table.getNumRows(); j++) {
							if (table.isRowSelected(j))
								for (int i=0; i<table.getNumColumns(); i++)
										data.setStateName(j,i, fillName);
						}
		 	   			table.repaintAll();
					}
				}
				else if (table.anyColumnSelected()) {
		   			if (table.isColumnSelected(column)) {
						for (int i=0; i<table.getNumColumns(); i++){
							if (table.isColumnSelected(i))
								for (int j=0; j<table.getNumRows(); j++) 
									data.setStateName(j,i, fillName);
						}
		 	   			table.repaintAll();
					}
				}
				else {
					data.setStateName(row, column, fillName);
					table.redrawCell(column, row);
				}
			}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Queries the user to fill the selected cells with text", null, commandName, "fillSelected")) {
			fillName = MesquiteString.queryString(this, "Fill cells", "Fill cells with names:", "");
   			if (table.anyCellSelected()) {
				for (int i=0; i<table.getNumColumns(); i++)
					for (int j=0; j<table.getNumRows(); j++)
						if (table.isCellSelected(i,j))
							data.setStateName(j,i, fillName);
 	   			table.repaintAll();
			}
			else if (table.anyRowSelected()) {
				for (int j=0; j<table.getNumRows(); j++) {
					if (table.isRowSelected(j))
						for (int i=0; i<table.getNumColumns(); i++)
								data.setStateName(j,i, fillName);
				}
 	   			table.repaintAll();
			}
			else if (table.anyColumnSelected()) {
				for (int i=0; i<table.getNumColumns(); i++){
					if (table.isColumnSelected(i))
						for (int j=0; j<table.getNumRows(); j++) 
							data.setStateName(j,i, fillName);
				}
 	   			table.repaintAll();
			}
    	 	}
     	 	else if (checker.compare(this.getClass(), "Queries the user what paint to use", null, commandName, "fillOptions")) {
   	 		if (table!=null && data !=null){
				fillName = MesquiteString.queryString(this, "Fill name", "Name with which to fill using paint bucket:", "");
		   		if (StringUtil.blank(fillName))
		   			return null;
	   			fillTool.setDescription("Fill with \"" + fillName+ " \"");
	   			dropperTool.setDescription("Copy name (current: " + fillName + ")");
				toolTextChanged();
			}
			
			
    	 	}
    	 	else if (checker.compare(this.getClass(), "Queries the user for the paint string", null, commandName, "touchTool")) {
   	 		if (table!=null && data !=null){
				String newFillString  = MesquiteString.queryString(this, "Fill string", "String with which to fill using paint bucket:", fillString);
		   		if (StringUtil.blank(newFillString))
		   			return null;
		   		fillString = newFillString;
		   		neverFilled = false;
	   			fillTool.setDescription("Fill with \"" + fillString + " \"");
	   			dropperTool.setDescription("Copy string (current: " + fillString + ")");
				this.toolTextChanged();
			}
    	 	}
   	 	else if (checker.compare(this.getClass(), "Fills the paint bucket with the string of the selected cell", "[column][row]", commandName, "dropperTouchCell")) {
   	 		if (table!=null && data !=null){
	   	 		MesquiteInteger io = new MesquiteInteger(0);
	   			int column= MesquiteInteger.fromString(arguments, io);
	   			int row= MesquiteInteger.fromString(arguments, io);
	   			if (!table.rowLegal(row)|| !table.columnLegal(column))
	   				return null;
	   			fillName = table.getText(column, row);
	   			fillTool.setDescription("Fill with \"" + fillName+ " \"");
	   			dropperTool.setDescription("Copy name (current: " + fillName + ")");
				toolTextChanged();
			}
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
   	 }
	public void selectRow(int row) {
		table.selectRow(row);
	}
	
	public void selectColumn(int column) {
		table.selectBlock(column, 0, column, table.getLastRow());
	}
	
	/*.................................................................................................................*/
	public void windowResized() {
	   	if (MesquiteWindow.checkDoomed(this))
	   		return;
   	 	if (table!=null && ((getHeight()!=windowHeight) || (getWidth()!=windowWidth))) {
   	 		windowHeight =getHeight();
   	 		windowWidth = getWidth();
   	 		table.setTableSize(windowWidth, windowHeight);
   	 	}
   	 	if (table!=null)
   	 		table.resetComponentSizes();
		MesquiteWindow.uncheckDoomed(this);
	}
	/*.................................................................................................................*/
    	 public MesquiteTable getTable() {
		return table; 
   	 }
}

/* ======================================================================== */
class StateNamesTable extends MesquiteTable {
	StateNamesWindow window;
	CategoricalData data;
	StateAnnotation cellAnnotated;
	public StateNamesTable (CategoricalData data, int totalWidth, int totalHeight, int columnNamesWidth, StateNamesWindow window) {
		super(data.getNumChars(), CategoricalState.maxCategoricalState+1, totalWidth, totalHeight, columnNamesWidth, window.getColorScheme(),true,false);
		this.data = data;
		this.window = window;
		autosizeColumns = true;
		cellAnnotated = new StateAnnotation(data);
		showRowGrabbers=true;
		showColumnGrabbers=true;
		//setAutoEditable(false, false, false, false);
		setEditable(true, true, false, false);
		setSelectable(true, true, true, true, false, false);
		setColumnNamesCopyPaste(false);
	}
	
	/*...............................................................................................................*/
	/** returns a tab delimited text version of the table*/
	public String getTextVersion(){
		int maxStateNamed = -1;
		for (int row=0; row<data.getNumChars(); row++)
			for (int column=0; column<CategoricalState.maxCategoricalState; column++)
				if (data.hasStateName(row, column) && maxStateNamed < column)
					maxStateNamed = column;
		
		StringBuffer outputBuffer = new StringBuffer(maxStateNamed*getNumRows());
		for (int column=0; column<maxStateNamed;column++)
			outputBuffer.append("\t" + columnNames.getText(column, -1));
		outputBuffer.append("\n");
		for (int row=0; row<getNumRows();row++) {
			outputBuffer.append(rowNames.getText(-1, row));
			for (int column=0; column<maxStateNamed;column++)
				outputBuffer.append("\t" + matrix.getText(column,row));
			outputBuffer.append("\n");
		}
		return outputBuffer.toString();
		
	}
	/*public void getCornerCellText(Graphics g, int x, int y,  int w, int h){  
		g.drawString(window.getItemTypeName(), x+2, y+h-2);
	}*/
	public String getMatrixText(int column, int row){  
		if (data.hasStateName(row, column))
			return data.getStateName(row, column); //return state name
		else return "";
	}
	
	/*...............................................................................................................*/
	public String getMatrixTextForDisplay(int column, int row){
		String s = getMatrixText(column, row);
		if (!StringUtil.blank(data.getStateNote(row, column))) 
			s += '*';
		return s;
	}
	public boolean useString(int column, int row){  
		return true;
	}
	public String getColumnNameText( int column){
		if (column>9 ||  Integer.toString(column).charAt(0) != data.getSymbol(column))
			return "State "  + Integer.toString(column) + " (\'" + data.getSymbol(column) + "\')";
		return "State " + Integer.toString(column);
	}
	public String getCornerText(){
		return "Character name";
	}
	
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
	 		if (checker.compare(this.getClass(), "Selects all cells in the editor", null, commandName, "selectAll")) {
 			if (matrix.getEditing()|| rowNames.getEditing()){
				TextField edit = null;
				if (matrix.getEditing())
					edit = matrix.getEditField();
				else if (rowNames.getEditing())
					edit = rowNames.getEditField();
				if (edit!=null) {
					edit.selectAll();
				}
			}
			else {
				boolean selectA = (!anythingSelected());
				
				if ((selectA || anyCellSelected()))
					selectBlock(0, 0, numColumnsTotal-1, numRowsTotal-1);
				if ((selectA || anyRowSelected())) {
					selectRows(0, numRowsTotal-1);
				}
				if ((selectA || anyColumnSelected())) {
					selectBlock(0, 0, numColumnsTotal-1, numRowsTotal-1);
				}
				if ((selectA || anyRowNameSelected()))
					selectRowNames(0,numRowsTotal-1);
				repaintAll();
			/*if (cornerSelectable)
				selectCorner;*/
			}
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
	}	
	public void selectColumn(int column) {
		selectBlock(column, 0, column, getLastRow());
	}
	public void selectColumns(int first, int last) {
		selectBlock(first, 0, last, getLastRow());
	}
	/*...............................................................................................................*/
	public void returnedMatrixText(int column, int row, String s, CommandRecord commandRec){
		if (StringUtil.blank(s)) {
			data.setStateName(row, column,null);
			if (data.hasStateName(row, column))
				if (autoSizeColumns(getGraphics()))
					repaintAll();
		}
		else if ((s.length()>1 || s.charAt(0)!=data.getSymbol(column)) && (!s.equals(data.getStateName(row, column)))) {
			data.setStateName(row, column, s);
			if (autoSizeColumns(getGraphics()))
				repaintAll();
		}
	}
	public String getRowNameText(int row){
		return data.getCharacterName(row);
	}
	/*...............................................................................................................*/
	public String getRowNameTextForDisplay(int row){
		String s = data.getCharacterName(row);
		if (data.getAnnotation(row) != null)
			 s += "*";
		return s;
		
	}
	/*...............................................................................................................*/
	public void returnedRowNameText(int row, String s, CommandRecord commandRec){
		if (StringUtil.blank(s) && data.characterHasName(row)){
			data.setCharacterName(row, null);
			if (autoSizeColumns(getGraphics()))
				repaintAll();
		}
		else if (!s.equals(data.getCharacterName(row))) {
			String warning = data.checkNameLegality(row, s);
			if (warning == null)
				data.setCharacterName(row, s);
			else
				window.ownerModule.discreetAlert(commandRec, warning);
			if (autoSizeColumns(getGraphics())) {
				repaintAll();
			}
		}
	}
	/*.................................................................................................................*/
 	protected void copyIt(StringBuffer s, boolean literal){
 		if (s==null)
 			return;
		int count = 0;
		boolean firstInLine = true;
		boolean allSelected = true;
		int numRows = numRowsTotal;
		int maxColumn = 0;
		boolean nothingSelected = !anythingSelected();
		for (int i = 0; i<numColumnsTotal; i++) {
			for (int j = 0; j<numRows; j++) {
				if (nothingSelected || isRowNameSelected(j) || isRowSelected(j)) {
					String t =null;
					if (literal)
						t = getMatrixTextForDisplay(i,j);
					else 
						t = getMatrixText(i,j);
					if (!StringUtil.blank(t)) {
						if (maxColumn<i)
							maxColumn = i;
					}
				}
			}
		}
		int numColumns = maxColumn+1;
		
		if (columnNamesCopyPaste) {
			for (int i = 0; i<numColumns; i++) {
				if (nothingSelected || isColumnNameSelected(i) || isColumnSelected(i)) {
					if (!firstInLine) 
						s.append('\t');
					firstInLine = false;
					String t = getColumnNameText(i);
					if (t!=null)
						s.append(t);
				}
				else allSelected = false;
			}
		}
		else
			allSelected = false;
		if (!firstInLine) 
			s.append(StringUtil.lineEnding());
		for (int j = 0; j<numRows; j++) {
			firstInLine = true;
			if (rowNamesCopyPaste) {
				if (nothingSelected || isRowNameSelected(j) || isRowSelected(j)) {
					String t = getRowNameText(j);
					if (t!=null)
						s.append(t);
					firstInLine = false;
				}
				else allSelected = false;
			}
			else allSelected = false;
			
			for (int i = 0; i<numColumns; i++) {
				if (nothingSelected || isCellSelected(i,j) || isRowSelected(j) || isColumnSelected(i)) {
					if (!firstInLine) 
						s.append('\t');
					firstInLine = false;
					String t =null;
					if (literal)
						t = getMatrixTextForDisplay(i,j);
					else 
						t = getMatrixText(i,j);
					if (t!=null)
						s.append(t);
				}
				else allSelected = false;
			}
			if (!firstInLine)
				s.append(StringUtil.lineEnding());
		}
		if (allSelected) //absolutely everyting is selected; prepend tab for corner cell
			s.insert(0, '\t');
			
 	}
	/*...............................................................................................................*/
	public void cellTouched(int column, int row, int modifiers, int clickCount) {
		if (((TableTool)window.getCurrentTool()== (TableTool)window.arrowTool) && (clickCount>1)) {
			window.setCurrentTool((TableTool)window.ibeamTool);
			window.getPalette().setCurrentTool((TableTool)window.ibeamTool); 
			((TableTool)window.getCurrentTool()).cellTouched(column, row, modifiers);
		}
		else if (((TableTool)window.getCurrentTool()).useTableTouchRules())
			super.cellTouched(column, row, modifiers, clickCount);
		else
			((TableTool)window.getCurrentTool()).cellTouched(column, row, modifiers);
		if (window.getCurrentTool()== window.arrowTool || window.getCurrentTool()== window.ibeamTool){
			//deselectAll();
			setFocusedCell(column, row);
			repaintAll();
		}
		else
			window.setAnnotation("", null);
	}
	/*...............................................................................................................*/
	public void cellDrag(int column, int row, int modifiers) {
		if (window.getCurrentTool()== window.arrowTool) 
			super.cellDrag(column, row, modifiers);
		else
		((TableTool)window.getCurrentTool()).cellDrag(column, row, modifiers);
	}
	/*...............................................................................................................*/
	public void cellDropped(int column, int row, int modifiers) {
		if (window.getCurrentTool()== window.arrowTool) 
			super.cellDropped(column, row, modifiers);
		else
		   ((TableTool)window.getCurrentTool()).cellDropped(column, row, modifiers);
	}
	public void setFocusedCell(int column, int row){
			cellAnnotated.setCell(column, row);
			window.setAnnotation(cellAnnotated);
			super.setFocusedCell(column, row);
	}
	/*...............................................................................................................*/
	public void rowNameTouched(int row, int modifiers, int clickCount) {
		if ((window.getCurrentTool()==window.arrowTool) && (clickCount>1)) {
			window.setCurrentTool((TableTool)window.ibeamTool);
			window.getPalette().setCurrentTool((TableTool)window.ibeamTool); 
			((TableTool)window.getCurrentTool()).cellTouched(-1, row, clickCount);
		}
		else if (window.getCurrentTool()== window.arrowTool) 
			super.rowNameTouched( row, modifiers, clickCount);
		else
			((TableTool)window.getCurrentTool()).cellTouched(-1, row, modifiers); 
		if (window.getCurrentTool()== window.arrowTool || window.getCurrentTool()== window.ibeamTool){
			//deselectAll();
			setFocusedCell(-1, row);
			repaintAll();
		}
		else
			window.setAnnotation("", null);
	}
	/*...............................................................................................................*/
	public void columnNameTouched(int column, int modifiers, int clickCount) {
		columnTouched(column, modifiers);
		/*if (window.getCurrentTool()== window.arrowTool) 
			super.columnNameTouched( column, modifiers, clickCount);
		else
			((TableTool)window.getCurrentTool()).cellTouched(column, -1, modifiers);
		if (window.getCurrentTool()== window.arrowTool || window.getCurrentTool()== window.ibeamTool){
			//deselectAll();
			setFocusedCell(column, -1);
			repaintAll();
		}
		else
			window.setAnnotation("", null);
			*/
	}
}

/* ======================================================================== */
class StateAnnotation implements Annotatable {
	String name;
	CategoricalData data;
	int state, character;
	public StateAnnotation(CategoricalData data) {
		this.data = data;
	}
	
	void setCell(int c, int r){
		character = r;
		state = c;
	}
	public String getName(){
		if (data == null)
			return "";
		if (character == -1) {
			if (state == -1)
				return "matrix \"" + data.getName() + "\"";
			return null;
		}
		else if (state == -1) {
			return "character \"" +data.getCharacterName(character) + "\"";
		}
		else {
			return "state " + (state) + " in " +data.getCharacterName(character) ;
		}
	}
	public String getAnnotation(){
		if (data == null)
			return null;
		if (character == -1)
			return null;
		if (state == -1) {
			return data.getAnnotation(character);
		}
		else if (character > -1){
			return data.getStateNote(character, state);
		}
		return null;
	}
	public void setAnnotation(String s, boolean notify){
		if (data == null)
			return;
		if ("".equals(s))
			s = null;
		if (state == -1) {
			data.setAnnotation(character, s);
		}
		else if (character > -1){
			data.setStateNote(character, state, s);
		}
	}
 }

