/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.charMatrices.BasicDataWindowCoord; 
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;

/** Coordinates the display of the spreadsheet editor window for character matrices.  This doesn't actually make the window (see BasicDataWindowMaker). */
public class BasicDataWindowCoord extends FileInit {
	MesquiteSubmenuSpec elementsSubmenu;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		if (getProject()==null || getFileCoordinator()==null)
			return false;
 		elementsSubmenu = getFileCoordinator().addSubmenu(MesquiteTrunk.charactersMenu, "Character Matrix Editor", makeCommand("showDataWindow",  this));
		elementsSubmenu.setList( getProject().datas);
		elementsSubmenu.setBehaviorIfNoChoice(MesquiteSubmenuSpec.ONEMENUITEM_ZERODISABLE);
		return true;
  	 }
  	 String dataRef(CharacterData d){
  	 	return getProject().getCharMatrixReference(d);
  	 }
	/*.................................................................................................................*/
   	 public boolean isSubstantive(){
   	 	return false;
   	 }
	/*.................................................................................................................*/
  	 public boolean isPrerelease() {
		return false;
   	 }
	/*.................................................................................................................*/
	/**Returns command to hire employee if clonable*/
	public String getClonableEmployeeCommand(MesquiteModule employee){
		if (employee!=null && employee.getEmployer()==this) {
			if (employee.getHiredAs()==DataWindowMaker.class) {
				CharacterData d = (CharacterData)employee.doCommand("getDataSet", null, CommandRecord.getRecNSIfNull(), CommandChecker.defaultChecker);
				if (d != null) {
					return ("showDataWindow " + dataRef(d) + "  " + StringUtil.tokenize(employee.getName()) + ";");//quote
				}
			}
		}
		return null;
	}
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) {
   	 	Snapshot temp = new Snapshot();
		for (int i = 0; i<getNumberOfEmployees(); i++) {
			Object e=getEmployeeVector().elementAt(i);
			if (e instanceof DataWindowMaker) {
				DataWindowMaker dwm = (DataWindowMaker)e;
				CharacterData d = (CharacterData)dwm.doCommand("getDataSet", null, CommandRecord.getRecNSIfNull(), CommandChecker.defaultChecker);
				if (d != null) {
					temp.addLine("showDataWindow " + dataRef(d), dwm);
				}
			}
		}
  	 	return temp;
  	 }
	MesquiteInteger pos = new MesquiteInteger();
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Shows the data editor window.  If a data editor window for this matrix already exists, it is brought to the front", "[number of data matrix to be shown] [name of data matrix to be shown]", commandName, "showDataWindow")) {  //IF WINDOW ALREADY SHOWN, JUST BRING IT TO FRONT
	   	 	//first, find the data set requested
	   	 	CharacterData data;
	   	 	if (StringUtil.blank(arguments)){
	   	 		int numDatas = getProject().getNumberCharMatrices();
	   	 		//if only one taxa block, use it
	   	 		if (numDatas<=0){
	   	 			return null;
	   	 		}
	   	 		else if (numDatas==1){
		   	 		data =  getProject().getCharacterMatrix(0);
	   	 		}
	   	 		else {
		   	 		data =  getProject().chooseData(containerOfModule(), null, null, "Which data matrix would you like to display in an data editor window?", commandRec);
	   	 			//else, query user
	   	 		}
	   	 	}
	   	 	else { //data ref doesn't need file
		   	 	data =  getProject().getCharacterData(checker.getFile(), parser.getFirstToken(arguments));
   	 		}
    	 		if (data == null)
    	 			return null;
    	 		//next, find if already has DataWindowMaker module for it
			for (int i = 0; i<getNumberOfEmployees(); i++) {
				Object e=getEmployeeVector().elementAt(i);
				if (e instanceof DataWindowMaker) {
					DataWindowMaker dwm = (DataWindowMaker)e;
					CharacterData d = (CharacterData)dwm.doCommand("getDataSet", null, commandRec, checker);
					if (d == data) {
						if (dwm.getModuleWindow() !=null) {
							dwm.getModuleWindow().setVisible(true);
						}
						return dwm;
					}
				}
			}
			//if no data window module active, hire one
			DataWindowMaker mb = (DataWindowMaker)hireEmployee(commandRec, DataWindowMaker.class, null);
			if (mb!=null) {
	   	 		mb.doCommand("makeWindow", getProject().getCharMatrixReference(data), commandRec, checker);
			}
 			return mb;
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
   	 }
	/*.................................................................................................................*/
    	 public String getName() {
		return "Data Window Coordinator";
   	 }
   	 
	/*.................................................................................................................*/
   	 
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Coordinates the creation of basic data windows." ;
   	 }
   	 
}
	
