/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.charMatrices.BasicDataWindowMaker; 
/*~~  */

import java.util.*;
import java.awt.*;
import java.io.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import java.awt.datatransfer.*;
import mesquite.lib.table.*;
import mesquite.categ.lib.*;

/** Makes and manages the spreadsheet editor for character matrices. */
public class BasicDataWindowMaker extends DataWindowMaker implements CommandableOwner  {
  	public static final short cellWidth = 24;
   	public static final short cellHeight = 16;
   	CharacterData data;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
 		makeMenu("Matrix");
		resetContainingMenuBar();
 		return true;
  	 }
  	 
  	public  Commandable[] getCommandablesForAccumulation(){
  	 	Commandable[] cs = new Commandable[1];
  	 	cs[0]= new BasicDataWindow();
  	 	return cs;
  	 }
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) { 
  	 	BasicDataWindow bdw = ((BasicDataWindow)getModuleWindow());
   	 	if (bdw == null)
   	 		return null;
   	 	Snapshot temp = new Snapshot();
  	 	temp.addLine("getWindow"); 
  	 	temp.addLine("tell It"); 
  		Snapshot sn = bdw.getSnapshot(file);
  		temp.incorporate(sn, true);
  	 	temp.addLine("endTell"); 
  	 	temp.addLine("showWindow"); 
   	 	if (!bdw.widthsSet && (bdw.table.tight.getValue() || bdw.table.showStates.getValue())) { //needed to ensure appears correctly first time under some OS's
	  	 	temp.addLine("getWindow"); 
	  	 	temp.addLine("tell It"); 
	  		temp.addLine("\tforceAutosize");
	  	 	temp.addLine("endTell"); 
  	 	}
 	 	return temp;
  	 }
	MesquiteInteger pos = new MesquiteInteger();
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Returns the data matrix shown by the data window", null, commandName, "getDataSet")) {
    	 		return data;
    	 	}
    	 	else if (checker.compare(this.getClass(), "Makes a data editor window (but doesn't display it)", "[number or reference string of data set to be shown]", commandName, "makeWindow")) {
    	 		if (getModuleWindow() != null)
    	 			return getModuleWindow();
   	 		CharacterData data = getProject().getCharacterData(checker.getFile(), parser.getFirstToken(arguments));
   	 		if (data !=null) {
				makeDataWindow(data, commandRec);
    	 			return getModuleWindow();
    	 		}
    	 	}

    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
   	 }
	/*.................................................................................................................*/
  	 public void makeDataWindow(CharacterData data, CommandRecord commandRec){
  	 	this.data=data;
		setModuleWindow(new BasicDataWindow(this, data, commandRec));
 		resetContainingMenuBar();
		resetAllWindowsMenus();
		if (!commandRec.scripting()){
 			getModuleWindow().setVisible(true);
 			getModuleWindow().contentsChanged();
 			getModuleWindow().toFront();
 		}
  	 }
	/*.................................................................................................................*/
 	public void windowGoAway(MesquiteWindow whichWindow) {
			whichWindow.hide();
			whichWindow.dispose();
			iQuit();
	}
	/*.................................................................................................................*/
  	 public boolean showCitation() {
		return true;
   	 }
	/*.................................................................................................................*/
  	 public boolean isPrerelease() {
		return false;
   	 }
	/*.................................................................................................................*/
    	 public String getName() {
		return "Character Matrix Editor";
   	 }
   	 
	/*.................................................................................................................*/
   	 
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Makes editor windows to edit character data." ;
   	 }
}
	
/* ======================================================================== */
class BasicDataWindow extends TableWindow implements MesquiteListener {
	CharacterData data;
	MatrixTable table;
	int windowWidth = 420;
	int windowHeight = 280;
	int columnWidth = 12;
   	boolean widthsSet = false;
	DataWindowPalette palette = null;
	TableTool arrowTool,ibeamTool;//, charWandTool;//, taxaWandTool
	MesquiteString cellColorerName;
	
	public BasicDataWindow () {
	}
	public BasicDataWindow (BasicDataWindowMaker ownerModule, CharacterData data, CommandRecord commandRec) {
		super(ownerModule, true); //INFOBAR
      		setWindowSize(windowWidth,windowHeight);
		ownerModule.setModuleWindow(this);
		this.data = data;
		if (data!=null)
			columnWidth = data.getDefaultColumnWidth();
		this.ownerModule = ownerModule;
		MesquiteSubmenuSpec mssa = ownerModule.addSubmenu(MesquiteTrunk.analysisMenu, "Analyses for Matrix", MesquiteModule.makeCommand("newAssistant",  this), DataWindowAssistantA.class);
		mssa.setCompatibilityCheck(data.getStateClass());
		table = new MatrixTable((BasicDataWindowMaker)ownerModule, this, data, data.getTaxa().getNumTaxa(), data.getNumChars(), windowWidth, windowHeight, 110);
		table.setQuickMode(true);
		setDefaultAnnotatable(data);
		data.getTaxa().addListener(this);
		data.addListener(this);
		setPalette(palette = new DataWindowPalette(ownerModule, this));
		addToWindow(table);
		table.setLocation(0,0);
		table.setVisible(true);
		table.setColumnAssociable(data);
		table.setRowAssociable(data.getTaxa());
		MesquiteSubmenuSpec mss = ownerModule.addSubmenu(null, "Display");

		/*if (data.pleaseAutoSizeColumns()){
			MesquiteMenuItemSpec mmis = ownerModule.addItemToSubmenu(null, mss, "Fixed Column Width...", MesquiteModule.makeCommand("setColumnWidth",  this));
			mmis.setEnabled(!data.pleaseAutoSizeColumns());  
		*/
		ownerModule.addCheckMenuItemToSubmenu(null, mss, "Narrow Columns", MesquiteModule.makeCommand("toggleTight",  this), table.tight);
		ownerModule.addCheckMenuItemToSubmenu(null, mss, "Show Changes Since Saved", MesquiteModule.makeCommand("toggleShowChanges",  this), table.showChanges);
		MesquiteSubmenuSpec mCC = ownerModule.addSubmenu(null, "Color cells", MesquiteModule.makeCommand("colorCells",  this), ownerModule.getEmployeeVector());
		mCC.setListableFilter(CellColorer.class);

		mCC.setCompatibilityCheck(data.getStateClass());
		cellColorerName = new MesquiteString();
		mCC.setSelected(cellColorerName);

		if (data instanceof CategoricalData && !(data instanceof DNAData) && !(data instanceof ProteinData)) 
			ownerModule.addCheckMenuItemToSubmenu(null, mss, "Show State Names", MesquiteModule.makeCommand("toggleShowNames",  this), table.showNames);
		ownerModule.addCheckMenuItemToSubmenu(null, mss, "Show States", MesquiteModule.makeCommand("toggleShowStates",  this), table.showStates);
		ownerModule.addCheckMenuItemToSubmenu(null, mss, "Auto-size includes Char. Names", MesquiteModule.makeCommand("toggleAutoWithCharNames",  this), table.autoWithCharNames);
		ownerModule.addItemToSubmenu(null, mss, "-", null);
		ownerModule.addItemToSubmenu(null, mss, "Restore Column Auto-Size", MesquiteModule.makeCommand("forceAutosize", this));
		
		ownerModule.addMenuItem("-", null);

		String selectExplanation = "This tool selects items in the matrix.  By holding down shift while clicking, the selection will be extended from the first to the last touched cell. ";
		selectExplanation += " A block of cells can be selected either by using shift-click to extend a previous selection, or by clicking on a cell and dragging with the mouse button still down";
		selectExplanation += " Discontinous selections are allowed, and can be obtained by a \"meta\"-click (right mouse button click, or command-click on a MacOS system). ";
		arrowTool = new TableTool(this, "arrow", MesquiteModule.getRootImageDirectoryPath(),"arrow.gif", 4,2,"Select", selectExplanation, MesquiteModule.makeCommand("arrowTouchCell",  this) , MesquiteModule.makeCommand("arrowDragCell",  this), MesquiteModule.makeCommand("arrowDropCell",  this));
		arrowTool.setIsArrowTool(true);
		arrowTool.setUseTableTouchRules(true);
		addTool(arrowTool);
  	 	setCurrentTool(arrowTool);
 			arrowTool.setInUse(true);
		//ibeam
		ibeamTool = new TableTool(this, "ibeam", MesquiteModule.getRootImageDirectoryPath(),"ibeam.gif", 7,7,"Edit", "This tool can be used to edit the contents of cells in the matrix.", MesquiteModule.makeCommand("editCell",  (Commandable)table) , null, null);
		ibeamTool.setWorksOnRowNames(true);
		ibeamTool.setWorksOnColumnNames(true);
		addTool(ibeamTool);
		

		ownerModule.hireNamedEmployee(commandRec, DataWindowAssistantI.class, "#AlterData");
		ownerModule.hireNamedEmployee(commandRec, DataWindowAssistantI.class, "#AddDeleteData");

		ownerModule.hireAllOtherCompatibleEmployees(commandRec, DataWindowAssistantI.class, data.getStateClass()); 
 		Enumeration enum=ownerModule.getEmployeeVector().elements();
		while (enum.hasMoreElements()){
			Object obj = enum.nextElement();
			if (obj instanceof DataWindowAssistantI) {
				DataWindowAssistantI init = (DataWindowAssistantI)obj;
				init.setTableAndData(table, data, commandRec);
			}
		}
		ownerModule.hireAllCompatibleEmployees(commandRec, CharTableAssistantI.class, data.getStateClass()); 
 		enum=ownerModule.getEmployeeVector().elements();
		while (enum.hasMoreElements()){
			Object obj = enum.nextElement();
			if (obj instanceof CharTableAssistantI) {
				CharTableAssistantI init = (CharTableAssistantI)obj;
				init.setTableAndData(table, data, false, commandRec);
			}
		}
		ownerModule.hireAllEmployees(commandRec, TaxaTableAssistantI.class); 
 		enum=ownerModule.getEmployeeVector().elements();
		while (enum.hasMoreElements()){
			Object obj = enum.nextElement();
			if (obj instanceof TaxaTableAssistantI) {
				TaxaTableAssistantI init = (TaxaTableAssistantI)obj;
				init.setTableAndTaxa(table, data.getTaxa(), true, commandRec);
			}
		}
		MesquiteSubmenuSpec mss2 = ownerModule.addSubmenu(null, "Utilities", MesquiteModule.makeCommand("doUtility",  this));
		mss2.setList(DataUtility.class);
		mss2.setCompatibilityCheck(data.getStateClass());

		if (data.colorCellsByDefault()) {
			MesquiteModule mbc = ownerModule.findEmployeeWithName("#ColorByState", true);
			setCellColorer(mbc, commandRec);
		}
		setShowAnnotation(true);
		table.requestFocus();
		MesquiteWindow.addKeyListener(this, table);

    		resetTitle();
	}
	/*.................................................................................................................*/
	/** When called the window will determine its own title.  MesquiteWindows need
	to be self-titling so that when things change (names of files, tree blocks, etc.)
	they can reset their titles properly*/
	public void resetTitle(){
		String t;
		if (data != null && data.hasTitle()) {
			t = "Character Matrix \"" + data.getName() + "\"";
		}
		else {
			t = "Character Matrix";
		}
		setTitle(t);
	}
	/*.................................................................................................................*/
	private boolean containsShift(String s){
		if (s==null)
			return false;
		else
			return (s.indexOf("shift")>=0);
	}
	/*.................................................................................................................*/
	int rowFirstTouched=0;
	int columnFirstTouched=0;
	int rowLastTouched=0;
	int columnLastTouched=0;
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) { 
   	 	Snapshot temp = new Snapshot();
  	 	temp.incorporate(super.getSnapshot(file), false);
  	 	temp.addLine("setTool " +  getCurrentTool().getName()); 

		if (table != null && table.cellColorer !=null)
			temp.addLine("colorCells ", (MesquiteModule)table.cellColorer);
		temp.addLine("toggleShowNames " + table.showNames.toOffOnString());
		temp.addLine("toggleTight " + table.tight.toOffOnString());
		temp.addLine("toggleShowChanges " + table.showChanges.toOffOnString());
		temp.addLine("toggleShowStates " + table.showStates.toOffOnString());
		temp.addLine("toggleAutoWithCharNames " + table.autoWithCharNames.toOffOnString());
  	 	if (widthsSet)
  	 		temp.addLine("setColumnWidth " + columnWidth);
		for (int i = 0; i<ownerModule.getNumberOfEmployees(); i++) {
			Object e=ownerModule.getEmployeeVector().elementAt(i);
			if (e instanceof DataWindowAssistant && !(e instanceof DataWindowAssistantI)) {
				temp.addLine("newAssistant " , ((MesquiteModule)e));
			}
		}
  	 	return temp;
  	 }
   	 MesquiteInteger pos = new MesquiteInteger(0);
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Sets the current tool", "[name of tool]", commandName, "setTool")) {
  	 		ToolPalette palette = getPalette();
  	 		if (palette == null)
  	 			return null;
  	 		setCurrentTool((TableTool)palette.getToolWithName(arguments));
    	 	}
    	 	else if (checker.compare(this.getClass(), "Not functioning", null, commandName, "moveSelection")) {
   	 		MesquiteInteger io = new MesquiteInteger(0);
   			int column= MesquiteInteger.fromString(arguments, io);
   			int row= MesquiteInteger.fromString(arguments, io);
    	 	}
    	 	else if (checker.compare(this.getClass(), "Adds or inserts characters into matrix", "[character number after which new characters to be inserted] [number of new characters]" , commandName, "addCharacters")) {
   	 		MesquiteInteger io = new MesquiteInteger(0);
   			int starting= CharacterStates.toInternal(MesquiteInteger.fromString(arguments, io));
   			int number= MesquiteInteger.fromString(arguments, io);
   			if (data.addParts(starting, number)) {
	   			table.setNumColumns(data.getNumChars());
				data.notifyListeners(this, new Notification(MesquiteListener.PARTS_ADDED, new int[] {starting, number}), commandRec);
	   			data.addInLinked(starting, number, true);
	   		}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Deletes characters", "[first character to be deleted] [number of characters]", commandName, "deleteCharacters")) {
   	 		MesquiteInteger io = new MesquiteInteger(0);
   			int starting= CharacterStates.toInternal(MesquiteInteger.fromString(arguments, io));
   			int number= MesquiteInteger.fromString(arguments, io);
   			if (data.deleteCharacters(starting, number, false)) {
				data.notifyListeners(this, new Notification(MesquiteListener.PARTS_DELETED, new int[] {starting, number}), commandRec);
   				data.deleteInLinked(starting, number, true);
	   			table.setNumColumns(data.getNumChars()); 
	   		}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Adds taxa", "[taxon number after which new taxa to be inserted] [number of new taxa]", commandName, "addTaxa")) {
   	 		MesquiteInteger io = new MesquiteInteger(0);
   			int starting= Taxon.toInternal(MesquiteInteger.fromString(arguments, io));
   			int number= MesquiteInteger.fromString(arguments, io);
   			if (data.getTaxa().addTaxa(starting, number, true))
	   			table.setNumRows(data.getNumTaxa());
    	 	}
    	 	else if (checker.compare(this.getClass(), "Deletes taxa", "[first taxon to be deleted] [number of taxa]", commandName, "deleteTaxa")) {
   	 		MesquiteInteger io = new MesquiteInteger(0);
   			int starting= Taxon.toInternal(MesquiteInteger.fromString(arguments, io));
   			int number= MesquiteInteger.fromString(arguments, io);
			if (starting == 0 && number == data.getTaxa().getNumTaxa()){
				ownerModule.discreetAlert(commandRec, "You cannot delete all taxa; the command will be ignored");
				return null;
		 	}
  			if (data.getTaxa().deleteTaxa(starting, number, true))
	   			table.setNumRows(data.getNumTaxa());
    	 	}
    	 	else if (checker.compare(this.getClass(), "Hires utility module to operate on the data", "[name of module]", commandName, "doUtility")) {
   	 		if (table!=null && data !=null){
	    	 		DataUtility tda= (DataUtility)ownerModule.hireNamedEmployee(commandRec, DataUtility.class, arguments);
				if (tda!=null) {
					boolean a = tda.operateOnData(data, commandRec);
	 	   			if (a) {
	 	   				table.repaintAll();
						data.notifyListeners(this, new Notification(MesquiteListener.DATA_CHANGED), commandRec);
					}
					if (!tda.pleaseLeaveMeOn())
						ownerModule.fireEmployee(tda);
				}
			}
    	 	}
	 	else if (checker.compare(this.getClass(), "Sets width of columns of matrix", "[width]", commandName, "setColumnWidth")) {
   	 		MesquiteInteger io = new MesquiteInteger(0);
    	 		widthsSet = true;
			int newWidth= MesquiteInteger.fromFirstToken(arguments, io);
			if (!MesquiteInteger.isCombinable(newWidth))
				newWidth = MesquiteInteger.queryInteger(getTheWindow(), "Set column width", "Column Width:", columnWidth, table.getMinColumnWidth(), table.getMaxColumnWidth() );
    	 		if (newWidth>=table.getMinColumnWidth() && newWidth<=table.getMaxColumnWidth() && newWidth!=columnWidth) {
    	 			columnWidth = newWidth;
				table.doAutosize = false;
				table.suppressAutosize = true;
    	 			table.setColumnWidthsUniform(columnWidth);
    	 			table.repaintAll();
    	 		}
    	 		
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets whether or not cells of matrix are colored (no longer used)", "[on]", commandName, "toggleColorCell")) {
			if ("on".equalsIgnoreCase(ParseUtil.getFirstToken(arguments, pos))) {
				MesquiteModule mb = ownerModule.findEmployeeWithName("#ColorByState", true);
				setCellColorer(mb, commandRec);
			}
    	 		table.repaintAll();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets by what the cells in the matrix are colored", "[name of module]", commandName, "colorCells")) {
			String s = ParseUtil.getFirstToken(arguments, pos);
			int i = MesquiteInteger.fromFirstToken(arguments, pos);
			if (MesquiteInteger.isCombinable(i)) {
				ListableVector emp = ownerModule.getEmployeeVector();
				Enumeration enum=emp.elements();
				int count = 0;
				while (enum.hasMoreElements()){
					MesquiteModule mb = (MesquiteModule)enum.nextElement();
					if (mb instanceof CellColorer) {
						if (count == i) {
							if (mb == table.cellColorer) {
								setCellColorer(null,commandRec);
							}
							else {
								setCellColorer(mb, commandRec);
							}
				    	 		table.repaintAll();
							return mb;
						}
							
						count++;
					}
				}
			}
			else {
					MesquiteModule mb = (MesquiteModule)ownerModule.findEmployeeWithName(s, true);
					if (mb instanceof CellColorer) { //note: if by name doesn't turn off!
							setCellColorer(mb, commandRec);
				    	 		table.repaintAll();
							return mb;
					}
			}

    	 	}
    	 	else if (checker.compare(this.getClass(), "Forces the columns to auto-size", null, commandName, "forceAutosize")) {
			table.doAutosize = true;
			table.suppressAutosize = false;
			//table.setColumnWidthsUniform(columnWidth);
    	 		table.repaintAll();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets whether or not states are to be written into cells", "[on or off]", commandName, "toggleShowStates")) {
			table.showStates.toggleValue(ParseUtil.getFirstToken(arguments, pos));
			table.doAutosize = true;
    	 		table.repaintAll();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets whether or not autosize is to use character names also", "[on or off]", commandName, "toggleAutoWithCharNames")) {
			table.autoWithCharNames.toggleValue(ParseUtil.getFirstToken(arguments, pos));
			table.doAutosize = true;
    	 		table.repaintAll();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets whether or not columns are drawn as narrowly as possible", "[on or off]", commandName, "toggleTight")) {
			table.tight.toggleValue(ParseUtil.getFirstToken(arguments, pos));
			table.suppressAutosize = false;
			table.doAutosize = true;
			table.setColumnWidthsUniform(columnWidth);
    	 		table.repaintAll();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets whether or not changes since last save are to be highlighted", "[on or off]", commandName, "toggleShowChanges")) {
			table.showChanges.toggleValue(ParseUtil.getFirstToken(arguments, pos));
    	 		table.repaintAll();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets whether or not full names are shown in the cells of matrix", "[on or off]", commandName, "toggleShowNames")) {
			table.showNames.toggleValue(ParseUtil.getFirstToken(arguments, pos));
			table.doAutosize = true;
    	 		table.repaintAll();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets whether or not birds eye view is shown", "[on or off]", commandName, "toggleshowBirdsEyeView")) {
			table.showBirdsEyeView.toggleValue(ParseUtil.getFirstToken(arguments, pos));
			if (table.showBirdsEyeView.getValue()) {
				table.setColumnWidthsUniform(4);
    	 			table.frameMatrixCells = false;
			}
			else {
    	 			table.setColumnWidthsUniform(columnWidth);
    	 			table.frameMatrixCells = true;
    	 		}
    	 		table.repaintAll();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Hires new data editor assistant module", "[name of module]", commandName, "newAssistant")) {
    	 		DataWindowAssistant dwa = (DataWindowAssistant)ownerModule.hireNamedEmployee(commandRec, DataWindowAssistant.class, arguments);
    	 		dwa.setTableAndData(table, data, commandRec); 
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
	return null;
   	 }
   	 
   	 private void setCellColorer(MesquiteModule mb, CommandRecord commandRec){
			if (table.cellColorer != null)
				table.cellColorer.setActiveColors(false, commandRec);
			table.cellColorer = (CellColorer)mb;
			if (mb !=null) {
				cellColorerName.setValue(mb.getName());
				((CellColorer)mb).setActiveColors(true, commandRec);
			}
			else {
				cellColorerName.setValue(null);
			}
   	 }
   	 
	/*.................................................................................................................*/
   	 public void pleaseUpdate() {
   	 	table.repaintAll();
   	 }
	/*.................................................................................................................*/
	/** passes which object changed, along with optional integer (e.g. for character) (from MesquiteListener interface)*/
	public void changed(Object caller, Object obj, Notification notification, CommandRecord commandRec){
		int code = Notification.getCode(notification);
		int[] parameters = Notification.getParameters(notification);
		if (obj instanceof Taxa &&  (Taxa)obj ==data.getTaxa()) {
			Taxa taxa = (Taxa)obj;
			if (code==MesquiteListener.NAMES_CHANGED) {
				table.redrawRowNames();
			}
			else if (code==MesquiteListener.SELECTION_CHANGED) {
				table.synchronizeRowSelection(taxa);
				table.repaintAll();
			}
			else if (code==MesquiteListener.PARTS_ADDED) {
	   			table.setNumRows(taxa.getNumTaxa());
				table.synchronizeRowSelection(taxa);
	   			table.repaintAll();
			}
			else if (code==MesquiteListener.PARTS_DELETED) {
	   			table.setNumRows(taxa.getNumTaxa());
				table.synchronizeRowSelection(taxa);
	   			table.repaintAll();
			}
			else if (code==MesquiteListener.PARTS_MOVED) {
	   			table.setNumRows(taxa.getNumTaxa());
				table.synchronizeRowSelection(taxa);
	   			table.repaintAll();
			}
			else {
	   			table.repaintAll();
			}
			if (getMode()>0)
				updateTextPage();
		}
		else if (obj instanceof CharacterData && (CharacterData)obj ==data) {
			if (getMode()>0)
				updateTextPage();
			else if (code==MesquiteListener.NAMES_CHANGED) {
				table.doAutosize = true;
				table.repaintAll();
			}
			else if (code==MesquiteListener.SELECTION_CHANGED) {
				if (caller != table){    //if object provoking notification is me, then don't repaint
					table.synchronizeColumnSelection(data);
					table.repaintAll();
				}
			}
			else if (code==MesquiteListener.PARTS_DELETED) {
				table.doAutosize = true;
				if (parameters!=null && parameters.length>1) {
					int starting = parameters[0];
					int num = parameters[1];
					if (data.getNumChars()<table.getNumColumns()) {
						for (int i = starting+num-1; i>=starting; i--)
							table.deleteColumn(i);
					}
				}
	   			else if (table.getNumColumns() != data.getNumChars())
	   				table.setNumColumns(data.getNumChars());
	   			if (table.getNumRows() != data.getTaxa().getNumTaxa())
	   				table.setNumRows(data.getTaxa().getNumTaxa());
				table.synchronizeColumnSelection(data);
	   			table.repaintAll();
			}
			else if (code==MesquiteListener.PARTS_ADDED) {
				table.doAutosize = true;
				if (parameters!=null && parameters.length>1) {
					int starting = parameters[0];
					int num = parameters[1];
					if (data.getNumChars()>table.getNumColumns()) {
						table.insertColumns(starting, num);
					}
				}
	   			else if (table.getNumColumns() != data.getNumChars())
	   				table.setNumColumns(data.getNumChars());
	   			if (table.getNumRows() != data.getTaxa().getNumTaxa())
	   				table.setNumRows(data.getTaxa().getNumTaxa());
				table.synchronizeColumnSelection(data);
	   			table.repaintAll();
			}
			else if (code==MesquiteListener.PARTS_MOVED) {
				table.doAutosize = true;
				if (parameters!=null && parameters.length>1) {
					int starting = parameters[0];
					int num = parameters[1];
					int justAfter = parameters[2];
					if (data.getNumChars()==table.getNumColumns()) {
						table.moveColumns(starting, num, justAfter);
					}
				}
	   			if (table.getNumRows() != data.getTaxa().getNumTaxa())
	   				table.setNumRows(data.getTaxa().getNumTaxa());
				table.synchronizeColumnSelection(data);
	   			table.repaintAll();
			}
			else if (code==MesquiteListener.DATA_CHANGED) {
				table.doAutosize = true;
	   			if (table.getNumRows() != data.getTaxa().getNumTaxa())
	   				table.setNumRows(data.getTaxa().getNumTaxa());
	   			if (table.getNumColumns() != data.getNumChars())
	   				table.setNumColumns(data.getNumChars());
				table.synchronizeColumnSelection(data);
	   			table.repaintAll();
			}
			else{
				table.doAutosize = true;
	   			if (table.getNumRows() != data.getTaxa().getNumTaxa())
	   				table.setNumRows(data.getTaxa().getNumTaxa());
	   			if (table.getNumColumns() != data.getNumChars())
	   				table.setNumColumns(data.getNumChars());
				table.synchronizeColumnSelection(data);
	   			table.repaintAll();
			}
		}
		table.setMessage(data.getCellContentsDescription());
		super.changed(caller, obj, notification, commandRec);
	}
	/*.................................................................................................................*/
	/** passes which object is being disposed (from MesquiteListener interface)*/
	public void disposing(Object obj){
		if ((obj instanceof Taxa &&  (Taxa)obj ==data.getTaxa())||(obj instanceof CharacterData && (CharacterData)obj ==data)) {
			ownerModule.windowGoAway(this);
		}
	}
	/*.................................................................................................................*/
	/** passes which object is being disposed (from MesquiteListener interface)*/
	public boolean okToDispose(Object obj, int queryUser){
		return true;
	}
	
	/*.................................................................................................................*/
	/** Query module as to whether conditions are such that it will have to quit soon -- e.g. if its taxa block has been doomed.  The tree window, data window, 
	etc. override this to return true if their object is doomed. This is useful in case MesquiteListener disposing method is not called for an employer before one of its
	employees discovers that it needs to quit.  If the employer is going to quit anyway,there is no use to use auto rehire for the quit employee.*/
	public boolean quittingConditions(){
		return (data.isDoomed() || data.getTaxa().isDoomed());
	}
	
	/*.................................................................................................................*/
	public void setWindowSize(int width, int height){
		super.setWindowSize(width, height);
		checkSizes();
	}
	void checkSizes(){
   	 	if (table !=null &&((getHeight()!=windowHeight) || (getWidth()!=windowWidth))) {
   	 		windowHeight =getHeight();
   	 		windowWidth = getWidth();
   	 		table.setTableSize(windowWidth, windowHeight);
   	 	}
	}
	/*.................................................................................................................*/
	public void setCurrentTool(MesquiteTool tool){
		super.setCurrentTool(tool);
		if ((getCurrentTool()!=ibeamTool) && (table!=null))
			table.offAllEdits();
		boolean myTool = (getCurrentTool()==ibeamTool || getCurrentTool()==arrowTool);
		setAEFocusSuppression(!myTool);
		if (!myTool)
			table.requestFocus();
			
   	 }
	/*.................................................................................................................*/
	public String getPrintMenuItem() {
		return "Print Matrix...";
	}
	/*.................................................................................................................*/
	public String getPrintToFitMenuItemName() {
		return "Print Matrix To Fit Page...";
	}
	/** Returns menu location for item to bring the window to the for (0 = custom or don't show; 1 = system area of Windows menu; 2 = after system area of Windows menu)*/
	public int getShowMenuLocation(){
		return 0;
	}
	/*.................................................................................................................*/
    	 public MesquiteTable getTable() {
		return table; 
   	 }
	/*.................................................................................................................*/
	public void printWindow(MesquitePrintJob pjob) {
    	 		table.printTable(pjob, this);
   	 		//TODO: optionally print modelsets, etc.
	}
 	/*.................................................................................................................*/
	public void paintContents(Graphics g) {
		table.repaintAll();
	}
	/*.................................................................................................................*/
	public String getTextContents() {
		String s = "Character matrix editor for matrix \"" + data.getName() + "\"\n";
		s+= "Type of matrix: " + data.getDataTypeName();
		if (data instanceof CategoricalData) {
			if (((CategoricalData)data).usingShortMatrix())
				s += " (compacted)\n";
			else
				s += " (uncompacted)\n";
		}
		else
			s += "\n";
		s += "Number of characters: " + data.getNumChars() + "\n";
		s += "Number of taxa: " + data.getNumTaxa() + "\n";
		int countExcluded = 0;
		int countInapplicable = 0;
		int countUnassigned = 0;
		for (int ic = 0; ic< data.getNumChars(); ic++){
		   	if (!data.isCurrentlyIncluded(ic))
		   		countExcluded++;
		   	for (int it=0; it<data.getNumTaxa(); it++){
		   		if (data.isInapplicable(ic, it))
		   			countInapplicable++;
		   		if (data.isUnassigned(ic, it))
		   			countUnassigned++;
		   	}
   			
   		}
		s += "Number of characters excluded: " + countExcluded + "\n";
		s += "Proportion of missing data: " + MesquiteDouble.toString(1.0*countUnassigned/(data.getNumChars()*data.getNumTaxa())) + "\n";
		s += "Proportion of inapplicable codings: " + MesquiteDouble.toString(1.0*countInapplicable/(data.getNumChars()*data.getNumTaxa())) + "\n\n";
		s += table.getTextVersion();
		return s;
	}
	
	/*.................................................................................................................*/
	public void windowResized(){
		checkSizes();
	}
   	 public void dispose(){
		disposing = true;
		waitUntilDisposable();
		table.dispose();
		if (data.getTaxa()!=null)
			data.getTaxa().removeListener(this);
		palette.dispose();
		data.removeListener(this);
		super.dispose();
   	 }

}

/* ======================================================================== */
class CellAnnotation implements Annotatable {
	String name;
	CharacterData data;
	int row, column;
	public CellAnnotation(CharacterData data) {
		this.data = data;
	}
	
	void setCell(int c, int r){
		row = r;
		column = c;
	}
	public String getName(){
		if (data == null)
			return "";
		if (row == -1) {
			if (column == -1)
				return "matrix \"" + data.getName() + "\"";
			return "character \"" +data.getCharacterName(column) + "\"";
		}
		else if (column == -1) {
			return "taxon \"" +data.getTaxa().getTaxonName(row) + "\"";
		}
		else {
			return "character " + (column +1) + " of taxon " + (row+1);
		}
	}
	public String getAnnotation(){
		if (data == null)
			return null;
		if (row == -1) {
			if (column == -1)
				return data.getAnnotation();
			return data.getAnnotation(column);
		}
		else if (column == -1) {
			return data.getTaxa().getAnnotation(row);
		}
		else {
			return data.getAnnotation(column, row);
		}
	}
	public void setAnnotation(String s, boolean notify){
		if (data == null)
			return;
		if ("".equals(s))
			s = null;
		if (row == -1) {
			if (column == -1) {
				data.setAnnotation(s, notify);
				return;
			}
			data.setAnnotation(column, s);
		}
		else if (column == -1) {
			data.getTaxa().setAnnotation(row, s);
		}
		else {
			data.setAnnotation(column, row, s);
		}
	}
 }
/* ======================================================================== */
class DataWindowPalette extends ToolPalette {

	public DataWindowPalette(BasicDataWindowMaker ownerModule, BasicDataWindow containingWindow) {  //in future pass general MesquiteWindow
		super( ownerModule,  containingWindow, 2);
		
	}
 }
/* ======================================================================== */
class MatrixTable extends MesquiteTable {
	BasicDataWindowMaker editorModule;
	Taxa taxa;
	MesquiteBoolean showStates;
	MesquiteBoolean autoWithCharNames;
	MesquiteBoolean showNames;
	MesquiteBoolean showBirdsEyeView;
	MesquiteBoolean tight;
	MesquiteBoolean showChanges;
	CellAnnotation cellAnnotated;
	CharacterData data;
   	Font oldFont=null;
   	Font boldFont;
   	Parser parser = new Parser();
	BasicDataWindow window;
	CellColorer cellColorer = null;
	
	public MatrixTable (BasicDataWindowMaker editorModule,  BasicDataWindow window, CharacterData data, int numRowsTotal, int numColumnsTotal, int totalWidth, int totalHeight, int taxonNamesWidth) {
		super(numRowsTotal, numColumnsTotal, totalWidth, totalHeight, taxonNamesWidth, ColorDistribution.getColorScheme(editorModule),true,true);
		this.window = window;
		setUserMove(true, true); 
		this.data = data;
		cellAnnotated = new CellAnnotation(data);
		showStates = new MesquiteBoolean(true);
		autoWithCharNames = new MesquiteBoolean(true);
		showBirdsEyeView = new MesquiteBoolean(false);
		tight = new MesquiteBoolean(false);
		showChanges = new MesquiteBoolean(true);
		showNames = new MesquiteBoolean(true);
		setUserAdjust(MesquiteTable.NOADJUST, MesquiteTable.RESIZE);
		showRowGrabbers=true;
		showColumnGrabbers=true;
		setEditable(true, true, true, false);
		setSelectable(true, true, true, true, true, false);
		this.editorModule=editorModule;
		this.taxa = data.getTaxa();
		setMessage(data.getCellContentsDescription());
		setBackground(ColorDistribution.project[ColorDistribution.getColorScheme(editorModule)]);
 		oldFont = getFont();
 		if (oldFont!=null)
 			boldFont = new Font(oldFont.getName(), Font.BOLD, oldFont.getSize());
 		synchronizeRowSelection(taxa);
 		synchronizeColumnSelection(data);
	}
	
	/*.................................................................................................................*/
	/** A request for the MesquiteModule to perform a command.  It is passed two strings, the name of the command and the arguments.
	This should be overridden by any module that wants to respond to a command.*/
 	public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) { 
		 if  (checker.compare(MesquiteTable.class, "Paste contents of clipboard into matrix", null, commandName, "paste")) {
			if (window.annotationHasFocus()){
				return super.doCommand(commandName, arguments, commandRec, checker);
			}
			Clipboard clip = Toolkit.getDefaultToolkit().getSystemClipboard();
			Transferable t = clip.getContents(this);
			try {
				String s = (String)t.getTransferData(DataFlavor.stringFlavor);
				if (s!=null) {
 					if (matrix.getEditing()|| rowNames.getEditing() || columnNames.getEditing()){
						TextField edit = null;
						if (matrix.getEditing())
							edit = matrix.getEditField();
						else if (rowNames.getEditing())
							edit = rowNames.getEditField();
						else if (columnNames.getEditing())
							edit = columnNames.getEditField();
						if (edit!=null) {
	 						String text = edit.getText();
	 						String newText = text.substring(0, edit.getSelectionStart()) + s + text.substring(edit.getSelectionEnd(), text.length());
	 						edit.setText(newText);
	 					}
 					}
					else if (clipboardDimensionsFit(s)){
						data.incrementNotifySuppress();
						pasteIt(s,commandRec);
						//deselectAll();
						data.notifyListeners(this, new Notification(MesquiteListener.DATA_CHANGED), commandRec);
						data.decrementNotifySuppress();
						
						repaintAll();
					}
					else {
						int[] lines = getTabbedLines(s); 
						if (lines==null || lines.length==0 || lines[0]==0) {
							editorModule.alert("Sorry, the pasted text was not appropriate for the matrix.");
							return null;
						}
						if (!IntegerArray.equalValues(lines)) {
							if (AlertDialog.query(editorModule.containerOfModule(), "Paste shape mismatch", "Sorry, the number of lines and of items pasted don't match the spaces selected to be filled.  Would you like Mesquite to attempt to adjust the selected region so that you can paste?  (Note: if you select OK, Mesquite will change the selection but will not paste. You will have to request again to paste.)", "OK", "Cancel", 1))
								setSelectionToShape(lines, commandRec);
						}
						else {
							if (AlertDialog.query(editorModule.containerOfModule(), "Paste shape mismatch", "Sorry, Clipboard is a different size or shape than the region selected, and thus you cannot paste.  Would you like Mesquite to attempt to adjust the selected region so that you can paste?  (Note: if you select OK, Mesquite will change the selection but will not paste. You will have to request again to paste.)", "OK", "Cancel", 1))
								setSelectionToShape(lines, commandRec);
						}
					}
				}
			}
			catch(Exception e){
				e.printStackTrace();
			}
		}
		else return super.doCommand(commandName, arguments, commandRec, checker);
 		return null;
 	}

	public boolean autoSizeColumns (Graphics g) {
		FontMetrics fm=g.getFontMetrics(g.getFont());
		int h = fm.getMaxAscent()+ fm.getMaxDescent() + MesquiteModule.textEdgeCompensationHeight; //2 + MesquiteString.riseOffset;
		setRowHeightsUniform(h);
		setColumnNamesHeight(h);
		String s;
		boolean changed = false;
		int def = fm.stringWidth("G"); //WPMMAT 12
		int max = def;
		for (int ic = 0; ic<data.getNumChars(); ic++) {
			if (!columnAdjusted(ic)){
				if (autoWithCharNames.getValue() && data.characterHasName(ic))
					max = fm.stringWidth(getColumnNameTextForDisplay(ic));
				else 
					max = def;
				if (showStates.getValue()){
					for (int it = 0; it<data.getNumTaxa(); it++) {
						s=getMatrixTextForDisplay(ic, it);
						int lengthString = fm.stringWidth(s);
						if (lengthString>max)
							max = lengthString;
					}
				}
				int newCW = 0;
				int current = getColumnWidth(ic);
				if (tight.getValue()) 
					newCW = max + 2; //WPMMAT + MesquiteModule.textEdgeCompensationHeight
				else
					newCW = max + 2 + MesquiteModule.textEdgeCompensationHeight;
				if (newCW != current) {
					setColumnWidth(ic, newCW);
					changed = true;
				}
			}
		}
		return changed;
	}
	
	
	/*...............................................................................................................*/
	public String getCornerText(){  
			return "Taxon  \\  Character";
	}
	/*...............................................................................................................*/
	/** sets which row is the first visible.*/
	public void setFirstRowVisible (int value) {
		super.setFirstRowVisible(value);
		window.setExplanation("Characters " + (getFirstColumnVisible()+1) + " to " + (getLastColumnVisible()) + "; Taxa " + + (getFirstRowVisible()+1) + " to " + (getLastRowVisible()));
	}
	/*...............................................................................................................*/
	/** sets which column is the first visible.*/
	public void setFirstColumnVisible (int value) {
		super.setFirstColumnVisible(value);
		window.setExplanation("Characters " + (getFirstColumnVisible()+1) + " to " + (getLastColumnVisible()) + "; Taxa " + + (getFirstRowVisible()+1) + " to " + (getLastRowVisible()));
	}
	/*...............................................................................................................*/
	/** returns color of row or number box. */
	public Color getRowNumberBoxColor(boolean isRow, int number) {
		Color color = null;
		if (!isRow)
			color = data.getDefaultCharacterColor(number);
		if (color!=null)
			return color;
		else if (!isRow)
			return Color.lightGray;
		return ColorDistribution.medium[colorScheme];
	}
	/*...............................................................................................................*/
	/** returns dark color of row or number box. */
	public Color getRowNumberBoxDarkColor(boolean isRow, int number) {
		Color color = null;
		if (!isRow)
			color = data.getDarkDefaultCharacterColor(number);
		if (color!=null)
			return color;
		return ColorDistribution.dark[colorScheme];
	}
	/*...............................................................................................................*/
	float[] hsb = new float[3];
	public void drawMatrixCell(Graphics g, int x, int y, int w, int h, int column, int row, boolean selected){
		if (data == null)
			return;
		boolean writeStates = !showBirdsEyeView.getValue() && showStates.getValue();
		boolean leaveEdges = writeStates && !tight.getValue();
		boolean changedSinceSave = showChanges.getValue() && data.getChangedSinceSave(column, row);
		Color drawn = null;
		Color c = g.getColor();
		hsb[0]=hsb[1]=hsb[2]= 1;
		if (selected){
			g.setColor(Color.black);
		}
		else {
			if (cellColorer !=null) {
				drawn = cellColorer.getCellColor(column, row);
				Color.RGBtoHSB(drawn.getRed(), drawn.getGreen(), drawn.getBlue(), hsb);
				g.setColor(drawn);
			}
			else
				g.setColor(Color.white);
		}
		if (leaveEdges)
			g.fillRect(x+1,y+1,w-1, h-1);
		else
			g.fillRect(x,y,w, h);
			
		if (writeStates) {
			if (selected)
				g.setColor(Color.white);
			else if (drawn==null || hsb[2]>0.5)
				g.setColor(Color.black);
			else
				g.setColor(Color.white);
			if (changedSinceSave){
				g.drawLine(x,y+1, x+1,y);
				g.drawLine(x,y+2, x+2,y);
				g.drawLine(x,y+3, x+3,y);
			}
			String st = getMatrixTextForDisplay(column, row);
			g.drawString(st, StringUtil.getStringCenterPosition(st,g,x,w), StringUtil.getStringVertPosition(g,y,h));

		}
		if (c!=null) g.setColor(c);
	}
	/*...............................................................................................................*/
	public  boolean useString(int column, int row){
		return false; //(cellColorer == null && !data.getChangedSinceSave(column, row));
	}
	/*...............................................................................................................*/
	public void drawColumnNameCell(Graphics g, int x, int y, int w, int h, int column){
		
		if (data == null)
			return;
		CharacterPartition part = (CharacterPartition)data.getCurrentSpecsSet(CharacterPartition.class);
		if (part!=null){
			CharactersGroup mi = (CharactersGroup)part.getProperty(column);
			if (mi!=null && mi.getColor()!=null) {
				Color c = g.getColor();
				g.setColor(mi.getColor());
				g.fillRect(x,y,w-1,h-1);
				if (c!=null) g.setColor(c);
			}
		}
		String name;
		if (data.characterHasName(column))
			name = data.getCharacterName(column);
		else
			name = Integer.toString(CharacterStates.toExternal(column));
		String s = data.getAnnotation(column);
		if (!StringUtil.blank(s)) 
			name = "*" + name;
			
		g.drawString(name, x+getNameStartOffset(), StringUtil.getStringVertPosition(g,y,h));
	}
	/*...............................................................................................................*/
	public void drawRowNameCell(Graphics g, int x, int y,  int w, int h, int row){
		if (taxa == null)
			return;
		TaxaPartition part = (TaxaPartition)taxa.getCurrentSpecsSet(TaxaPartition.class);
		if (part!=null){
			TaxaGroup mi = (TaxaGroup)part.getProperty(row);
			if (mi!=null && mi.getColor()!=null) {
				Color c = g.getColor();
				g.setColor(mi.getColor());
				g.fillRect(x,y,w-1,h-1);
				if (c!=null) g.setColor(c);
			}
		}
		String s = taxa.getAnnotation(row);
		
		int svp = StringUtil.getStringVertPosition(g,y,h);
		int xgnso = x+getNameStartOffset();

		if (!StringUtil.blank(s)) 
			g.drawString("*" + taxa.getTaxon(row).getName(), xgnso, svp);
		else
			g.drawString(taxa.getTaxon(row).getName(), xgnso, svp);
		
	}
	StringBuffer sb = new StringBuffer(50);
	/*...............................................................................................................*/
	public String getMatrixText(int column, int row){
		sb.setLength(0);
		data.statesIntoStringBuffer(column,row, sb, false);
		return sb.toString();
	}

	boolean displayModifications = false;
	private boolean checkDisplayModifications(){
		//go through all assistants to see if they claim hasDisplayModifications
 		Enumeration enum=window.ownerModule.getEmployeeVector().elements();
		while (enum.hasMoreElements()){
			Object obj = enum.nextElement();
			if (obj instanceof DataWindowAssistant) {
				DataWindowAssistant init = (DataWindowAssistant)obj;
				if (init.hasDisplayModifications()) {
					return true;
				}
			}
		}
		return false;
	}
	/*...............................................................................................................*/
	public String getMatrixTextForDisplay(int column, int row){
		sb.setLength(0);
		data.statesIntoStringBuffer(column,row, sb, showNames.getValue());
		if (!StringUtil.blank(data.getAnnotation(column, row))) {
			sb.append('*');
		}
		if (displayModifications){
			if (data.getCellObjectDisplay(column, row)) {
				//go through assistants that claim to have modifications to see what each wants to add
		 		Enumeration enum=window.ownerModule.getEmployeeVector().elements();
				while (enum.hasMoreElements()){
					Object obj = enum.nextElement();
					if (obj instanceof DataWindowAssistant) {
						DataWindowAssistant init = (DataWindowAssistant)obj;
						if (init.hasDisplayModifications()) {
							String s = init.getDisplayModString(column, row);
							if (s!=null)
								sb.append(s);
						}
					}
				}
			}
		}
		return sb.toString();
	}
	
	/*.................................................................................................................*/
	public String getColumnComment(int column) {
		return data.getAnnotation(column);
	}
	/*.................................................................................................................*/
	public String getRowComment(int row) {
		return taxa.getAnnotation(row);
	}
	/*...............................................................................................................*/
	/** Returns text in row name.  */
	public String getRowNameText(int row){
		Taxon t = taxa.getTaxon(row);
		if (t!=null) {
			return t.getName();
		}
		else
			return "";
	}
	/*...............................................................................................................*/
	/** Returns text in column name.  */
	public String getColumnNameText(int column){
		return data.getCharacterName(column);
	}
	/*...............................................................................................................*/
	/** Returns text in column name.  */
	public String getColumnNameTextForDisplay(int column){
		if (data.characterHasName(column))
			return data.getCharacterName(column);
		else
			return Integer.toString(CharacterStates.toExternal(column));
	}
	/*...............................................................................................................*/
	StringBuffer esb = new StringBuffer(100);
	private String getCellExplanation(int column, int row){
			String s = "[";
			if (row>=0 && row<data.getNumTaxa())
				s += "t." + (row+1);
			if (column>=0 && column<data.getNumChars())
				s += " c." + (column+1);
			if (row>=0 && row<data.getNumTaxa() && column>=0 && column<data.getNumChars()){
				esb.setLength(0);
				data.statesIntoStringBuffer(column,row, esb, false);
				s += " s." + esb;
			}
			s += "] ";
			if (column>=0)
				s += data.getCharacterName(column) + statesExplanation(column, row);
			if (row>=0 && row<data.getNumTaxa() && column>=0)
				s += " [in taxon \"" + data.getTaxa().getTaxonName(row) + "\"]";
			return s;
	}
	public void setFocusedCell(int column, int row){
		cellAnnotated.setCell(column, row);
		window.setAnnotation(cellAnnotated);
		window.setExplanation(getCellExplanation(column, row));
		if ((window.getCurrentTool()== window.arrowTool) && (isEditing(column, row))) {
			window.setCurrentTool(window.ibeamTool);
			window.getPalette().setCurrentTool(window.ibeamTool); 
		}
		super.setFocusedCell(column, row);
	}
	/*...............................................................................................................*/
	public void cellTouched(int column, int row, int modifiers, int clickCount) {
		if ((window.getCurrentTool()== window.arrowTool) && (clickCount>1)) {
			window.setCurrentTool(window.ibeamTool);
			window.getPalette().setCurrentTool(window.ibeamTool); 
			((TableTool)window.getCurrentTool()).cellTouched(column, row, modifiers);
		}
		else if (((TableTool)window.getCurrentTool()).useTableTouchRules())
			super.cellTouched(column, row, modifiers, clickCount);
		else
			((TableTool)window.getCurrentTool()).cellTouched(column, row, modifiers);
			
		if (window.getCurrentTool()== window.arrowTool){
			setFocusedCell(column, row);
		}
		else if (window.getCurrentTool()== window.ibeamTool){
			setFocusedCell(column, row);
		}
		else
			window.setAnnotation("", null);
	}
	/*...............................................................................................................*/
	public void cellDrag(int column, int row, int modifiers) {
		if (((TableTool)window.getCurrentTool()).useTableTouchRules()) 
			super.cellDrag(column, row, modifiers);
		else
		((TableTool)window.getCurrentTool()).cellDrag(column, row, modifiers);
	}
	/*...............................................................................................................*/
	public void cellDropped(int column, int row, int modifiers) {
		if (((TableTool)window.getCurrentTool()).useTableTouchRules())
			super.cellDropped(column, row, modifiers);
		else
		   ((TableTool)window.getCurrentTool()).cellDropped(column, row, modifiers);

	}
	
	/*...............................................................................................................*/
	public void rowNameTouched(int row, int modifiers, int clickCount) {
		if (window.getCurrentTool()== window.arrowTool){
			cellAnnotated.setCell(-1, row);
			window.setAnnotation(cellAnnotated);
		}
		else if (window.getCurrentTool()== window.ibeamTool){
			cellAnnotated.setCell(-1, row);
			window.setAnnotation(cellAnnotated);
		}
		else
			window.setAnnotation("", null);

		if (window.getCurrentTool()== window.arrowTool) 
			if (clickCount>1){
				window.setCurrentTool(window.ibeamTool); 
				window.getPalette().setCurrentTool(window.ibeamTool); 
				((TableTool)window.getCurrentTool()).cellTouched(-1, row, modifiers);
			}
			else
				super.rowNameTouched( row, modifiers, clickCount);
		else
			((TableTool)window.getCurrentTool()).cellTouched(-1, row, modifiers); 
	}
	/*...............................................................................................................*/
	public void rowTouched(int row, int modifiers) {
		if (((TableTool)window.getCurrentTool()).useTableTouchRules())
			super.rowTouched( row, modifiers);
		else
			((TableTool)window.getCurrentTool()).cellTouched(-1, row, modifiers); 
	}
	/*...............................................................................................................*/
	private String statesExplanation(int column,int row){
		String s = "";
		if (data.getClass() == CategoricalData.class){
			s += ":  ";
			CategoricalData cData = (CategoricalData)data;
			for (int i = 0; i<CategoricalState.maxCategoricalState; i++)
				if (cData.hasStateName(column, i))
					s +=  "(" + cData.getSymbol(i) + ") " + cData.getStateName(column, i) + "; ";
		}
		return s;
	}
	/*...............................................................................................................*/
	public void mouseInCell(int modifiers, int column,int row, MesquiteTool tool){
		if (column>=0 && column <= data.getNumChars() && !(matrix.getEditing()|| rowNames.getEditing() || columnNames.getEditing()) && !singleTableCellSelected()) {
			window.setExplanation(getCellExplanation(column, row));
		}
	}
	/*...............................................................................................................*/
	public void columnNameTouched(int column, int modifiers, int clickCount) {
		if (window.getCurrentTool()== window.arrowTool || window.getCurrentTool()== window.ibeamTool){
			cellAnnotated.setCell(column, -1);
			window.setAnnotation(cellAnnotated);
		}
		else
			window.setAnnotation("", null);
		
		if (window.getCurrentTool()== window.arrowTool) {
			if (clickCount>1){
				window.setCurrentTool(window.ibeamTool); 
				window.getPalette().setCurrentTool(window.ibeamTool);
				((TableTool)window.getCurrentTool()).cellTouched(column, -1, modifiers);
			}
			else
				super.columnNameTouched( column, modifiers, clickCount);
		}
		else {
			((TableTool)window.getCurrentTool()).cellTouched(column, -1, modifiers);
		}
	}
	/*...............................................................................................................*/
	public void columnTouched(int column, int modifiers) {
		if (((TableTool)window.getCurrentTool()).useTableTouchRules()) {
			super.columnTouched( column, modifiers);
		}
		else {
			((TableTool)window.getCurrentTool()).cellTouched(column, -1, modifiers);
		}
	}
	/*...............................................................................................................*/
	public void moveOneColumn(int column, int after){
		int width = getColumnWidth(column);
		if (column<after) {
			for (int i=column; i<after; i++) {
				setColumnWidth(i,getColumnWidth(i+1));
				if (isColumnSelected(i+1))
					selectColumn(i);
				else
					deselectColumn(i);
			}
			setColumnWidth(after,width);
		} 
		else if (column>after) {
			for (int i=column; i>after+1; i--)
				setColumnWidth(i,getColumnWidth(i-1));
			setColumnWidth(after+1,width);
		} 
		
		data.moveParts(column,1,after);//TODO: not correct! need to adjust and see what new "after" is
		data.moveInLinked(column, 1, after, false);
		
	}
	/*...............................................................................................................*/
	public void moveColumns(int starting, int num, int justAfter){
		//todo: check for illegal move
		if (justAfter<starting) { //moving columns left, closer to zero
			for (int i=starting; i<starting+num-1; i++)
				moveOneColumn(i,justAfter++);//after each move, starting is now one column on
		} 
		else {//moving columns right
			for (int i=0; i<num; i++) 
				moveOneColumn(starting,justAfter);  //after each move, starting is now column moved up, and justafter has moved up
		}

		synchronizeColumnSelection(data);
		data.notifyListeners(this, new Notification(MesquiteListener.PARTS_MOVED));
		data.notifyInLinked(new Notification(MesquiteListener.PARTS_MOVED));

 		editorModule.getModuleWindow().contentsChanged();
  	}
	/*...............................................................................................................*/
	public void moveOneRow(int row, int after){
		taxa.moveParts(row,1,after);
	}
	/*............................................................................................................*/
	/** Called if row was dragged and dropped. 
	(after = -1 if dropped above first row; -2 if below last.)*/
	public void selectedRowsDropped(int after) {
		int i = 0;
		Bits sel = getRowsSelected();
		while (i<getNumRows()) {
			if (sel.isBitOn(i)) {
				deselectRow(i);
				sel.clearBit(i);
				moveOneRow(i,after);
				sel.moveParts(i, 1, after);
				if (i>after)
					after++;
				i=0;
			}
			else
				i++;
		}
		synchronizeRowSelection(data);
		taxa.notifyListeners(this, new Notification(MesquiteListener.PARTS_MOVED));

 		editorModule.getModuleWindow().contentsChanged();
	}
	/*...............................................................................................................*/
	public void selectedColumnsDropped(int after){
		int i = 0;
		Bits sel = getColumnsSelected();
		while (i<getNumColumns()) {
			if (sel.isBitOn(i)) {
				deselectColumn(i);
				sel.clearBit(i);
				moveOneColumn(i,after);
				sel.moveParts(i, 1, after);
				if (i>after)
					after++;
				i=0;
			}
			else
				i++;
		}
		synchronizeColumnSelection(data);
		data.notifyListeners(this, new Notification(MesquiteListener.PARTS_MOVED));
		data.notifyInLinked(new Notification(MesquiteListener.PARTS_MOVED));

 		editorModule.getModuleWindow().contentsChanged();
  	}

	/*...............................................................................................................*/
	boolean doAutosize = false;
	boolean suppressAutosize = false;
	/*.................................................................................................................*/
 	protected void clearIt(CommandRecord commandRec){
		for (int i = 0; i<numColumnsTotal; i++) {
			if (isColumnNameSelected(i) || isColumnSelected(i)) {
				returnedColumnNameText(i,"", commandRec);
			}
		}
		for (int j = 0; j<numRowsTotal; j++) {
			/*if (isRowNameSelected(j) || isRowSelected(j)) {
				returnedRowNameText(j,"", commandRec);
			}
			*/
			for (int i = 0; i<numColumnsTotal; i++) {
				if (isCellSelected(i,j) || isRowSelected(j) || isColumnSelected(i)) {
					returnedMatrixText(i,j,"?", commandRec);
				}
			}
		}
 	}
 	boolean turnOffWarnings = false;
	/*...............................................................................................................*/
	public void returnedMatrixText(int column, int row, String s, CommandRecord commandRec){ //TODO: SHOULD CHECK FOR ERRORS
		if (window.ownerModule.isDoomed())
			return;
		CharacterState csBefore = data.getCharacterState(null, column,row);
		parser.setString(s);
		MesquiteString result = new MesquiteString("");
		int response = data.setState(column, row, parser, true, result); //receive errors?
		if (response == CharacterData.OK){
			CharacterState csAfter = data.getCharacterState(null, column,row);
			if (csBefore !=null && !csBefore.equals(csAfter)) {
				data.notifyListeners(this, new Notification(MesquiteListener.DATA_CHANGED, new int[] {column, row}));
				doAutosize = true;
			}
		}
		else if (!result.isBlank()) {
			String ws = "Illegal entry for character " + (column+1) + " in taxon "  + (row+1) + ": " + result.toString();
			if (turnOffWarnings || commandRec.scripting())
				window.ownerModule.logln(ws);
			else
				turnOffWarnings = !AlertDialog.query(window, "Illegal character state",  ws, "OK", "Don't warn again");
		}
	}
	/*...............................................................................................................*/
	public void returnedRowNameText(int row, String s, CommandRecord commandRec){
		if (s!=null && !s.equals(taxa.getTaxonName(row))) {
				String warning = taxa.checkNameLegality(row, s);
				if (warning == null)
					taxa.setTaxonName(row, s);
				else
					window.ownerModule.discreetAlert(commandRec, warning);
		}
	}
	/*...............................................................................................................*/
	public void returnedColumnNameText(int column, String s, CommandRecord commandRec){
		if (s!=null && !s.equals(data.getCharacterName(column))) {
			String warning = data.checkNameLegality(column, s);
			if (warning == null)
				data.setCharacterName(column, s);
			else
				window.ownerModule.discreetAlert(commandRec, warning);
		}
	}
	
	/*...............................................................................................................*/
	public int getMinColumnWidth(){
		return 3;
	}
   	 public void repaintAll(){
   	 	displayModifications = checkDisplayModifications();
   	 	if (data !=null)
   	 		data.checkTaxaIDs(null); //seeing that taxa id's still OK!
   	 	super.repaintAll();
   	 }
	/*...............................................................................................................*/
   	 public void paint(Graphics g) {
	   	if (MesquiteWindow.checkDoomed(this))
	   		return;
   	 	displayModifications = checkDisplayModifications();
   	 	Font gF = g.getFont();
   	 	boolean resized = false;
   	 	if (oldFont == null || !gF.equals(oldFont)) {
   	 		if ((data.pleaseAutoSizeColumns()) && !showBirdsEyeView.getValue()) //TODO: 
   	 			resized =autoSizeColumns(g);
   	 		oldFont = g.getFont();
   	 		boldFont = new Font(oldFont.getName(), Font.BOLD, oldFont.getSize());
   	 	}
   	 	else if (doAutosize && !suppressAutosize) {
   	 		doAutosize = false;
   	 		resized =autoSizeColumns(g);
   	 	}
   	 	if (resized) {
	   		MesquiteWindow.uncheckDoomed(this);
	   		resetComponentSizes();
   	 		repaintAll();
   	 	}
   	 	else {
   	 		super.paint(g);
	   		MesquiteWindow.uncheckDoomed(this);
	   	}
   	 }
	/*...............................................................................................................*/
}


