/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.charMatrices.ConcatenateMatrices; 

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;

/* ======================================================================== */
public class ConcatenateMatrices extends DataUtility { 
	CharacterData data;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName){
		return true;
	}
	/*.................................................................................................................*/
	/** returns whether this module is requesting to appear as a primary choice */
   	public boolean requestPrimaryChoice(){
   		return true;  
   	}
	/*.................................................................................................................*/
   	 public boolean isSubstantive(){
   	 	return true;
   	 }
	/*.................................................................................................................*/
   	 public boolean isPrerelease(){
   	 	return false;
   	 }
   	/** Called to operate on the data in all cells.  Returns true if data altered*/
   	public boolean operateOnData(CharacterData data, CommandRecord commandRec){ 
		this.data = data;
		CharacterData oData =getProject().getDifferentButCompatibleMatrix(data,  commandRec, "Sorry, there are no other compatible data matrices available for concatenation.  If the other matrix is in another file, open the file as a linked file before attempting to concatenate.", "Concatenate which matrix?");
		if (oData==null)
			return false;
		if (oData.isLinked(data) || data.isLinked(oData)) {
			discreetAlert(commandRec, "Sorry, those two matrices cannot be concatenated because they are linked");
			return false;
		}
		int origNumChars = data.getNumChars();
		data.addParts(data.getNumChars()+1, oData.getNumChars());
		data.addInLinked(data.getNumChars()+1, oData.getNumChars(), true);
		CharacterState cs = null;
		for (int it = 0; it<data.getNumTaxa() && it<oData.getNumTaxa(); it++){
			for (int ic = 0; ic<oData.getNumChars(); ic++){
				cs= oData.getCharacterState(cs, ic, it);
				data.setState(ic+origNumChars, it, cs);
				data.setCharacterName(ic+origNumChars, oData.getCharacterName(ic));
				//note that does not set other stuff (state names, weights, footnotes)
			}
		}
		data.notifyListeners(this, new Notification(MesquiteListener.PARTS_ADDED, new int[] {origNumChars, oData.getNumChars()}),  commandRec);
		return true;
	}
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) {
  	 	Snapshot temp = new Snapshot();
  	 	return temp;
  	 }
	/*.................................................................................................................*/
    	 public String getName() {
		return "Concatenate Other Matrix";
   	 }
   	 
	/*.................................................................................................................*/
  	 public String getExplanation() {
		return "Concatenates matrix onto one in data editor.  Footnotes and assumptions like weights and character models are NOT transferred.  For categorical data, state names are not included.  For continuous data, new items may need to be created to accommodate differences it items between the matrices.";
   	 }
}


	

