/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.charMatrices.ManageCharPartitions;
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;

/** Manages specifications of character partitions, including reading and writing from NEXUS files */
public class ManageCharPartitions extends CharSpecsSetManager {
	CharactersGroupVector groups; 
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		groups = new CharactersGroupVector();
 		return true;
	}
	public NexusBlock elementAdded(FileElement e){
		if (e instanceof CharactersGroup){
			if (groups.indexOf(e)<0) {
				groups.addElement(e, false);
			}
			e.setManager(this);
			return null;
		}
		else
			return super.elementAdded(e);
	}
	public void elementDisposed(FileElement e){
		if (groups!=null)
			groups.removeElement(e, false);
	}
	public void projectEstablished(){
		groups.addToFile(getProject().getHomeFile(), getProject(), this);
		super.projectEstablished();
	}
	public Class getElementClass(){
		return CharacterPartition.class;
	}
	public String upperCaseTypeName(){
		return "Character Partition";
	}
	public String lowerCaseTypeName(){
		return "character partition";
	}
	public String nexusToken(){
		return "CHARPARTITION";
	}
	private CharactersGroup findGroup(String token){ 
		if (token ==null)
			return null;
		for (int i = 0; i< groups.size(); i++){
			CharactersGroup cg = (CharactersGroup)groups.elementAt(i);
			if (token.equalsIgnoreCase(cg.getName())){
				return cg;
			}
		}
		return null;
	}
	
	private CharactersGroup makeGroup(String name, Parser subcommands, MesquiteFile file){
			CharactersGroup group = findGroup(name);
			if (group==null) {
				group = new CharactersGroup();
				group.setName(name);
				group.addToFile(file, getProject(), this);
				if (groups.indexOf(group)<0) 
					groups.addElement(group, false);
			}
			if (subcommands !=null){ //this should be passed into group to handle?
				String token = null;
				while ((token = subcommands.getNextToken())!=null){
					if (token.equalsIgnoreCase("COLOR")){
						token = subcommands.getNextToken(); //=
						token = subcommands.getNextToken(); // (
						token = subcommands.getNextToken(); // (
							if (token!=null && token.equalsIgnoreCase("RGB")) {
								double red = MesquiteDouble.fromString(subcommands.getNextToken()); //Red
								double green = MesquiteDouble.fromString(subcommands.getNextToken()); //green
								double blue = MesquiteDouble.fromString(subcommands.getNextToken()); //blue
								if (MesquiteDouble.isCombinable(red) && MesquiteDouble.isCombinable(green) && MesquiteDouble.isCombinable(blue)){
									Color c = new Color((float)red, (float)green, (float)blue);
									group.setColor(c);
								}
							}
					}
				}
			}
			return group;
	}
	public Object getSpecification(String token){ //NEED TO PASS FILE
		return makeGroup(token, null, getProject().getHomeFile());
	}
	public void setSpecification(SpecsSet specsSet, Object specification, int ic){
		if (specsSet==null || !(specsSet instanceof CharacterPartition) || !(specification instanceof CharactersGroup))
			return;
		CharacterPartition characterPartition = (CharacterPartition)specsSet;
		characterPartition.setProperty(specification,ic);
	}
	public SpecsSet getNewSpecsSet(String name, CharacterData data){
		return new CharacterPartition(name, data.getNumChars(), null, data);
	}
	public boolean appropriateBlockForWriting(String blockName){
		if (blockName == null)
			return false;
		return blockName.equalsIgnoreCase("SETS");
	}
	public boolean appropriateBlockForReading(String blockName){
		if (blockName == null)
			return false;
		return blockName.equalsIgnoreCase("SETS") || blockName.equalsIgnoreCase("ASSUMPTIONS");
	}
	/*.................................................................................................................*/
	public String nexusStringForSpecsSet(CharSpecsSet specsSet, CharacterData data, MesquiteFile file, boolean isCurrent){
		if (specsSet ==null || !(specsSet instanceof CharacterPartition))
			return null;
		CharacterPartition characterPartition = (CharacterPartition)specsSet;
		String s= "";
		if (characterPartition !=null && (characterPartition.getFile()==file || (characterPartition.getFile()==null && data.getFile()==file))) {
			String sT = " ";
			CharactersGroup[] parts = characterPartition.getGroups();
			boolean firstTime = true;
			
			if (parts!=null)
				for (int i=0; i<parts.length; i++) {
					String q = ListableVector.getListOfMatches((Listable[])characterPartition.getProperties(), parts[i], CharacterStates.toExternal(0));
					if (q != null) {
						if (!firstTime)
							sT += ", ";
						firstTime = false;
						sT += StringUtil.tokenize(parts[i].getName()) + " : " + q;
					}
				}
			
			if (!StringUtil.blank(sT)) {
				s+= "CHARPARTITION " ;
				if (isCurrent)
					s += "* ";
				s+= StringUtil.tokenize(characterPartition.getName()) + " ";
				if (file.getProject().getNumberCharMatrices()>1)
					s+= " (CHARACTERS = " +  StringUtil.tokenize(data.getName()) + ")";
				s+= " = "+  sT + ";" + StringUtil.lineEnding();
			}
		}
		return s;
   	}
	/*.................................................................................................................*/
	public boolean readNexusCommand(MesquiteFile file, String blockName, String command, MesquiteString comment){ 
		if (blockName.equalsIgnoreCase("LABELS")) {
			String commandName = parser.getFirstToken(command);
			if ("CHARGROUPLABEL".equalsIgnoreCase(commandName)) {
				String name = parser.getNextToken();
				makeGroup(name, parser, file); //pass whole command
				return true;
			}
			return false;
		}
		else 
			return super.readNexusCommand(file, blockName, command, comment);
	}
	/*.................................................................................................................*/
	public String getNexusCommands(MesquiteFile file, String blockName){ 
		if (blockName.equalsIgnoreCase("LABELS")) {
			String s = "";
			for (int i = 0; i< groups.size(); i++){
				CharactersGroup cg = (CharactersGroup)groups.elementAt(i);
				if (cg.getFile() == file){
					s += "\tCHARGROUPLABEL " + ParseUtil.tokenize(cg.getName());
					if (cg.colorSet()){
						Color c = cg.getColor();
						s += " COLOR = (RGB " + MesquiteDouble.toString(c.getRed()/255.0) + " " + MesquiteDouble.toString(c.getGreen()/255.0) + " " + MesquiteDouble.toString(c.getBlue()/255.0) + ") ";
					}
					s += ";" + StringUtil.lineEnding();
				}
			}
			if (StringUtil.blank(s))
				return null;
			else
				return s;
		}
		else  {
			return super.getNexusCommands(file, blockName);
		}
		}
	/*.................................................................................................................*/
	public NexusCommandTest getNexusCommandTest(){ 
		return new PartitionNexusCommandTest();
	}
	/*.................................................................................................................*/
   	 public boolean isPrerelease(){
   	 	return false;
   	 }
	/*.................................................................................................................*/
    	 public String getName() {
		return "Manage character partititions";
   	 }
	/*.................................................................................................................*/
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Manages (including NEXUS read/write) character partitions." ;
   	 }
	/*.................................................................................................................*/
   	 
}

/* ======================================================================== */
class PartitionNexusCommandTest  extends NexusCommandTest{
	public boolean readsWritesCommand(String blockName, String commandName, String command){  //returns whether or not can deal with command
		if ((blockName.equalsIgnoreCase("SETS") || blockName.equalsIgnoreCase("ASSUMPTIONS")) && commandName.equalsIgnoreCase("CHARPARTITION"))
			return true;
		
		if ((blockName.equalsIgnoreCase("LABELS")) && commandName.equalsIgnoreCase("CHARGROUPLABEL"))
			return true;
		
		return false;
	}
}

