/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.charMatrices.SortChar; 

import java.util.*;
import java.awt.*;
import java.text.Collator;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import mesquite.lib.table.*;


/* ======================================================================== */
public class SortChar extends DataWindowAssistantI { 
	TableTool charSortTool;
	CharacterData data;
	MesquiteTable table;
	Collator collator;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName){
		if (containerOfModule() instanceof MesquiteWindow) {
			charSortTool = new TableTool(this, "charSort", getPath(), "charSort.gif", 2, 8,"Sorts characters", "This tool sorts characters according to row selected", MesquiteModule.makeCommand("charSortTouch",  this) , null, null);
			charSortTool.setOptionImageFileName("revCharSort.gif", 13, 8);
			charSortTool.setWorksOnColumnNames(true);
			((MesquiteWindow)containerOfModule()).addTool(charSortTool);
		}
		else return false;
		collator = Collator.getInstance();
		return true;
	}
	/*.................................................................................................................*/
   	 public boolean isSubstantive(){
   	 	return true;
   	 }
	/*.................................................................................................................*/
   	 public boolean isPrerelease(){
   	 	return false;
   	 }
	public void setTableAndData(MesquiteTable table, CharacterData data, CommandRecord commandRec){
		this.table = table;
		this.data = data;
	}
	/*.................................................................................................................*/
  	 boolean compare(boolean greaterThan, String one, String two){
		int order = collator.compare(one, two);
		if ((order == 1 && greaterThan) ||(order == -1 && !greaterThan))
			return true;
		return false;
  	 }
	/*.................................................................................................................*/
 	/** Swaps parts first and second*/
	void swapParts(CharacterData data, int first, int second, String[] text){  
  		String temp = text[first];
  		text[first] = text[second];
  		text[second] = temp;
		data.swapParts(first, second); 
	}
	MesquiteInteger pos = new MesquiteInteger();
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(),  "Touches on a cell with the sort characters tool", "[column touched][row touched]", commandName, "charSortTouch")) {
	   	 		if (data == null)
	   	 			return null;
	   	 		if (data.isMolecularSequence())
	   	 			if (!AlertDialog.query(MesquiteTrunk.mesquiteTrunk.containerOfModule(), "Sort Characters", "Are you sure you want to reorder the sequences?  It cannot be undone.", "Sort", "Cancel", 2))
	   	 				return null;
	   	 		MesquiteInteger io = new MesquiteInteger(0);
	   			int column= MesquiteInteger.fromString(arguments, io);
	   			int row= MesquiteInteger.fromString(arguments, io);
	   			boolean gT = true;
    				if (arguments.indexOf("option")>=0)
    					gT = false;
	   			if (!MesquiteInteger.isCombinable(column) && !MesquiteInteger.isCombinable(row))
	   				return null;
				if (column>=0 && row >=0) {
					String[] text = new String[data.getNumChars()];
					for (int i=0; i<data.getNumChars(); i++)
						text[i] = table.getMatrixText(i, row);
					for (int i=1; i<data.getNumChars(); i++) {
						if (i % 10 == 0) commandRec.tick("Sorting from character " + i);
						for (int j= i-1; j>=0 && compare(gT, text[j], text[j+1]); j--) {
							swapParts(data, j, j+1, text);
						}
					}
					commandRec.tick("Sorting finished");
					data.notifyListeners(this, new Notification(MesquiteListener.PARTS_MOVED), commandRec);
					data.notifyInLinked(new Notification(MesquiteListener.PARTS_MOVED));  
				}
				else if (row == -1 && column >=0) { //row names selected; sort by taxon name
					for (int i=1; i<data.getNumChars(); i++) {
						if (i % 10 == 0) commandRec.tick("Sorting from character " + i);
						for (int j= i-1; j>=0 && compare(gT, data.getCharacterName(j), data.getCharacterName(j+1)); j--) {
							data.swapParts(j, j+1);
						}
					}
					commandRec.tick("Sorting finished");
					data.notifyListeners(this, new Notification(MesquiteListener.PARTS_MOVED), commandRec);
					data.notifyInLinked(new Notification(MesquiteListener.PARTS_MOVED));  
				}
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
   	 }
	/*.................................................................................................................*/
    	 public String getName() {
		return "Character Sort (data)";
   	 }
   	 
	/*.................................................................................................................*/
  	 public String getExplanation() {
		return "Provides a tool with which to sort characters automatically.";
   	 }
}


	

