/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.charMatrices.Wand; //DRM started April 02

import java.util.*;
import java.awt.*;
import java.text.Collator;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import mesquite.lib.table.*;

/*   to do:
	- deal with Option cursor, Shift cursor
	- pass taxaAreRows into this
*/


/* ======================================================================== */
public class Wand extends DataWindowAssistantI {
	TableTool magicWandTool;
	CharacterData data;
	MesquiteTable table;
	MesquiteBoolean wholeMatrix, singleTaxon, singleCharacter, equals, greaterthan, lessthan;
	boolean defaultEquals = true;
	boolean defaultGT = false;
	boolean defaultLT = false;
	boolean defaultSingleTaxon = false;
	boolean defaultSingleCharacter = false;
	boolean defaultWholeMatrix = true;
	boolean taxaAreRows = true;
	Collator collator;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName){
		if (containerOfModule() instanceof MesquiteWindow) {
			magicWandTool = new TableTool(this, "magicWand", getPath(), "magicWand.gif", 4,4,"Select cells with same", "This tool selects cells according to criteria of being the same, greater than, or less than the value in the cell touched.  The criteria can be set using the menu that appears when you click on the tool button", MesquiteModule.makeCommand("magicWandTouch",  this) , null, null);
			((MesquiteWindow)containerOfModule()).addTool(magicWandTool);
			magicWandTool.setPopUpOwner(this);
			setUseMenubar(false); //menu available by touching oning button
		}
		else return false;
		singleTaxon = new MesquiteBoolean(defaultSingleTaxon);
		singleCharacter = new MesquiteBoolean(defaultSingleCharacter);
		wholeMatrix = new MesquiteBoolean(defaultWholeMatrix);
		equals = new MesquiteBoolean(defaultEquals);
		greaterthan = new MesquiteBoolean(defaultGT);
		lessthan = new MesquiteBoolean(defaultLT);
		addCheckMenuItem(null, "Equal", makeCommand("toggleEquals",  this), equals);
		addCheckMenuItem(null, "Greater than", makeCommand("toggleGT",  this), greaterthan);
		addCheckMenuItem(null, "Less than", makeCommand("toggleLT",  this), lessthan);
		addMenuItem("-", null);
		addCheckMenuItem(null, "Whole Matrix", makeCommand("toggleWholeMatrix",  this), wholeMatrix);
		addCheckMenuItem(null, "Restrict to single taxon", makeCommand("toggleSingleTaxon",  this), singleTaxon);
		addCheckMenuItem(null, "Restrict to single character", makeCommand("toggleSingleCharacter",  this), singleCharacter);
		collator = Collator.getInstance();
		return true;
	}
	/*.................................................................................................................*/
   	 public boolean isSubstantive(){
   	 	return false;
   	 }
	public void setTableAndData(MesquiteTable table, CharacterData data, CommandRecord commandRec){
		this.table = table;
		this.data = data;
	}
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) {
  	 	Snapshot temp = new Snapshot();
		if (wholeMatrix.getValue()!=defaultWholeMatrix)
			temp.addLine("toggleWholeMatrix " + wholeMatrix.toOffOnString());
		if (singleTaxon.getValue()!=defaultSingleTaxon)
			temp.addLine("toggleSingleTaxon " + singleTaxon.toOffOnString());
		if (singleCharacter.getValue()!=defaultSingleCharacter)
			temp.addLine("toggleSingleCharacter " + singleCharacter.toOffOnString());
		if (equals.getValue()!=defaultEquals)
			temp.addLine("toggleEquals " + equals.toOffOnString());
		if (greaterthan.getValue()!=defaultGT)
			temp.addLine("toggleGT " + greaterthan.toOffOnString());
		if (lessthan.getValue()!=defaultLT)
			temp.addLine("toggleLT " + lessthan.toOffOnString());
  	 	return temp;
  	 }
  	 boolean satisfiesCriteria(String one, String two){
		if (equals.getValue() && one.equals(two))
			return true;
		double dOne = MesquiteDouble.fromString(one);
		double dTwo = MesquiteDouble.fromString(two);
		if (MesquiteDouble.isCombinable(dOne) && MesquiteDouble.isCombinable(dTwo)) {
			if (greaterthan.getValue() && (dTwo>dOne))
				return true;
			if (lessthan.getValue() && (dTwo<dOne))
				return true;
			return false;
		}
		int order = collator.compare(one, two);
		//int order =0;
		if (greaterthan.getValue() && (order == -1))
			return true;
		if (lessthan.getValue() && (order == 1))
			return true;
		if (equals.getValue() && (order == 0))
			return true;
		return false;
  	 }
	MesquiteInteger pos = new MesquiteInteger();
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(),  "Touches on a cell with the wand to select cells similar as defined by the options", "[column touched][row touched]", commandName, "magicWandTouch")) {
	   	 		MesquiteInteger io = new MesquiteInteger(0);
	   			int column= MesquiteInteger.fromString(arguments, io);
	   			int row= MesquiteInteger.fromString(arguments, io);
				if (MesquiteInteger.isNonNegative(column)&& (MesquiteInteger.isNonNegative(row))) {
	   				boolean commandSelected =  (arguments.indexOf("command")>=0);
	   				boolean subtractFromSelection = commandSelected && table.isCellSelected(column, row);
		   			if ((arguments.indexOf("shift")<0) && !commandSelected)
		   				table.deselectAll();
		   			table.offAllEdits();
					String text = table.getMatrixText(column, row);
					int rowStart = 0;
					int rowEnd = table.getNumRows();
					int columnStart = 0;
					int columnEnd = table.getNumColumns();
					if (singleTaxon.getValue()) {
						if (taxaAreRows) {
							rowStart = row;
							rowEnd = row+1;
						}
						else {
							columnStart = column;
							columnEnd = column+1;
						}
					}
					else if (singleCharacter.getValue()) {
						if (!taxaAreRows) {
							rowStart = row;
							rowEnd = row+1;
						}
						else {
							columnStart = column;
							columnEnd = column+1;
						}
					}
					for (int j=rowStart; j<rowEnd; j++){
						for (int i=columnStart; i<columnEnd; i++){
							if (satisfiesCriteria(text, table.getMatrixText(i, j))) {
								if (subtractFromSelection)
									table.deselectCell(i, j); //deselect cell
								else
									table.selectCell(i, j); //select cell
							}
						}
					}
					data.notifyListeners(this, new Notification(MesquiteListener.SELECTION_CHANGED), commandRec);
						
				}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets whether the wand selects through the matrix", "[on = whole matrix; off]", commandName, "toggleWholeMatrix")) {
    	 		boolean current = wholeMatrix.getValue();
    	 		wholeMatrix.toggleValue(parser.getFirstToken(arguments));
    	 		if (wholeMatrix.getValue()) {
    	 			singleTaxon.setValue(false);
    	 			singleCharacter.setValue(false);
    	 		}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets whether the wand selects only cells of the taxon that was touched", "[on = single taxon; off]", commandName, "toggleSingleTaxon")) {
    	 		boolean current = singleTaxon.getValue();
    	 		singleTaxon.toggleValue(parser.getFirstToken(arguments));
    	 		if (singleTaxon.getValue()) {
    	 			wholeMatrix.setValue(false);
    	 			singleCharacter.setValue(false);
    	 		}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets whether the wand selects only cells of the character that was touched", "[on = single character; off]", commandName, "toggleSingleCharacter")) {
    	 		boolean current = singleCharacter.getValue();
    	 		singleCharacter.toggleValue(parser.getFirstToken(arguments));
  	 		if (singleCharacter.getValue()) {
    	 			wholeMatrix.setValue(false);
    	 			singleTaxon.setValue(false);
    	 		}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets whether the wand selects cells with value equal to that in cell touched", "[on = selects equal; off]", commandName, "toggleEquals")) {
    	 		boolean current = equals.getValue();
    	 		equals.toggleValue(parser.getFirstToken(arguments));
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets whether the wand selects cells with value greater than that in cell touched", "[on = selects greater than; off]", commandName, "toggleGT")) {
    	 		boolean current = greaterthan.getValue();
    	 		greaterthan.toggleValue(parser.getFirstToken(arguments));
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets whether the wand selects cells with value less than that in cell touched", "[on = selects less than; off]", commandName, "toggleLT")) {
    	 		boolean current = lessthan.getValue();
    	 		lessthan.toggleValue(parser.getFirstToken(arguments));
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
   	 }
	/*.................................................................................................................*/
    	 public String getName() {
		return "Cell Wand (data)";
   	 }
   	 
	/*.................................................................................................................*/
  	 public String getExplanation() {
		return "Provides a tool with which to select cells in a matrix automatically.";
   	 }
}


	

