/* Mesquite source code.  Copyright 1997-2004 W. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.coalesce.ContainedCoalescence;
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;
import mesquite.coalesce.lib.*;
import mesquite.assoc.lib.*;
/* ======================================================================== */
public class ContainedCoalescence extends TreeSimulate {
	AssociationSource associationTask;
	Coalescer coalescenceTask;
	int oldPopulationSize = 0;
	TaxaAssociation association;
	OneTreeSource oneTreeSourceTask;
	Tree speciesTree;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		//TODO: allow subsequent choosing from menus
		associationTask = (AssociationSource)hireEmployee(commandRec, AssociationSource.class, "Source of taxon association");
		if (associationTask == null)
			return sorry(commandRec, getName() + " couldn't start because no source of taxon associations obtained.");
		coalescenceTask = (Coalescer)hireEmployee(commandRec, Coalescer.class, "Coalescence simulator");
		if (coalescenceTask == null)
			return sorry(commandRec, getName() + " couldn't start because no coalescence module obtained.");
		if (oneTreeSourceTask == null) {
			oneTreeSourceTask = (OneTreeSource)hireEmployee(commandRec, OneTreeSource.class, "Source of containing tree");
		}
 		return true;
  	 }
  	 public void endJob(){
  	 	if (savedBush!=null)
  	 		savedBush.dispose();
  	 	super.endJob();
  	 }
	/*.................................................................................................................*/
  	 public void employeeQuit(MesquiteModule m){
  	 	iQuit();
  	 }
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) { 
   	 	Snapshot temp = new Snapshot();
  	 	temp.addLine("getAssociationTask ", associationTask); 
  	 	temp.addLine("getTreeSource ", oneTreeSourceTask); 
  	 	temp.addLine("forgetAssociation "); 
  	 	
  	 	return temp;
  	 }
  	 /*---*/
  	
	MesquiteInteger pos = new MesquiteInteger();
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if  (checker.compare(this.getClass(), "Returns the species tree being used for contained coalescence", null, commandName, "getSpeciesTree")) {
    	 		return speciesTree;
    	 	}
    	 	else if  (checker.compare(this.getClass(), "Returns the containing tree context", null, commandName, "getTreeSource")) {
    	 		return oneTreeSourceTask;
    	 	}
    	 	else if  (checker.compare(this.getClass(), "Returns the module supplying associations", null, commandName, "getAssociationTask")) {
    	 		return associationTask;
    	 	}
    	 	else if  (checker.compare(this.getClass(), "Sets the current association to null", null, commandName, "forgetAssociation")) {
    	 		association = null;
    	 		return null;
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
   	 }
	/*.................................................................................................................*/
   	private boolean check(Taxa taxa, CommandRecord commandRec){
   		if (taxa!=null) {
	        	if (association == null || (association.getTaxa(0)!= taxa && association.getTaxa(1)!= taxa)) {
	        		association = associationTask.getCurrentAssociation(taxa, commandRec);
	        		if (association==null) {
	        			String s = "Association null in Contained Coalescence (for taxa " + taxa.getName() + ")";
	        			discreetAlert(commandRec, s);
	        			return false;
	        		}
	        	}
			if (oneTreeSourceTask == null) {
				oneTreeSourceTask = (OneTreeSource)hireEmployee(commandRec, OneTreeSource.class, "Source of containing tree");
				if (oneTreeSourceTask == null) {
	        			String s = "Source for containing tree null in Contained Coalescence (for taxa " + taxa.getName() + ")";
	        			discreetAlert(commandRec, s);
	        			return false;
				}
					
			}
   		}
   		return true;
   	}
	/*.................................................................................................................*/
	NameReference widthNameReference = NameReference.getNameReference("width");
	DoubleArray widths = null;
	double maxWidth = 0;
	/*.................................................................................................................*/
	double branchNEAdjustment(int node){
		if (widths !=null) {
			double w = widths.getValue(node);
			if (MesquiteDouble.isCombinable(w) && w>=0) {
				return w;
			}
		}	
		return 1.0;
	}
   	public void initialize(Taxa taxa, CommandRecord commandRec){
   		if (!check(taxa, commandRec))
   			return;
   		if (oneTreeSourceTask!=null && association!=null)
   			oneTreeSourceTask.initialize(association.getOtherTaxa(taxa), commandRec);

   	}
	/*.................................................................................................................*/
	boolean first = true;
	/*.................................................................................................................*/
   	int[] containing; //length of geneTree.getNumNodeSpaces();
	/*.................................................................................................................*/
	/*  coalesces the gene tree nodes contained within the species tree node "node". */
	private synchronized int coalesceDown (Tree speciesTree, MesquiteTree geneTree, Taxa geneTaxa, int node, int root, MesquiteLong seed, CommandRecord commandRec) {
		int nodeNumber = -2;
		double br=speciesTree.getBranchLength(node);  //for calculating the number of generations in which coalescence can occur here
		long generations=0;
		//branch length is NOT scaled by population size; thus branch length is now treated as the number of generations to coalesce
		if (MesquiteDouble.isCombinable(br) && (long)(br)>1) //&& node != speciesTree.getRoot() 
			generations =  (long)(br); 
		else 
			generations =1;
		if (speciesTree.nodeIsTerminal(node)) { //if terminal node, make CoalescedNode array, one element for each contained gene
			int taxonNum = speciesTree.taxonNumberOfNode(node);
			Taxon species =speciesTree.getTaxa().getTaxon(taxonNum);
			nodeNumber=node;
			Taxon[] containedGenes = association.getAssociates(species);
			if (containedGenes != null) {
				for (int i=0; i<containedGenes.length; i++)  {
					int taxNum = geneTaxa.whichTaxonNumber(containedGenes[i]);
					int cN = geneTree.nodeOfTaxonNumber(taxNum);
					if (!geneTree.nodeExists(cN)) {
						MesquiteMessage.warnProgrammer("oops ContainedCoalescence containedGenes[i] " + containedGenes[i] + " taxNum " + taxNum + " cN " + cN);
						MesquiteMessage.warnProgrammer("   " + geneTree.writeTree());
					}
					containing[cN] = node;  //can't use taxonNum because if 1, screws things up
				}
			}
		}
		else {
			for (int daughter = speciesTree.firstDaughterOfNode(node); speciesTree.nodeExists(daughter); daughter = speciesTree.nextSisterOfNode(daughter)){
				int nn = coalesceDown(speciesTree, geneTree, geneTaxa, daughter, root, seed, commandRec);
				if (nodeNumber==-2)
					nodeNumber = nn;
				else {
					//change all occurrences of nn in containing to nodeNumber
					for (int i =0; i<containing.length; i++)
						if (containing[i]== nn) {
							if (!geneTree.nodeExists(i))
								MesquiteMessage.warnProgrammer("oops ContainedCoalescence nn " + nn);
							containing[i]=nodeNumber;
						}
				}
			}
		}
		// now about to coalesce the gene tree nodes listed in containing as belonging to nodeNumber.
		if (MesquiteLong.isInfinite(generations)) {
			coalescenceTask.coalesce(geneTree, containing, nodeNumber, branchNEAdjustment(node), 0, seed, true, commandRec); //speciesTree.getRoot() == node);
		}
		else {

			coalescenceTask.coalesce(geneTree, containing, nodeNumber,  branchNEAdjustment(node), generations, seed, root == node, commandRec); //speciesTree.getRoot() == node);
		}
		return nodeNumber;
	}
	/*.................................................................................................................*/
   	public int getNumberOfTrees(Taxa taxa, CommandRecord commandRec) {
   		return MesquiteInteger.infinite;
   	}
   	MesquiteTree savedBush;
   	long oldTaxaVersion = -1;
   	boolean inProgress = false;
	/*.................................................................................................................*/
   	public Tree getSimulatedTree(Taxa taxa, Tree geneTree, int treeNumber, ObjectContainer extra, MesquiteLong seed, CommandRecord commandRec) {
   		if (!check(taxa, commandRec))
   			return null;
   		if (association==null) {
   			if (first)
   				alert("No association found for use by Contained Coalescence; no tree could be made");
   			first = false;
   			return null;
   		}
		if (inProgress)
			return null;
		inProgress = true;
		speciesTree = oneTreeSourceTask.getTree(association.getOtherTaxa(taxa), commandRec);
   		if (speciesTree==null) {
   			if (first)
   				alert("No species tree found for use by Contained Coalescence.  Taxa blocks: " + taxa + " AND " + association.getOtherTaxa(taxa));
   			first = false;
   			inProgress = false;
   			return null;
   		}
        	widths = speciesTree.getWhichAssociatedDouble(widthNameReference);
		MesquiteTree t=null;
		if (geneTree==null || !(geneTree instanceof MesquiteTree)) 
			 t = new MesquiteTree(taxa);
		else
			t = (MesquiteTree)geneTree;
		if (savedBush==null || savedBush.getTaxa()!=t.getTaxa() || oldTaxaVersion != t.getVersionNumber()) {
			if (savedBush != null)
				savedBush.dispose();
			t.setToDefaultBush(t.getNumTaxa(), false);
			Taxa geneTaxa = t.getTaxa();
			oldTaxaVersion = t.getVersionNumber();
			Taxon[] assocs = null;
			for (int i=0; i<geneTaxa.getNumTaxa(); i++){
				assocs = association.getAssociates(geneTaxa.getTaxon(i), assocs);  //get containing taxa
				//if any of containing taxa in speciesTree then keep, otherwise delete contained taxon from savedbush
				if (!inSpeciesTree(speciesTree, assocs, speciesTree.getTaxa())){
					t.snipClade(t.nodeOfTaxonNumber(i), false);
				}
			}
			savedBush = t.cloneTree();//no need to establish listener to Taxa, as will be remade when needed?
		}
		else {
			t.setToClone(savedBush);
		}
		if (containing ==null || containing.length!=t.getNumNodeSpaces())
			containing = new int[t.getNumNodeSpaces()];
		for (int i=0; i<containing.length; i++)
			containing[i] = 0;
		int root = speciesTree.getRoot();
		coalesceDown(speciesTree, t, taxa, root, root, seed, commandRec);
   		inProgress = false;
   		return t;
   	}
   	public boolean inSpeciesTree(Tree speciesTree, Taxon[] assocs, Taxa taxa){
   		if (assocs == null)
   			return false;
   		for (int i=0; i<assocs.length; i++){
   			int t = taxa.whichTaxonNumber(assocs[i]);
   			if (speciesTree.nodeOfTaxonNumber(t)>=0)
   				return true;
   		}
   		return false;
   	}
	/*.................................................................................................................*/
    	 public String getName() {
		return "Coalescence Contained within Current Tree";
   	 }
   	 
	/*.................................................................................................................*/
   	 
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Generates tree by a simple coalescence model of a neutral gene with constant population size, within a current species tree from a Tree window or other tree context. "
 		+"Branch lengths are assigned according to generation of coalescence." 
 		+"The species tree used is a current tree found in a Tree Window or other tree context." ;
   	 }
	/*.................................................................................................................*/
 	/** returns current parameters, for logging etc..*/
 	public String getParameters() {
		if (speciesTree!=null && coalescenceTask!=null)
			return "Species tree: " + speciesTree.getName() + "; Coalescence simulated by: " + coalescenceTask.getName();
		return "";
   	 }
   	public boolean isPrerelease(){
   		return false;
   	}
	/*.................................................................................................................*/
   	 public boolean showCitation(){
   	 	return true;
   	 }
}

