/* Mesquite (package mesquite.io).  Copyright 2000-2004 D. Maddison and W. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.io.InterpretTabbedConts;
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import mesquite.categ.lib.*;
import mesquite.cont.lib.*;

/* ============  a file interpreter for tabbed continuous files ============*/

public class InterpretTabbedConts extends FileInterpreterI {
/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
 		return true;  //make this depend on taxa reader being found?)
  	 }
  	 
/*.................................................................................................................*/
	public boolean canExport() {  
		 return getProject().getNumberCharMatrices(ContinuousState.class) > 0; //
	}
/*.................................................................................................................*/
	public boolean canExport(Class dataClass) {  
		return (dataClass==ContinuousState.class);
	}
/*.................................................................................................................*/
	public boolean canImport() {  
		 return true;
	}
/*.................................................................................................................*/
	public void readFile(MesquiteProject mf, MesquiteFile file, String arguments, CommandRecord commandRec) {
		incrementMenuResetSuppression();
		ProgressIndicator progIndicator = new ProgressIndicator(mf,"Importing File "+ file.getName(), file.existingLength());
		progIndicator.start();
		file.linkProgressIndicator(progIndicator);
		if (file.openReading()) {
			boolean abort = false;
			TaxaManager taxaTask = (TaxaManager)findElementManager(Taxa.class);
			 CharactersManager charTask = (CharactersManager)findElementManager(CharacterData.class);
			
			Taxa taxa = taxaTask.makeNewTaxa(getProject().getTaxas().getUniqueName("Untitled Block of Taxa"), 0, false);
			taxa.addToFile(file, getProject(), taxaTask);
			CharacterData data = charTask.newCharacterData(taxa, 0, "Continuous Data");
			boolean wassave = data.saveChangeHistory;
			data.saveChangeHistory = false;
			data.addToFile(file, getProject(), null);
			
			int numTaxa = 0;
			StringBuffer sb = new StringBuffer(1000);
			file.readLine(sb);
			String line = sb.toString();
			
			String token;

			while (!StringUtil.blank(line)&& !abort) {
				parser.setString(line);
				token = parser.getNextToken();
				taxa.addTaxa(numTaxa-1, 1, true);
				Taxon t = taxa.getTaxon(numTaxa);
				if (t!=null) {
					t.setName(token);
					progIndicator.setText("Reading taxon: "+token);
					int ic = 0;
					while (parser.getPosition()<line.length()) {
						if (data.getNumChars() <= ic) {
							data.addParts(data.getNumChars()-1, 1);   // add a character if needed
						}
						data.setState(ic, numTaxa, parser, false, null);
						ic += 1;
					}
				}
				numTaxa++;
				file.readLine(sb);
				line = sb.toString();		
				if (file.getFileAborted()) {
					abort = true;
				}
			}
			data.saveChangeHistory = wassave;
			data.resetChangedSinceSave();
			finishImport(progIndicator, file, abort, commandRec);
		}
		decrementMenuResetSuppression();
	}
	


/* ============================  exporting ============================*/
	boolean includeGaps = false;
	/*.................................................................................................................*/
	
	public boolean getExportOptions(boolean dataSelected, boolean taxaSelected){
		MesquiteInteger buttonPressed = new MesquiteInteger(1);
		ExporterDialog exportDialog = new ExporterDialog(this,containerOfModule(), "Export Tabbed Continuous Options", buttonPressed);
		
		exportDialog.completeAndShowDialog(dataSelected, taxaSelected);
			
		boolean ok = (exportDialog.query(dataSelected, taxaSelected)==0);
		
		exportDialog.dispose();
		return ok;
	}	

	/*.................................................................................................................*/

	public void exportFile(MesquiteFile file, String arguments, CommandRecord commandRec) { //if file is null, consider whole project open to export
		Arguments args = new Arguments(new Parser(arguments), true);
		boolean usePrevious = args.parameterExists("usePrevious");

		ContinuousData data = (ContinuousData)getProject().chooseData(containerOfModule(), file, null, ContinuousState.class, "Select data to export", commandRec);
		if (data ==null) {
			showLogWindow(true);
			logln("WARNING: No suitable data available for export to a file of format \"" + getName() + "\".  The file will not be written.\n");
			return;
		}
		Taxa taxa = data.getTaxa();
		if (!commandRec.scripting() && !usePrevious)
			if (!getExportOptions(data.anySelected(), taxa.anySelected()))
				return;

		int numTaxa = taxa.getNumTaxa();
		int numChars = data.getNumChars();
		StringBuffer outputBuffer = new StringBuffer(numTaxa*(20 + numChars));
		
		for (int it = 0; it<numTaxa; it++){
			if (!writeOnlySelectedTaxa || (taxa.getSelected(it))){
				outputBuffer.append(ParseUtil.tokenize(taxa.getTaxonName(it)));
				for (int ic = 0; ic<numChars; ic++) {
					if (!writeOnlySelectedData || (data.getSelected(ic))){
						outputBuffer.append(ParseUtil.tokenize("\t"));
						data.statesIntoStringBuffer(ic, it, outputBuffer, false);
					}
				}
				outputBuffer.append(getLineEnding());
			}
		}
		
		saveExportedFileWithExtension(outputBuffer, arguments, "txt", commandRec);
	}

	/*.................................................................................................................*/
    	 public String getName() {
		return "Tab-delimited continuous data file";
   	 }
	/*.................................................................................................................*/
   	 
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Imports and exports simple tab-delimited files of continuous data." ;
   	 }
	/*.................................................................................................................*/
   	 
   	 
}
	
