/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib;

import java.awt.*;
import java.util.*;



/* ======================================================================== */
/**  */
public class ChangeHistory {
	Vector events;
	
	public ChangeHistory(){
		events = new Vector();
	}
	public ChangeEvent addEvent(int author, long time, String change){
		ChangeEvent e = null;
		events.addElement(e = new ChangeEvent(author, time, change));
		return e;
	}
	public long lastModified(){
		if (events.size()==0)
			return 0;
		long max = 0;
		for (int i=events.size()-1; i>=0;i--) {
			ChangeEvent ce = (ChangeEvent)events.elementAt(i);
			if (ce.time > max)
				max = ce.time;
		}
		return max;
	}
	public boolean timeExists(long time){
		if (events.size()==0)
			return false;
		for (int i=events.size()-1; i>=0;i--) {
			ChangeEvent ce = (ChangeEvent)events.elementAt(i);
			if (ce.time == time)
				return true;
		}
		return false;
	}
	public ChangeEvent updateEvent(int author, long time, String change){
		if (events.size()==0)
			return null;
		for (int i=events.size()-1; i>=0;i--) {
			ChangeEvent ce = (ChangeEvent)events.elementAt(i);
			if (ce.time == time) {
				ce.change = change;
				return ce;
			}
		}
		return null;
	}
	public String toString(){
		if (events.size()==0)
			return null;
		StringBuffer sb = new StringBuffer(1000);
		for (int i=events.size()-1; i>=0;i--) {
			ChangeEvent ce = (ChangeEvent)events.elementAt(i);
			sb.append("Author " + ce.author + "; Time " + new Date(ce.time) + " Changed to: " + ce.change);
			ChangeAuthority ca = ce.getAuthority();
			if (ca !=null) {
				sb.append("; Authority: ");
				sb.append(ca.toString());
			}
			sb.append('\n');
		}
		return sb.toString();
	}
	public String getNexusString(){
		if (events.size()==0)
			return null;
		StringBuffer sb = new StringBuffer(1000);
		for (int i=0; i<events.size();i++) {
			ChangeEvent ce = (ChangeEvent)events.elementAt(i);
			sb.append("( " + ce.author + " " + ce.time + " " + ParseUtil.tokenize(ce.change));
			ChangeAuthority ca = ce.getAuthority();
			if (ca !=null) {
				sb.append(' ');
				sb.append(ca.getNexusString());
			}
			sb.append(" ) ");
		}
		return sb.toString();
	}

}

