/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib;

import java.awt.*;
import java.awt.event.*;
import mesquite.lib.duties.*;
/* ======================================================================== */
/** A container fitting within the InterContentArea, below the information bar.  It is within the ContentArea that
modules draw their stuff.*/
public class ContentArea extends MousePanel {
	protected ToolPalette palette;
	protected MousePanel mainPanel;
	public ContentArea () {
		setLayout(null);
		mainPanel = new MousePanel();
		mainPanel.setLayout(null);
		mainPanel.setBounds(0,0,getBounds().width, getBounds().height);
		//mainPanel.setBackground(Color.green);
		super.add(mainPanel);
		mainPanel.requestFocus(); //this may address a MRJ 2.2.3 bug
	}
	public void setPalette(ToolPalette palette){
		if (palette== null) {
			super.remove(this.palette);
			this.palette = null;
			setBounds(0,0,getBounds().width, getBounds().height);
		}
		else {
			this.palette = palette;
			super.add(palette);
			setBounds(0,0,getBounds().width, getBounds().height);
			palette.setVisible(true);
			palette.repaint(); //attempt to deal with bug in OS X 10.2
		}
	}
	public ToolPalette getPalette(){
		return palette;
	}
	public Panel getMainPanel(){
		return mainPanel;
	}
	public Component add(Component comp, int i){
		if (comp==null)
			return null;
		Component c = mainPanel.add(comp,  i);
		mainPanel.requestFocus();

		return c;
	}
	public void add(Component comp, Object s, int i){
		if (comp==null)
			return;
		mainPanel.add(comp, s, i);
		mainPanel.requestFocus();
	}
	public void add(Component comp, Object s){
		if (comp==null)
			return;
		mainPanel.add(comp, s);
		mainPanel.requestFocus();
	}
	public Component add(Component comp){
		if (comp==null)
			return null;
		Component c = mainPanel.add(comp);
		mainPanel.requestFocus();
		return c;
	}
	public Component add(String name, Component comp){
		if (comp==null)
			return null;
		Component c = mainPanel.add(name, comp);
		mainPanel.requestFocus();
		return c;
	}
	public void remove(Component comp){
		if (comp==null)
			return;
		mainPanel.remove(comp);
		mainPanel.requestFocus();
	}
	public void paint (Graphics g) {
	   	if (MesquiteWindow.checkDoomed(this))
	   		return;
		super.paint(g);
		Container c = getParent();
		while (c!=null && !(c instanceof MesquiteWindow))
			 c = c.getParent();
		if (c!=null)
			((MesquiteWindow)c).paintContents(g);
		MesquiteWindow.uncheckDoomed(this);
	}
	public void update (Graphics g) {
		try {
			super.update(g);
		}
		catch (OutOfMemoryError e){
		}
	}
	public void setBounds(int x, int y, int w, int h){
		if (palette!=null) {
			palette.setBounds(0,0,palette.getWidth(),h);
			palette.recheckSize();
			palette.setBounds(0,0,palette.getWidth(),h);
			mainPanel.setBounds(palette.getWidth(),0,w-palette.getWidth(),h );
		}
		else {
			mainPanel.setBounds(0,0,w,h);
		}
		super.setBounds(x,y,w,h);
	}
	public void setSize(int w, int h){
		if (palette!=null) {
			palette.setSize(palette.getWidth(),h);
			palette.recheckSize();
			palette.setSize(palette.getWidth(),h);
			mainPanel.setSize(w-palette.getWidth(),h);
		}
		else {
			mainPanel.setSize(w,h);
		}
		super.setSize(w,h);
	}
}
