/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib;

import java.awt.*;

/*Last documented:  August 1999 */
/* ======================================================================== */
public class Double2DArray {
	double[][] values;
	int numC;
	 int numT;
	NameReference name=null;
	public Double2DArray(int numC, int numT){
		this.numC = numC;
		this.numT = numT;	
		values = new double[numC][numT];
		for (int i=0; i<numC; i++)
			for (int j=0; j<numT; j++)
				values[i][j] =  MesquiteDouble.unassigned;
	}
	public Double2DArray(double[][] input){
		this(numFullColumns(input), numFullRows(input));
		for (int i=0; i<numC; i++)
			for (int j=0; j<numT; j++)
				values[i][j] =  input[i][j];
	}
	/*...........................................................*/
	public static double[][] clone(double[][] d){
		if (d==null || d.length==0)
			return d;
		double[][] q = new double[d.length][d[0].length];
		for (int i=0; i< d.length; i++)
			for (int j=0; j< d[i].length;j++)
				q[i][j] = d[i][j];
		return q;
	}
	/*...........................................................*/
	public void dispose(){
		values = null;
	}
	/*...........................................................*/
	public static int numFullColumns(double[][] matrix){
		if (matrix==null || matrix.length==0 || matrix[0] == null  || matrix[0].length == 0 )
			return 0;
		int n = matrix[0].length;
		for (int i=0; i<matrix.length; i++) {
			if (matrix[i]==null || n!= matrix[i].length)
				return 0;
		}
		return matrix.length;
	}
	/*...........................................................*/
	public static int numFullRows(double[][] matrix){
		if (matrix==null || matrix.length==0 || matrix[0] == null  || matrix[0].length == 0 )
			return 0;
		int n = matrix[0].length;
		for (int i=0; i<matrix.length; i++) {
			if (matrix[i]==null || n!= matrix[i].length)
				return 0;
		}
		return n;
	}
	/*...........................................................*/
	public double getValue(int ic, int it){
		if (values==null || (ic <0 || ic >= values.length) || (it <0 || it >= values[ic].length))
			return MesquiteDouble.unassigned;
		else
			return values[ic][it];
	}
	/*...........................................................*/
	public double[] getValues(int ic) {
		if (values == null || ic <0 || ic >= values.length)
			return null;
		else
			return values[ic];
	}
	/*...........................................................*/
	public double[][] getMatrix() {
		return values;
	}
	/*...........................................................*/
	public void setValue(int ic, int it, double value) {
		if (values!=null && ic >=0 && ic < values.length)
		 	if (it >=0 && it < values[ic].length)
				values[ic][it] = value;
	}
	/*...........................................................*/
	public void setValues(Double2DArray incoming) {
		if (incoming !=null) {
			resetSize(incoming.getSizeC(), incoming.getSizeT());
			for (int it=0; it < getSizeT(); it++) {
				for (int ic=0; ic < getSizeC(); ic++) {
					setValue(ic, it, incoming.getValue(ic, it));
				}
			}
		}
	}
	/*...........................................................*/
	public void setValues(double[][] matrix) {
		if (matrix !=null) {
			numC = matrix.length;
			if (numC>0)
				numT = matrix[0].length;	
			else
				numT=0;
			values = matrix;
		}
	}
	/*...........................................................*/
	public void zeroArray(){
		for (int ic=0; ic<values.length; ic++)
			for (int it=0; it<values[ic].length; it++)
			values[ic][it] =  0;
	}
	/*...........................................................*/
	public static void zeroArray(double[][] values){
		if (values==null)
			return;
		for (int ic=0; ic<values.length; ic++)
			for (int it=0; it<values[ic].length; it++)
			values[ic][it] =  0;
	}
	/*...........................................................*/
	public void deassignArray(){
		for (int ic=0; ic<values.length; ic++)
			for (int it=0; it<values[ic].length; it++)
			values[ic][it] =  MesquiteDouble.unassigned;
	}
	/*...........................................................*/
	public static void deassignArray(double[][] values){
		if (values==null)
			return;
		for (int ic=0; ic<values.length; ic++)
			for (int it=0; it<values[ic].length; it++)
			values[ic][it] =  MesquiteDouble.unassigned;
	}
	/*...........................................................*/
	public static void setToIdentityMatrix(double[][] values){
		if (values==null)
			return;
		for (int ic=0; ic<values.length; ic++)
			for (int it=0; it<values[ic].length; it++)
				if (it==ic)
					values[ic][it] =  1;
				else
					values[ic][it] =  0;
	}
	/*...........................................................*/
	/** Changes the array size to the new dimensions*/
	public void resetSize(int newNumC, int newNumT) {
		if (newNumC == numC && newNumT == numT)
			return;
		double[][] newDoubleValues = new double[newNumC][newNumT];
		for (int i=0; i<newNumC; i++)
			for (int j=0; j<newNumT; j++) {
				if (i<numC && j<numT)
					newDoubleValues[i][j]=values[i][j];
				else
					newDoubleValues[i][j]=0;
			}
		values=newDoubleValues;
		
		numC=newNumC;
		numT=newNumT;
	}
	/*...........................................................*/
	public static double maximumInColumn(double[][] values, int column){
		if (values==null || values.length<=column || values[column]==null)
			return MesquiteDouble.unassigned;
		MesquiteNumber d = new MesquiteNumber(MesquiteDouble.unassigned);
		for (int i=0; i<values[column].length; i++)
			d.setMeIfIAmLessThan(values[column][i]);
		return d.getDoubleValue();
	}
	/*...........................................................*/
	public static double minimumInColumn(double[][] values, int column){
		if (values==null || values.length<=column || values[column]==null)
			return MesquiteDouble.unassigned;
		MesquiteNumber d = new MesquiteNumber(MesquiteDouble.unassigned);
		for (int i=0; i<values[column].length; i++)
			d.setMeIfIAmMoreThan(values[column][i]);
		return d.getDoubleValue();
	}
      /*...........................................................*/
	/** borrowed from rhetenor.MatrixUtil*/
	public static double[][] multiply(double[][] matrix1, double[][] matrix2){
		int numRows1 = numFullRows(matrix1);
		int numRows2 = numFullRows(matrix2);
		int numColumns1 = numFullColumns(matrix1);
		int numColumns2 = numFullColumns(matrix2);
		boolean flag = (numColumns1!= numRows2 || numRows1*numRows2*numColumns1*numColumns2 ==0);
		if (flag) MesquiteMessage.println("Wow! You are trying to multiply a:   " 
		        +Integer.toString(numRows1) +"x" +Integer.toString(numColumns1)  
		             +" by a  " 
		        +Integer.toString(numRows2) +"x" +Integer.toString(numColumns2)  ); 
		if (flag)
		    return null; 
		double[][] result = new double[numColumns2][numRows1];
		for (int i=0; i<numColumns2; i++) {
			for (int j=0; j<numRows1; j++) {
				double sum =0;
				for (int k=0; k<numColumns1; k++) {
					sum += matrix1[k][j]* matrix2[i][k];
				}
				result[i][j]=sum;
			}
		}
		return result;
	}
	/*...........................................................*/
	/** borrowed from rhetenor.MatrixUtil*/
	public static double[][] transpose(double[][] matrix){
		int numRows = numFullRows(matrix);
		int numColumns = numFullColumns(matrix);
		if (numRows==0 ||  numColumns==0)
			return null;  
		double[][] result = new double[numRows][numColumns];
		for (int i=0; i<numColumns; i++) {
			for (int j=0; j<numRows; j++) {
				result[j][i]=matrix[i][j];
			}
		}
		return result;
	}
	/*...........................................................*/
	/** flip matrix both ways*/
	public static double[][] squnch(double[][] matrix){
		int numRows = numFullRows(matrix);
		int numColumns = numFullColumns(matrix);
		if (numRows==0 ||  numColumns==0)
			return null;  
		double[][] result = new double[numRows][numColumns];
		//flip columns
		for (int i=0; i<numColumns; i++) {
			for (int j=0; j<numRows; j++) {
				result[i][numRows - j - 1]=matrix[i][j];
			}
		}
		//flip rows
		for (int i=0; i<numColumns/2; i++) {
			for (int j=0; j<numRows; j++) {
				double t = result[numColumns-i-1][j];
				result[numColumns-i-1][j]=result[i][j];
				result[i][j] = t;
			}
		}
		return result;
	}
	/*...........................................................*/
	public int getSizeC() {
		return numC;
	}
	/*...........................................................*/
	public int getSizeT() {
		return numT;
	}
	/*...........................................................*/
	public static void swapColumns(double[][] d, int first, int second) {
		if (first<0 || d==null || first>=d.length || second<0 || second>=d.length) 
			return;
		double[] temp = d[first];
		d[first]=d[second];
		d[second] = temp;
	}
	/*...........................................................*/
	public static void moveColumns(double[][] d, int starting, int num, int justAfter) {
		if (num<=0 || d==null || starting>=d.length || (justAfter>=starting && justAfter<=starting+num-1)) //starting???
			return;
		if (justAfter>=d.length)
			justAfter = d.length-1;
		if (justAfter<0)
			justAfter = -1;
		double[][] newValues = new double[d.length][];
		if (starting>justAfter){
			int count =0;
			for (int i=0; i<=justAfter; i++)
				newValues[count++]=d[i];
			
			for (int i=starting; i<=starting+num-1; i++)
				newValues[count++]=d[i];
			for (int i=justAfter+1; i<=starting-1; i++)
				newValues[count++]=d[i];
			for (int i=starting+num; i<d.length; i++)
				newValues[count++]=d[i];
		}
		else {
			int count =0;
			for (int i=0; i<=starting-1; i++)
				newValues[count++]=d[i];
			
			for (int i=starting+num; i<=justAfter; i++)
				newValues[count++]=d[i];
			for (int i=starting; i<=starting+num-1; i++)
				newValues[count++]=d[i];
			for (int i=justAfter+1; i<d.length; i++)
				newValues[count++]=d[i];
		}
		for (int i=0; i<d.length; i++)
			d[i]=newValues[i];
	}
	/*...........................................................*/
	public void setNameReference(NameReference nr){
		name = nr;
	}
	/*...........................................................*/
	public NameReference getNameReference(){
		return name;
	}
	/*...........................................................*/
	public String getName(){
		if (name!=null)
			return name.getValue();
		else
			return "";
	}
	public static String toString(double[][] matrix){
		if (matrix == null)
			return "Matrix null";
		int numRows = numFullRows(matrix);
		int numColumns = numFullColumns(matrix);
		if (numRows==0 ||  numColumns==0)
			return null;  
		StringBuffer result = new StringBuffer(2*numRows*numColumns);
		for (int j=0; j<numRows; j++) {
			result.append('[');
			for (int i=0; i<numColumns; i++) {
				if (!MesquiteDouble.isCombinable(matrix[i][j]))
					MesquiteDouble.toString(matrix[i][j], result);
				else
					result.append(Double.toString(matrix[i][j]));
				result.append(' ');
			}
			result.append(']');
			result.append('\n');
		}
		return result.toString();
	}
	public static String toStringRC(double[][] matrix){
		int numRows = numFullRows(matrix);
		int numColumns = numFullColumns(matrix);
		StringBuffer result = new StringBuffer(2*numRows*numColumns);
		for (int i=0; i<matrix.length; i++) {
			result.append('[');
			for (int j=0; j<matrix[i].length; j++) {
				if (!MesquiteDouble.isCombinable(matrix[i][j]))
					MesquiteDouble.toString(matrix[i][j], result);
				else
					result.append(Double.toString(matrix[i][j]));
				result.append(' ');
			}
			result.append(']');
			result.append('\n');
		}
		return result.toString();
	}
}

