/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib;

import java.awt.*;
import mesquite.lib.duties.*;
/* ======================================================================== */
public class DoubleSliderWindow extends MesquiteWindow {
	TextField tF1,tF2;
	Label label1, label2;
	MiniSliderWithText slider1, slider2;
	MesquiteMenuItemSpec minItem, maxItem;
	GridBagLayout gridBag;
	GridBagConstraints constraints;
	String title ="Slider";
	static int insetFromWindow = 40;
	static int minWindowWidth = 260;
	boolean allowEstimation=false;

	public DoubleSliderWindow(MesquiteModule module, String title, String name1, MesquiteCommand command1, double initialValue1, double min1, double max1,  double minSweet1, double maxSweet1, String name2, MesquiteCommand command2, double initialValue2, double min2, double max2, double minSweet2, double maxSweet2) {
		super(module, false);
		setVisible(false);
		setWindowSize(minWindowWidth, 200);
		//minItem = module.addMenuItem("Slider Core Minimum...", module.makeCommand("setSweetMin", this));
		//maxItem = module.addMenuItem("Slider Core Maximum...", module.makeCommand("setSweetMax", this));
		this.title = title;

		Panel contents = getGraphicsArea();
	        gridBag = new GridBagLayout();
	        constraints = new GridBagConstraints();
	        constraints.gridx=1;
	       	constraints.gridy = 1;
	        constraints.weightx=1;
	        constraints.weighty=1;
	        constraints.ipadx=1;
	        constraints.ipady=1;
	        constraints.insets=new Insets(0,8,0,8);
	        constraints.fill=GridBagConstraints.BOTH;
	     	constraints.anchor=GridBagConstraints.CENTER;
		contents.setLayout(gridBag);
		
		label1 = new Label();
   	 	if (!StringUtil.blank(name1))
   	 		label1.setText(name1);
		contents.add(label1,constraints);

		slider1 = new MiniSliderWithText(command1, initialValue1, min1, max1, minSweet1, maxSweet1);
	       	constraints.gridy ++;
		contents.add(slider1,constraints);
		slider1.setWidth(getBounds().width-insetFromWindow);
		slider1.setVisible(true);
	/*
		tF2= new TextField();
   	 	if (!StringUtil.blank(name2))
   	 		tF2.setText(name2);
		tF2.setEditable(false);
		tF2.setBackground(Color.white);
		tF2.setVisible(true);
		contents.add(tF2);
	*/
		label2 = new Label();
   	 	if (!StringUtil.blank(name2))
   	 		label2.setText(name2);
		constraints.gridy ++;
		contents.add(label2,constraints);

		slider2 = new MiniSliderWithText(command2, initialValue2, min2, max2, minSweet2, maxSweet2);
		constraints.gridy ++;
		contents.add(slider2, constraints);
		slider2.setWidth(getBounds().width-insetFromWindow);
		slider2.setVisible(true);
	/*
		Panel buttons = new Panel();
		contents.add(buttons);
		Button ok;
		buttons.add("South", ok = new WindowButton("Close", this));
		Font df = new Font("Dialog", Font.PLAIN, 12);
		ok.setFont(df);
	*/
		//ok.setBackground(Color.white);
		
		//setVisible(true);  Windows should set themselves visible!!!!
		resetTitle();
		slider1.getEnterButton().setEnabled(false);
		slider2.getEnterButton().setEnabled(false);
	}
	/*.................................................................................................................*/
	/** When called the window will determine its own title.  MesquiteWindows need
	to be self-titling so that when things change (names of files, tree lists, etc.)
	they can reset their titles properly*/
	public void resetTitle(){
		setTitle(title);
	}
	
	/*.................................................................................................................*/
  	 public void useExponentialScale(boolean exp) {
  	 	slider1.useExponentialScale(exp);
  	 	slider2.useExponentialScale(exp);
  	 }
	/*.................................................................................................................*/
  	 public void setAllowEstimation(boolean allow) {
  	 	allowEstimation = allow;
  	 	if (slider1!=null) 
  	 		slider1.setAllowEstimation(allow);
	 	if (slider2!=null) 
  	 		slider2.setAllowEstimation(allow);
  	 }
	/*.................................................................................................................*/
  	 public Dimension getMinimumSize() {
  	 	return new Dimension(minWindowWidth,getWindowHeight());
  	 }
	/*.................................................................................................................*/
  	 public Dimension getMaximumSize() {
  	 	return new Dimension(super.getMaximumSize().width, getWindowHeight());  // forces height to not change
  	 }
	/*.................................................................................................................*/
	public void windowResized(){
		if (slider1!= null)
			slider1.setWidth(getBounds().width-insetFromWindow);
		if (slider2!= null)
			slider2.setWidth(getBounds().width-insetFromWindow);
	}
	/*.................................................................................................................*
  	 public Snapshot getSnapshot(MesquiteFile file) {
   	 	Snapshot temp = new Snapshot();
	 	temp.addLine("setSweetMin " + sliderWithText.getMinimumSweetValue());
  	 	temp.addLine("setSweetMax " + sliderWithText.getMaximumSweetValue());
  	 	temp.incorporate(super.getSnapshot(file), false);
  	 	return temp;
  	 }
	/*.................................................................................................................*
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
   	 	if (checker.compare(this.getClass(), "Sets the lower boundary for the core area of the slider, over which values change slowly", "[minimum value]", commandName, "setSweetMin")) {
   	 		MesquiteInteger io = new MesquiteInteger(0);
   			double x= MesquiteDouble.fromString(arguments, io);
   			if (!MesquiteDouble.isCombinable(x)){
   				x = MesquiteDouble.queryDouble(MesquiteWindow.frameOfComponent(this), "Slider Core Minimum...", "Set lower boundary for slider's core:", sliderWithText.getMinimumSweetValue().getDoubleValue());
   			}
   			 if (MesquiteDouble.isCombinable(x)) 
	   			 if (x>=maxSweetValue)
	   				MesquiteMessage.notifyUser("Lower boundary cannot be more than the upper boundary.");
	   			else if (x>minValue || minValue==MesquiteDouble.negInfinite){
	   				sliderWithText.setMinimumSweetValue(x);
	   				sliderWithText.repaint();
	   			}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets the upper boundary for the core area of the slider, over which values change slowly", "[maximum value]", commandName, "setSweetMax")) {
   	 		MesquiteInteger io = new MesquiteInteger(0);
   			double x= MesquiteDouble.fromString(arguments, io);
   			if (!MesquiteDouble.isCombinable(x)){
   				x = MesquiteDouble.queryDouble(MesquiteWindow.frameOfComponent(this), "Slider Core Maximum...", "Set upper boundary for slider's core:", sliderWithText.getMaximumSweetValue().getDoubleValue());
   			}
 			 if (MesquiteDouble.isCombinable(x)) 
	   			 if (x<=minSweetValue)
	   				MesquiteMessage.notifyUser("Upper boundary cannot be less than the lower boundary.");
	   			else if (x<=maxValue || maxValue==MesquiteDouble.infinite){
   					sliderWithText.setMaximumSweetValue(x);
   					sliderWithText.repaint();
   				}
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
    	 	return null;
    	 }
	/*.................................................................................................................*/
	public void setColor(Color c){
		setBackground(c);
		label1.setBackground(c);
		label2.setBackground(c);
		Panel contents = getGraphicsArea();
		contents.setBackground(c);
		repaintAll();
	}
	public void setText(int which, String s) {
		if (which==1){
			label1.setText(s);
			label1.repaint();
		}
		else if (which ==2){
			label2.setText(s);
			label2.repaint();
		}
		repaint();
	}
	public String getText(int which) {
		if (which==1)
			return label1.getText();
		else if (which ==2)
			return label2.getText();
		return null;
	}
	
	public void setValue(int which, double d) {
		if (which==1){
			slider1.setCurrentValue(d);
			slider1.repaint();
		}
		else if (which ==2){
			slider2.setCurrentValue(d);
			slider2.repaint();
		}
		repaint();
	}
	public double getCurrentValue(int which) {
		if (which==1)
			return slider1.getCurrentValue().getDoubleValue();
		else if (which ==2)
			return slider2.getCurrentValue().getDoubleValue();
		return 0;
	}
	public void dispose(){
		//getOwnerModule().deleteMenuItem(minItem);
		//getOwnerModule().deleteMenuItem(maxItem);
		super.dispose();
	}
	/*=============*/
	public void buttonHit(String label, Button button) {
		if (label.equalsIgnoreCase("Close")) {
			getOwnerModule().windowGoAway(this);
		}
	}
}

