/* Mesquite source code.  Copyright 2001-2004 D. Maddison and W. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib;

import java.awt.*;
import java.awt.event.*;

/*
	To do:
		- add ability to set radio buttons so that they are in a row
		- clean up event handling
		
*/

/*===============================================*/
/** An extensible dialog box.  */
public class ExtensibleDialog extends MesquiteDialog implements ActionListener, WindowListener, DoubleClickListener, ImagePanelListener, ComponentListener {
	public MesquiteInteger buttonPressed;
	String buttonLabel0;
	String buttonLabel1;
	String buttonLabel2;
	MesquiteModule helpURLOwnerModule=null;
	Button button0, button1, button2;
	Image helpImage;
	boolean useManualPage = false;
	public String defaultOKLabel = "OK";
	public String defaultLabel = defaultOKLabel;
	public String defaultCancelLabel="Cancel";
	String helpString = "";
	String helpURL = "";
	int numButtons = 0;
	int numExtraButtons = 5;
	int currentExtraButton = -1;
	String[] extraButtonLabels = new String[numExtraButtons];
//	CardPanels cardPanels = null;
//	Panel cardPanel = null;
	Panel addPanel=null;
	int cardPanelNumber = 0;
	Panel lastPanel;
	int defaultButtonNumber = 0;
	int dialogWidth=20;
//	public static final int checkBoxHeight = 24;
//	public static final int buttonHeight = 30;
//	public static final int popUpHeight = 30;
//	public static final int textFieldHeight = 30;
//	public static final int labelHeight = 24;
	int dialogHeight=50;	
	static final int minDialogWidthWithCanvas=240;
	int minDialogWidth=200;
	int minDialogHeight=120;
	Dimension d;
	
	static final int maxDialogWidthWithCanvas=740;
	int maxDialogWidth=660;
	boolean samePanelAsLast = false;
	boolean wasSamePanelAsLast = false;
        GridBagLayout gridBag;
        GridBagConstraints constraints;
        Object customConstraints=null;
        Component focalComponent=null;
        TextComponent defaultTextComponent=null;
        
        
        public int sideBuffer = 12;
        
        public static Font defaultBigFont = new Font ("Dialog", Font.PLAIN, 12);
        public static Font boldFont = new Font ("Dialog", Font.BOLD, 12);
	public static Font defaultSmallFont = new Font ("SanSerif", Font.PLAIN, 10);
	
	/*.................................................................................................................*/
	public ExtensibleDialog (Frame parent, String title, MesquiteInteger buttonPressed) {
		super(title);
		setFont(defaultBigFont);
		prepareDialog(parent,title,buttonPressed);
		addWindowListener(this);
	}
	/*.................................................................................................................*/
	public ExtensibleDialog (Frame parent, String title) {
		super(title);
		setFont(defaultBigFont);
		buttonPressed = new MesquiteInteger(1);
		prepareDialog(parent,title,buttonPressed);
		addWindowListener(this);
	}
	/*.................................................................................................................*/
	void prepareDialog (Frame parent, String title, MesquiteInteger buttonPressed) {
	        d = new Dimension(minDialogWidth, minDialogHeight);
	        gridBag = new GridBagLayout();
	        constraints = new GridBagConstraints();
	        constraints.gridx=0;
	       	constraints.gridy = GridBagConstraints.RELATIVE;
	        constraints.gridwidth=1;
	        constraints.gridheight=1;
	        constraints.fill=GridBagConstraints.BOTH;
	     	constraints.anchor=GridBagConstraints.CENTER;
		setLayout(gridBag);
		this.buttonPressed = buttonPressed;
		this.setResizable(false);
	}
	/*.................................................................................................................*/
	public void setAddPanel (Panel addPanel, LayoutManager layout) {
	       this.addPanel = addPanel;
	       if (layout==null)   // in this case, inherit the basic layout structure from dialog
	       		addPanel.setLayout(gridBag);
	       	else {
	       		addPanel.setLayout(layout);
	       	}
	}
	/*.................................................................................................................*/
	public void setAddPanel (Panel addPanel) {
	       setAddPanel(addPanel,null);
	}
	/*.................................................................................................................*/
	public Panel getAddPanel () {
	    	return addPanel;
	}
	/*.................................................................................................................*/
	public void nullifyAddPanel () {
	      	addPanel = null;
	     //  	lastPanel = addPanel;
	}
	/*.................................................................................................................*/
	public void suppressNewPanel () {
		samePanelAsLast = true;
	}
	/*.................................................................................................................*/
	public void forceNewPanel () {
		samePanelAsLast = false;
	}
	/*.................................................................................................................*/
	public Panel addNewDialogPanel (Panel panel,GridBagConstraints c) {
		wasSamePanelAsLast = samePanelAsLast;
		if (!samePanelAsLast) {
			Panel newPanel;
			if (panel != null)
				newPanel = panel;
			else
				newPanel = new Panel();
			addToDialog(newPanel, c);
			newPanel.setVisible(true);
			gridBag.setConstraints(newPanel,constraints);
			lastPanel = newPanel;
			return newPanel;
		}
		samePanelAsLast = false;
		return lastPanel;
	}
	/*.................................................................................................................*/
	public Panel addNewDialogPanel () {
		return addNewDialogPanel(null,null);
	}
	/*.................................................................................................................*/
	public Panel addNewDialogPanel (Panel panel) {
		return addNewDialogPanel(panel, null);
	}
	/*.................................................................................................................*/
	public Panel addNewDialogPanel (GridBagConstraints c) {
		return addNewDialogPanel(null, c);
	}
	/*.................................................................................................................*/
	public void addToDialog (Component component, Object c) {
	       if (component!=null) {
		        if (addPanel!=null && component !=addPanel) {
		        	if (customConstraints!=null)
		        		addPanel.add(component,(String)customConstraints);
		        	else
		        		addPanel.add(component);
		        	component.setBackground(addPanel.getBackground());
			}
		        else {
		        	//if (c==null)
		        		add(component);
		        	//else 
		        	//	add(component, c);
		        	component.setBackground(getBackground());
		        }
	        }
	}
	/*.................................................................................................................*/
	public void addToDialog (Component component) {
	       addToDialog(component, null);
	}
	/*.................................................................................................................*/
	public GridBagLayout getGridBagLayout () {
	       	return gridBag;
	}
	/*.................................................................................................................*/
	public GridBagConstraints getGridBagConstraints () {
	       	return constraints;
	}
	/*.................................................................................................................*/
	public void setCustomConstraints (Object customConstraints) {
	       this.customConstraints = customConstraints;
	}
	/*.................................................................................................................*/
	public void nullifyCustomConstraints () {
	      customConstraints = null;
	}
	/*.................................................................................................................*/
	public Object getCustomConstraints () {
	       	return customConstraints;
	}
	/*.................................................................................................................*/
	public void setGridBagConstraints (GridBagConstraints constraints) {
	       this.constraints = constraints;
	}
	/*.................................................................................................................*/
	public void setDefaultTextComponent (TextComponent textComponent) {
		defaultTextComponent = textComponent;
	}
	/*.................................................................................................................*/
	public void appendToHelpString (String s) {
		helpString += s + " ";
	}
	/*.................................................................................................................*/
	public String getHelpString () {
		return helpString;
	}

	/*.................................................................................................................*/
	public boolean hasHelpURL () {
		return (!StringUtil.blank(getHelpURL()) || (helpURLOwnerModule!=null));
	}

	/*.................................................................................................................*/
	public boolean getUseManualPage () {
		return useManualPage;
	}
	/*.................................................................................................................*/
	public void setUseManualPage (boolean useManual) {
		 useManualPage = useManual;
	}
	/*.................................................................................................................*/
	public MesquiteModule getHelpURLOwnerModule () {
		return helpURLOwnerModule;
	}
	/*.................................................................................................................*/
	public void setHelpURLOwnerModule (MesquiteModule ownerModule) {
		helpURLOwnerModule = ownerModule;
	}
	/*.................................................................................................................*/
	public void showManual(){
		if (getHelpURLOwnerModule()!=null) {
			if (!StringUtil.blank(helpURL)) 
				MesquiteModule.showWebPage(getHelpURLOwnerModule().getPath() + getHelpURL(), true);
			else if (useManualPage)
				getHelpURLOwnerModule().showManual();
			else
				MesquiteModule.showWebPage(getHelpURLOwnerModule().getPackageIntroModule().getSplashURL(), true);
		}
		else if (!StringUtil.blank(helpURL)) 
				MesquiteModule.showWebPage(getHelpURL(), true);
	}
	/*.................................................................................................................*/
	public void setHelpURL (MesquiteModule ownerModule, String s) {
		helpURLOwnerModule = ownerModule;
		helpURL = s;
	}
	/*.................................................................................................................*/
	public void setHelpURL (MesquiteModule ownerModule, String s, boolean useManual) {
		helpURLOwnerModule = ownerModule;
		helpURL = s;
		useManualPage = useManual;
	}
	/*.................................................................................................................*/
	public void setHelpURL (String s) {
		helpURL = s;
	}
	/*.................................................................................................................*/
	public String getHelpURL () {
		return helpURL;
	}
	/*.................................................................................................................*/
	public String getDefaultOKLabel () {
		return defaultLabel;
	}
	/*.................................................................................................................*/
	public String getDefaultCancelLabel () {
		return defaultCancelLabel;
	}
	/*.................................................................................................................*/
	public Button getMainButton () {
		return button0;
	}
	/*.................................................................................................................*/
	public Button getCancelButton () {
		int cancel = getButtonNumber(getDefaultCancelLabel());
		 if (cancel==0)
			return button0;
		else if (cancel==1)
			return button1;
		else if (cancel==2)
			return button2;
		return null;
	}
	/*.................................................................................................................*/
	void drawLine(int x1, int y1, int x2, int y2, Color color) {
		Graphics g = getGraphics();
		if (g!=null) {
			g.setColor(color);
			g.drawLine(x1,y1,x2, y2);
		}
	}
	/*.................................................................................................................*/
	public int getButtonNumber (String buttonLabel) {
		if (buttonLabel == null)
			return -1;
		else if (buttonLabel.equalsIgnoreCase(buttonLabel0))
			return 0;
		else if (buttonLabel.equalsIgnoreCase(buttonLabel1))
			return 1;
		else if (buttonLabel.equalsIgnoreCase(buttonLabel2))
			return 2;
		else
			return -1; 
	}
	/*.................................................................................................................*/
	public void setDefaultButton (String defaultLabel) {
		super.setDefaultButton(defaultLabel);
		if (defaultLabel != null)
			this.defaultLabel = defaultLabel;
		this.defaultButtonNumber = getButtonNumber(defaultLabel);		
	}
	/*.................................................................................................................*/
	void setPrimaryButtonConstraints(Panel buttons) {
		gridBag.setConstraints(buttons,constraints);
	}
	/*.................................................................................................................*/

	public boolean addExtraButtons (Panel panel, ActionListener actionListener) {
		boolean buttonAdded=false;
		for (int i=0; i<numExtraButtons; i++) {
			if (!StringUtil.blank(extraButtonLabels[i])) {
				if (actionListener!=null)
					addAListenedButton(extraButtonLabels[i],panel, actionListener);
				else
					addAButton(extraButtonLabels[i],panel);
				buttonAdded = true;
			}
		}
		if (buttonAdded)
			addAnEmptyImage(panel);
		return buttonAdded;
	}
	/*.................................................................................................................*/

	public boolean addExtraButtons (Panel panel) {
		return addExtraButtons(panel, null);
	}
	/*.................................................................................................................*/

	public void setExtraButton (String s) {
		currentExtraButton++;
		if (currentExtraButton<numExtraButtons)
			extraButtonLabels[currentExtraButton]=s;
	}
	/*.................................................................................................................*/

	public void addHelpButtons (Panel panel) {
		boolean buttonAdded=false;
		if (hasHelpURL()) {
			addAListenedImage("manual", "manual.gif","manualPressed.gif",true,panel, this);
			buttonAdded = true;
		}
		if (!StringUtil.blank(getHelpString())) {
			//addAListenedButton("?",buttons,this);
			addAListenedImage("?", "help.gif","helpPressed.gif",true,panel, this);
			buttonAdded = true;
		}
		if (buttonAdded)
			addAnEmptyImage(panel);
	}
	/*.................................................................................................................*/

	public void addPrimaryButtonRow (String buttonLabel0, String buttonLabel1, String buttonLabel2) {
		Panel buttons = new Panel();
//		buttons.setSize(dialogWidth, buttonHeight);
		String singleButton = null;
		boolean noButtons=true;
		if (hasHelpURL())
			noButtons=false;
		if (!StringUtil.blank(getHelpString()))
			noButtons=false;
		addHelpButtons(buttons);
		if (addExtraButtons(buttons))
			noButtons=false;
		if (buttonLabel2!=null) {
			button2=addAButton(buttonLabel2,buttons);
			this.buttonLabel2 = buttonLabel2;
			if (noButtons) {
				singleButton = buttonLabel2;
				noButtons = false;
			}
		}
		if (buttonLabel1!=null) {
			button1=addAButton(buttonLabel1,buttons);
			this.buttonLabel1 = buttonLabel1;
			if (noButtons) {
				singleButton = buttonLabel1;
				noButtons = false;
			}
			else if (!noButtons)
				singleButton = null;
		}
		if (buttonLabel0!=null) {
			button0=addAButton(buttonLabel0,buttons);
			this.buttonLabel0 = buttonLabel0;
			if (noButtons) {
				singleButton = buttonLabel0;
				noButtons = false;
			}
			else if (!noButtons)
				singleButton = null;
		}
		addToDialog(buttons);	
		setPrimaryButtonConstraints(buttons);	
		if (singleButton!=null)
			setDefaultButton(singleButton);	
	}
	/*.................................................................................................................*/
	public void addPrimaryButtonRowListener (String buttonLabel0, String buttonLabel1, ActionListener actionListener) {
		Panel buttons = new Panel();
//		buttons.setSize(dialogWidth, buttonHeight);
		addHelpButtons(buttons);
		addExtraButtons(buttons, actionListener);
		button1=addAListenedButton(buttonLabel1,buttons,actionListener);
		this.buttonLabel1 = buttonLabel1;
		button0=addAListenedButton(buttonLabel0,buttons,actionListener);
		this.buttonLabel0 = buttonLabel0;
		addToDialog(buttons);			
	        setPrimaryButtonConstraints(buttons);
	        addBlankLine();
	}
	/*.................................................................................................................*/
	public void addPrimaryButtonRow (String buttonLabel0, String buttonLabel1) {
		addPrimaryButtonRowListener(buttonLabel0,buttonLabel1,null);
	}
	/*.................................................................................................................*/
	public void addPrimaryButtonRow (String buttonLabel0) {
		Panel buttons = new Panel();
//		buttons.setSize(dialogWidth, buttonHeight);
		addHelpButtons(buttons);
		addExtraButtons(buttons);
		button0=addAButton(buttonLabel0,buttons);
		this.buttonLabel0 = buttonLabel0;
		addToDialog(buttons);	
		setPrimaryButtonConstraints(buttons);	
		setDefaultButton(buttonLabel0);	
	        addBlankLine();
	}
	/*.................................................................................................................*/
	public Button addAButton (String s, Panel buttons) { 
		return addAButton(s,buttons,null);
	}
	/*.................................................................................................................*/
	public Button addAButton (String s, Panel buttons, GridBagConstraints c) { 
		Button button;
		if (buttons==null && lastPanel!=null)
			buttons = lastPanel;
		if (buttons == null)
			return null;
		if (c==null)
			buttons.add(button = new Button(s));
		else
			buttons.add(button = new Button(s),c);
		numButtons++;
		return button;
	}
	/*.................................................................................................................*/
	public Button addAListenedButton (String s, Panel buttons, ActionListener actionListener) { //David: modified to return button
		if (StringUtil.blank(s))
			return null;
		Button button=null;
		if (buttons==null) {
			if (lastPanel!=null) {
				lastPanel.add(button = new Button(s));
				numButtons++;
			}
		}
		else 	{
			buttons.add(button = new Button(s));
			numButtons++;
		}
		if ((actionListener!=null) && (button!=null))
			button.addActionListener(actionListener);
		return button;
	}
	/*.................................................................................................................*/
	public ImagePanel addAListenedImage (String imageCommandName, String imageName, String imagePressedName, boolean inRoot, Panel panel, ImagePanelListener imageListener) {
		ImagePanel imagePanel = null;
		Image image=null;
		Image imagePressed=null;
		if (inRoot) {
			image = MesquiteImage.getImage(MesquiteModule.getRootImageDirectoryPath() + imageName);
			imagePressed = MesquiteImage.getImage(MesquiteModule.getRootImageDirectoryPath() + imagePressedName);
		}
			imagePanel = new ImagePanel(image,imagePressed, imageCommandName, imageListener);
		if (panel==null) {
			if (lastPanel!=null) 
				lastPanel.add(imagePanel);
		}
		else 	
			panel.add(imagePanel);
		return imagePanel;
	}
	/*.................................................................................................................*/
	public ImagePanel addAnEmptyImage (Panel panel) {
		Image image = MesquiteImage.getImage(MesquiteModule.getRootImageDirectoryPath() + "empty.gif");
		ImagePanel imagePanel = new ImagePanel(image);
		
		if (panel==null) {
			if (lastPanel!=null) 
				lastPanel.add(imagePanel);
		}
		else 	
			panel.add(imagePanel);
		return imagePanel;
	}
	/*.................................................................................................................*/
	public Button addButton (String s) {
		Panel newPanel = addNewDialogPanel();
		Button button = new Button(s);
		newPanel.add(button);
		numButtons++;
		return button;
	}
	/*.................................................................................................................*/
	int currentWidth=0;
	/*.................................................................................................................*/
	public PopUpPanelOfCards addPopUpPanelOfCards (String name) {
		PopUpPanelOfCards panelOfCards = new PopUpPanelOfCards(this,name);
		addNewDialogPanel(panelOfCards);
		return panelOfCards;
	}
	/*.................................................................................................................*/
	private void resetTextCanvasSizes(Component c){
		if (c==null)
			return;
		if (c instanceof TextCanvas) {
			minDialogWidth = minDialogWidthWithCanvas;
			maxDialogWidth = maxDialogWidthWithCanvas;
			if (currentWidth>maxDialogWidth)
				currentWidth = maxDialogWidth;
			else if (currentWidth<minDialogWidth)
				currentWidth = minDialogWidth;
			((TextCanvas)c).setCanvasRows(currentWidth);
			
		}
		if (c instanceof Container){
			Component[] cc = ((Container)c).getComponents();
			if (cc!=null && cc.length>0)
				for (int i=0; i<cc.length; i++)
					resetTextCanvasSizes(cc[i]);
		}
	}
	/*.................................................................................................................*/
	public void prepareDialog () {


		pack();
		d = getPreferredSize();

		if (MesquiteTrunk.isMacOSXJaguar()){
	//		int targetWidth = 300;
	//		int targetHeight = 160;
			int targetWidth = d.width;
			int targetHeight = d.height;
			int h, v;
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
				h = (screenSize.width-targetWidth)/2;
			int menuBarHeight = 0;
			if (!StringUtil.blank(System.getProperty("mrj.version")))
				menuBarHeight = 32;
			v = (screenSize.height-targetHeight - menuBarHeight)/2 + menuBarHeight;
			if (v < menuBarHeight)
				v = menuBarHeight;
			setLocation(h, v);
		}
		
	//	pack();
	//	d = getPreferredSize();
		currentWidth = d.width;
		if (currentWidth < 240) {
			currentWidth = 240;
		}
		if (currentWidth >600) {
			currentWidth = 600;
		}
		resetTextCanvasSizes(this);
		pack();
		d = getPreferredSize();
		if (d.width<minDialogWidth) {
			d.width = minDialogWidth;
		}
		else if (d.width> maxDialogWidth) {
			d.width = maxDialogWidth;
		}
		if (d.height<minDialogHeight) {
			d.height = minDialogHeight;
		}
		pack();
		d.width += 20;
		if (MesquiteTrunk.isMacOSX() && MesquiteTrunk.getJavaVersionAsDouble()>=1.4)
			d.height +=25;
		setSize(d);
		doLayout();
		MesquiteWindow.centerWindow(this);
		if (MesquiteTrunk.isMacOSXJaguar())
			pack(); //add for Jaguar bug, possibly also linux
	}
	/*.................................................................................................................*/
	public void prepareDialogHideFirst () {
		boolean wasVisible = isVisible();
		//if (wasVisible && MesquiteTrunk.isMacOSXJaguar())
		//	setVisible(false);
		prepareDialog();
	//	if (focalComponent!=null)
	//		focalComponent.requestFocus();
		//if (wasVisible && MesquiteTrunk.isMacOSXJaguar())
		//	setVisible(true);
	}
	/*.................................................................................................................*/
	public void prepareAndDisplayDialog () {
		prepareDialog();
		if (focalComponent!=null)
			focalComponent.requestFocus();
		if (defaultTextComponent!=null) {
			defaultTextComponent.selectAll();
		}
		setVisible(true);
	}
	
	/*.................................................................................................................*/
	public void setSize(int w, int h){
		super.setSize(w,h);
	}
	/*.................................................................................................................*/
	public void setBounds(int x, int y, int w, int h){
		super.setBounds(x, y, w,h);
	}
	/*.................................................................................................................*/
	public void componentResized(ComponentEvent e){
		if (e==null || e.getComponent()!= this)
			return;
		if (d == null){
		}
		else if (getBounds().width< d.width || getBounds().height<d.height) {
			setSize(d);
		}
	}
	/*.................................................................................................................*/
	public void setDefaultComponent (Component component) {
		focalComponent = component;
	}
	/*.................................................................................................................*/
	/**Can be overrided to provide extra panels above button row*/
	public void addAuxiliaryDefaultPanels(){//************
	}
	/*.................................................................................................................*/
	public void completeAndShowDialog () {
		addAuxiliaryDefaultPanels();//************
		addPrimaryButtonRow(defaultLabel, defaultCancelLabel);
		prepareAndDisplayDialog();
	}
	/*.................................................................................................................*/
	public void completeAndShowDialog (boolean setDefaultToOK) {
		addDefaultPanelsListener(setDefaultToOK, null);
	}
	/*.................................................................................................................*/
	public void completeAndShowDialog (boolean setDefaultToOK, ActionListener actionListener) {
		addDefaultPanelsListener(setDefaultToOK, actionListener);
	}
	/*.................................................................................................................*/
	public void addDefaultPanelsListener (boolean setDefaultToOK, ActionListener actionListener) {
		addAuxiliaryDefaultPanels();//************
		addPrimaryButtonRowListener(defaultLabel, defaultCancelLabel, actionListener);
		if (setDefaultToOK)
			setDefaultButton(defaultLabel);
		else 
			setDefaultButton(defaultCancelLabel);
		prepareAndDisplayDialog();
	}
	/*.................................................................................................................*/
	public void completeAndShowDialog (String buttonLabel0, String buttonLabel1, boolean setDefaultToOK, ActionListener actionListener) {
		addAuxiliaryDefaultPanels();//************
		addPrimaryButtonRowListener(buttonLabel0, buttonLabel1,actionListener);
		if (setDefaultToOK)
			setDefaultButton(buttonLabel0);
		prepareAndDisplayDialog();
	}
	/*.................................................................................................................*/
	public void completeAndShowDialog (String buttonLabel0, String buttonLabel1, String buttonLabel2, String defaultButton) {
		addAuxiliaryDefaultPanels();//************
		addPrimaryButtonRow(buttonLabel0, buttonLabel1,buttonLabel2);
		//if (defaultButton!=null)  //David: why is this here? If not here, how do we disable default buttons?
			setDefaultButton(defaultButton);
		prepareAndDisplayDialog();
	}
	/*.................................................................................................................*/
	public DoubleClickList createListenedList (Object names, MesquiteInteger selected, int numElements, ActionListener actionListener, ItemListener itemListener, boolean multipleMode) {
		String[] strings = null;
		Listable[] listables = null;
		if (names != null)
			if (names instanceof Listable[])
				listables = (Listable[])names;
			else if (names instanceof String[])
				strings = (String[])names;
		DoubleClickList list = new DoubleClickList (numElements, multipleMode); 
		list.setDoubleClickListener(this);
		list.setBackground(Color.white);
		list.addItemListener(itemListener);
		list.addActionListener(actionListener);
		if (listables!=null){
			for (int i=0; i<listables.length; i++) {
				if (listables[i]!=null){
					if (listables[i] instanceof SpecialListName && ((SpecialListName)listables[i]).getListName()!=null)
						list.add(((SpecialListName)listables[i]).getListName(), -1);
					else if (listables[i].getName()!=null)
						list.add(listables[i].getName(), -1);
				}
			}
			if (selected!=null && selected.getValue()>=0 && selected.getValue()<listables.length)
				list.select(selected.getValue());
		}
		else if (strings!=null){
			for (int i=0; i<strings.length; i++)
				if (strings[i]!=null)
					list.add(strings[i], -1);
			if (selected!=null && selected.getValue()>=0 && selected.getValue()<strings.length)
				list.select(selected.getValue());
		}
		return list;
	}
	/*.................................................................................................................*/
	/**Adds a panel with three labels closely stacked, one on top of the other. */
	public Panel addStackedLabels (String s1, String s2, String s3, int boldString) {  
		Panel newPanel = addNewDialogPanel();
	     	GridBagConstraints gridConstraints=null;
	     	GridBagLayout gridBagLabels = new GridBagLayout();
	        gridConstraints = new GridBagConstraints();
	        gridConstraints.gridwidth=1;
	        gridConstraints.gridheight=1;
	     /*  enable these if you want it left-justified
	       gridConstraints.ipadx=1;
	       gridConstraints.ipady=1;
	        gridConstraints.weightx=1;
	       gridConstraints.weighty=1;
	   */
	/* enable this if you want them in the center, but left-justified with respect to each other
	        gridConstraints.fill=GridBagConstraints.HORIZONTAL;
	 */
	        
		newPanel.setLayout(gridBagLabels);
	        gridBagLabels.setConstraints(newPanel,gridConstraints);
	     	gridConstraints.gridy = 0;
	        gridConstraints.gridx=1; 
	     /*
	        if (StringUtil.blank(s3))
	        	newPanel.setLayout(new GridLayout(2,1));
	        else
	        	newPanel.setLayout(new GridLayout(3,1));
	       */
	        Label label;
		if (!StringUtil.blank(s1)) {
	     		gridConstraints.gridy++;
			label = new Label(s1);
			if (boldString==1)
				label.setFont(boldFont);
			newPanel.add(label,gridConstraints);
		}
		if (!StringUtil.blank(s2)) {
	     		gridConstraints.gridy++;
			label = new Label(s2);
			if (boldString==2)
				label.setFont(boldFont);
			newPanel.add(label,gridConstraints);
		}
		if (!StringUtil.blank(s3)) {
	     		gridConstraints.gridy++;
			label = new Label(s3);
			if (boldString==3)
				label.setFont(boldFont);
			newPanel.add(label);
		}
		return newPanel;
	}
	/*.................................................................................................................*/
	/**Adds a panel with two labels closely stacked, one on top of the other. */
	public Panel addStackedLabels (String s1, String s2, int boldString) {
		return addStackedLabels(s1,s2,null,boldString);
	}
	/*.................................................................................................................*/
	/**Adds a panel with two labels closely stacked, one on top of the other, with the second on in a smaller font. */
	public void addStackedLabelsBottomSmall (String s1, String s2) {  
		Panel newPanel = addNewDialogPanel();
//		newPanel.setLayout(new GridLayout(1,1));
	        GridBagLayout gridBagLabels = new GridBagLayout();
	        GridBagConstraints gridConstraints = new GridBagConstraints();
	        gridConstraints.gridwidth=1;
	        gridConstraints.gridheight=1;
	        gridConstraints.fill=GridBagConstraints.BOTH;
	        gridConstraints.anchor = GridBagConstraints.WEST;
	        
		newPanel.setLayout(gridBagLabels);
	        gridBagLabels.setConstraints(newPanel,gridConstraints);
	     	gridConstraints.gridy = 0;
	        gridConstraints.gridx=1;
	        newPanel.setLayout(new GridLayout(2,1));
	        Label label;
		if (!StringUtil.blank(s1)) {
	     		gridConstraints.gridy++;
			label = new Label(s1);
			newPanel.add(label);
		}
		if (!StringUtil.blank(s2)) {
	     		gridConstraints.gridy++;
			label = new Label(s2);
			label.setFont(defaultSmallFont);
			newPanel.add(label);
		}
	}
	/*.................................................................................................................*/
	public Label addLabel (String s, int alignment, boolean doAlignment, boolean isBold) {

		Panel newPanel;
	      	 Label label = new Label(s);

		if (doAlignment && alignment == Label.LEFT) {
			newPanel = addNewDialogPanel();
			newPanel.add(label);
			newPanel.setLayout(new GridLayout(1,1));
			label.setAlignment(Label.LEFT);
			if (isBold)
				label.setFont(boldFont);
		}
		else {
			label = new Label(s);
			newPanel = addNewDialogPanel();
			newPanel.add(label);
			if (isBold)
				label.setFont(boldFont);
		}
		return label;
	}
	/*.................................................................................................................*/
	public Label addLabel (String s, int alignment, boolean doAlignment) {
		return addLabel(s,alignment,doAlignment, false);
	}
	/*.................................................................................................................*/
	public Label addBoldLabel (String s) {
		return addLabel(s,0,false,true);
	}
	/*.................................................................................................................*/
	public Label addLabel (String s) {
		return addLabel(s,0,false,false);
	}
	/*.................................................................................................................*/
	public Label addLabel (String s, int alignment) {
		return addLabel(s,alignment,true,false);
	}
	/*.................................................................................................................*/
	public void addLargeOrSmallTextLabel (String message) {
		if (message!=null) {
			int totalLength = StringUtil.getStringDrawLength(this, message);
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			if (totalLength>(screenSize.width/4) || message.indexOf("\n")>=0 || message.indexOf("\r")>=0)
				addLargeTextLabel(message); // can be way too wide
			else
				addLabel(message);
		}
	}
	/*.................................................................................................................*/
	public Choice addPopUpMenu (String message, String[] choices, int defaultChoice) {
		Panel newPanel = addNewDialogPanel();
		Label label = new Label (message);
		newPanel.add(label);

		Choice choice = new Choice();
		if (choices!=null){
			int numChoices=0;
			for (int i=0; i<choices.length; i++) 
				if (!StringUtil.blank(choices[i])) {
					choice.add(choices[i]);
					numChoices++;
				}
			if ((numChoices>0) && (defaultChoice<numChoices))
				choice.select(defaultChoice);
		}
		newPanel.add(choice);
		return choice;
	}
	/*.................................................................................................................*/
	public Choice addPopUpMenu (String message, String string1, String string2, String string3, String string4, int defaultChoice) {
		Panel newPanel = addNewDialogPanel();
		Label label = new Label (message);
		newPanel.add(label);

		Choice choice = new Choice();
		if (string1!=null)
			choice.add(string1);
		if (string2!=null)
			choice.add(string2);
		if (string3!=null)
			choice.add(string3);
		if (string4!=null)
			choice.add(string4);
		if (choice.getItemCount()>0)
			choice.select(defaultChoice);
		newPanel.add(choice);
		return choice;
	}
	/*.................................................................................................................*/
	public Choice addPopUpMenu (String message, Listable[] choices, int defaultChoice) {
		Panel newPanel = addNewDialogPanel();
		Label label = new Label (message);
		newPanel.add(label);

		Choice choice = new Choice();
		if (choices!=null)
			for (int i=0; i<choices.length; i++)
				choice.add(choices[i].getName());
		if (choice.getItemCount()>0)
			choice.select(defaultChoice);
		newPanel.add(choice);
		return choice;
	}
	/*.................................................................................................................*/
	public Choice addPopUpMenu (String message, String string1, String string2, String string3, int defaultChoice) {
		return addPopUpMenu(message, string1, string2, string3, null, defaultChoice);
	}
	/*.................................................................................................................*/
	public Choice addPopUpMenu (String message, String string1, String string2, int defaultChoice) {
		return addPopUpMenu(message, string1, string2, null, null, defaultChoice);
	}
	/*.................................................................................................................*/
	public Choice addPopUpMenu (String message) {
		Panel newPanel = addNewDialogPanel();
		Label label = new Label (message);
		newPanel.add(label);

		Choice choice = new Choice();
		newPanel.add(choice);
		return choice;
	}
	/*.................................................................................................................*/
	public void addBlankLine () {
		Panel newPanel = addNewDialogPanel();
	}
	/*.................................................................................................................*/
	public void addHorizontalLine (int thickness) {
		addHorizontalLine(thickness,0);
	}
	/*.................................................................................................................*/
	public void addHorizontalLine (int thickness, int startx) {
		forceNewPanel();
		Panel newPanel = addNewDialogPanel();
		newPanel.add(new HorizontalLine(this, thickness,startx));
	}
	/*.................................................................................................................*/
	public void addNormalHorizontalLine (int thickness) {
		Panel newPanel = addNewDialogPanel();
		//addABlackImage(newPanel,thickness);
		//newPanel.setLayout(gridBag);
		newPanel.add(new HorizontalLine(this, thickness));
	}
	/*.................................................................................................................*
	public ImagePanel addABlackImage (Panel panel,int height) {
		Image image = MesquiteImage.getImage(MesquiteModule.getRootImageDirectoryPath() + "blackPixel.gif");
		ImagePanel imagePanel = new ImagePanel(image);
		imagePanel.setForcedImageHeight(height);
		imagePanel.setForcedImageWidth(20);
		
		if (panel==null) {
			if (lastPanel!=null) 
				lastPanel.add(imagePanel);
		}
		else 	
			panel.add(imagePanel);
		return imagePanel;
	}
	/*.................................................................................................................*/
	public Checkbox addCheckBox (String s, boolean value) {
		Panel newPanel = addNewDialogPanel();
		Checkbox checkbox =new Checkbox(s, null, value);
		newPanel.add(checkbox);
		return checkbox;
	}

	/*.................................................................................................................*/
	public List addList (Object names, MesquiteInteger selected, String message) {
		 if (message!=null)
		 	 constraints.fill=GridBagConstraints.NONE;
		 Panel newPanel = addNewDialogPanel();
		if (message!=null) {
			newPanel.add(new Label(message));
		}
		List list = new List(4,false);

		newPanel.setLayout(new GridLayout(1,1));
		newPanel.add(list);
		focalComponent = list;
		constraints.fill=GridBagConstraints.BOTH;
		
		String[] strings = null;
		Listable[] listables = null;
		if (names != null)
			if (names instanceof Listable[])
				listables = (Listable[])names;
			else if (names instanceof ListableVector)
				listables = ((ListableVector)names).getListables();
			else if (names instanceof String[])
				strings = (String[])names;
		list.setBackground(Color.white);
		if (listables!=null){
			for (int i=0; i<listables.length; i++) {
				if (listables[i]!=null){
					if (listables[i] instanceof SpecialListName && ((SpecialListName)listables[i]).getListName()!=null)
						list.add(((SpecialListName)listables[i]).getListName(), -1);
					else if (listables[i].getName()!=null)
						list.add(listables[i].getName(), -1);
				}
			}
			if (selected!=null && selected.getValue()>=0 && selected.getValue()<listables.length)
				list.select(selected.getValue());
		}
		else if (strings!=null){
			for (int i=0; i<strings.length; i++)
				if (strings[i]!=null)
					list.add(strings[i], -1);
			if (selected!=null && selected.getValue()>=0 && selected.getValue()<strings.length)
				list.select(selected.getValue());
		}
		return list;
	}
	/*.................................................................................................................*/
	public SingleLineTextField addTextField (String message, String initialString, int fieldLength) {
		 if (message!=null)
		 	 constraints.fill=GridBagConstraints.NONE;
		 Panel newPanel = addNewDialogPanel();
		if (message!=null) {
			newPanel.add(new Label(message));
		}
		SingleLineTextField textField;
		if (initialString == null)
			initialString = "";
		if (fieldLength>=1) {
			textField =new SingleLineTextField(initialString,fieldLength);
		}
		else {
			newPanel.setLayout(new GridLayout(1,1));
			textField =new SingleLineTextField(initialString);
			textField.setSize(new Dimension(dialogWidth-sideBuffer*2, textField.getSize().height));
			newPanel.setSize(textField.getSize());
		}
		textField.setBackground(Color.white);
		newPanel.add(textField);
		focalComponent = textField;
		textField.selectAll();
		constraints.fill=GridBagConstraints.BOTH;
		return textField;
	}
	/*.................................................................................................................*/
	public SingleLineTextField addTextField (String initialString, int fieldLength) {
		return addTextField(null, initialString, fieldLength);
	}
	/*.................................................................................................................*/
	public SingleLineTextField addTextField (String initialString) {
		return addTextField(null, initialString, -1);
	}
	/*.................................................................................................................*/
	public SingleLineTextArea addSingleLineTextArea (String initialString, int numRows) {
		forceNewPanel();
		if (initialString == null)
			initialString = "";
		Panel newPanel = addNewDialogPanel();
		newPanel.setLayout(new GridLayout(1,1));
		SingleLineTextArea textArea = new SingleLineTextArea(initialString,numRows,40,TextArea.SCROLLBARS_VERTICAL_ONLY);
//		Dimension d = textArea.getPreferredSize();
		textArea.setSize(new Dimension(dialogWidth-sideBuffer*2, textArea.getSize().height));
		textArea.setBackground(Color.white);
		newPanel.add(textArea);
		focalComponent = textArea;
		return textArea;
	}
	/*.................................................................................................................*/
	public TextArea addTextArea (String initialString, int numRows, Font font) {
		return addTextArea(initialString, numRows, 40, font);
	}
	/*.................................................................................................................*/
	public TextArea addTextArea (String initialString, int numRows, int numColumns, Font font, int scrollbars, boolean textAreaFullDialogWidth) {
		forceNewPanel();
		if (initialString == null)
			initialString = "";
		Panel newPanel = addNewDialogPanel();
		if (textAreaFullDialogWidth)
			newPanel.setLayout(new GridLayout(1,1)); 
		TextArea textArea = new TextArea(initialString,numRows,numColumns,scrollbars);
		//textArea.setSize(new Dimension(dialogWidth-sideBuffer*2, textArea.getSize().height));
		textArea.setBackground(Color.white);
		if (font !=null)
			textArea.setFont(font);
		newPanel.add(textArea);
		//lastTextPanel = newPanel;
		focalComponent = textArea;
		return textArea;
	}
	/*.................................................................................................................*/
	public TextArea addTextArea (String initialString, int numRows, int numColumns, Font font) {
		return addTextArea(initialString, numRows, numColumns, font, TextArea.SCROLLBARS_VERTICAL_ONLY, true);
	}
	/*.................................................................................................................*/
	public TextArea addTextArea (String initialString, int numRows) {
		return addTextArea(initialString,numRows,null);
	}
	/*.................................................................................................................*/
	public TextArea addTextAreaSmallFont (String initialString, int numRows, int numColumns) {
		return addTextArea(initialString,numRows,numColumns, defaultSmallFont);
	}
	/*.................................................................................................................*/
	public TextArea addTextAreaSmallFontNoScroll (String initialString, int numRows, boolean textAreaFullDialogWidth) {
		return addTextArea(initialString,numRows,40, defaultSmallFont, TextArea.SCROLLBARS_NONE, textAreaFullDialogWidth);
	}
	/*.................................................................................................................*/
	public TextArea addTextAreaSmallFont (String initialString, int numRows) {
		return addTextArea(initialString,numRows, defaultSmallFont);
	}
	/*.................................................................................................................*/
	public void addLargeTextLabel (String message, Font font) {
		if (message!=null) {
			Panel newPanel = addNewDialogPanel();

/*	        GridBagLayout panelGrid = new GridBagLayout();
	        GridBagConstraints panelGridConstraints = new GridBagConstraints();
	        panelGridConstraints.gridx=0;
	       	panelGridConstraints.gridy = 0;
	        panelGridConstraints.gridwidth=1;
	        panelGridConstraints.gridheight=1;
	        panelGridConstraints.fill=GridBagConstraints.HORIZONTAL;
	        panelGridConstraints.anchor=GridBagConstraints.CENTER;
	       	panelGridConstraints.weightx = 1.0;


			newPanel.setLayout(panelGrid);

*/
			newPanel.setLayout(new GridLayout(1,1));
			pack();
			Dimension d = getPreferredSize();
			dialogWidth = d.width;
			if (dialogWidth<minDialogWidthWithCanvas) //this had been hardcoded at 240
				 dialogWidth = minDialogWidthWithCanvas;
			else if (dialogWidth> maxDialogWidthWithCanvas) 
				 dialogWidth =maxDialogWidthWithCanvas;
			if (font==null)
				font = getFont();
			TextCanvas textCanvas = TextCanvas.getTextCanvas(dialogWidth-sideBuffer*2, font, message);
			newPanel.add(textCanvas);
			
			//newPanel.setSize(textCanvas.getPreferredSize());
		}
	}
	/*.................................................................................................................*/
	public void addLargeTextLabel (String message) {
		addLargeTextLabel(message, null);
	}
	/*.................................................................................................................*/
	public LongField addLongField (String message, long initialValue, int fieldLength) {
		return new LongField(this,message,initialValue, fieldLength);
	}
	/*.................................................................................................................*/
	public IntegerField addIntegerField (String message, int initialValue, int fieldLength) {
		return new IntegerField(this,message,initialValue, fieldLength);
	}
	/*.................................................................................................................*/
	public IntegerField addIntegerField (String message, int fieldLength) {
		return new IntegerField(this,message, fieldLength);
	}
	/*.................................................................................................................*/
	public DoubleField addDoubleField (String message, double initialValue, int fieldLength) {
		return new DoubleField(this,message,initialValue, fieldLength);
	}
	/*.................................................................................................................*/
	public DoubleField addDoubleField (String message, int fieldLength) {
		return new DoubleField(this,message, fieldLength);
	}
	/*.................................................................................................................*/
	public RadioButtons addRadioButtons (String[] labels, int defaultBox) {
		return new RadioButtons(this,labels, defaultBox);
	}
	/*.................................................................................................................*/
	public int getDialogWidth() {
		return dialogWidth;
	}
	/*.................................................................................................................*/
	public void setDialogWidth(int width) {
		dialogWidth = width;
	}
	/*.................................................................................................................*/
	public void buttonHit(String buttonLabel, Button button) {
		if (buttonPressed!=null) {
			buttonPressed.setValue(getButtonNumber(buttonLabel));
		}
	}
	/*.................................................................................................................*/
	/* Override this to provide a check on whether the dialog box should be dismissed or not in response to hitting OK. 
	*/
	public boolean dialogValuesAcceptable () {
		return true;
	}
	/*.................................................................................................................*/
	/* Override this to change the button that will be affected by the dialogValuesAcceptable check
	*/
	public int checkValuesAcceptableButton () {
		return 0;
	}
	/*.................................................................................................................*/
	/* Override this to change the message that will be presented if the dialogValuesAcceptable check fails
	*/
	public void dialogValuesNotAcceptableWarning () {
		MesquiteMessage.notifyUser("Values unacceptable.");
	}
	/*.................................................................................................................*/
	/* David: this overridden here because dlog was getting disposed for any button, even if not primary.  Might be 
	good to store Buttons directly and not just strings; that way to know easily if button hit was button desired */
	public void mouseReleased(MouseEvent e){
		if (e.getComponent() instanceof Button) {
			if (e.getComponent().getBounds().contains(e.getComponent().getBounds().x+e.getX(),e.getComponent().getBounds().y+e.getY())) {
				String label = ((Button)e.getComponent()).getLabel();
				if (getButtonNumber(label)>=0) {
					if ((getButtonNumber(label)==checkValuesAcceptableButton()) && !dialogValuesAcceptable())
						dialogValuesNotAcceptableWarning();
					else {
						buttonHit(label, (Button)e.getComponent());
						if (autoDispose)
							dispose();
					}
				}
			}
		}
	}

	public void selectButton(String label){ //for use by scripting & console
			buttonHit(label, null);
			if (autoDispose)
				dispose();
	}
	/*.................................................................................................................*/
	public static int query(Frame parent, String title) {
		MesquiteInteger buttonPressed = new MesquiteInteger(1);
		ExtensibleDialog extensibleDialog = new ExtensibleDialog(parent, title,buttonPressed);
		extensibleDialog.completeAndShowDialog();
		extensibleDialog.dispose();
		return buttonPressed.getValue();
	}
	/*.................................................................................................................*/
	public void resetButtonPressed() {
		buttonPressed.setValue(1);
	}
	/*.................................................................................................................*/
	public int query() {
		return buttonPressed.getValue();
	}

	/*.................................................................................................................*/
 	public void windowOpened(WindowEvent e){ }
	/*.................................................................................................................*/
 	public void windowClosing(WindowEvent e){
		if (getButtonNumber(defaultCancelLabel)>=0) {
			buttonHit(defaultCancelLabel,null);
			dispose();
		}
		else if (getButtonNumber("Done")>=0) {
			buttonHit("Done",null);
			dispose();
		}
		else if (numButtons==1) {
			buttonPressed.setValue(0);
			dispose();
		}
 	} 	
	/*.................................................................................................................*/
 	public void windowClosed(WindowEvent e){}
	/*.................................................................................................................*/
	 public void windowIconified(WindowEvent e){}
	/*.................................................................................................................*/
 	public void windowDeiconified(WindowEvent e){}
	/*.................................................................................................................*/
	 public void windowActivated(WindowEvent e){}
	/*.................................................................................................................*/
	 public void windowDeactivated(WindowEvent e){}
	/*.................................................................................................................*/
	/** This displays the help note.  If you override this, make sure you call super.actionPerformed(e) at the end of your method so that the help system still works */
	 public  void actionPerformed(ActionEvent e) {
	 	if   ("?".equals(e.getActionCommand())) {
			MesquiteTrunk.mesquiteTrunk.alert(getHelpString(),"Mesquite Help", "NOTE");
		}
//		else
//			super.actionPerformed(e);
	 }
	/*.................................................................................................................*/
	 public void mouseOnImage(String imageName) {
	 	if   ("?".equals(imageName)) 
			MesquiteTrunk.mesquiteTrunk.alert(getHelpString(),"Mesquite Help", "NOTE");
		else if (imageName =="manual")
			showManual();
	 }
	/*.................................................................................................................*/
	/** This should be overridden to process double-click events, if you have added this as a DoubleClickListener to some component */
	public void doubleClicked(Component c){
	}
}



/*===============================================*/
/** A field for displaying uneditable text. */
class TextCanvas extends TextArea  {
	String message;
	int width = 0;
	int totalLength;
	int totalHeight;
	int lineHeight;
	int numReturns=0;
	Font font;
	/*.................................................................................................................*/
	public TextCanvas (int width, Font font, String message, int scrollbars) {
		super(message, 1, 40, TextArea.SCROLLBARS_NONE);
		this.message = message;
		numReturns = StringUtil.getNumMatchingChars(message, '\n');
		this.width = width;
		if (font == null)
			font = ExtensibleDialog.defaultBigFont;
		this.font = font;
		setFont(font);
		reCalcRows(width);
		setEditable(false);
		disableEvents(AWTEvent.MOUSE_EVENT_MASK+AWTEvent.MOUSE_MOTION_EVENT_MASK);
	}
	/*.................................................................................................................*/
	public void reCalcRows (int width) {
		totalLength = StringUtil.getStringDrawLength(this,message);
		lineHeight = StringUtil.getTextLineHeight(this);
		int estimatedRows;
		if (width == 12)
			estimatedRows = 2+numReturns; //added 13 Dec 01
		else
			estimatedRows = totalLength / (width-12) +2+numReturns;
		if (estimatedRows <= 0)
			estimatedRows = 3;
		else if (estimatedRows >20)
			estimatedRows = 20;
		setRows(estimatedRows);
		
	}
	/*.................................................................................................................*/
	public void setCanvasRows (int width) {
		Font font = getFont();
		if (font==null)
			setFont(new Font("Dialog", Font.PLAIN, 12));
		reCalcRows(width);
		if (getBackground() != getParent().getBackground())
			setBackground(getParent().getBackground());
	}
	
	public static TextCanvas getTextCanvas(int width, Font font, String message){
		TextCanvas tc = new TextCanvas(width, font, message, TextArea.SCROLLBARS_NONE);
		if (tc.getRows()>50){
			return new TextCanvas(width, font, message, TextArea.SCROLLBARS_VERTICAL_ONLY);
		}
		else
			return tc;
	}
	public void paint(Graphics g){
		if (getBackground() != getParent().getBackground())
			setBackground(getParent().getBackground());
		super.paint(g);
	}

}



