/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib;

import java.awt.*;
import java.util.*;



/* ======================================================================== */
/**Represents a raw block of command strings from a NEXUS file.  Temporarily created in file reading.
 */
public class FileBlock {
	Vector commands;
	Vector commandComments;
	MesquiteFile f;
	long countRequested; //count of number of instances of target command
	
	public FileBlock(){
		commands = new Vector();
		commandComments = new Vector();
	}
	public void setFile(MesquiteFile f){
		this.f = f;
	}
	public MesquiteFile getFile(){
		return f;
	}
	public void addCommand(String c, String comment){
		commands.addElement(c);
		if (comment ==null)
			commandComments.addElement(new String(""));
		else
			commandComments.addElement(comment);
	}
	public String getCommand(int ic, MesquiteString comment){
		if (ic>=0 && ic< commands.size()){
			if (comment !=null)
				comment.setValue((String)commandComments.elementAt(ic));
			return (String)commands.elementAt(ic);
		}
		return null;
	}
	public int getNumCommands(){
		return commands.size();
	}
	public String toString(){
		if (commands.size()==0)
			return null;
		StringBuffer sb = new StringBuffer(1000);
		for (int i=0; i<commands.size();i++)
			sb.append((String)commands.elementAt(i) + StringUtil.lineEnding());
		return sb.toString();
	}

}

