/* Mesquite source code.  Copyright 2001-2004 D. Maddison and W. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib;

import java.awt.*;
import java.awt.event.*;

/*===============================================*/
/** A little dialog box with a list.*/
public class ListDialog extends ExtensibleDialog implements ItemListener{
	DoubleClickList list;
	MesquiteInteger selected;
	IntegerArray indicesSelected = new IntegerArray(0);
	String[] strings = null;
	Listable[] listables = null;
	Listable[] listablesUsed = null;
	TextArea explanation = null;
	//Button ok;
	String thirdButton = null;
	Checkbox showSecondaryCheckbox;
	boolean showSecondary;
	Class priorityClass;
	String showSecString = "Show Secondary Choices";
	boolean prioritize = false;
	boolean hasDefault=true;
	
	public ListDialog (Frame parent, String title, String message, boolean autoComplete, String helpString, Object names, MesquiteInteger selected, String okButton, String thirdButton, boolean prioritize, Class priorityClass, boolean hasDefault, boolean multipleMode) {
		super(parent,title);
		if (names instanceof Listable[])
			listables = (Listable[])names;
		else if (names instanceof String[])
			strings = (String[])names;
		listablesUsed = listables; 
		this.selected = selected;
		this.thirdButton =thirdButton;
		this.prioritize = prioritize;
		this.priorityClass=priorityClass;
		this.hasDefault=hasDefault;

		if (!priorityDifferenceExists()){
			this.prioritize = false;
			this.showSecondary = true;
		}
		
		if (helpString!=null)
			appendToHelpString(helpString);
		
		if (message!= null) {
			addLargeOrSmallTextLabel(message);
		}
		
		Panel mainPanel = addNewDialogPanel();
		list = createListenedList (names, selected, 12, this,this, multipleMode);
		list.setForceSize(true);
		mainPanel.add(list);

		if (anyDocumentables(listables)){
			explanation = addTextAreaSmallFont("", 2);
			explanation.setBackground(ColorDistribution.brighter(explanation.getParent().getBackground(),0.5));
			explanation.setEditable(false);
		}


		if (this.prioritize){
			showSecondaryCheckbox = addCheckBox(showSecString, false);
			showSecondaryCheckbox.setState(showSecondary);
			showSecondaryCheckbox.addItemListener(this);
			resetShowSecondary(showSecondary);
		}
		else {
			showSecondaryCheckbox = null;
			resetShowSecondary(true);
		}


		if (!StringUtil.blank(thirdButton)) 
			setExtraButton(thirdButton);
			
		setAutoDispose(false);
		
		list.requestFocus();
		if (numSelected() == 1)
			showExplanation(list.getSelectedIndex());
		

		if (autoComplete){
			if (!StringUtil.blank(okButton))
				completeAndShowDialog(okButton,defaultCancelLabel, hasDefault,this);
			else
				completeAndShowDialog(defaultOKLabel,defaultCancelLabel, hasDefault,this);
		}
		//ok = getMainButton();
		
	}
	public ListDialog (Frame parent, String title, String message, boolean autoComplete, String helpString, Object names, MesquiteInteger selected, String okButton,String thirdButton, boolean hasDefault, boolean multipleMode) {
		this(parent,title,message, autoComplete, helpString, names,selected,okButton,thirdButton,false,null,hasDefault, multipleMode);
	}
	public ListDialog (Frame parent, String title, String message, boolean autoComplete, String helpString, Object names, MesquiteInteger selected, String thirdButton, boolean hasDefault, boolean multipleMode) {
		this(parent,title,message, autoComplete, helpString, names,selected,"OK",thirdButton,false,null,hasDefault, multipleMode);
	}
	
	private boolean priorityDifferenceExists(){
		
		if (listables!=null){
			boolean secondPriorityFound = false;
			boolean firstPriorityFound = false;
			for (int i=0; i<listables.length; i++) {
				if (listables[i]!=null){
					if (listables[i] instanceof Prioritizable && !((Prioritizable)listables[i]).isFirstPriority(priorityClass)){
						secondPriorityFound = true;
						if (firstPriorityFound)
							return true;
					}
					else {
						firstPriorityFound = true;
						if (secondPriorityFound)
							return true;
					}
				}
			}
		}
		return false;
	}
	private void resetShowSecondary(boolean showSec){
		if (listables!=null){
			list.setVisible(false);
			list.removeAll();
			if (showSec && priorityDifferenceExists())
				listablesUsed = listables;
			else {
				int count = 0;
				for (int i=0; i<listables.length; i++) {
					if (listables[i]!=null && (showSec || !(listables[i] instanceof Prioritizable) || ((Prioritizable)listables[i]).isFirstPriority(priorityClass))){
						count++;
					}
				}
				listablesUsed = new Listable[count];
				count = 0;
				for (int i=0; i<listables.length; i++) {
					if (listables[i]!=null && (showSec || !(listables[i] instanceof Prioritizable) || ((Prioritizable)listables[i]).isFirstPriority(priorityClass))){
						listablesUsed[count++] = listables[i];
					}
				}
			}
			for (int i=0; i<listablesUsed.length; i++) {
				if (listablesUsed[i] instanceof SpecialListName && ((SpecialListName)listablesUsed[i]).getListName()!=null)
					list.add(((SpecialListName)listablesUsed[i]).getListName(), -1);
				else if (listablesUsed[i].getName()!=null)
					list.add(listablesUsed[i].getName(), -1);
			}
			if (selected!=null && selected.getValue()>=0 && selected.getValue()<listables.length)
				list.select(selected.getValue());
			if (numSelected() == 1)
				showExplanation(list.getSelectedIndex());
			list.setVisible(true);
			list.invalidate();
			list.validate();
			invalidate();
			validate();
			list.repaint();
		}
	}
	protected void resetList(){
		if (listables!=null){
			resetShowSecondary(showSecondary);
		}
		else if (strings!=null){
			list.setVisible(false);
			int vis = list.getVisibleIndex();
			boolean[] sel = new boolean[list.getItemCount()];
			for (int i = 0; i<list.getItemCount(); i++){
				sel[i] = list.isIndexSelected(i);
			}
			list.removeAll();
			for (int i=0; i<strings.length; i++)
				if (strings[i]!=null)
					list.add(strings[i], -1);
			for (int i = 0; i<sel.length && i < list.getItemCount(); i++){
				if (sel[i])
					list.select(i);
			}
			if (selected!=null && selected.getValue()>=0 && selected.getValue()<strings.length)
				list.select(selected.getValue());
			if (numSelected() == 1)
				showExplanation(list.getSelectedIndex());
			list.makeVisible(vis);
			list.setVisible(true);
			list.invalidate();
			list.validate();
			invalidate();
			validate();
			list.repaint();
		}
	}
/*	private void setPrioritize(boolean p, Class priorityClass){
		if (prioritize==p)
			return;
		this.priorityClass = priorityClass;
		if (!priorityDifferenceExists()){
			prioritize = false;
			showSecondary = true;
		}
		else
			prioritize = p;
		if (prioritize){
			buttons.remove(ok);
			buttons.remove(cancel);
			buttons.add("West", showSecondaryCheckbox);
			buttons.add("Center", cancel);
			buttons.add("East", ok);
			showSecondaryCheckbox.setState(showSecondary);
			resetShowSecondary(showSecondary);
		}
		else {
			buttons.remove(showSecondaryCheckbox);
			buttons.remove(ok);
			buttons.remove(cancel);
			buttons.add("West", cancel);
			buttons.add("East", ok);
			resetShowSecondary(true);
		}
	}
*/		
	private boolean anyDocumentables(Listable[] L){
		if (L==null)
			return false;
		for (int i=0; i<L.length; i++)
			if (L[i] instanceof Explainable)
				return true;
		return false;
	}
	protected int numSelected(){
		int count = 0;
		for (int i=0; i<list.getItemCount(); i++)
			if (list.isIndexSelected(i))
				count++;
		return count;
	}
	private void setSelected(boolean[] selected){
		if (selected == null)
			return;
		for (int i=0; i<list.getItemCount(); i++)
			list.deselect(i);
		
		for (int i=0; i<list.getItemCount() && i<selected.length; i++)
			if (selected[i])
				list.select(i);
	}
	private void showExplanation(int selected){
		if (selected <0) {
			if (explanation !=null) {
				explanation.setText("");
				return;
			}
		}
		else if (listablesUsed !=null && selected < listablesUsed.length && listablesUsed[selected] instanceof Explainable) {
			if (explanation !=null) {
				String exp = ((Explainable)listablesUsed[selected]).getExplanation();
				if (exp == null)
					exp = "";
				explanation.setText(exp);
				return;
			}
		}
		if (explanation !=null) 
			explanation.setText("");
	}
	public void itemStateChanged(ItemEvent e){
		if (listablesUsed==null)
			return;
		if (e.getItem() instanceof String && showSecString.equals((String)e.getItem())){
			showSecondary = showSecondaryCheckbox.getState();
			resetShowSecondary(showSecondaryCheckbox.getState());
		}
		else if (e.getStateChange() == ItemEvent.SELECTED && e.getItem() instanceof Integer){
			if (numSelected() > 1)
				showExplanation(-1);
			else {
				showExplanation(((Integer)e.getItem()).intValue());
			}
		}
	}
/*	public void actionPerformed(ActionEvent e){
		if (defaultButtonString!=null) {
			buttonHit(defaultButtonString, null);
			dispose();
		}
	}
*/
	public static Listable queryList(Frame parent, String title, String message, String helpString, ListableVector vector, int current, boolean doShow) {
		if (vector==null) 
			return null;
		MesquiteInteger io = new MesquiteInteger(current);
		Listable[] names = vector.getListables();
		String thirdButton = null;
		if (doShow)
			thirdButton = "Show";
		while (true) {
			ListDialog id = new ListDialog(parent, title, message, true, helpString, names, io, thirdButton,true, false);
			//id.setVisible(true);
			if (io.isUnassigned()) {
				id.dispose();
				return null;
			}
			int result = io.getValue();
			if (result <0) { //negative value; must be asking to show
				result = - result - 1;  
				if (result>=0 && result<vector.size()) {
					Listable li =  (Listable)vector.elementAt(result);
					if (li instanceof Showable) {
						((Showable)li).showMe();
					}
					io.setValue(result);
				}
			}
			else if (result>=0 && result<vector.size()) {
				id.dispose();
				return (Listable)vector.elementAt(result);
			}
			else {
				id.dispose();
				return null;
			}
		}
	}
	//TODO: probably better if all of these return array of selected!
	public static Listable queryList(Frame parent, String title, String message, String helpString, ListableVector vector, int current) {
		if (vector==null) 
			return null;
		MesquiteInteger io = new MesquiteInteger(current);
		Listable[] names = vector.getListables();
		if (names==null)
			return null;
		ListDialog id = new ListDialog(parent, title, message, true,helpString, names, io, null,true, false);
		//id.setVisible(true);
		if (id!=null)
			id.dispose();
		int result = io.getValue();
		if (io.isUnassigned())
			return null;
		if (result>=0 && result<vector.size())
			return (Listable)vector.elementAt(result);
		else return null;
	}
	public static Listable[] queryListMultiple(Frame parent, String title, String message, String helpString, ListableVector vector, boolean[] selected) {
		return queryListMultiple(parent, title, message, helpString, null, true, vector, selected);
	}
	public static Listable[] queryListMultiple(Frame parent, String title, String message, String helpString, String okButton, boolean hasDefault, ListableVector vector, boolean[] selected) {
		if (vector==null) 
			return null;
		Listable[] names = vector.getListables();
		if (names==null)
			return null;
		ListDialog id = new ListDialog(parent, title, message, true,helpString, names, null, okButton,null,hasDefault, true);
//		if (okButton!=null)
//			id.ok.setLabel(okButton);
		id.list.setMultipleMode(true);
		id.setSelected(selected);
		//id.setVisible(true);
		IntegerArray result = id.indicesSelected;
		if (result==null || result.getSize()==0) {
			id.dispose();
			return null;
		}
		Listable[] L = new Listable[result.getSize()];
		for (int i=0; i<result.getSize(); i++)
			L[i] = (Listable)vector.elementAt(result.getValue(i));
		id.dispose();
		return L;
	}
	public static Listable queryList(Frame parent, String title, String message, String helpString, Listable[] objects, boolean prioritize, Class priorityClass, int current) {
		if (objects==null) 
			return null;
		MesquiteInteger io = new MesquiteInteger(current);
		// (Frame parent, String title, String message, Object names, MesquiteInteger selected, String okButton, String thirdButton, boolean prioritize, Class priorityClass, boolean hasDefault) 
		ListDialog id = new ListDialog(parent, title, message, true,helpString, objects, io, null,null,prioritize, priorityClass, true, false);
	//	id.setPrioritize(prioritize, priorityClass);
		//id.setVisible(true);
		id.dispose();
		int result = io.getValue();
		if (io.isUnassigned())
			return null;
		if (result>=0 && result<objects.length) ///ERROR here it's returning item in original list even if prioritized!!!!!!!!!!
			return objects[result];
		else return null;
	}
	public static Listable queryList(Frame parent, String title, String message, String helpString, Listable[] objects, int current) {
		if (objects==null) 
			return null;
		MesquiteInteger io = new MesquiteInteger(current);
		ListDialog id = new ListDialog(parent, title, message, true,helpString, objects, io, null,true, false);
		//id.setVisible(true);
		id.dispose();
		int result = io.getValue();
		if (io.isUnassigned())
			return null;
		if (result>=0 && result<objects.length)
			return objects[result];
		else return null;
	}
	public static int queryList(Frame parent, String title, String message, String helpString, Context[] contexts, int current, boolean doShow) {
		if (contexts==null)
			return MesquiteInteger.unassigned;
		MesquiteInteger io = new MesquiteInteger(current);
		String[] names = new String[contexts.length];
		for (int i = 0; i<names.length; i++)
			names[i]=contexts[i].getContextName();
		
		String thirdButton = null;
		if (doShow)
			thirdButton = "Show";
		while (true) {
			ListDialog id = new ListDialog(parent, title, message, true,helpString, names, io, thirdButton,true, false);
			//id.setVisible(true);
			id.dispose();
			if (io.isUnassigned())
				return MesquiteInteger.unassigned;
			int result = io.getValue();
			if (result <0) { //negative value; must be asking to show
				result = - result - 1;  
				if (result>=0 && result<contexts.length) {
					Context li =  contexts[result];
					if (li instanceof Showable) {
						((Showable)li).showMe();
					}
					io.setValue(result);
				}
			}
			else if (result>=0 && result<contexts.length)
				return result;
		
			else return MesquiteInteger.unassigned;
		}
	}
	public static int queryList(Frame parent, String title, String message, String helpString, String[] names, int current) {
		if (names==null)
			return MesquiteInteger.unassigned;
		MesquiteInteger io = new MesquiteInteger(current);
		ListDialog id = new ListDialog(parent, title, message, true,helpString, names, io, null,true, false);
		//id.setVisible(true);
			id.dispose();
		return io.getValue();
	}
	public static String queryList(Frame parent, String title, String message, String helpString, Class dutyClass, Object condition, MesquiteModule prospectiveEmployer) {
		Listable[] names = MesquiteTrunk.mesquiteModulesInfoVector.getModulesOfDuty(dutyClass, condition, prospectiveEmployer);
		if (names==null)
			return null;
		MesquiteInteger io = new MesquiteInteger(0);
		ListDialog id = new ListDialog(parent, title, message, true,helpString, names, io, null,true, false);
		//id.setVisible(true);
			id.dispose();
		if (io.isUnassigned())
			return null;
		if (io.getValue()>=0)
			return names[io.getValue()].getName();
		else
			return null;
	}
	protected DoubleClickList getList(){
		return list;
	}
	private int[] translateIndicesUsedToOriginal(int[] indices){
		if (indices !=null && listablesUsed !=null && listablesUsed != listables){
			int count = 0;
			for (int i=0; i< indices.length; i++){
				if (translateIndexUsedToOriginal(indices[i]) >=0)
					count++;
			}
			if (count == 0)
				return indices;
			int[] usedIndices = new int[count];
			count =0;
			for (int i=0; i< indices.length; i++){
				int used = translateIndexUsedToOriginal(indices[i]);
				if (used >=0)
					usedIndices[count++] = used;
			}
			return usedIndices;
		}
		return indices;
	}
	private int findListable(Listable[] lArray, Listable target){
		if (lArray==null)
			return -1;
		for (int i=0; i< lArray.length; i++)
			if (target == lArray[i])
				return i;
		return -1;
	}
	private int translateIndexUsedToOriginal(int index){
		if (listablesUsed !=null && listablesUsed != listables && index < listables.length && index < listablesUsed.length){
			Listable listable = listablesUsed[index];
			int which = findListable(listables, listable);
			return which;
		}
		return index;
	}
/*.................................................................................................................*/
	public void doubleClicked(Component c){
	  	if (c==list) {
			try {
				if (selected!=null) {
					selected.setValue(translateIndexUsedToOriginal(list.getSelectedIndex()));
				}
				indicesSelected.setValues(translateIndicesUsedToOriginal(list.getSelectedIndexes()));
			}
			catch (NumberFormatException eq){}
			dispose();
	  	}
	}
/*.................................................................................................................*/
	public void buttonHit(String buttonLabel, Button button) {
		if (buttonLabel!=null)
			if (buttonLabel.equalsIgnoreCase(getMainButton().getLabel())){
				try {
					if (selected!=null) {
						selected.setValue(translateIndexUsedToOriginal(list.getSelectedIndex()));
					}
					indicesSelected.setValues(translateIndicesUsedToOriginal(list.getSelectedIndexes()));
				}
				catch (NumberFormatException eq){}
				dispose();
			}
			else if (buttonLabel.equalsIgnoreCase("Show")){
				int i = translateIndexUsedToOriginal(list.getSelectedIndex());
				if (i>=0 && selected!=null) 
					selected.setValue(-i-1); //if nothing selected do not respond!
			}
			else if (buttonLabel.equalsIgnoreCase(thirdButton)){
					selected.setValue(-1); 
			}
			else {
				if (selected!=null)
					selected.setToUnassigned();
				indicesSelected.setValues(null);
				dispose();
			}
	}
}

