/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib;

import java.awt.*;
import java.util.*;
import mesquite.lib.duties.*;



/* ======================================================================== */
/**Represents a block in a NEXUS file.  Often contains a reference to the object to be written (e.g., a CharacterData
or TreeVector.
 */
public abstract class NexusBlock implements Listable, Identifiable {
	MesquiteModule manager;
	MesquiteFile file;
	String name;
	Vector unrec; //to store unrecognized commands
	public static Vector classesCreated, classesFinalized, countsOfClasses, countsOfClassesFinalized; //to detect memory leaks
	public static long totalCreated =0;
	public static long totalDisposed =0;
	public static long totalFinalized =0;
	long id;
	public static boolean suppressTITLE = false;
	public static boolean suppressLINK = false;
	static {
		if (MesquiteTrunk.checkMemory) {
			classesCreated = new Vector();
			classesFinalized = new Vector();
			countsOfClassesFinalized = new Vector();
			countsOfClasses = new Vector();
		}
	}
	
	public NexusBlock(MesquiteFile f, MesquiteModule manager){
		totalCreated++;
		if (MesquiteTrunk.checkMemory)
			countCreated();
		id = totalCreated;
		file = f;
		this.manager = manager;
		/*
		if (f==null)
			MesquiteMessage.warnProgrammer("Warning: file null in NexusBlock " + this.getClass());
		if (manager==null)
			MesquiteMessage.warnProgrammer("Warning: manager null in NexusBlock" + this.getClass());
		*/
	}
	void countCreated(){
		if (classesCreated.indexOf(getClass())<0) {
			classesCreated.addElement(getClass());
			countsOfClasses.addElement(new MesquiteInteger(1));
			countsOfClassesFinalized.addElement(new MesquiteInteger(0));
		}
		else {
			MesquiteInteger c = (MesquiteInteger)countsOfClasses.elementAt(classesCreated.indexOf(getClass()));
			if (c!=null)
				c.increment();
		}
	}
	public void dispose() {
		manager = null;
		file = null;
		totalDisposed++;
	}
	public void finalize() throws Throwable {
		totalFinalized++;
		if (MesquiteTrunk.checkMemory) {
			if (classesFinalized.indexOf(getClass())<0) {
				classesFinalized.addElement(getClass());
			}
			MesquiteInteger c = (MesquiteInteger)countsOfClassesFinalized.elementAt(classesCreated.indexOf(getClass()));
			if (c!=null)
				c.increment();
		}
		super.finalize();
	}
	public long getID(){
		return id;
	}
	
	/** Returns the offical NEXUS block name (e.g. "TAXA")*/
	public abstract String getBlockName();
	
	/** DOCUMENT */
	public MesquiteFile getFile() { 
		return file;
	}
	/** DOCUMENT */
	public void setFile(MesquiteFile f) { 
		file = f;
	}
	/** DOCUMENT */
	public String getName() { 
		if (name == null)
			return "Untitled";
		else
			return name;
	}
	/** DOCUMENT */
	public void setName(String n) { 
		name = n;
	}
	/** DOCUMENT */
	public MesquiteModule getManager(){
		return manager;
	}
	/** DOCUMENT */
	public void setManager(MesquiteModule manager){
		this.manager = manager;
	}
	/** DOCUMENT */
	public abstract boolean contains(FileElement e);
	
	/** Returns true if this block must occur after the given block*/
	public abstract boolean mustBeAfter(NexusBlock block);

	/** DOCUMENT */
	public String getUnrecognizedCommands(){
		if (unrec==null)
			return "";
		String result = "";
		Enumeration enum=unrec.elements();
		while (enum.hasMoreElements()){
			Object obj = enum.nextElement();
			result += (String)obj + StringUtil.lineEnding();
		}
		return result;
	}
	/** DOCUMENT */
	void storeUnrecognizedCommand(String s){
		if (StringUtil.blank(s))
			return;
		if (unrec ==null)
			unrec = new Vector();
		unrec.addElement(s);
		if (file !=null && file.foreignElements!=null)
			file.foreignElements.addElement("Command: " + s + " in " + getBlockName() + " block ");
	}
	/** Returns the NEXUS block as a string for writing into the file.  Block must override either this or writeNEXUSBlock to write to file*/
	protected String getNEXUSBlock(){
		return null;
	}
	/** Writes the NEXUS block into the file  Block must override either this or getNEXUSBlock to write to file*/
	public void writeNEXUSBlock(MesquiteFile file, ProgressIndicator progIndicator){
		String s = getNEXUSBlock();
		if (!StringUtil.blank(s))
			file.writeLine(s);
	}
	/*   */
}

