/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib;

import java.awt.*;
import java.text.*;



/* ======================================================================== */
/** An object that contains information about a notification event (e.g., call to notifyListeners or parametersChanged)*/
public class Notification implements Identifiable {
	int code = MesquiteListener.UNKNOWN;
	int[] parameters = null;
	long id;
	long nn = -1;
	static long numCreated = 0;
	public Notification(int code, int[] parameters){
		this();
		this.code = code;
		this.parameters = parameters;
	}
	public Notification(int code){
		this();
		this.code = code;
	}
	public Notification(){
		id = numCreated++;
	}
	public long getID(){
		return id;
	}
	public Notification setNotificationNumber(long nn){
		this.nn = nn;
		return this;
	}
	public long getNotificationNumber(){
		if (nn<0)
			return id;
		else
			return nn;
	}
	public int getCode(){
		return code;
	}
	public int[] getParameters(){
		return parameters;
	}
	
	public static int getCode(Notification n){
		if (n==null)
			return MesquiteListener.UNKNOWN;
		else
			return n.getCode();
	}
	public static int[] getParameters(Notification n){
		if (n==null)
			return null;
		else
			return n.getParameters();
	}
	public static boolean appearsCosmetic(Notification n){
		if (n==null)
			return false;
		else
			return n.getCode() == MesquiteListener.ANNOTATION_CHANGED || n.getCode() == MesquiteListener.NAMES_CHANGED ;
	}
}

