/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib;

import java.awt.*;


/* ======================================================================== */
/** A class for parsing strings for NEXUS files and commands.*/
public class Parser extends StringUtil {
	StringBuffer buffer, buffer2;
	StringBuffer commandBuffer;
	String whitespaceString = null;
	String whitespaceStringSet = null;
	String punctuationString = null;
	String punctuationStringSet = null;
	char quoteChar = defaultQuote;
	boolean lineEndingsDark = false;
	MesquiteInteger pos;
	StringBuffer line = new StringBuffer(1000);
	public Parser(){
		pos = new MesquiteInteger(0);
		buffer = new StringBuffer(100); 
		buffer2 = new StringBuffer(100); 
	}
	public Parser(String line){
		this();
		setString(line);
	}
	public void setString(String s){
		if (s == null)
			s = "";
		line.setLength(0);
		line.append(s);
		pos.setValue(0);
	}
	public StringBuffer getBuffer(){
		return line;
	}
	public String getString(){
		if (line == null)
			return null;
		return line.toString();
	}
	public int getPosition(){
		return pos.getValue();
	}
	public void setPosition(int p){
		pos.setValue(p);
	}
	public String getWhitespaceString(){
		return whitespaceString;
	}
	public void setWhitespaceString(String s){
		if (s==null){
			whitespaceString = defaultWhitespace;
			whitespaceStringSet = null;
		}
		else {
			whitespaceString = s;
			whitespaceStringSet = s;
		}
		if (lineEndingsDark){
			if (whitespaceString ==null)
				whitespaceString = StringUtil.defaultWhitespace;
			whitespaceString = StringUtil.delete(whitespaceString, '\n');
			whitespaceString = StringUtil.delete(whitespaceString, '\r');
		}
	}
	public boolean getLineEndingsDark(){
		return lineEndingsDark;
	}
	public void setLineEndingsDark(boolean b){
		lineEndingsDark = b;
		if (lineEndingsDark){
			if (whitespaceString ==null)
				whitespaceString = StringUtil.defaultWhitespace;
			whitespaceString = StringUtil.delete(whitespaceString, '\n');
			whitespaceString = StringUtil.delete(whitespaceString, '\r');
			if (punctuationString == null)
				punctuationString = defaultPunctuation;
			if (punctuationString.indexOf('\n')<0);
				punctuationString = punctuationString + '\n';
			if (punctuationString.indexOf('\r')<0);
				punctuationString = punctuationString + '\r';
		}
		else {
			whitespaceString =  whitespaceStringSet;
			punctuationString = punctuationStringSet;
		}
	}
	public void addToDefaultPunctuationString(String s){
		if (s==null)
			return;
		punctuationString = defaultPunctuation+s;
		punctuationStringSet = defaultPunctuation+s;
		if (lineEndingsDark){
			if (punctuationString.indexOf('\n')<0);
				punctuationString = punctuationString + '\n';
			if (punctuationString.indexOf('\r')<0);
				punctuationString = punctuationString + '\r';
		}
	}
	
	public void setPunctuationString(String s){
		if (s==null){
			punctuationString = defaultPunctuation;
			punctuationStringSet = null;
		}
		else {
			punctuationString = s;
			punctuationStringSet = s;
		}
		if (lineEndingsDark){
			if (punctuationString == null)
				punctuationString = defaultPunctuation;
			if (punctuationString.indexOf('\n')<0);
				punctuationString = punctuationString + '\n';
			if (punctuationString.indexOf('\r')<0);
				punctuationString = punctuationString + '\r';
		}
	}
        public String getPunctuationString (){
                return punctuationString;
        }
	public void setQuoteCharacter(char c){
		quoteChar = c;
	}
	private String getQuoted() {
		if (line==null)
			return null;
		buffer2.setLength(0);
		char c;
		boolean done = false;
		while (!done) {
			int where = pos.getValue();
			if (where<line.length()) {
				c= line.charAt(where);
				if (c==quoteChar){
					int np = where + 1;
					if (np<line.length()) {
						char nxt= line.charAt(np);
						if (nxt==quoteChar) {
							buffer2.append(quoteChar);
							pos.increment();  //skip to \'
							pos.increment(); //skip to next
						}
						else 
							done = true;
					}
					else done = true;
				}
				else if (c=='^'){
					int np = where + 1;
					if (np<line.length()) {
						char nxt= line.charAt(np);
						if (nxt=='n') {
							buffer2.append('\n');
							pos.increment();  //skip to \n
							pos.increment(); //skip to next
						}
						else
							pos.increment(); //skip to next
					}
					else done = true;
				}
				else {
					buffer2.append(c);
					pos.increment();
				}
			}
			else
				done = true;
			
		}
		pos.increment();
		return buffer2.toString();
	}
	/*.........................................................................................................*/
	/*  doesn't replace '' by ' */
	private StringBuffer getQuotedUnaltered(StringBuffer buf) {
		if (line==null)
			return null;
		buf.setLength(0);
		char c;
		boolean done = false;
		while (!done) {
			int where = pos.getValue();
			if (where<line.length()) {
				c= line.charAt(where);
				if (c==quoteChar){
					int np = where + 1;
					if (np<line.length()) {
						char nxt= line.charAt(np);
						if (nxt==quoteChar) {
							buf.append('\'');
							buf.append('\'');
							pos.increment();  //skip to \'
							pos.increment(); //skip to next
						}
						else 
							done = true;
					}
					else done = true;
				}
				else {
					buf.append(c);
					pos.increment();
				}
			}
			else
				done = true;
			
		}
		pos.increment();
		return buf;
	}
	/*.........................................................................................................*/
	/** returns remainder of line starting at pos; including square bracket comments*/
	public String getRemaining() {
		if (line == null)
			return null;
		else
			return line.toString().substring(pos.getValue(), line.length());
	}

	/*............................................  ....................................................*/
	/** if first character of string is a quote ('), takes to contents of the quote and returns them (calling getFirstToken).
	Otherwise returns the string*/
	public String getUnquotedToken() {
		if (line == null || line.length()==0 || line.charAt(0)!= quoteChar)
			return line.toString();
		else
			return getFirstToken(line.toString());
	}
	/*............................................  ....................................................*/
	/** returns first token from string; sets pos to end of token; excluding square bracket comments*/
	public String getFirstToken(String line) {
		setString(line);
		pos.setValue(0);
		return getNextToken();
	}
	/*............................................  ....................................................*/
	/** returns first token from string; afterward sets pos to end of token; excluding square bracket comments*/
	public String getFirstToken() {
		pos.setValue(0);
		return getNextToken();
	}
	/*............................................  ....................................................*/
	/** returns characters from pos to first instance of \r or \n*/
	public String getRawNextLine() {
		if (line==null)
			return null;
		else if (line.equals(""))
			return "";
		 if (pos.getValue() >= line.length())
			return null;
		char c=getNextChar();
		buffer.setLength(0);
		while (!lineEndCharacter(c) && pos.getValue()<line.length()) {  // or should this be <= line.length?
			buffer.append(c);
			c=getNextChar();
		}
		return buffer.toString();
	}
	/*............................................  ....................................................*/
	/** returns token from line starting at pos; excluding square bracket comments*/
	public String getNextToken() {
		if (line==null)
			return null;
		else if (line.equals(""))
			return "";
		 if (pos.getValue() >= line.length())
			return null;
		char c;
		buffer.setLength(0);
		try {
			boolean continu = true;
			while (whitespace(c=getNextChar(), whitespaceString))
				;
			if (punctuationOrQuote(c, punctuationString, quoteChar)) {
				continu = false;
				if (c == quoteChar) {
					buffer.setLength(0);
					buffer.append(getQuoted());
				}
				else if (c == '-') {
					char cod = charOnDeck(1);
					char cod2 = charOnDeck(2);
					if ((digits.indexOf(cod)>=0)||(cod=='.' && (digits.indexOf(cod2)>=0))) {
						buffer.append(c);
						c=getNextChar();
						continu = true;
					}
					else {
						buffer.append(c);
					}
				}
				else
					buffer.append(c);
				if (lineEndCharacter(c)) {
					char r = charOnDeck(1);
					//if next character is also lineEndCharacter but of opposite sort (i.e. \n instead of \r) then eat it up
					if (r!=c && lineEndCharacter(r))
						pos.increment();
				}
			}
			if (continu) {
				while ((!whitespace(c, whitespaceString) && !punctuationOrQuote(c, punctuationString, quoteChar)) && c!=0) {
					if (c=='_')
						c = ' ';
					else if (c=='^'){
						char nxt = getNextChar();
						if (nxt=='n') 
							c='\n';
					}
					buffer.append(c);
					c=getNextChar();
				}
				if (punctuationOrQuote(c, punctuationString, quoteChar)) {
					pos.decrement();
				}
			}
		}
		catch (StringIndexOutOfBoundsException e) {
			System.out.println("string bounds exceeded 0");
			pos.setValue(line.length());
			}
		return buffer.toString();
	}
	/*.........................................................................................................*/
	/** returns token from line starting at pos; excluding square bracket comments*/
	public String getUnalteredToken(boolean includeWhitespace) {
		if (line==null)
			return null;
		else if (line.equals(""))
			return "";
		 if (pos.getValue() >= line.length())
			return null;
		buffer.setLength(0);
		char c;
		try {
			boolean continu = true;
			while (whitespace(c=getNextChar(), whitespaceString))
				if (includeWhitespace)
					buffer.append(c);
			if (punctuationOrQuote(c, punctuationString, quoteChar)) {
				continu = false;
				if (c == quoteChar) {
					buffer.setLength(0);
					buffer.append(quoteChar);
					buffer.append(getQuotedUnaltered(buffer2));
					buffer.append(quoteChar);
				}
				else if (c == '-') {
					char cod = charOnDeck(1);
					char cod2 = charOnDeck(2);
					if ((digits.indexOf(cod)>=0)||(cod=='.' && (digits.indexOf(cod2)>=0))) {
						buffer.append(c);
						c=getNextChar();
						continu = true;
					}
					else {
						buffer.setLength(0);
						buffer.append(c);
					}
				}
				else {
					buffer.setLength(0);
					buffer.append(c);
				}
				if (lineEndCharacter(c)) {
					char r = charOnDeck(1);
					//if next character is also lineEndCharacter but of opposite sort (i.e. \n instead of \r) then eat it up
					if (r!=c && lineEndCharacter(r))
						pos.increment();
				}
			}
			if (continu) {
				while ((!whitespace(c, whitespaceString) && !punctuationOrQuote(c, punctuationString, quoteChar)) && c!=0) {
					buffer.append(c);
					c=getNextChar();
				}
				if (punctuationOrQuote(c, punctuationString, quoteChar)) {
					pos.decrement();
				}
			}
		}
		catch (StringIndexOutOfBoundsException e) {
			System.out.println("string bounds exceeded 0");
			pos.setValue(line.length());
			}
		if (blankByCurrentWhitespace(buffer)) 
			return null;
		return buffer.toString();
	}
	/*.........................................................................................................*/
	/** returns token from line starting at pos; keeps track of pending square bracket closure, etc.*/
	public String getUnalteredToken(boolean includeWhitespace, MesquiteInteger pendingBrackets, StringBuffer comment, MesquiteBoolean suppressComment) {
		if (line==null)
			return null;
		else if (line.equals(""))
			return "";
		 if (pos.getValue() >= line.length())
			return null;
		buffer.setLength(0);

		char c = 0;
		try {
			boolean continu = true;
			while (whitespace(c=getNextChar(pendingBrackets, comment, suppressComment), whitespaceString))  //skipping whitespace in front
				if (includeWhitespace)
					buffer.append(c);
			if (punctuationOrQuote(c, punctuationString, quoteChar)) {
				continu = false;
				if (c == quoteChar) {
					buffer.setLength(0);
					buffer.append(quoteChar);
					buffer.append(getQuotedUnaltered(buffer2));
					buffer.append(quoteChar);
				}
				else if (c == '-') {
					char cod = charOnDeck(1);
					char cod2 = charOnDeck(2);
					if ((digits.indexOf(cod)>=0)||(cod=='.' && (digits.indexOf(cod2)>=0))) {
						buffer.append(c);
						c=getNextChar(pendingBrackets, comment, suppressComment);
						continu = true;
					}
					else {
						buffer.setLength(0);
						buffer.append(c);
					}
				}
				else {
					buffer.setLength(0);
					buffer.append(c);
				}
				if (lineEndCharacter(c)) {
					char r = charOnDeck(1);
					//if next character is also lineEndCharacter but of opposite sort (i.e. \n instead of \r) then eat it up
					if (r!=c && lineEndCharacter(r)) {
						pos.increment();
					}
				}
			}
			if (continu) {
				while ((!whitespace(c, whitespaceString) && !punctuationOrQuote(c, punctuationString, quoteChar)) && c !=0) {
					buffer.append(c);
					c=getNextChar(pendingBrackets, comment, suppressComment);
				}
				if (punctuationOrQuote(c, punctuationString, quoteChar)) {
					pos.decrement();
				}
			}
		}
		catch (StringIndexOutOfBoundsException e) {
			System.out.println("string bounds exceeded 1");
			pos.setValue(line.length());
			}
		if (blankByCurrentWhitespace(buffer)) 
			return null;
		return buffer.toString();
	}
	/*.........................................................................................................*/
	/** Fills StringBuffer with token from line starting at pos; keeps track of pending square bracket closure, etc.  Returns false if null string to be returned*/
	public boolean getUnalteredToken(StringBuffer token, boolean includeWhitespace, MesquiteInteger pendingBrackets, StringBuffer comment, MesquiteBoolean suppressComment) {
		if (token == null)
			return false;
		token.setLength(0);
		if (line==null)
			return false;
		 if (pos.getValue() >= line.length())
			return false;
		if (line.equals(""))
			return true;

		char c = 0;
		try {
			boolean continu = true;
			while (whitespace(c=getNextChar(pendingBrackets, comment, suppressComment), whitespaceString))  //skipping whitespace in front
				if (includeWhitespace)
					token.append(c);
			if (punctuationOrQuote(c, punctuationString, quoteChar)) {
				continu = false;
				if (c == quoteChar) {
					token.setLength(0);
					token.append(quoteChar);
					/**/StringUtil.append(token, getQuotedUnaltered(buffer2));
					token.append(quoteChar);
				}
				else if (c == '-') {
					char cod = charOnDeck(1);
					char cod2 = charOnDeck(2);
					if ((digits.indexOf(cod)>=0)||(cod=='.' && (digits.indexOf(cod2)>=0))) {
						token.append(c);
						c=getNextChar(pendingBrackets, comment, suppressComment);
						continu = true;
					}
					else {
						token.setLength(0);
						token.append(c);
					}
				}
				else {
					token.setLength(0);
					token.append(c);
				}
				if (lineEndCharacter(c)) {
					char r = charOnDeck(1);
					//if next character is also lineEndCharacter but of opposite sort (i.e. \n instead of \r) then eat it up
					if (r!=c && lineEndCharacter(r)) {
						pos.increment();
					}
				}
			}
			if (continu) {
				while ((!whitespace(c, whitespaceString) && !punctuationOrQuote(c, punctuationString, quoteChar)) && c !=0) {
					token.append(c);
					c=getNextChar(pendingBrackets, comment, suppressComment);
				}
				if (punctuationOrQuote(c, punctuationString, quoteChar)) {
					pos.decrement();
				}
			}
		}
		catch (StringIndexOutOfBoundsException e) {
			System.out.println("string bounds exceeded 1");
			pos.setValue(line.length());
			}
		if (blankByCurrentWhitespace(token)) 
			return false;
		return true;
	}
	/*.................................................................................................................*/
	/** returns token from line starting at pos; keeps track of pending square bracket closure, etc.*/
	public String getToken(MesquiteInteger pendingBrackets, StringBuffer comment, MesquiteBoolean suppressComment) {
		if (line==null)
			return null;
		else if (line.equals(""))
			return "";
		 if (pos.getValue() >= line.length())
			return null;
		buffer.setLength(0);
		char c;
		try {
			boolean continu = true;
			while (whitespace(c=getNextChar(pendingBrackets, comment, suppressComment), whitespaceString))  //skipping whitespace in front
				;
			if (punctuationOrQuote(c, punctuationString, quoteChar)) {
				continu = false;
				if (c == quoteChar) {
					buffer.append(getQuoted());
				}
				else if (c == '-') {
					char cod = charOnDeck(1);
					char cod2 = charOnDeck(2);
					if ((digits.indexOf(cod)>=0)||(cod=='.' && (digits.indexOf(cod2)>=0))) {
						buffer.append(c);
						c=getNextChar(pendingBrackets, comment, suppressComment);
						continu = true;
					}
					else {
						buffer.append(c);
					}
				}
				else
					buffer.append(c);
				if (lineEndCharacter(c)) {
					char r = charOnDeck(1);
					//if next character is also lineEndCharacter but of opposite sort (i.e. \n instead of \r) then eat it up
					if (r!=c && lineEndCharacter(r)) {
						pos.increment();
					}
				}
			}
			if (continu) {
				while ((!whitespace(c, whitespaceString) && !punctuationOrQuote(c, punctuationString, quoteChar)) && c !=0) {
					if (c=='_')
						c = ' ';
					buffer.append(c);
					c=getNextChar(pendingBrackets, comment, suppressComment);
				}
				if (punctuationOrQuote(c, punctuationString, quoteChar)) {
					pos.decrement();
				}
			}
		}
		catch (StringIndexOutOfBoundsException e) {
			System.out.println("string bounds exceeded 1");
			pos.setValue(line.length());
		}
		return buffer.toString();
	}
	/*.................................................................................................................*/
	private boolean storeCharToComment(String c, int debt, StringBuffer comment, MesquiteBoolean suppressComment){
		if (debt >0 && comment!=null && !suppressComment.getValue()) {
			comment.append(c);
			return true;
		}
		else
			return false;
	}
	/*.................................................................................................................*/
	private boolean storeCharToComment(char c, int debt, StringBuffer comment, MesquiteBoolean suppressComment){
		if (debt >0 && comment!=null && !suppressComment.getValue()) {
			comment.append(c);
			return true;
		}
		else
			return false;
	} 
	int pendingPComment = 0;
	/*.................................................................................................................*/
	char getNextChar(MesquiteInteger pendingBrackets, StringBuffer comment, MesquiteBoolean suppressComment) {
		int posTemp = pos.getValue();
		if (posTemp>=line.length()) {
			return 0;
		}
		char c = line.charAt(posTemp);
		posTemp++;
		int debt = pendingBrackets.getValue();
		boolean hadBrack = false;
		boolean checkExclamation = false;
		if (openingSquareBracket(c)) {
			char next = 0;
			if (posTemp<line.length())
				next = line.charAt(posTemp);
			if (debt==0 && next == '%') {
				c = '<';
				posTemp++; //done to go past %
				line.setCharAt(posTemp-1, c); //done in case punctuation passed back will cause pos to decrement
				pendingPComment = 1;
			}
			else {
				if (!storeCharToComment(c, debt, comment, suppressComment))
					checkExclamation = true;
					if (comment !=null && comment.length()>0)
						comment.append(' ');
				debt++;
			}
			
		}
		else if (closingSquareBracket(c)) {
			if (debt==0 && pendingPComment >0) {
				c = '>';
				line.setCharAt(posTemp-1, c); //done in case punctuation passed back will cause pos to decrement
				pendingPComment = 0;
			}
			else {
				if (debt>0){
					debt--;
					storeCharToComment(c, debt, comment, suppressComment);
					if (debt ==0)
						suppressComment.setValue(false);

				}
				c=0;
			}
		}
		else storeCharToComment(c, debt, comment, suppressComment);
		int count = 0;
		while  (debt>0 && posTemp< line.length()) {
			count++;
			c=line.charAt(posTemp);
			if (!whitespace(c) && count == 1 && checkExclamation){
				if (checkExclamation && debt ==1)
					if (c != '!' && c!= '&')
						suppressComment.setValue(true);
				checkExclamation = false;
			}
			hadBrack=true;
			if (openingSquareBracket(c)) {
				storeCharToComment(c, debt, comment, suppressComment);
				debt++;
			}
			else if (closingSquareBracket(c)) {
				if (debt>0){
					debt--;
					storeCharToComment(c, debt, comment, suppressComment);
					if (debt ==0)
						suppressComment.setValue(false);
				}
				c=0;
			}
			else storeCharToComment(c, debt, comment, suppressComment);
			posTemp++;
		}
		if (posTemp>=line.length()) {
			storeCharToComment(StringUtil.lineEnding(), debt, comment, suppressComment);
		}
		pendingBrackets.setValue(debt);
		pos.setValue(posTemp);
		if (posTemp>line.length())
			c=0;
		else if (hadBrack) {
			return getNextChar(pendingBrackets, comment, suppressComment);
		}
		return c;
	}
	
	/*.................................................................................................................*/
	public char charOnDeck(int ahead) {
		int posTemp = pos.getValue() +ahead;
		if (posTemp>=line.length()) {
			return 0;
		}
		return line.charAt(posTemp);
	}
	/*.................................................................................................................*/
	public char getNextChar() {
		int posTemp = pos.getValue();
		if (posTemp>=line.length()) {
			return 0;
		}
		char c = line.charAt(posTemp);
		posTemp++;
		int debt = 0;
		boolean hadBrack = false;
		if (openingSquareBracket(c))
			debt++;
		while  (debt>0 && posTemp< line.length()) {
			c=line.charAt(posTemp);
			hadBrack = true;
			if (openingSquareBracket(c)) {
				debt++;
			}
			else if (closingSquareBracket(c)) {
				if (debt>0)
					debt--;
				c=0;
			}
			posTemp++;
		}
		pos.setValue(posTemp);
		if (posTemp>line.length())
			c=0;
		else if (hadBrack) {
			return getNextChar();
		}
		return c;
	}
	/*.................................................................................................................*/
	public String getTokenNumber(String s, int tokenNumber) {
			setString(s);
			return getTokenNumber(tokenNumber);
	}
	/*.................................................................................................................*/
	public String getTokenNumber(int tokenNumber) {
			pos.setValue(0);
			String token=null;
			for (int i=1; i<= tokenNumber; i++)
				token = getNextToken();
			return token;
	}
	/*.................................................................................................................*/
	public String getTokens(String s, int first, int last) {
			setString(s);
			return getTokens(first, last);
	}
	/*.................................................................................................................*/
	public String getTokens(int first, int last) {
			pos.setValue(0);
			String token=null;
			for (int i=1; i<= first; i++)
				token = getNextToken();
			String tokens = null;
			if (token !=null) {
				tokens = StringUtil.tokenize(token);
				for (int i=first+1; i<= last; i++)
					tokens += StringUtil.tokenize(getNextToken());
			}
			return tokens;
	}
	/*.................................................................................................................*/
	public int getNumberOfTokens(String s) {
		setString(s);
		return getNumberOfTokens();
	}
	/*.................................................................................................................*/
	public int getNumberOfTokens() {
			pos.setValue(0);
			return getNumberOfTokensRemaining();
	}
	/*.................................................................................................................*/
	public int getNumberOfTokensRemaining() {
			String token=null;
			int count=0;
			do {
				token = getNextToken();
				if (token!=null)
					count++;
			} while (token!=null);
			return count;
	}
	/*.................................................................................................................*/
	void skipComment() {
		try {
			int index = pos.getValue();
			char c;
			while ((c = line.charAt(index)) != ']') {
				if (c=='[') {
					pos.setValue(index);
					skipComment();
					index=pos.getValue()-1;
				} 
				index++;
			}
			index++;
			pos.setValue(index);
		}
		catch (StringIndexOutOfBoundsException e) {}
	}
	/*.................................................................................................................*/
	void skipToDarkspace() {
		try {
			int index = pos.getValue();
			while (whitespace(line.charAt(index), whitespaceString))
				index++;
			pos.setValue(index);
		}
		catch (StringIndexOutOfBoundsException e) {
			pos.setValue(line.length());
		}
	}
	/*.................................................................................................................*/
	public char nextDarkChar() {
		try {
			int index = pos.getValue();
			while (whitespace(line.charAt(index), whitespaceString))
				index++;
			char dark =line.charAt(index);
			pos.setValue(++index);
			if (whitespace(dark, whitespaceString))
				return '\0';
			else
				return dark;
		}
		catch (StringIndexOutOfBoundsException e) {
			pos.setValue(line.length());
		}
		return '\0';
	}
	/*.................................................................................................................*/
	public char firstDarkChar() {
		try {
			int index = 0;
			while (whitespace(line.charAt(index), whitespaceString))
				index++;
			return line.charAt(index);
		}
		catch (StringIndexOutOfBoundsException e) {}
		return '\0';
	}
	/*.................................................................................................................*/
	/*.................................................................................................................*/
	public boolean whitespace(char c) {
		return whitespace(c, whitespaceString);
	}
	/*.................................................................................................................*/
	public boolean punctuation(char c) {
		return punctuation(c, punctuationString);
	}
	/*.................................................................................................................*/
	/** Returns the next command in the string passed starting at the given character */
	public String getNextCommand(MesquiteInteger p) {
		if (p!=null)
			pos.setValue(p.getValue());
		String token = getNextCommand();
		if (p!=null)
			p.setValue(pos.getValue());
		return token;
	}
	/*.................................................................................................................*/
	/** Returns the next command in the string passed starting at the given character */
	public String getNextCommand() {
		if (commandBuffer==null)
			commandBuffer = new StringBuffer(300);
		commandBuffer.setLength(0);
		if (line == null || pos.getValue()>=line.length())
			return null;
		String token = getUnalteredToken(false);
		if (!blankByCurrentWhitespace(token)) {
			commandBuffer.append(token);
			while (!blankByCurrentWhitespace(token) && !token.equals(";")) {
				token = getUnalteredToken( false);
				if (token != null && token.length() >0) {
					if (token.charAt(0)!=';') 
						commandBuffer.append(' ');
					commandBuffer.append(token);
				}
			}
		}
		return commandBuffer.toString();
	}
	/*.................................................................................................................*/
	public boolean blankByCurrentWhitespace(String line) {
		if (line==null)
			return true;
		else if (line.length()==0)
			return true;
		else {
			for (int i=0; i<line.length(); i++)
				if (!whitespace(line.charAt(i)))
					return false;
			return true;
		}
	}
	/*.................................................................................................................*/
	public boolean blankByCurrentWhitespace(StringBuffer line) {
		if (line==null)
			return true;
		else if (line.length()==0)
			return true;
		else {
			for (int i=0; i<line.length(); i++)
				if (!whitespace(line.charAt(i)))
					return false;
			return true;
		}
	}
	/*.................................................................................................................*/
	protected static boolean openingSquareBracket(char c) {
		return (c=='[');
	}
	/*.................................................................................................................*/
	protected static boolean closingSquareBracket(char c) {
		return (c==']');
	}
}
