/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib;

import java.awt.*;


/* ======================================================================== */
/** A set of static methods for Strings, independent of NEXUS parsing rules*/
public class StringUtil {
	static String lineSeparator;
	static {
		/*
		if (MesquiteTrunk.isMacOSXJaguar())
			lineSeparator = "\r";
		else */
			lineSeparator = System.getProperty("line.separator"); //"\n"
	}
	
	public static FontMetrics defaultFontMetrics = null; 
	public static final String defaultWhitespace = " \t\n\r\f\b" + (char)0;
	public static final String defaultPunctuation = "(){}:,;-+<>=\\*/\''\"[]"; //Note: & isn't punctuation by NEXUS format //[] added Nov 03
	public static final char defaultQuote = '\''; 
	public static final String digits = "0123456789";
	public static final char argumentMarker = '@'; 
	
	/*.................................................................................................................*/
	public static String lineEnding(){
		return lineSeparator;
	}
	public static void highlightString(Graphics g, String s, int x, int y, Color fore, Color back){
		if (s==null)
			return;
		Color c = g.getColor();
		g.setColor(back);
		//g.drawString(s, x,y+1);
		//g.drawString(s, x,y-1);
		//g.drawString(s, x+1,y);
		//g.drawString(s, x-1,y);
		g.drawString(s, x+1,y+1);
		g.drawString(s, x-1,y+1);
		g.drawString(s, x+1,y-1);
		g.drawString(s, x-1,y-1);
		g.setColor(fore);
		g.drawString(s, x,y);
		if (c!=null) g.setColor(c);
	}

	/*.................................................................................................................*/
	/** replaces all instance of the old character by the nnew character*/
	public static String delete(String line, char old) {
		if (line==null)
			return null;
		else if (line.equals(""))
			return "";
		else if (line.indexOf(old)<0)
			return line;
		else {
			StringBuffer sb = new StringBuffer("");
			int len = line.length();
			for (int i=0; i<len; i++)
				if (line.charAt(i)!=old)
					sb.append(line.charAt(i));
			return sb.toString();
		}
	}
	/*.................................................................................................................*/
	/** replaces all instance of the old character by the nnew character*/
	public static String replace(String line, char old, char nnew) {
		if (line==null)
			return null;
		else if (line.equals(""))
			return "";
		else if (line.indexOf(old)<0)
			return line;
		else {
			StringBuffer sb = new StringBuffer(line);
			int len = sb.length();
			for (int i=0; i<len; i++)
				if (sb.charAt(i)==old)
					sb.setCharAt(i, nnew);
			return sb.toString();
		}
	}
/*.................................................................................................................*/
	public static int characterCount(String s,char c){
		if (s==null || s.length() == 0)
			return 0;
		int count = 0;
		for (int i=0; i<s.length(); i++)
			if (c == s.charAt(i))
				count++;
		return count;
	}
/*.................................................................................................................*/
	public static String replace(String s,String from,String to)
	{
		String newString = s;
		int indexCounter = 0;
		int pos = s.indexOf(from);
		while (pos>=indexCounter) {
			newString = newString.substring(0, pos)+to+newString.substring(pos+from.length(), newString.length());
			indexCounter = pos +to.length();
			pos = newString.indexOf(from);
		}
		return newString;
	}
/*.................................................................................................................*/
	/** returns all text between startBoundary through endBoundary*/
	public static String getFirstSubString(String line,String startText, String endText)
	{
		if (line==null)
			return null;
		else if (line.equals(""))
			return "";
		else if (startText ==null ||  endText ==null || line.indexOf(startText)<0 || line.indexOf(endText)<0)
			return "";
		else {
			int placeStart = line.indexOf(startText)+startText.length();
			int placeEnd = line.indexOf(endText,placeStart);  // find ending portion
			if (placeEnd<0)
				return line;
			StringBuffer sb = new StringBuffer(placeEnd-placeStart+1);
			for (int i=placeStart; i<placeEnd; i++)
				sb.append(line.charAt(i));
			return sb.toString();
		}
	}
/*.................................................................................................................*/
	/** replaces first instance of the all text in startOld through endOld inclusive by the new substring*/
	public static String replaceFirst(String line,String startOld, String endOld,String nnew)
	{
		if (line==null)
			return null;
		else if (line.equals(""))
			return "";
		else if (startOld ==null ||  endOld ==null || line.indexOf(startOld)<0 || line.indexOf(endOld)<0)
			return line;
		else {
			int place = line.indexOf(startOld);
			int placeEnd = line.indexOf(endOld,place+startOld.length());  // find ending portion
			if (placeEnd<0)
				return line;
			int len = line.length();
			StringBuffer sb = new StringBuffer(line.length());
			for (int i=0; i<place; i++)
				sb.append(line.charAt(i));
			if (nnew!=null)
				for (int i=0; i<nnew.length(); i++)
					sb.append(nnew.charAt(i));
			for (int i=placeEnd+endOld.length(); i<len; i++)
				sb.append(line.charAt(i));
			return sb.toString();
		}
	}
	/*.................................................................................................................*/
	/** replaces first instance of the old substring by the nnew substring*/
	public static String replaceFirst(String line, String old, String nnew) {
		if (line==null)
			return null;
		else if (line.equals(""))
			return "";
		else if (old ==null || line.indexOf(old)<0)
			return line;
		else {
			int place = line.indexOf(old);
			int len = line.length();
			StringBuffer sb = new StringBuffer(line.length());
			for (int i=0; i<place; i++)
				sb.append(line.charAt(i));
			if (nnew!=null)
				for (int i=0; i<nnew.length(); i++)
					sb.append(nnew.charAt(i));
			for (int i=place+old.length(); i<len; i++)
				sb.append(line.charAt(i));
			return sb.toString();
		}
	}
	/*.................................................................................................................*/
	/** returns the last item in a string, separated into parts by the separator*/
	public static String getLastItem(String line, String separator) {
		if (line==null)
			return null;
		else if (line.equals(""))
			return "";
		return line.substring(line.lastIndexOf(separator)+1, line.length());
	}
	/*.................................................................................................................*/
	/** returns the last item in a string, separated into parts by whichever is last of the two separators*/
	public static String getLastItem(String line, String separator1, String separator2) {
		if (line==null)
			return null;
		else if (line.equals(""))
			return "";
		int last = MesquiteInteger.maximum(line.lastIndexOf(separator1), line.lastIndexOf(separator2));
		return line.substring(last+1, line.length());
	}
	/*.................................................................................................................*/
	/** returns everything in front of the last item in a string, separated into parts by the separator*/
	public static String getAllButLastItem(String line, String separator) {
		if (line==null)
			return null;
		else if (line.equals(""))
			return "";
		else if (line.lastIndexOf(separator) <0)
			return line;
		return line.substring(0, line.lastIndexOf(separator));
	}
	/*.................................................................................................................*/
	/** returns everything in front of the last item in a string, separated into parts by whichever is last of the two separators*/
	public static String getAllButLastItem(String line, String separator1, String separator2) {
		if (line==null)
			return null;
		else if (line.equals(""))
			return "";
		int last = MesquiteInteger.maximum(line.lastIndexOf(separator1), line.lastIndexOf(separator2));
		if (last <0 )
			return line;
		return line.substring(0, last);
	}
	/*.................................................................................................................*/
	public static byte[] pascalByteArrayFromString(String s){
		byte[] nameBytes = s.getBytes();
		byte[] pstrName = new byte[nameBytes.length + 1];
		System.arraycopy(nameBytes, 0, pstrName, 1, nameBytes.length);
		pstrName[0] = (byte)nameBytes.length;
		return pstrName;
	}
	/*.................................................................................................................*/
	public static String stringFromPascalByteArray(byte[] nameBytes){
		byte[] pstrName = new byte[nameBytes.length - 1];
		System.arraycopy(nameBytes, 1, pstrName, 0, nameBytes.length-1);
		String s = new String(pstrName);
		return s;
	}
	/*...............................................................................................................*/
	private static FontMetrics getFontMetrics(Component c){ //built because of crashes in Linux & Windows
		if (c==null)
			return null;
		try {
			FontMetrics fm = c.getFontMetrics(c.getFont());
			return fm;
		}
		catch (Exception e){
		}
		return null;
	}
	private static FontMetrics getFontMetrics(Graphics g){ //built because of crashes in Linux & Windows
		if (g==null)
			return null;
		try {
			FontMetrics fm = g.getFontMetrics();
			return fm;
		}
		catch (Exception e){
		}
		return null;
	}
	/*...............................................................................................................*/
	
	public static int getStringVertPosition(Graphics g, int top, int height) {
		FontMetrics fm = getFontMetrics(g);
		return getStringVertPosition(fm, top, height);
	}
	public static int getStringVertPosition(FontMetrics fm, int top, int height) {
		if (fm!=null) {
			return top+height-(height-fm.getAscent()-fm.getDescent())/2-fm.getDescent();
		}
		else
			return top+height-MesquiteString.riseOffset;
	}
	/*...............................................................................................................*/
	public static int getStringCenterPosition(String s, Graphics g, int left, int width){
		FontMetrics fm = getFontMetrics(g);
		return getStringCenterPosition(s, fm, left, width);
	}
	/*...............................................................................................................*/
	public static int getStringCenterPosition(String s, FontMetrics fm, int left, int width){
		if (fm!=null) {
			return left+(width-fm.stringWidth(s))/2;
		}
		else
			return left+2;
	}
	/*...............................................................................................................*/
	public static int getStringDrawLength(Graphics g, String s){
		if (g!=null) {
			FontMetrics fm = getFontMetrics(g);
			if (fm!=null) 
				return fm.stringWidth(s);
		}
		if (defaultFontMetrics!=null) 
				return defaultFontMetrics.stringWidth(s); //added Dec 2001
		return 0;
	}
	/*...............................................................................................................*/
	public static int getStringDrawLength(Component component, String s){
		if (s == null)
			return 0;
		if (component!=null) {
			FontMetrics fm = getFontMetrics(component);
			if (fm== null)
				fm = defaultFontMetrics; //added Dec 2001
			if (fm!=null) {
				int sum = 0;
				for (int i=0; i<s.length(); i++)
					sum += fm.charWidth(s.charAt(i));
				return sum; //this is done instead of stringWidth because of apparent bug when stringWidth > 32767, possibly only on Mac
			}
		}
		return 0;
	}
	/*...............................................................................................................*/
	public static int getTextLineHeight(Graphics g){
		if (g!=null) {
			FontMetrics fm = getFontMetrics(g);
			if (fm!=null) 
				return fm.getHeight();
		}
		return 0;
	}
	/*...............................................................................................................*/
	public static int getMaxCharWidth(Component component){
		if (component!=null) {
			FontMetrics fm = getFontMetrics(component);
			if (fm!=null) 
				return fm.getMaxAdvance();
		}
		return 0;
	}
	/*...............................................................................................................*/
	public static int getTextLineHeight(Component component){
		if (component!=null) {
			FontMetrics fm = getFontMetrics(component);
			if (fm!=null) 
				return fm.getHeight();
		}
		return 0;
	}
	/*.................................................................................................................*/
	public static boolean titled(String line) {
		if (blank(line))
			return false;
		else if (line.equalsIgnoreCase("untitled"))
			return false;
		else
			return true;
	}
	public static String deTokenize(String token) {
		if (token == null)
			return null;
		if (token.indexOf('_')>=0) {
			StringBuffer sb = new StringBuffer(token);
			for (int i=0; i<token.length(); i++)
				if (sb.charAt(i) == '_')
					sb.setCharAt(i, ' ');
			return sb.toString();
		}
		else
			return token;
	}
	/*.................................................................................................................*/
	//TODO: quote should not be punctuation, and should be treated separately!!!!
	private static String quote(String token) {
		if (token == null)
			return "";
		if (token.indexOf(defaultQuote)>=0) {
			StringBuffer sb = new StringBuffer(token);
			int extra = 0;
			for (int i=0; i<token.length()+extra; i++) {
				if (sb.charAt(i) == defaultQuote) {
					sb.insert(i, defaultQuote);
					extra ++;
					i++;
				}
				else if (sb.charAt(i) == '\n') {
					sb.setCharAt(i, '^');
					sb.insert(i+1, 'n');
					extra ++;
					i++;
				}
			}
			return "'" + sb.toString() + defaultQuote;
		}
		else if (token.indexOf('\n')>=0) {
			StringBuffer sb = new StringBuffer(token);
			int extra = 0;
			for (int i=0; i<token.length() + extra; i++)
				if (sb.charAt(i) == '\n') {
					sb.setCharAt(i, '^');
					sb.insert(i+1, 'n');
					i++;
					extra ++;
				}
			return "'" + sb.toString() + defaultQuote;
		}
		else 
			return  "'" + token + defaultQuote;
	}
	/*.................................................................................................................*/
	public static String tokenize(String token, String punctuationString) {
		if (token == null)
			return "";
		if (hasPunctuation(token,punctuationString) || hasSpecial(token))
			return quote(token);  
		StringBuffer sb = new StringBuffer(token);
		int extra = 0;
		for (int i=0; i<token.length()+extra; i++)
			if (sb.charAt(i) == ' ')
				sb.setCharAt(i, '_');
			else if (sb.charAt(i) == '\n'){
				sb.setCharAt(i, '^');
				sb.insert(i+1, 'n');
				extra ++;
				i++;
			}
		return sb.toString();
	}
	/*.................................................................................................................*/
	public static int getNumMatchingChars(String token, char c) {
		int numMatches = 0;
		for (int i=0; i<token.length(); i++) {
			if (token.charAt(i)==c) 
				numMatches++;
		}
		return numMatches;
	}
	/*.................................................................................................................*/
	public static String removeLastCharacterIfMatch(String token, char c) {
		String timmedString = token.trim();
		if (token == null)
			return "";
		String trimmedString = token.trim();
		int charPos = trimmedString.lastIndexOf((int)c) ;  
		if (charPos == trimmedString.length()-1)   // then the last none-whitespace is the character c
			return token.substring(0,token.lastIndexOf((int)c)-1);
		else
			return token;
	}
	/*.................................................................................................................*/
	public static String removeCharacters(String token, String characters) {
		if (token == null)
			return "";
		if (characters == null || characters.length()<1)
			return token;
		StringBuffer sb = new StringBuffer();
		for (int i=0; i<token.length(); i++) {
			char c = token.charAt(i);
			if (characters.indexOf(c)<0)
				sb.append(c);
		}
		return sb.toString();
	}
	/*.................................................................................................................*/
	public static String stripLeadingWhitespace(String token) { //added 22 Dec 01
		if (token == null)
			return "";
		int firstDark = -1;
		for (int i=0; i<token.length(); i++) {
			char c = token.charAt(i);
			if (defaultWhitespace.indexOf(c)<0){
				firstDark = i;
				break;
			}
		}
		if (firstDark == -1)
			return "";
		else
			return token.substring(firstDark, token.length());
	}
	/*.................................................................................................................*/
	public static String stripTrailingWhitespace(String token) {
		if (token == null)
			return "";
		int firstDark = -1;
		for (int i=token.length()-1;  i>=0; i--) {
			char c = token.charAt(i);
			if (defaultWhitespace.indexOf(c)<0){
				firstDark = i;
				break;
			}
		}
		if (firstDark == -1)
			return "";
		else
			return token.substring(0, firstDark+1);
	}
	/*.................................................................................................................*/
	/** Removes all new line characters from the string. */
	public static String removeNewLines(String line) {
		if (line==null)
			return null;
		else if (line.equals(""))
			return "";
		else if (line.indexOf("\r")<0 && line.indexOf("\n")<0)
			return line;
		else {   // there's at least one \r or \n
			line = removeLastCharacterIfMatch(line,'\n');  // remove trailing \n
			line = removeLastCharacterIfMatch(line,'\r');    // remove trailing \r
			if (line.indexOf("\r")<0 && line.indexOf("\n")<0)  // check to see if there are any left
				return line;
			StringBuffer sb = new StringBuffer(line);
			int len = sb.length();
			for (int i=0; i<len; i++)
				if (sb.charAt(i)=='\n' || sb.charAt(i)=='\r')
					sb.setCharAt(i, ' ');
			line = sb.toString().trim();
			return line;
		}
	}
	/*.................................................................................................................*/
	public static String tokenize(String token) {
		return tokenize(token, null);
	}

	/*.................................................................................................................*/
	public static boolean hasPunctuation(String token, String punctuationString) {
		if (token == null)
			return false;
		for (int i=0; i<token.length(); i++)
			if (punctuation (token.charAt(i), punctuationString))
				return true;
		return false;
	}
	/*.................................................................................................................*/
	public static boolean hasPunctuation(String token) {
		return hasPunctuation(token, null);
	}
	/*.................................................................................................................*/
	public static boolean hasSpecial(String token) {
		if (token == null)
			return false;
		for (int i=0; i<token.length(); i++) {
			char c = token.charAt(i);
			if ('^'==c || '\n'==c || '\t'==c || '\r'==c)
				return true;
		}
		return false;
	}
	/*.................................................................................................................*/
	protected static boolean openingSquareBracket(char c) {
		return (c=='[');
	}
	/*.................................................................................................................*/
	protected static boolean closingSquareBracket(char c) {
		return (c==']');
	}
	/*.................................................................................................................*
	public static int numTokens(String s, boolean useNEXUSRules) {
		if (blank(s))
			return 0;
		int num=0;
		int pos=0;
		if (useNEXUSRules) {
			MesquiteMessage.warnProgrammer("numTokens doesn't support NEXUS rules yet!!!");
		}
		else 
			try {
				while (pos<s.length()) {
					if (!whitespace(s.charAt(pos),defaultWhitespace)) {// we are at start of a new token
						num++;
						pos++;
						while (!whitespace(s.charAt(pos),defaultWhitespace))  // go through until we hit whitespace again
							pos++;
					}
					pos++;
				}
			}
			catch (Exception e) {
			}
		return num;
	}
	/*.................................................................................................................*/
	public static boolean whitespace(char c, String whitespaceString) {
	// in the future, we might want to add  if (c<0) return true;
		if (c<0)
			return true;
		if (c==0)
			return false;  
		if (whitespaceString!=null)
			return whitespaceString.indexOf(c)>=0;
		return defaultWhitespace.indexOf(c)>=0;
	}
	/*.................................................................................................................*/
 	public static boolean punctuation(char c, String punctuationString) {
		if (c==0)
			return false;
		else if (punctuationString!=null)
			return punctuationString.indexOf(c)>=0;
		else  //NOTE:  * is assumed not to be punctuation!!!!!!!!!!!!!!!  at least, if this changes, search for it (ManageMesquiteBlock)
			return defaultPunctuation.indexOf(c)>=0;
	}
	/*.................................................................................................................*/
 	public static boolean punctuationOrQuote(char c, String punctuationString, char quotechar) {
		if (c==0)
			return false;
		else if (c==quotechar)
			return true;
		else if (punctuationString!=null)
			return punctuationString.indexOf(c)>=0;
		else  //NOTE:  * is assumed not to be punctuation!!!!!!!!!!!!!!!  at least, if this changes, search for it (ManageMesquiteBlock)
			return defaultPunctuation.indexOf(c)>=0;
	}
	/*.................................................................................................................*/
 	public static boolean lineEndCharacter(char c) {
		return (c=='\r' || c=='\n');
	}
 	/*.................................................................................................................*/
	/*.................................................................................................................*/
	public static String blankIfNull(String line) {
		if (line==null)
			return "";
		else 
			return line;
	}
	/*.................................................................................................................*/
	public static boolean blank(String line) {
		if (line==null)
			return true;
		else if (line.length()==0)
			return true;
		else {
			for (int i=0; i<line.length(); i++) {
				char c = line.charAt(i);
				if (c > 0 && !whitespace(c, null))
					return false;
			}
			return true;
		}
	}
	/*.................................................................................................................*/
	public static boolean blankDebugg(String line) {
		if (line==null)
			return true;
		else if (line.length()==0)
			return true;
		else {
			for (int i=0; i<line.length(); i++) {
				char c = line.charAt(i);
				if (c > 0 && !whitespace(c, null) )
					return false;
			}
			return true;
		}
	}
	/*.................................................................................................................*/
	public static boolean blank(String line, String temporaryWhiteSpace) {
		if (temporaryWhiteSpace == null)
			return blank(line);
		if (line==null)
			return true;
		else if (line.length()==0)
			return true;
		else {
			for (int i=0; i<line.length(); i++) {
				if ((!whitespace(line.charAt(i), null)) && (!(temporaryWhiteSpace.indexOf(line.charAt(i))>=0))) {
					return false;
				}
			}
			return true;
		}
	}
	/*.................................................................................................................*/
	public static boolean blank(StringBuffer line) {
		if (line==null)
			return true;
		else if (line.length()==0)
			return true;
		else {
			for (int i=0; i<line.length(); i++) {
				char c = line.charAt(i);
				if (c >0 && !whitespace(c, null))
					return false;
			}
			return true;
		}
	}
	/*.................................................................................................................*/
	public static boolean blank(FileBlock line) {
		if (line==null)
			return true;
		else if (line.getNumCommands()==0)
			return true;
		else {
			return false;
		}
	}

	/*.................................................................................................................*/
	public static void append(StringBuffer recipient, StringBuffer donor) {
		if (recipient==null || donor == null || donor.length() == 0)
			return;
		for (int i=0; i<donor.length(); i++)
			recipient.append(donor.charAt(i));
	}

}

