/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib;

import java.awt.*;
import mesquite.lib.duties.*;

/*  commands  */
/* includes commands,  buttons, miniscrolls
/* ======================================================================== */
	/** This subclass of MesquiteTool is used in data windows for manipulating data.
	*/
public class TableTool extends MesquiteTool {
	MesquiteCommand touchedCommand;
	MesquiteCommand droppedCommand;
	MesquiteCommand dragCommand;
	boolean useTableTouch;
	boolean worksOnRowNames = false;
	boolean worksOnColumnNames = false;
	boolean worksBeyondLastRow = false;
	boolean worksBeyondLastColumn = false;
	boolean worksOnCornerPanel = false;
	boolean worksOnMatrixPanel = true;
	
	public TableTool (Object initiator, String name, String imageDirectoryPath, String imageFileName, int hotX, int hotY, String fullDescription, String explanation, MesquiteCommand touchedCommand, MesquiteCommand dragCommand, MesquiteCommand droppedCommand) {
		super(initiator, name, imageDirectoryPath, imageFileName, hotX, hotY, fullDescription, explanation);
		this.touchedCommand = touchedCommand;
		this.dragCommand = dragCommand;
		this.droppedCommand = droppedCommand;
		useTableTouch = touchedCommand == null;
	}
	
	
	public void dispose(){ 
		if (touchedCommand!=null) 
			touchedCommand.dispose();
		else if (droppedCommand!=null) 
			droppedCommand.dispose();
		else if (dragCommand!=null) 
			dragCommand.dispose();
		touchedCommand=null;
		droppedCommand=null;
		dragCommand=null;
		super.dispose();
	}
	public void setWorksOnCornerPanel(boolean u){
		worksOnCornerPanel = u;
	}
	public boolean getWorksOnCornerPanel(){
		return worksOnCornerPanel;
	}
	public void setWorksOnMatrixPanel(boolean u){
		worksOnMatrixPanel = u;
	}
	public boolean getWorksOnMatrixPanel(){
		return worksOnMatrixPanel;
	}
	public void setWorksBeyondLastRow(boolean u){
		worksBeyondLastRow = u;
	}
	public boolean getWorksBeyondLastRow(){
		return worksBeyondLastRow;
	}
	public void setWorksBeyondLastColumn(boolean u){
		worksBeyondLastColumn = u;
	}
	public boolean getWorksBeyondLastColumn(){
		return worksBeyondLastColumn;
	}
	public void setWorksOnColumnNames(boolean u){
		worksOnColumnNames = u;
	}
	public boolean getWorksOnColumnNames(){
		return worksOnColumnNames;
	}
	public void setWorksOnRowNames(boolean u){
		worksOnRowNames = u;
	}
	public boolean getWorksOnRowNames(){
		return worksOnRowNames;
	}
	public void setUseTableTouchRules(boolean u){
		useTableTouch = u;
	}
	public boolean useTableTouchRules(){
		return useTableTouch;
	}
	public void cellTouched (int column, int row, int modifiers) {
		if (touchedCommand!=null)
			touchedCommand.doItMainThread(Integer.toString(column) + " " + row + " " + MesquiteEvent.modifiersToString(modifiers), CommandChecker.getQueryModeString("Tool", touchedCommand, this), false, false);  
	}
	public void cellDrag (int column, int row, int modifiers) {
		if (dragCommand!=null)
			dragCommand.doItMainThread(Integer.toString(column) + " " + row + " " +  MesquiteEvent.modifiersToString(modifiers), CommandChecker.getQueryModeString("Tool", dragCommand, this), false, false);  
	}
	public void cellDropped (int column, int row, int modifiers) {
		if (droppedCommand!=null)
			droppedCommand.doItMainThread(Integer.toString(column) + " " + row + " " + MesquiteEvent.modifiersToString(modifiers), CommandChecker.getQueryModeString("Tool", droppedCommand, this), false, false);  
	}
}
