/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib;

import java.awt.*;
import java.math.*;


/*  taxa  */
/* ======================================================================== */
/** A taxon, including the taxon name, its number, an other information (e.g., notes and pictures attached to it).*/
public class Taxon  implements Listable, Illustratable, Identifiable {
	private String name;
	public int number;
	private Image illustration;
	private String illustrationPath;
	public static long totalCreated = 0;
	private long id;
	private String link = null;
	private Taxa taxa = null;
	//need footnotes etc.
	
	public Taxon(Taxa taxa){
		this.taxa = taxa;
		totalCreated++;
		id = totalCreated;
	}
	public Taxa getTaxa() {
		return taxa;
	}
	public long getID() {
		return id;
	}
	public void setName(String name) {
		this.name = name;
	}
	public String getName() {
		return name;
	}
	public void setLink(String link) {
		this.link = link;
	}
	public String getLink() {
		return link;
	}
	/**Translates internal numbering system to external (currently, 0 based to 1 based*/
	public static int toExternal(int i){
		if (!MesquiteInteger.isCombinable(i))
			return i;
		else
			return i+1;
	}
	/**Translates external numbering system to internal (currently, 1 based to 0 based*/
	public static int toInternal(int i){
		if (!MesquiteInteger.isCombinable(i))
			return i;
		else
		return i-1;
	}
	public void setIllustration(Image illustration, String path) {
		this.illustration = illustration;
		this.illustrationPath = path;
	}
	public String getIllustrationPath() {
		return illustrationPath;
	}
	public Image getIllustration() {
		return illustration;
	}
}

