/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib;

import java.awt.*;
import mesquite.lib.duties.*;


/* ======================================================================== */
/** Legends for TreeDisplays, e.g. the TraceCharacter Legend. */
public abstract class TreeDisplayLegend extends ResizableLegend {
	boolean fixToRoot;
	private TreeDisplay treeDisplay;
	private int oldOrientation;
	
	public TreeDisplayLegend (TreeDisplay treeDisplay, int defaultWidth, int defaultHeight, boolean fixToRoot) {
		super(defaultWidth, defaultHeight);
		this.treeDisplay = treeDisplay;
		this.fixToRoot = fixToRoot;
		defaultOffsets();
	}
	/*.................................................................................................................*/
	public void defaultOffsets() {
		if (treeDisplay==null)
			return;
		int buffer = 20;
		if (treeDisplay.getOrientation() == TreeDisplay.DOWN) {
			setOffsetX(buffer);
			setOffsetY(buffer);
		}
		else if (treeDisplay.getOrientation() == TreeDisplay.RIGHT) {
			setOffsetX(buffer);
			setOffsetY(-(defaultHeight+buffer));
		}
		else {
			setOffsetX(-(defaultWidth+buffer));
			setOffsetY(-(defaultHeight+buffer));
		}
		oldOrientation = treeDisplay.getOrientation();
	}
	public void adjustLocation() {
		if (treeDisplay==null)
			return;
		if (oldOrientation != treeDisplay.getOrientation())
			defaultOffsets();
		int legendX, legendY;
		if (fixToRoot) {
			int rootX =treeDisplay.getTreeDrawing().x[treeDisplay.getTree().getRoot()];
			int rootY =treeDisplay.getTreeDrawing().y[treeDisplay.getTree().getRoot()];
			 legendX = rootX+getOffsetX();
			 legendY = rootY+getOffsetY();
			if (legendX + legendWidth>treeDisplay.getBounds().width) {
				setOffsetX(treeDisplay.getBounds().width-rootX-legendWidth);
				legendX = rootX+getOffsetX();
			}
			if (legendX<0) {
				setOffsetX(-rootX+4);
				legendX = rootX+getOffsetX();
			}
				
			if (legendY + legendHeight>treeDisplay.getBounds().height) {
				setOffsetY(treeDisplay.getBounds().height-rootY-legendHeight);
				legendY = rootY+getOffsetY();
			}
			if (legendY<0) {
				setOffsetY(-rootY+4);
				legendY = rootY+getOffsetY();
			}
		}
		else {
			int baseX;
			int baseY;
			if (treeDisplay.getOrientation() == TreeDisplay.UP) {
				baseX = treeDisplay.getBounds().width;
				baseY = treeDisplay.getBounds().height;
			}
			else if (treeDisplay.getOrientation() == TreeDisplay.DOWN) {
				baseX = 0;
				baseY = 0;
			}
			else if (treeDisplay.getOrientation() == TreeDisplay.RIGHT) {
				baseX = 0;
				baseY = treeDisplay.getBounds().height;
			}
			else if (treeDisplay.getOrientation() == TreeDisplay.LEFT) {
				baseX = treeDisplay.getBounds().width;
				baseY = treeDisplay.getBounds().height;
			}
			else {
				baseX = treeDisplay.getBounds().width;
				baseY = treeDisplay.getBounds().height;
			}
			legendX = baseX+getOffsetX();
			legendY = baseY+getOffsetY();
			
			if (legendX + legendWidth>treeDisplay.getBounds().width) {
				setOffsetX(treeDisplay.getBounds().width-baseX-legendWidth);
				legendX = baseX+getOffsetX();
			}
			if (legendX<0) {
				setOffsetX(-baseX+4);
				legendX = baseX+getOffsetX();
			}
				
			if (legendY + legendHeight>treeDisplay.getBounds().height) {
				setOffsetY(treeDisplay.getBounds().height - baseY-legendHeight);
				legendY = baseY+getOffsetY();
			}
			if (legendY<0) {
				setOffsetY(-baseY+4);
				legendY = baseY+getOffsetY();
			}
		}
		if ((legendX!=getBounds().x) || (legendY!=getBounds().y)) {
			setLocation(legendX, legendY);
			repaint();
		}
	}
	
	/*public void mouseUp (int modifiers, int x, int y, MesquiteTool tool) {
	   	super.mouseUp(modifiers, x, y, tool);
	   	treeDisplay.pleaseUpdate(false, commandRec);
		repaint();
	}*/
}
