/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib;

import java.awt.*;
import mesquite.lib.duties.*;

/*  commands  */
/* includes commands,  buttons, miniscrolls
/* ======================================================================== */
	/** This subclass of MesquiteTool is used in Tree windows for manipulating the tree, etc.  Each tool
	represents a cursor in the tool palette.  Each has MesquiteCommands attached.<p>
	When a cursor is chosen, its corresponding MesquiteTool is set as
	the current tool.  The tool When a mousedown etc. event occurs on the tree, the tool is passed the tree and the mouse click 
	parameters -- the branch first touched, the branch dropped upon, etc.  If the tool has a touchedCommand, or droppedCommand,
	or touchedTaxonCommand, then this command is executed when the tool is informed of the event.  As argument to the command,
	the branch number(s) are passed in a string.
	*/
public class TreeTool extends MesquiteTool {
	MesquiteCommand touchedCommand;
	MesquiteCommand droppedCommand;
	MesquiteCommand transferredCommand;
	MesquiteCommand draggedCommand;
	MesquiteCommand touchedTaxonCommand;
	MesquiteCommand touchedFieldCommand;
	MesquiteCommand movedCommand;


	public TreeTool (Object initiator,  String name, String imageDirectoryPath, String imageFileName, int hotX, int hotY, String fullDescription, String explanation) {
		super(initiator, name, imageDirectoryPath, imageFileName, hotX, hotY, fullDescription, explanation);
	}

	public void dispose(){ 
		if (touchedCommand!=null) 
			touchedCommand.dispose();
		else if (droppedCommand!=null) 
			droppedCommand.dispose();
		else if (transferredCommand!=null) 
			transferredCommand.dispose();
		else if (draggedCommand!=null) 
			draggedCommand.dispose();
		else if (touchedTaxonCommand!=null) 
			touchedTaxonCommand.dispose();
		else if (touchedFieldCommand!=null) 
			touchedFieldCommand.dispose();
		else if (movedCommand != null)
			movedCommand.dispose();
		touchedCommand=null;
		droppedCommand=null;
		transferredCommand=null;
		draggedCommand=null;
		touchedTaxonCommand=null;
		touchedFieldCommand = null;
		movedCommand = null;
		super.dispose();
	}
	public boolean informTransfer() {
		return (transferredCommand!=null);
	}
	public boolean informDrop() {
		return (droppedCommand!=null);
	}
	public boolean informTouch() {
		return (touchedCommand!=null);
	}
	public boolean informDrag() {
		return (draggedCommand!=null);
	}
	public boolean informMove() {
		return (movedCommand!=null);
	}
	public boolean informTouchField() {
		return (touchedFieldCommand!=null);
	}
	public void setTransferredCommand(MesquiteCommand transferredCommand){
		this.transferredCommand = transferredCommand;
	}
	public void setTouchedCommand(MesquiteCommand touchedCommand){
		this.touchedCommand = touchedCommand;
	}
	public void setDraggedCommand(MesquiteCommand draggedCommand){
		this.draggedCommand = draggedCommand;
	}
	public void setDroppedCommand(MesquiteCommand droppedCommand){
		this.droppedCommand = droppedCommand;
	}
	public void setTouchedTaxonCommand(MesquiteCommand touchedTaxonCommand){
		this.touchedTaxonCommand = touchedTaxonCommand;
	}
	public void setTouchedFieldCommand(MesquiteCommand touchedFieldCommand){
		this.touchedFieldCommand = touchedFieldCommand;
	}
	public void setMovedCommand(MesquiteCommand movedCommand){
		this.movedCommand = movedCommand;
	}

	public void branchTouched (int N, int x, int y, Tree tree, int modifiers) {
		if (touchedCommand!=null) {
			touchedCommand.doItMainThread(Integer.toString(N) + " " + Integer.toString(x)+ " " + Integer.toString(y) + " " + MesquiteEvent.modifiersToString(modifiers), CommandChecker.getQueryModeString("Tool", touchedCommand, this), false, false);  
		}
	}
	public void branchDragged (int N, int x, int y, Tree tree, int modifiers) {
		if (draggedCommand!=null)
			draggedCommand.doItMainThread(Integer.toString(N) + " " + Integer.toString(x)+ " " + Integer.toString(y) + " " + MesquiteEvent.modifiersToString(modifiers), CommandChecker.getQueryModeString("Tool", draggedCommand, this), false, false);  
	}
	public void branchTransferred (int fromN, int toN, Tree tree, int modifiers) {
		if (transferredCommand!=null)
			transferredCommand.doItMainThread(Integer.toString(fromN) + " " + Integer.toString(toN) + " " + MesquiteEvent.modifiersToString(modifiers), CommandChecker.getQueryModeString("Tool", transferredCommand, this), false, false);  
	}
	public void branchDropped (int N, int x, int y, Tree tree, int modifiers) {
		if (droppedCommand!=null)
			droppedCommand.doItMainThread(Integer.toString(N) + " " + Integer.toString(x)+ " " + Integer.toString(y) + " " + MesquiteEvent.modifiersToString(modifiers), CommandChecker.getQueryModeString("Tool", droppedCommand, this), false, false);  
	}
	public void taxonTouched (int M, Tree tree, int modifiers) {
		if (touchedTaxonCommand!=null)
			touchedTaxonCommand.doItMainThread(Integer.toString(M) + " "  + MesquiteEvent.modifiersToString(modifiers), CommandChecker.getQueryModeString("Tool", touchedTaxonCommand, this), false, false);  
	}
	public boolean fieldTouched (int x, int y, Tree tree, int modifiers) {
		if (touchedFieldCommand!=null) { 
			touchedFieldCommand.doItMainThread(Integer.toString(x)+ " " + Integer.toString(y) + " " + MesquiteEvent.modifiersToString(modifiers), CommandChecker.getQueryModeString("Tool", touchedFieldCommand, this), false, false);  
			return true;
		}
		else
			return false;
	}
	public boolean moved (int x, int y, Tree tree, int modifiers) {
		if (movedCommand!=null) {
			movedCommand.doItMainThread(Integer.toString(x)+ " " + Integer.toString(y) + " " + MesquiteEvent.modifiersToString(modifiers), CommandChecker.getQueryModeString("Tool", movedCommand, this), false, false);  
			return true;
		}
		else
			return false;
	}
}
