/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib.duties;

import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.table.*;
import mesquite.lib.characters.*;


/* ======================================================================== */
/**This is superclass of modules to search a portion a data matrix.*/

public abstract class DataSearcher extends MesquiteModule  {

   	 public Class getDutyClass() {
   	 	return DataSearcher.class;
   	 }
 	public String getDutyName() {
 		return "Data Searcher";
   	}
   	
	/*.................................................................................................................*/
   	/** Called to search data in those cells selected in table.  Returns true if search successful*/
   	public abstract boolean searchData(CharacterData data, MesquiteTable table, CommandRecord commandRec);
   	
	/*.................................................................................................................*/
   	/** True iff one cannot ask for a search involving all or part of multiple taxa*/
   	public  boolean canSearchMoreThanOnePiece() {
   		return false;
   	}
   	
	/*.................................................................................................................*/
   	/** Called to search data in a table. This is used if the searching procedure can be done on the selected region in one taxon
   	at a time, independent of all other taxa.  If the searching procedure involves dependencies between taxa,
   	then a different method must be built.  */
   	public boolean searchSelectedTaxa(CharacterData data, MesquiteTable table, CommandRecord commandRec){
		boolean did=false;
 		if (table==null && data!=null){    // alter entire matrix
 			if (canSearchMoreThanOnePiece() || data.getNumTaxa()==1)
 				return false;
			for (int j=0; j<data.getNumTaxa(); j++)
				searchOneTaxon(data,j,0,data.getNumChars(), commandRec);
			did = true;
			return true;
 		}
 		else if (table!=null && data !=null){
 		//	table.resetLastInSelectedBlock();
 			MesquiteInteger row = new MesquiteInteger();
 			MesquiteInteger firstColumn = new MesquiteInteger();
 			MesquiteInteger lastColumn = new MesquiteInteger();
 	
 			while (table.nextSingleRowBlockSelected(row, firstColumn, lastColumn)) { 
 				searchOneTaxon(data,row.getValue(), firstColumn.getValue(), lastColumn.getValue(), commandRec);
 				did = true;
 			}
 			if (!did)
				discreetAlert(commandRec, "Sorry, to use the search you need to have one or more stretches of character states (e.g. a section of sequence) selected in one or more taxa.");
		}
		return did;
   	}
	/*.................................................................................................................*/
   	/** Called to search the data selected .  If you use the searchSelectedTaxa method of this class, 
   	then you must supply a real method for this, not just this stub. */
   	public void searchOneTaxon(CharacterData data, int it, int icStart, int icEnd, CommandRecord commandRec){
   	}
	/*.................................................................................................................*/
	/** Returns CompatibilityTest so other modules know if this is compatible with some object. */
	public CompatibilityTest getCompatibilityTest(){
		return new CharacterStateTest();
	}
}




