/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib.duties;

import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.table.*;


/* ======================================================================== */
/**This is superclass of modules to alter a data matrix.*/

public abstract class TaxonNameAlterer extends MesquiteModule  {
	protected MesquiteTable table;
	protected Taxa taxa;
	
   	public Class getDutyClass() {
   	 	return TaxonNameAlterer.class;
   	 }
 	public String getDutyName() {
 		return "Taxon Name Alterer";
   	}
   	
   	
	/*.................................................................................................................*/
   	/** Called to alter the taxon names in a single cell.  If you use the alterContentOfCells method of this class, 
   	then you must supply a real method for this, not just this stub. */
   	public boolean alterName(Taxa taxa, int it, CommandRecord commandRec){
   		return false;
   	}
   	
	/*.................................................................................................................*/
   	/** A stub method for querying the user about options. If alterIndividualTaxonNames is used to 
   	alter the names, and options need to be specified for the operation, then optionsQuery should be overridden.*/
   	public boolean getOptions(CommandRecord commandRec){
   		return true;
   	}
	/*.................................................................................................................*/
   	/** Called to alter taxon names in table. This is used if the altering procedure can be done on one name
   	at a time, independent of all other name.  If the altering procedure involves dependencies between names,
   	then alterTaxonNames should be overridden with a method that uses another procedure.  */
   	public boolean alterIndividualTaxonNames(Taxa taxa, MesquiteTable table, CommandRecord commandRec){
   		this.taxa = taxa;
   		this.table = table;
		if (!getOptions(commandRec))
			return false;
			
		boolean anyChanged = false;
		boolean okDoIt = !taxa.anySelected() && !table.anythingSelected();
		for (int it=0; it<taxa.getNumTaxa(); it++){
			
			if ((okDoIt || taxa.getSelected(it) || table.isRowSelected(it) || table.isRowNameSelected(it)) && alterName(taxa, it, commandRec))
				anyChanged = true;
		}
   		return anyChanged;
   	}

	/*.................................................................................................................*/
   	/** Called to alter taxon names in those cells selected in table.  Returns true if any taxon names are altered.
   	This should be overridden if the is doing something that involves dependencies between names.*/
   	public boolean alterTaxonNames(Taxa taxa, MesquiteTable table, CommandRecord commandRec){
   		return alterIndividualTaxonNames(taxa, table, commandRec);
   	}

   	public boolean isSubstantive(){
   		return false;  
   	}
}




