/* Mesquite Class Library.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib.table;

import java.awt.*;
import java.awt.event.*;
import mesquite.lib.*;

/* ======================================================================== */
/** A panel for column headings for use in MesquiteTable*/
public class ColumnNamesPanel extends EditorPanel  {
	MesquiteTable table;
	public int width,  height;
	int touchX = -1;
	int touchColumn = -1;
	int lastX=-1;
	
	public ColumnNamesPanel (MesquiteTable table , int w, int h) {
		super(table);
		this.table=table;
		this.width=w;
		this.height=h;
		//setBackground(ColorDistribution.medium[table.colorScheme]);
		setBackground(Color.white);
		setCursor(table.getHandCursor());
		setSize(w, h);
	}
	public void setTableUnitSize (int w, int h) {
		this.width=w;
		this.height=h;
		setSize(width, h);
	}
	public void setHeight (int h) {
		this.height=h;
		setSize(width, h);
	}
	public int startOfRow(int row){
		return table.getRowGrabberWidth()-2;
	}
	public int firstRowVisible(){
		return -1;
	}
	public int numRowsVisible(){
		return 1;
	}
	public int rowHeight(int row) {
		return height -table.getColumnGrabberWidth();
	}
	public void textReturned(int column, int row, String text, CommandRecord commandRec){
		table.returnedColumnNameText(column, text, commandRec);
	}
	public String getText(int column, int row){
		return table.getColumnNameText(column);
	}
	public void deselectCell(int column,int row){
		table.deselectColumnName(column);
	}
	public void redrawCell(int column, int row){
		Graphics g = getGraphics();
		if (g!=null) {
			redrawName(g, column);
			g.dispose();
		}
	}
	public void redrawName(Graphics g, int column) {
		int left = table.getFirstColumnVisible();
		
		if (column<left) //TODO: should also fail to draw if to big
			return;  
		if (column == returningColumn){
			return; //don't draw if text about to be returned to cell, and will soon be redrawn anyway
		}
		int leftSide = startOfColumn(column);
		if (leftSide>getBounds().width || leftSide+columnWidth(column)<0)
			return;
		int topSide = startOfRow(-1);
		Shape clip = g.getClip();
		g.setClip(leftSide,0,columnWidth(column), height);
		
		prepareCell(g, leftSide+1, 1,columnWidth(column), height, table.focusColumn == column, table.isColumnNameSelected(column) || table.isColumnSelected(column), table.getCellDimmed(column, -1), table.isColumnNameEditable(column));
		
		g.setClip(0,0, getBounds().width, getBounds().height);

		if (table.frameColumnNames) {
			Color cg = g.getColor();
			g.setColor(Color.gray); 
			g.drawLine(leftSide+columnWidth(column), 0, leftSide+columnWidth(column), height);
			g.setColor(cg);
		}
		Font fnt= null;
		boolean doFocus = table.focusColumn == column && table.boldFont !=null;
		if (doFocus){
			fnt = g.getFont();
			g.setFont(table.boldFont);
		}
		if (table.showColumnGrabbers) {
			if (table.showColumnNumbers) {
				
				table.drawRowColumnNumber(g,column,false,leftSide+1,0, columnWidth(column)-2, table.getColumnGrabberWidth());
			}
			else
				table.drawRowColumnNumberBox(g,column, false,leftSide+1,0, columnWidth(column)-2, table.getColumnGrabberWidth());
			g.setClip( leftSide,0 + table.getColumnGrabberWidth(), columnWidth(column), height-table.getColumnGrabberWidth()-1);
			table.drawColumnNameCell(g, leftSide,0 + table.getColumnGrabberWidth(), columnWidth(column), height-table.getColumnGrabberWidth(), column);
				
		}
		else {
			g.setClip( leftSide,0, columnWidth(column), height-1);
			table.drawColumnNameCell(g, leftSide,0, columnWidth(column), height, column);
		}
		if (doFocus && fnt !=null){
			g.setFont(fnt);
		}
		g.setClip(0,0, getBounds().width, getBounds().height);

		g.setColor(Color.black);
		if (table.getDropDown(column, -1)) {
			int offset = 0;
			if (table.showColumnGrabbers)
				offset = table.getColumnGrabberWidth();
			
			dropDownTriangle.translate(leftSide - 8 + columnWidth(column),1 + offset);
			g.setColor(Color.white);
			g.drawPolygon(dropDownTriangle);
			g.setColor(Color.black);
			g.fillPolygon(dropDownTriangle);
			dropDownTriangle.translate(-(leftSide - 8 + columnWidth(column)),-(1 + offset));
		}
		g.setClip(clip);
		g.drawLine(0, height-1, width, height-1);
	}
	
	/*...............................................................................................................*/
	public void paint(Graphics g) {
	   	if (MesquiteWindow.checkDoomed(this))
	   		return;
		int lineX = 0;
		int oldLineX=lineX;
		int resetWidth = getBounds().width;
		int resetHeight = getBounds().height;
		width = resetWidth;//this is here to test if width/height should be reset here
		height = resetHeight;
		checkEditFieldLocation();
		Shape clip = g.getClip();
		for (int c=table.firstColumnVisible; (c<table.numColumnsTotal) && (lineX<width); c++) { // or lineX+table.columnWidths[c]
			redrawName(g, c);
		}
		g.setClip(0,0, getBounds().width, getBounds().height);
		if ((endOfLastColumn()>=0) && (endOfLastColumn()<table.matrixWidth)) {
			g.setColor(ColorDistribution.medium[table.colorScheme]);
			g.fillRect(endOfLastColumn(), 0, getBounds().width, getBounds().height);
		}
		g.setColor(Color.black);
		if (table.frameColumnNames)
			g.drawRect(0, 0, width-1, height);
		g.drawLine(0, height-1, width, height-1);
		g.setClip(clip);
		width = resetWidth;
		
		MesquiteWindow.uncheckDoomed(this);
	}
	/*...............................................................................................................*/
	public void print(Graphics g) {
		int lineX = 0;
		int oldLineX=lineX;
		Shape clip = g.getClip();
		for (int c=0; (c<table.numColumnsTotal); c++) { 
			lineX += table.columnWidths[c];
			g.setColor(Color.black);
			//g.setClip( oldLineX,0, table.columnWidths[c], height);
			table.drawColumnNameCell(g, oldLineX,0, table.columnWidths[c], height, c);
			g.setColor(Color.black);
				
			oldLineX = lineX;
		}
		g.setColor(Color.black);
		g.setClip(0,0, table.getTotalColumnWidth(), height);
		g.drawLine(0, height-1,table.getTotalColumnWidth(), height-1);
		g.setClip(clip);
	}
	/*...............................................................................................................*/
	public void enterPressed(KeyEvent e){
		if (!getEditing())
			return;
		if (table.getCellsEditable())
			table.editMatrixCell(editField.getColumn(), 0);
	}
	/*...............................................................................................................*/
	public void downArrowPressed(KeyEvent e){
		if (getEditing())
			enterPressed(e);
		//here should move selection into matrix
	}
	  
	  
	  
	/*...............................................................................................................*/
	/*...............................................................................................................*/
	/*...............................................................................................................*/
   	public void shimmerOff(int x) {
		table.shimmerVerticalOff(table.columnNames,x);
		table.shimmerVerticalOff(table.matrix,x);
   	 }
	/*...............................................................................................................*/
   	public void shimmerOn(int x) {
		table.shimmerVerticalOn(table.columnNames,x);
		table.shimmerVerticalOn(table.matrix,x);
   	 }
	/*...............................................................................................................*/
	   public void mouseDown(int modifiers, int clickCount, long when, int x, int y, MesquiteTool tool) {
		if (!(tool instanceof TableTool))
			return;

		table.adjustingColumnWidth = false;
		touchX=-1;
		touchColumn=-1;
			/*@@@*/
		int possibleTouch = table.findColumn(x);
		if (possibleTouch<table.numColumnsTotal && possibleTouch>=0) {
			if (tool != null && tool.isArrowTool() && table.getUserAdjustColumn()==MesquiteTable.RESIZE && table.nearColumnBoundary(x)  && !MesquiteEvent.shiftKeyDown(modifiers) && !MesquiteEvent.controlKeyDown(modifiers)) {
				touchX=x;
				lastX = x;
				touchColumn=possibleTouch;
				shimmerOn(touchX);
				table.adjustingColumnWidth = true;
			}
			/*@@@*/
			else if (tool != null && tool.isArrowTool() && table.getUserMoveColumn() && table.isColumnSelected(possibleTouch) && !MesquiteEvent.shiftKeyDown(modifiers) && !MesquiteEvent.controlKeyDown(modifiers)) {
				touchX=x;
				lastX = x;
				touchColumn=possibleTouch;
				shimmerOn(touchX);
			}
			else if ((table.showColumnGrabbers) && (y<table.getColumnGrabberWidth())) {
				table.columnTouched(possibleTouch,modifiers);
			}
			else {
				table.columnNameTouched(possibleTouch,modifiers, clickCount);
			}
		}
		else if (possibleTouch==-2 && ((TableTool)tool).getWorksBeyondLastColumn())
			table.columnTouched(possibleTouch,modifiers);
		else{
	   		table.offAllEdits();
	   		if (table.anythingSelected()) {
	   			table.deselectAllNotify();
	   			table.repaintAll();
	   		}
	   	}
   	 }
	/*...............................................................................................................*/
   	public void mouseDrag(int modifiers, int x, int y, MesquiteTool tool) {
		if (touchColumn>=0 && tool != null && tool.isArrowTool()) {
			
			if (table.getUserAdjustColumn()==MesquiteTable.RESIZE) {
				shimmerOff(lastX);
				shimmerOn(x);
				lastX=x;
			}
			else if (table.getUserMoveColumn()) {
				shimmerOff(lastX);
				shimmerOn(x);
				lastX=x;
			}
		}
   	 }
	/*...............................................................................................................*/
   	public void mouseUp(int modifiers, int x, int y, MesquiteTool tool) {
		if (touchColumn>=0 && tool != null && tool.isArrowTool()) {
			if (table.getUserAdjustColumn()==MesquiteTable.RESIZE && table.adjustingColumnWidth) {
				shimmerOff(lastX);

				int newColumnWidth = table.columnWidths[touchColumn] + x-touchX;
				if ((newColumnWidth >= table.getMinColumnWidth()) && (touchX>=0)) {
					table.setColumnWidth(touchColumn, newColumnWidth);
					table.columnWidthsAdjusted.setBit(touchColumn);
					table.repaintAll();
					//touchX=-1;
				}
				
			}
			else if (table.getUserMoveColumn()) {
				shimmerOff(lastX);
				int dropColumn = table.findHalfColumn(x);
				if (dropColumn != touchColumn && !table.isColumnSelected(dropColumn)) //don't move dropped on column included in selection
					table.selectedColumnsDropped(dropColumn);
			}
		table.adjustingColumnWidth = false;
		}
   	 }
   	/*...............................................................................................................*/
   	public void mouseExited(int modifiers, int x, int y, MesquiteTool tool) {
		if (!table.editingAnything() && !table.singleTableCellSelected()) 
				setWindowAnnotation("", null);
		setCursor(Cursor.getDefaultCursor());
		int column = table.findColumn(x);
		table.mouseExitedCell(modifiers, column, -1, tool);
  	}
	/*...............................................................................................................*/
	public void setCurrentCursor(int modifiers, int column, int x, MesquiteTool tool) {
		if (tool == null || !(tool instanceof TableTool))
				setCursor(getDisabledCursor());
		else if (column<table.numColumnsTotal && column>=0) {   //within bounds of normal columns
			if (tool.isArrowTool()) {
				if (table.getUserAdjustColumn()==MesquiteTable.RESIZE && table.nearColumnBoundary(x)  && !MesquiteEvent.shiftKeyDown(modifiers) && !MesquiteEvent.controlKeyDown(modifiers)) 
					setCursor(table.getEResizeCursor());
				else {
					setCursor(table.getHandCursor());
					if (!(table.editingAnything() || table.singleTableCellSelected())) {
						String s = table.getColumnComment(column);
						if (s!=null)   
							setWindowAnnotation(s, "Annotation above refers to " + table.getColumnNameText(column));
						else
							setWindowAnnotation("", null);
					}
				}
			}
			else if (((TableTool)tool).getWorksOnColumnNames())
				setCursor(tool.getCursor());
			else
				setCursor(getDisabledCursor());
		}
		else if (((TableTool)tool).getWorksBeyondLastColumn() && (column==-2))
			setCursor(tool.getCursor());
		else
			setCursor(getDisabledCursor());

	}
	/*...............................................................................................................*/
	public void mouseEntered(int modifiers, int x, int y, MesquiteTool tool) {
		int column = table.findColumn(x);
		setCurrentCursor(modifiers, column ,  x, tool);
		table.mouseInCell(modifiers, column, -1, tool);
	}
	/*...............................................................................................................*/
	public void mouseMoved(int modifiers, int x, int y, MesquiteTool tool) {
		int column = table.findColumn(x);
		setCurrentCursor(modifiers, column ,  x, tool);
		table.mouseInCell(modifiers, column, -1, tool);
	}
}



