/* Mesquite Class Library.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib.table;

import java.awt.*;
import java.awt.event.*;
import mesquite.lib.*;
//import mesquite.comm.lib.*;


/* ======================================================================== */
/** A panel for the main body of cells (i.e., excluding the column and row headings) of a MesquiteTable */
public class MatrixPanel extends EditorPanel {
	MesquiteTable table;
	public int width,  height;
	boolean drawingNow=false;
	
	public MatrixPanel (MesquiteTable table , int w, int h) {
		super(table);
		this.table=table;
		this.width=w;
		this.height=h;
		//setBackground(ColorDistribution.medium[table.colorScheme]);
		setBackground(Color.white);
	}
	/*...............................................................................................................*/
	public void setTableUnitSize (int w, int h) {
		this.width=w;
		this.height=h;
	}
	
	
	/*...............................................................................................................*/
	public String getText(int column, int row){
		return table.getMatrixText(column, row);
	}
	public void deselectCell(int column,int row){
		table.deselectCell(column, row);
	}
	/*...............................................................................................................*/
	boolean redrawingCellNow = false;
	static long redrawCells = 0;
	static long paints = 0;
	/*...............................................................................................................*/
	public void redrawCell(int column, int row){
		Graphics g = getGraphics();
		if (g!=null) {
			redrawCell(g, column, row);
			g.dispose();
		}
	}
	/*...............................................................................................................*/
	public void redrawCell(Graphics g, int column, int row) {
		redrawCells++;
		int left = table.getFirstColumnVisible();
		int top = table.getFirstRowVisible();
		if (column<left || row<top)  
			return;  
		if (column== returningColumn && row == returningRow){
			return; //don't draw if text about to be returned to cell, and will soon be redrawn anyway
		}
		if (drawingNow) {
			//MesquiteMessage.warnProgrammer("drawingNow MatrixPanel");
			return;
		}
		while (redrawingCellNow) {
			//MesquiteMessage.warnProgrammer("redrawingNow MatrixPanel");
			try {
				Thread.sleep(20);
			}
			catch (InterruptedException e) {
				//MesquiteMessage.warnProgrammer("interrupted MatrixPanel");
			return;
			}
		}
		redrawingCellNow=true;
		int leftSide = startOfColumn(column);
		int topSide = startOfRow(row);
		if (leftSide>getBounds().width || leftSide+columnWidth(column)<0) {
			redrawingCellNow = false;
			return;
		}
		if (topSide>getBounds().height || topSide+rowHeight(row)<0) {
			redrawingCellNow = false;
			return;
		}
		Shape clip = null;
		if (!table.useQuickMode()) {
			clip = g.getClip();
			g.setClip(leftSide,topSide,table.columnWidths[column], table.rowHeights[row]);
		}
		boolean selected = table.isCellSelected(column, row) || table.isRowSelected(row)|| table.isColumnSelected(column);
		if (!table.useString(column,row)) {
			table.drawMatrixCell(g, leftSide,topSide,table.columnWidths[column], table.rowHeights[row], column, row, selected);
		}
		else  {
			String supplied = table.getMatrixTextForDisplay(column,row);
			prepareCell(g, leftSide+1,topSide+1,table.columnWidths[column]-1, table.rowHeights[row]-1, false, selected, table.getCellDimmed(column, row), table.isCellEditable(column, row));
			table.drawMatrixCellString(g, null, leftSide,topSide,table.columnWidths[column], table.rowHeights[row], column, row, supplied);
		}
		
		
		if (table.getDropDown(column, row)) {
			dropDownTriangle.translate(leftSide+1,topSide + 1);
			g.setColor(Color.white);
			g.drawPolygon(dropDownTriangle);
			g.setColor(Color.black);
			g.fillPolygon(dropDownTriangle);
			dropDownTriangle.translate(-(leftSide+1),-(topSide + 1));
		}
		if (!table.useQuickMode() && clip !=null) {
			g.setClip(clip);
		}
		redrawingCellNow=false;
	}
	/*...............................................................................................................*/
	public void blankCell(int column, int row) {
		Graphics g = getGraphics();
		if (g==null)
			return;
		int left = table.getFirstColumnVisible();
		int top = table.getFirstRowVisible();
		
		if (column<left || row<top)  
			return;  
		int leftSide = startOfColumn(column);
		int topSide = startOfRow(row);
		if (leftSide>getBounds().width || leftSide+columnWidth(column)<0)
			return;
		if (topSide>getBounds().height || topSide+rowHeight(row)<0)
			return;
		Shape clip = g.getClip();
		g.setClip(leftSide,topSide,table.columnWidths[column], table.rowHeights[row]);
		
		prepareCell(g, leftSide+1,topSide+1,table.columnWidths[column]-1, table.rowHeights[row]-1, false, table.isCellSelected(column, row) || table.isRowSelected(row)|| table.isColumnSelected(column), false, table.isCellEditable(column, row));
		
		g.setClip(clip);
		g.dispose();
	}
	
	/*...............................................................................................................*/
	public void textReturned(int column, int row, String text, CommandRecord commandRec){
		table.returnedMatrixText(column, row, text,commandRec);
	}
	/*...............................................................................................................*/
	public void print(Graphics g) {
		int lineY;
		int lineX;
		if (table.frameMatrixCells) {
			int columnHeight = table.getTotalRowHeight();
			g.setColor(Color.gray);
			lineX = 0;
			for (int c=0; c<table.numColumnsTotal; c++) {
				lineX += table.columnWidths[c];
				g.drawLine(lineX, 0, lineX, columnHeight);//matrixHeight + columnNamesHeight
			}
			int rowLength = table.getTotalColumnWidth();

			lineY = 0;
			for (int r=0; (r<table.numRowsTotal); r++) {
				lineY += table.rowHeights[r];
				g.drawLine(0, lineY, rowLength, lineY);//rowNamesWidth+matrixWidth
			}
		}

		g.setColor(Color.black);
		lineY = 0;
		int oldLineY=lineY;
			
		int numCells = 0;//
		
		Shape clip = g.getClip();
		for (int r=0; (r<table.numRowsTotal); r++) {
			lineY += table.rowHeights[r];
			
			lineX = 0;
			int oldLineX=lineX;
			for (int c=0; (c<table.numColumnsTotal); c++) {
				numCells++;
				lineX += table.columnWidths[c];
				g.setClip(oldLineX,oldLineY,table.columnWidths[c], table.rowHeights[r]);
				String supplied = table.getMatrixTextForDisplay(c,r);
				if (fillColor!=null)
					g.setColor(fillColor);
				else if (table.isCellEditable(c, r))
					g.setColor(Color.white);
				else
					g.setColor(ColorDistribution.uneditable);
				g.fillRect(oldLineX+1,oldLineY+1,table.columnWidths[c]-1, table.rowHeights[r]-1);
				g.setColor(Color.black);
				if (!table.useString(c,r))
					table.drawMatrixCell(g, oldLineX,oldLineY,table.columnWidths[c], table.rowHeights[r], c, r, false);
				else 
					table.drawMatrixCellString(g, null, oldLineX,oldLineY,table.columnWidths[c], table.rowHeights[r], c, r, supplied);
				oldLineX = lineX;
			}
			oldLineY=lineY;
		}
		g.setClip(clip);
	}
	/*...............................................................................................................*/
	public void update(Graphics g){
		paint(g);
	}
	/*...............................................................................................................*/
	public void paint(Graphics g) {
	   	if (MesquiteWindow.checkDoomed(this))
	   		return;
		paints++;
		if (drawingNow) {//don't remember what this protects against!
			//MesquiteMessage.warnProgrammer("drawingNow2 MatrixPanel");
			drawingNow = false; 
			MesquiteWindow.uncheckDoomed(this);
			return;
		}
		if (redrawingCellNow){
			//MesquiteMessage.warnProgrammer("redrawingCellNow2 MatrixPanel");
			MesquiteWindow.uncheckDoomed(this);
			return;
		}

		drawingNow=true;
		int lineY;
		int lineX;
		width = getBounds().width;//this is here to test if width/height should be reset here
		height = getBounds().height;
		if (table.useQuickMode()){
			g.setColor(Color.white);
			g.fillRect(0,0,width, height);
		}
		if (table.frameMatrixCells) {
			int columnHeight = 0;
			for (int r=table.firstRowVisible; (r<table.numRowsTotal) && (columnHeight<height); r++) {
				columnHeight += table.rowHeights[r];
			}
			if  (columnHeight>height)
				columnHeight=height;

			g.setColor(Color.gray);
			lineX = 0;
			for (int c=table.firstColumnVisible; (c<table.numColumnsTotal) && (lineX<width); c++) {
				lineX += table.columnWidths[c];
				g.drawLine(lineX, 0, lineX, columnHeight);//matrixHeight + columnNamesHeight
			}
			int rowLength;
			if  (lineX>width)
				rowLength=width;
			else
				rowLength = lineX;

			lineY = 0;
			for (int r=table.firstRowVisible; (r<table.numRowsTotal) && (lineY<height); r++) {
				lineY += table.rowHeights[r];
				g.drawLine(0, lineY, rowLength, lineY);//rowNamesWidth+matrixWidth
			}
		}
		g.setColor(Color.black);
		lineY = 0;
		int oldLineY=lineY;
		int resetWidth = getBounds().width;
		int resetHeight = getBounds().height;
		checkEditFieldLocation();
			
		int numCells = 0;//
		FontMetrics fm = g.getFontMetrics(g.getFont());
		Shape clip = g.getClip();
		for (int r=table.firstRowVisible; (r<table.numRowsTotal) && (lineY<resetHeight); r++) {
			lineY += table.rowHeights[r];
			
			lineX = 0;
			int oldLineX=lineX;
			for (int c=table.firstColumnVisible; (c<table.numColumnsTotal) && (lineX<resetWidth); c++) {
				numCells++;
				lineX += table.columnWidths[c];
				if (c!= returningColumn || r != returningRow){ //don't draw if text about to be returned to cell, and will soon be redrawn anyway
					if (!table.useQuickMode())
						g.setClip(oldLineX,oldLineY,table.columnWidths[c], table.rowHeights[r]);
					boolean selected = table.isCellSelected(c, r) || table.isRowSelected(r)|| table.isColumnSelected(c);
					if (!table.useString(c,r)) {
						table.drawMatrixCell(g, oldLineX,oldLineY,table.columnWidths[c], table.rowHeights[r], c, r, selected);
					}
					else {
						String supplied = table.getMatrixTextForDisplay(c,r);
						Color color = null;
						if (selected)
							color = Color.black;
						else if (fillColor!=null)
							color = fillColor;
						else if (table.isCellEditable(c, r)){
							if (!table.useQuickMode())
								color = Color.white;
						}
						else
							color = ColorDistribution.uneditable;
						if (color !=null){
							g.setColor(color);
							g.fillRect(oldLineX+1,oldLineY+1,table.columnWidths[c]-1, table.rowHeights[r]-1);
						}
						Color textColor;
						if (selected)
							textColor = Color.white;
						else if (table.getCellDimmed(c,r))
							textColor = Color.gray;
						else
							textColor = Color.black;
						g.setColor(textColor);
						table.drawMatrixCellString(g, fm, oldLineX,oldLineY,table.columnWidths[c], table.rowHeights[r], c, r, supplied);
					}
				}
				oldLineX = lineX;
			}
			oldLineY=lineY;
		}
		if ((endOfLastColumn()>=0) && (endOfLastColumn()<table.matrixWidth)) {
			g.setClip(endOfLastColumn()+1, 0, table.matrixWidth-1, table.matrixHeight-1);
			g.setColor(ColorDistribution.medium[table.colorScheme]);
			g.fillRect(endOfLastColumn()+1, 0, table.matrixWidth-1, table.matrixHeight-1);
		}
		if ((endOfLastRow()>=0) && (endOfLastRow()<table.matrixHeight)) {
			g.setClip(0, endOfLastRow()+1, table.matrixWidth-1, table.matrixHeight-1);
			g.setColor(ColorDistribution.medium[table.colorScheme]);
			g.fillRect(0, endOfLastRow()+1, table.matrixWidth-1, table.matrixHeight-1);
		}
		g.setClip(clip);
		g.setColor(Color.black);

		g.drawRect(0, 0, resetWidth-1, resetHeight-1);
		drawingNow=false;
		MesquiteWindow.uncheckDoomed(this);
	}
	public void upArrowPressed(KeyEvent e){ //
		if (getEditing() && (editField.getRow() == 0) && table.isColumnNameEditable(editField.getColumn()))
			table.editColumnNameCell(editField.getColumn());
		else
			super.upArrowPressed(e);
	}
	
	public void leftArrowPressed(KeyEvent e){
		if (getEditing() && (editField.getColumn() == 0) && table.isRowNameEditable(editField.getRow()))
			table.editRowNameCell(editField.getRow());
		else
			super.leftArrowPressed(e);
	}
	/*...............................................................................................................*/
	public void mouseDown(int modifiers, int clickCount, long when, int x, int y, MesquiteTool tool) {
		if (!(tool instanceof TableTool))
			return;
		int column = table.findColumn(x);
		int row = table.findRow(y);
   		
		if (column>-1 && row > -1 && column<table.numColumnsTotal && row<table.numRowsTotal)
			table.cellTouched(column, row,modifiers, clickCount);
		else if (column==-2 && ((TableTool)tool).getWorksBeyondLastColumn())
			table.cellTouched(column, row,modifiers, clickCount);
		else if (row==-2 && ((TableTool)tool).getWorksBeyondLastRow())
			table.cellTouched(column, row,modifiers, clickCount);
		else
			setWindowAnnotation("", null);
		
		
   	 }
	/*_________________________________________________*/
	public void mouseDrag(int modifiers, int x, int y, MesquiteTool tool) {
		int column = table.findColumn(x);
		int row = table.findRow(y);
   		
		if (column>-1 && row > -1 && column<table.numColumnsTotal && row<table.numRowsTotal) {
			table.cellDrag(column, row,modifiers);
		}
	}
	/*_________________________________________________*/
	public void mouseUp(int modifiers, int x, int y, MesquiteTool tool) {
		if (!(tool instanceof TableTool))
			return;
		int column = table.findColumn(x);
		int row = table.findRow(y);
   		
		if (column>-1 && row > -1 && column<table.numColumnsTotal && row<table.numRowsTotal)
			table.cellDropped(column, row,modifiers);
		else if (column==-2 && ((TableTool)tool).getWorksBeyondLastColumn())
			table.cellDropped(column, row,modifiers);
		else if (row==-2 && ((TableTool)tool).getWorksBeyondLastRow())
			table.cellDropped(column, row,modifiers);
		else {
	   		table.offAllEdits();
	   		if (table.anythingSelected()) {
	   			table.deselectAllNotify();
	   			table.repaintAll();
	   		}
		}
	}
   	/*...............................................................................................................*/
   	public void mouseExited(int modifiers, int x, int y, MesquiteTool tool) {
		if (!table.editingAnything() && !table.singleTableCellSelected()) 
				setWindowAnnotation("", null);
		setCursor(Cursor.getDefaultCursor());
		int column = table.findColumn(x);
		int row = table.findRow(y);
		table.mouseExitedCell(modifiers, column, row, tool);
  	}
	/*...............................................................................................................*/
	public void setCurrentCursor(int modifiers, int column, int row, MesquiteTool tool) {
		if (tool == null || !(tool instanceof TableTool))
				setCursor(getDisabledCursor());
		else if (!((TableTool)tool).getWorksOnMatrixPanel()) {
			setCursor(getDisabledCursor());
		}
		else if ((!((TableTool)tool).getWorksBeyondLastRow() && ((row>table.numRowsTotal)||(row<0)))||(!((TableTool)tool).getWorksBeyondLastColumn() && ((column>table.numColumnsTotal)||(column<0)))) 
			setCursor(getDisabledCursor());
		else 
			setCursor(tool.getCursor());
	}
  	/*...............................................................................................................*/
   	public void mouseEntered(int modifiers, int x, int y, MesquiteTool tool) {
		int column = table.findColumn(x);
		int row = table.findRow(y);
		setCurrentCursor(modifiers, column, row, tool);
		table.mouseInCell(modifiers, column, row, tool);
  	}
	/*...............................................................................................................*/
	public void mouseMoved(int modifiers, int x, int y, MesquiteTool tool) {
		int column = table.findColumn(x);
		int row = table.findRow(y);
		setCurrentCursor(modifiers, column, row, tool);
		table.mouseInCell(modifiers, column, row, tool);
	}
}
