/* Mesquite Class Library.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lib.table;

import java.awt.*;
import java.awt.event.*;
import mesquite.lib.*;


/* ======================================================================== */
/** A subclass of window designed to contain a MesquiteTable .*/
public abstract class TableWindow extends MesquiteWindow {
	/** constructor to be used only for accumulating list of commands*/
	public TableWindow () {
	}
	public TableWindow (MesquiteModule ownerModule, boolean showInfoBar) {
		super(ownerModule, showInfoBar); //INFOBAR
		setShowExplanation(true);
	}
	public abstract MesquiteTable getTable();
	/*.................................................................................................................*/
	public void pleaseUpdate() {
		repaint();
		if (getTable()==null) return;
		getTable().repaintAll();
	}
	
	/** Return Mesquite commands that will put the table back to its current state (approximately). */
  	 public Snapshot getSnapshot(MesquiteFile file) {  //this allows employees to be dealt with
	   	 	if (getTable()==null)
	   	 		return null;
	   	 	Snapshot temp = new Snapshot();
	   	 	Snapshot ts = getTable().getSnapshot(file);
	   	 	if (ts!=null) {
				temp.addLine("getTable");
				
				temp.addLine("tell It");
				temp.incorporate(ts, true);
				temp.addLine("endTell");
	   	 	}
			temp.incorporate(super.getSnapshot(file), false);
	 	 	return temp;
  	 }
	/*.................................................................................................................*/
	/** A request for the MesquiteModule to perform a command.  It is passed two strings, the name of the command and the arguments.
	This should be overridden by any module that wants to respond to a command.*/
 	public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) { 
		if (checker.compare(MesquiteTable.class, "Returns the table object for the list window", null, commandName, "getTable")) {
			return getTable();
		}
		else return super.doCommand(commandName, arguments, commandRec, checker);
 	}
	/*.................................................................................................................*/
	public void paintContents(Graphics g) {
		if (getTable()==null) return;
		getTable().repaintAll();
	}
	/*.................................................................................................................*/
	public String getTextContents() {
		if (getTable()==null) return null;
		return getTable().getTextVersion();
	}
	public void repaintAll() {
		if (getTable()==null) return;
		getTable().repaintAll();
	}
	/*.................................................................................................................*/
	public void printWindow(MesquitePrintJob pjob) {
		if (getTable()==null) return;
    	 	getTable().printTable(pjob, this);
	}
	public void resetCursor(){  
		MesquiteTool tool = getCurrentTool();
		if (!(tool instanceof TableTool))
			return;
		if (((TableTool)tool).getWorksOnColumnNames()) {
			Cursor c = tool.getCursor();
			if (c != null)
				getTable().getColumnNamesPanel().setCursor(tool.getCursor());
		}
		if (((TableTool)tool).getWorksOnRowNames()) {
			Cursor c = tool.getCursor();
			if (c != null)
				getTable().getRowNamesPanel().setCursor(tool.getCursor());
		}
		if (((TableTool)tool).getWorksOnCornerPanel()) {
			Cursor c = tool.getCursor();
			if (c != null)
				getTable().getCornerPanel().setCursor(tool.getCursor());
		}
		if (((TableTool)tool).getWorksOnMatrixPanel()) {
			Cursor c = tool.getCursor();
			if (c != null)
				getTable().getMatrixPanel().setCursor(tool.getCursor());
		}
		setCursor(Cursor.getDefaultCursor());
	}
	/*.................................................................................................................*/
	public void setCurrentTool(MesquiteTool tool){
		if (!(tool instanceof TableTool))
			return;
		super.setCurrentTool(tool);
		if (getTable()==null) return;
		resetCursor();
	}
	/*.................................................................................................................*/
    	 public void copyGraphicsPanel() {
		if (getTable()==null || getTable().getCopyCommand() == null) 
			return;
		getTable().getCopyCommand().doItMainThread("", null, this);  // command invoked
   	 }
    	 public String getCopySpecialName() {
		return "Copy Literal"; 
   	 }
    	 public MesquiteCommand getCopySpecialCommand() {
		if (getTable()==null) return null;
		return getTable().getCopyLiteralCommand(); 
   	 }
    	 public MesquiteCommand getPasteCommand() {
		if (getTable()==null) return null;
		return getTable().getPasteCommand(); 
   	 }
    	 public MesquiteCommand getCutCommand() {
		if (getTable()==null) return null;
		return getTable().getCutCommand(); 
   	 }
    	 public MesquiteCommand getClearCommand() {
		if (getTable()==null) return null;
		return getTable().getClearCommand(); 
   	 }
  	public void selectAllGraphicsPanel(){
		if (getTable()==null && getTable().getSelectAllCommand()!=null) 
			return;
		getTable().getSelectAllCommand().doItMainThread(null, "", this); 
   	 }
}


