/* Mesquite (package mesquite.lists).  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lists.CharListPartition;
/*~~  */

import mesquite.lists.lib.*;
import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import mesquite.lib.table.*;

/* ======================================================================== */
public class CharListPartition extends CharListAssistant {
	CharacterData data=null;
	MesquiteTable table=null;
	MesquiteSubmenuSpec mss, mEGC, mEGN;
	MesquiteMenuItemSpec mScs, mStc, mRssc, mLine, nNG, mLine2, mss2;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		return true;
  	 }
  	 MesquiteInteger pos = new MesquiteInteger(0);
	/*.................................................................................................................*/
  	private void setPartition(CharactersGroup group, String arguments, CommandRecord commandRec){
    	 		if (table !=null && data!=null) {
    	 			boolean changed=false;
    	 			String name = parser.getFirstToken(arguments);
    	 			if (group == null && StringUtil.blank(name))
    	 				return;
				CharacterPartition partition = (CharacterPartition) data.getCurrentSpecsSet(CharacterPartition.class);
				if (partition==null){
				 		partition= new CharacterPartition("Untitled Partition", data.getNumChars(), null, data);
				 		partition.addToFile(data.getFile(), getProject(), findElementManager(CharacterPartition.class));
						data.setCurrentSpecsSet(partition, CharacterPartition.class);
				}
				if (group == null){
					CharactersGroupVector groups = (CharactersGroupVector)getProject().getFileElement(CharactersGroupVector.class, 0);
					Object obj = groups.getElement(name);
					group = (CharactersGroup)obj;
				}
				if (group != null) {
		 			if (partition != null) {
	    	 				if (employer!=null && employer instanceof ListModule) {
	    	 					int c = ((ListModule)employer).getMyColumn(this);
		    	 				for (int i=0; i<data.getNumChars(); i++) {
		    	 					if (table.isCellSelectedAnyWay(c, i)) {
		    	 						partition.setProperty(group, i);
		    	 						if (!changed)
										outputInvalid(commandRec);
		    	 						changed = true;
		    	 					}
		    	 				}
	    	 				}
	    	 			}
	    	 			if (changed)
		    	 			data.notifyListeners(this, new Notification(MesquiteListener.NAMES_CHANGED), commandRec); //TODO: bogus! should notify via specs not data???
					outputInvalid(commandRec);
	   	 			parametersChanged(null, commandRec);
	   	 		}
   	 		}
  	}
  	private void removePartition(CommandRecord commandRec){
    	 		if (table !=null && data!=null) {
    	 			boolean changed=false;
				CharacterPartition partition = (CharacterPartition) data.getCurrentSpecsSet(CharacterPartition.class);
				if (partition!=null){
    	 				if (employer!=null && employer instanceof ListModule) {
    	 					int c = ((ListModule)employer).getMyColumn(this);
	    	 				for (int i=0; i<data.getNumChars(); i++) {
	    	 					if (table.isCellSelectedAnyWay(c, i)) {
	    	 						partition.setProperty(null, i);
	    	 						if (!changed)
									outputInvalid(commandRec);
	    	 						changed = true;
	    	 					}
	    	 				}
    	 				}
	    	 			
	    	 			if (changed)
		    	 			data.notifyListeners(this, new Notification(MesquiteListener.NAMES_CHANGED), commandRec); //TODO: bogus! should notify via specs not data???
	   	 			outputInvalid(commandRec);
	   	 			parametersChanged(null, commandRec);
		   	 		
				}
   	 		}
  	}
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	 if (checker.compare(this.getClass(), "Sets the character group of the selected characters", "[name of group]", commandName, "setPartition")) {
    	 		setPartition(null, arguments, commandRec);
    	 	}
    	 	else if (checker.compare(this.getClass(), "Edits the name and color of a character group label", "[name of group]", commandName, "editGroup")) {
    	 			String name = parser.getFirstToken(arguments);
    	 			if (StringUtil.blank(name))
    	 				return null;
    	 			String num = parser.getNextToken();
    	 			int i = MesquiteInteger.fromString(num);
				CharactersGroupVector groups = (CharactersGroupVector)getProject().getFileElement(CharactersGroupVector.class, 0);
				Object obj;
				if (MesquiteInteger.isCombinable(i) && i< groups.size())
					obj = groups.elementAt(i);
				else
					obj = groups.getElement(name);
				if (obj != null) {
					CharactersGroup group = (CharactersGroup)obj;

    	 				GroupDialog d = new GroupDialog(containerOfModule(), "Edit Character Group", group.getName(), group.getColor());
		    	 		d.completeAndShowDialog();
					name = d.getName();
					boolean ok = d.query()==0;
					Color c = d.getColor();
					d.dispose();
					if (!ok)
						return null;


					if (!StringUtil.blank(name)) {
						group.setName(name);
						group.setColor(c);
		    	 			data.notifyListeners(this, new Notification(MesquiteListener.NAMES_CHANGED), commandRec); //TODO: bogus! should notify via specs not data???
					}
					outputInvalid(commandRec);
					parametersChanged(commandRec);
				}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Creates a new group for use in character partitions", null, commandName, "newGroup")) {
    	 		String n = "Untitled Group";
    	 		if (data.getFile()!=null)
    	 			n = data.getFile().getFileElements().getUniqueName(n);
    	 		GroupDialog d = new GroupDialog(containerOfModule(), "New Character Group", n, Color.white);
    	 		d.completeAndShowDialog();
			String name = d.getName();
			boolean ok = d.query()==0;
			Color c = d.getColor();
			d.dispose();
			if (!ok)
				return null;
			//String name = MesquiteString.queryString(containerOfModule(), "New character group", "New character group label", "Untitled Group");
			if (StringUtil.blank(name))
				return null;
			CharactersGroup group = new CharactersGroup();
			group.setName(name);
			group.addToFile(data.getFile(), getProject(), null);
			if (c!=null) {
				group.setColor(c);
			}
	 		setPartition(group, name, commandRec);
	    	 }
    	 	else if (checker.compare(this.getClass(), "Stores the current character partition set", null, commandName, "storeCurrent")) {
    	 		if (data!=null){
    	 			SpecsSetVector ssv = data.getSpecSetsVector(CharacterPartition.class);
    	 			if (ssv == null || ssv.getCurrentSpecsSet() == null) {
			 		CharacterPartition partition= new CharacterPartition("Untitled Partition", data.getNumChars(), null, data);
			 		partition.addToFile(data.getFile(), getProject(), findElementManager(CharacterPartition.class));
					data.setCurrentSpecsSet(partition, CharacterPartition.class);
					ssv = data.getSpecSetsVector(CharacterPartition.class);
    	 			}
    	 			if (ssv!=null) {
    	 				SpecsSet s = ssv.storeCurrentSpecsSet();
    	 				s.setName(ssv.getUniqueName("Untitled Character Partition"));
    	 				String name = MesquiteString.queryString(containerOfModule(), "Name", "Name of character partition to be stored", "Untitled Character Partition");
    	 				if (!StringUtil.blank(name))
    	 					s.setName(name);
    	 				ssv.notifyListeners(this, new Notification(MesquiteListener.NAMES_CHANGED), commandRec);  
    	 			}
     	 			else MesquiteMessage.warnProgrammer("sorry, can't store because no specssetvector");
   	 		}
    	 		//return ((ListWindow)getModuleWindow()).getCurrentObject();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Replaces a stored character partition set by the current one", null, commandName, "replaceWithCurrent")) {
    	 		if (data!=null){
    	 				SpecsSetVector ssv = data.getSpecSetsVector(CharacterPartition.class);
  	 				if (ssv!=null) {
  	 					SpecsSet chosen = (SpecsSet)ListDialog.queryList(containerOfModule(), "Replace stored set", "Choose stored partition to replace by current set",MesquiteString.helpString, ssv, 0);
		    	 			if (chosen!=null){
		    	 				SpecsSet current = ssv.getCurrentSpecsSet();
		    	 				ssv.replaceStoredSpecsSet(chosen, current);
		    	 			}
	    	 			}
    	 			
				outputInvalid(commandRec);
				parametersChanged(commandRec);
    	 		}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Loads the stored character partition to be the current one", "[number of partition to load]", commandName, "loadToCurrent")) {
 			if (data !=null) {
 				int which = MesquiteInteger.fromFirstToken(arguments, stringPos);
 				if (MesquiteInteger.isCombinable(which)){
		 			SpecsSetVector ssv = data.getSpecSetsVector(CharacterPartition.class);
					if (ssv!=null) {
						SpecsSet chosen = ssv.getSpecsSet(which);
		    	 			if (chosen!=null){
		    	 				ssv.setCurrentSpecsSet(chosen.cloneSpecsSet()); 
		    	 				data.notifyListeners(this, new Notification(AssociableWithSpecs.SPECSSET_CHANGED), commandRec); //TODO: bogus! should notify via specs not data???
		    	 				return chosen;
		    	 			}
			 		}
 				}
				outputInvalid(commandRec);
				parametersChanged(commandRec);
	 		}
	    	 }
    	 	else if (checker.compare(this.getClass(), "Removes the group designation from the selected characters", null, commandName, "removeGroup")) {
				removePartition(commandRec);
	    	 }
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
   	 }
   
	/*.................................................................................................................*/
	public void setTableAndData(MesquiteTable table, CharacterData data, CommandRecord commandRec){
		/* hire employees here */
		deleteMenuItem(mss);
		deleteMenuItem(mss2);
		deleteMenuItem(mScs);
		deleteMenuItem(mRssc);
		deleteMenuItem(mLine);
		deleteMenuItem(mLine2);
		deleteMenuItem(mStc);
		deleteMenuItem(nNG);
		deleteMenuItem(mEGC);
		deleteMenuItem(mEGN);
		deleteMenuItem(nNG);
		mss = addSubmenu(null, "Set Group", makeCommand("setPartition", this));
		mss.setList((StringLister)getProject().getFileElement(CharactersGroupVector.class, 0));
		mss2 = addMenuItem("Remove Group Designation", makeCommand("removeGroup", this));
		mLine2 = addMenuItem("-",null);
		nNG = addMenuItem("New Group...", makeCommand("newGroup",  this));
		mEGC = addSubmenu(null, "Edit Group...", makeCommand("editGroup", this));
		mEGC.setList((StringLister)getProject().getFileElement(CharactersGroupVector.class, 0));
		mLine = addMenuItem("-",null);
		mScs = addMenuItem("Store current set", makeCommand("storeCurrent",  this));
		mRssc = addMenuItem("Replace stored set by current", makeCommand("replaceWithCurrent",  this));
		if (data !=null)
			mStc = addSubmenu(null, "Load", makeCommand("loadToCurrent",  this), data.getSpecSetsVector(CharacterPartition.class));
		this.data = data;
		this.table = table;
	}
	public void changed(Object caller, Object obj, Notification notification, CommandRecord commandRec){
		if (Notification.appearsCosmetic(notification))
			return;
		outputInvalid(commandRec);
		parametersChanged(notification, commandRec);
	}
	public String getTitle() {
		return "Group";
	}
	public String getStringForCharacter(int ic){
		if (data!=null) {
			CharacterPartition partition = (CharacterPartition)data.getCurrentSpecsSet(CharacterPartition.class);
			if (partition != null) {
				CharactersGroup group = (CharactersGroup)partition.getProperty(ic);
				if (group!=null) {
					return group.getName();
				}
			}
		}
		return "?";
	}
	public boolean useString(int ic){
		return false;
	}
	public void drawInCell(int ic, Graphics g, int x, int y,  int w, int h, boolean selected){
		if (data==null || g==null)
			return;
		boolean colored = false;
		Color c = g.getColor();
		CharacterPartition part = (CharacterPartition)data.getCurrentSpecsSet(CharacterPartition.class);
		if (part!=null) {
			CharactersGroup tg = part.getCharactersGroup(ic);
			if (tg!=null){
				Color cT = tg.getColor();
				if (cT!=null){
					g.setColor(cT);
					g.fillRect(x+1,y+1,w-1,h-1);
					colored = true;
				}
			}
		}
		if (!colored){ 
			if (selected)
				g.setColor(Color.black);
			else
				g.setColor(Color.white);
			g.fillRect(x+1,y+1,w-1,h-1);
		}
			
		String s = getStringForRow(ic);
		if (s!=null){
			FontMetrics fm = g.getFontMetrics(g.getFont());
			if (fm==null)
				return;
			int sw = fm.stringWidth(s);
			int sh = fm.getMaxAscent()+ fm.getMaxDescent();
			if (selected)
				g.setColor(Color.white);
			else
				g.setColor(Color.black);
			g.drawString(s, x+(w-sw)/2, y+h-(h-sh)/2);
			if (c!=null) g.setColor(c);
		}
	}
	public String getWidestString(){
		return "Partition     ";
	}
	/*.................................................................................................................*/
    	 public String getName() {
		return "Group Membership (characters)";
   	 }
	/*.................................................................................................................*/
	/** returns whether this module is requesting to appear as a primary choice */
   	public boolean requestPrimaryChoice(){
   		return true;  
   	}
   	 
	/*.................................................................................................................*/
   	public boolean isPrerelease(){
   		return false;  
   	}
	/*.................................................................................................................*/
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Lists and allows changes to group membership in the current partition of characters, for List of Characters window." ;
   	 }
}

