/* Mesquite (package mesquite.lists).  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lists.ModelsList;
/*~~  */

import mesquite.lists.lib.*;
import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import mesquite.lib.table.*;

/* ======================================================================== */
public class ModelsList extends ListModule {
	ListableVector models;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		return true;
  	 }
  	 public boolean showing(Object obj){
    	 		return (getModuleWindow()!=null && obj == models);
  	 }
  	 
  	 public void showListWindow(Object obj, CommandRecord commandRec){
		setModuleWindow(new ModelsListWindow(this));
     	 	((ModelsListWindow)getModuleWindow()).setObject(models = getProject().getCharacterModels(), commandRec);
		//makeMenu("Character_Models");
		makeMenu("List");
		
		if (!commandRec.scripting()){
			ModelsListAssistant assistant = (ModelsListAssistant)hireNamedEmployee(commandRec, ModelsListAssistant.class, StringUtil.tokenize("#ModelsListType"));
	 		if (assistant!= null){
	 			((ModelsListWindow)getModuleWindow()).addListAssistant(assistant, commandRec);
				assistant.setUseMenubar(false);
			}
			assistant = (ModelsListAssistant)hireNamedEmployee(commandRec, ModelsListAssistant.class, StringUtil.tokenize("#ModelsListParadigm"));
	 		if (assistant!= null){
	 			((ModelsListWindow)getModuleWindow()).addListAssistant(assistant, commandRec);
				assistant.setUseMenubar(false);
			}
			assistant = (ModelsListAssistant)hireNamedEmployee(commandRec, ModelsListAssistant.class, StringUtil.tokenize("#ModelsListWhole"));
	 		if (assistant!= null){
	 			((ModelsListWindow)getModuleWindow()).addListAssistant(assistant, commandRec);
				assistant.setUseMenubar(false);
			}
	  		/**/
	  		assistant = (ModelsListAssistant)hireNamedEmployee(commandRec, ModelsListAssistant.class, StringUtil.tokenize("Data type for model"));
	 		if (assistant!= null){
	 			((ModelsListWindow)getModuleWindow()).addListAssistant(assistant, commandRec);
				assistant.setUseMenubar(false);
			}
			
		}
 		resetContainingMenuBar();
		resetAllWindowsMenus();
 		//getModuleWindow().setVisible(true);
  	 }
	/*.................................................................................................................*/
	/* following required by ListModule*/
  	 public Object getMainObject(){
  	 	return getProject().getCharacterModels();
  	 }
  	 public int getNumberOfRows(){
  	 	if (getProject().getCharacterModels()==null)
  	 		return 0;
  	 	else
  	 		return getProject().getCharacterModels().size();
  	 }
	public Class getAssistantClass(){
		return ModelsListAssistant.class;
	}
	public String getItemTypeName(){
		return "Character model";
	}
	public String getItemTypeNamePlural(){
		return "Character models";
	}
	/*.................................................................................................................*/
	public boolean rowsDeletable(){
		return true;
	}
	public boolean deleteRow(int row){
		CharacterModel model = ((ModelsListWindow)getModuleWindow()).getModel(row);
		if (model!=null){
			if (model.isBuiltIn()) {
				alert("That character model cannot be deleted, because the model is built in");
				return false;
			}
			else {
				getProject().removeFileElement(model);//must remove first, before disposing
				model.dispose();
				return true;
			}
		}
		return false;
	}

	/** returns a String of annotation for a row*/
	public String getAnnotation(int row){ 
		if (row<0 || row>= getNumberOfRows())
			return null;
		CharacterModel model = ((ModelsListWindow)getModuleWindow()).getModel(row);
		return model.getAnnotation();
	}

	/** sets the annotation for a row*/
	public void setAnnotation(int row, String s, boolean notify){
		if (row<0 || row>= getNumberOfRows())
			return;
		CharacterModel model = ((ModelsListWindow)getModuleWindow()).getModel(row);
		model.setAnnotation(s, notify);
	}
	/** returns a String of explanation for a row*/
	public String getExplanation(int row){
		CharacterModel model = ((ModelsListWindow)getModuleWindow()).getModel(row);
		if (model!=null)
			return model.getExplanation();
		return null;
	}
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) { 
   	 	if (getModuleWindow()==null || !getModuleWindow().isVisible())
   	 		return null;
   	 	Snapshot temp = new Snapshot();
      	 	if (getModuleWindow()!=null)
			getModuleWindow().incorporateSnapshot(temp, file);
  	 	//if (getModuleWindow()!=null && !getModuleWindow().isVisible())
  	 	temp.addLine("showWindow"); 
 	 	return temp;
  	 }
	/*.................................................................................................................*/
    	 public String getName() {
		return "Character Models List";
   	 }
   	 
	/*.................................................................................................................*/
   	 
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Makes windows listing character models and information about them." ;
   	 }
	/*.................................................................................................................*/
   	 
   	 
	/*.................................................................................................................*/
	/** Requests a getModuleWindow() to close.  In the process, subclasses of MesquiteWindow might close down their owning MesquiteModules etc.*/
 	public void windowGoAway(MesquiteWindow whichWindow) {
 			//Debug.println("disposing of getModuleWindow()");
			whichWindow.hide();
	}
   	 
   	 
}

/* ======================================================================== */
class ModelsListWindow extends ListWindow implements MesquiteListener {
	ListableVector models;
	public ModelsListWindow (ModelsList ownerModule) {
		super(ownerModule);
		MesquiteTable t = getTable();
		if (t!=null)
			t.setAutoEditable(false, false, false, false);
	}
	/*.................................................................................................................*/
	/** When called the getModuleWindow() will determine its own title.  MesquiteWindows need
	to be self-titling so that when things change (names of files, tree blocks, etc.)
	they can reset their titles properly*/
	public void resetTitle(){
			setTitle("Character models"); 
	}
	/*.................................................................................................................*/
	public Object getCurrentObject(){
		return models;
	}
	public void setCurrentObject(Object obj){
		if (obj instanceof ListableVector) {
			if (models!=null)
				models.removeListener(this);
			models = (ListableVector)obj;
			models.addListener(this);
			getTable().synchronizeRowSelection(models);
		}
		super.setCurrentObject(obj);
	}
	/*...............................................................................................................*/
	/** returns whether or not a row name of table is editable.*/
	public boolean isRowNameEditable(int row){
		CharacterModel model = getModel(row);
		if (model==null)
			return false;
		else
			return !model.isBuiltIn();
	}
	CharacterModel getModel(int row){
		if (models!=null) {
			if (row>=0 && row<models.size())
				return(CharacterModel)models.elementAt(row);
		}
		return null;
	}
	public boolean interceptRowNameTouch(int row, int modifiers){
		CharacterModel model = getModel(row);
		if (model!=null){
			if (model.isBuiltIn()) {
				((ListTable)getTable()).superRowNameTouched(row, modifiers,1);
			}
			else
				getTable().editRowNameCell(row);
		}
		return true;
	}
	public void setRowName(int row, String name, CommandRecord commandRec){
		CharacterModel model = getModel(row);
		if (model!=null){
			if (model.isBuiltIn())
				ownerModule.alert("The name of that character model cannot be changed, because the model is built in");
			else {
				model.setName(name);
				resetAllTitles();
				getOwnerModule().resetAllMenuBars();
			}
		}
	}
	public String getRowName(int row){
		if (models!=null)
			return ((Listable)models.elementAt(row)).getName();
		else
			return null;
	}
	/*.................................................................................................................*/
	/** passes which object is being disposed (from MesquiteListener interface)*/
	public void disposing(Object obj){
		if (obj ==models)
			ownerModule.windowGoAway(this);
	}
	/*.................................................................................................................*/
	/** passes which object is being disposed (from MesquiteListener interface)*/
	public boolean okToDispose(Object obj, int queryUser){
		return true;  //TODO: respond
	}
	/*.................................................................................................................*/
	/** passes which object changed*/
	public void changed(Object caller, Object obj, Notification notification, CommandRecord commandRec){
		int code = Notification.getCode(notification);
		if (obj instanceof ListableVector && (ListableVector)obj ==models) {
			if (code==MesquiteListener.NAMES_CHANGED) {
				getTable().redrawRowNames();
			}
			else if (code==MesquiteListener.SELECTION_CHANGED) { //listable vector, not Associable, hence dont' synchronize
				getTable().synchronizeRowSelection(models);
				getTable().repaintAll();
			}
			else if (code==MesquiteListener.PARTS_ADDED || code==MesquiteListener.PARTS_DELETED || code==MesquiteListener.PARTS_CHANGED || code==MesquiteListener.PARTS_MOVED) {
				getTable().setNumRows(models.size());
				getTable().synchronizeRowSelection(models);
				getTable().repaintAll();
			}
			else if (code!=MesquiteListener.ANNOTATION_CHANGED) {
				getTable().setNumRows(models.size());
				getTable().synchronizeRowSelection(models);
				getTable().repaintAll();
			}
		}
		super.changed(caller, obj, notification, commandRec);
	}
	/*.................................................................................................................*/
	
	public void dispose(){
		if (models!=null)
			models.removeListener(this);
		super.dispose();
	}
}

