/* Mesquite (package mesquite.lists).  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lists.TaxonList;
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;
import mesquite.lists.lib.*;
import mesquite.lib.table.*;

/* ======================================================================== */
public class TaxonList extends ListModule {
	Taxa taxa;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		return true;
  	 }
	/*.................................................................................................................*/
	public void endJob(){
			if (taxa!=null)
				taxa.removeListener(this);
			super.endJob();
	}
	/*.................................................................................................................*/
	/** passes which object is being disposed (from MesquiteListener interface)*/
	public void disposing(Object obj){
		if (obj instanceof Taxa && (Taxa)obj == taxa) {
			iQuit();
		}
	}
  	 public boolean showing(Object obj){
	 	if (obj instanceof String) {
    	 		String arguments = (String)obj;
    	 		int query = MesquiteInteger.fromString(arguments);
    	 		return (query == getProject().getTaxaNumber(taxa) && getModuleWindow()!=null);
	 	}
	 	if (obj instanceof Taxa) {
    	 		return (this.taxa == obj && getModuleWindow()!=null);
	 	}
	 	return false;
  	 }
  	 
  	 public void showListWindow(Object obj, CommandRecord commandRec){
 	 	if (obj instanceof Taxa) {
    	 		taxa = (Taxa)obj;
			TaxonListWindow window = new TaxonListWindow(this, commandRec);
			setModuleWindow( window);

	     	 	window.setObject(taxa, commandRec);
			//makeMenu("Taxon_List");
			makeMenu("List");
			
			MesquiteSubmenuSpec mss2 = addSubmenu(null, "Utilities", MesquiteModule.makeCommand("doUtility",  this));
			mss2.setList(TaxonListUtility.class);
			MesquiteSubmenuSpec mss3 = addSubmenu(null, "Taxon Names", MesquiteModule.makeCommand("doNames",  this));
			mss3.setList(TaxonNameAlterer.class);
	 		addMenuItem( "Save selected as set...", makeCommand("saveSelectedRows", this));
	 		addMenuItem( "-", null);
	 		
			/*added default columns below May 02*/
			TaxonListAssistant assistant = (TaxonListAssistant)hireNamedEmployee(commandRec, TaxonListAssistant.class, StringUtil.tokenize("#TaxonListCurrPartition"));
	 		if (assistant!= null){
	 			((TaxonListWindow)window).addListAssistant(assistant, commandRec);
				assistant.setUseMenubar(false);
			}
	 		resetContainingMenuBar();
			resetAllWindowsMenus();
		}
  	 }
	/*.................................................................................................................*/
	/* following required by ListModule*/
  	 public Object getMainObject(){
  	 	return taxa;
  	 }
  	 public int getNumberOfRows(){
  	 	if (taxa==null)
  	 		return 0;
  	 	else
  	 		return taxa.getNumTaxa();
  	 }
	public Class getAssistantClass(){
		return TaxonListAssistant.class;
	}
	public String getItemTypeName(){
		return "Taxon";
	}
	public String getItemTypeNamePlural(){
		return "Taxa";
	}
	/*.................................................................................................................*/
	public boolean rowsMovable(){
		return true;
	}
	/*.................................................................................................................*/
	public boolean rowsDeletable(){
		return true;
	}
	public boolean deleteRow(int row){
		if (taxa!=null && taxa.getNumTaxa() > 1) {
			taxa.deleteTaxa(row,1, false);
			return true;
		}
		return false;
	}
	/*.................................................................................................................*/
	public boolean rowsAddable(){
		return true;
	}
	public boolean addRow(CommandRecord commandRec){
		if (taxa!=null) {
			taxa.addTaxa(taxa.getNumTaxa(), 1, false);
			return true;
		}
		return false;
	}
	/** returns either a String (if not editable) or a MesquiteString (if to be editable) */
	public String getAnnotation(int row){
		if (taxa !=null) {
			return taxa.getAnnotation(row);
		}
		return null;
	}
	/** sets the annotation for a row*/
	public void setAnnotation(int row, String s, boolean notify){
		if (taxa !=null) {
			taxa.setAnnotation(row, s);
		}
	}
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) { 
   	 	if (getModuleWindow()==null || !getModuleWindow().isVisible())
   	 		return null;
   	 	Snapshot temp = new Snapshot();
		

  	 	temp.addLine("setTaxa " + getProject().getTaxaReference(taxa)); 
      	 	if (getModuleWindow()!=null)
			getModuleWindow().incorporateSnapshot(temp, file);
  	 	//if (getModuleWindow()!=null && !getModuleWindow().isVisible())
  	 		temp.addLine("showWindow"); 
 	 	return temp;
  	 }
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Sets the taxa block used", "[block reference, number, or name]", commandName, "setTaxa")) {
   	 		Taxa t = getProject().getTaxa(checker.getFile(), parser.getFirstToken(arguments));
   	 		if (getModuleWindow()!=null && t!=null){
	   	 		if (taxa!=null)
	   	 			taxa.removeListener(this);
	   	 		taxa = t;
	   	 		if (taxa!=null)
	   	 			taxa.addListener(this);
    	 			((TaxonListWindow)getModuleWindow()).setObject(taxa, commandRec);
    	 			((TaxonListWindow)getModuleWindow()).repaintAll();
	   	 		return taxa;
   	 		}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Hires utility module to operate on the taxa", "[name of module]", commandName, "doUtility")) {
   	 		if (taxa !=null){
	    	 		TaxonListUtility tda= (TaxonListUtility)hireNamedEmployee(commandRec, TaxonListUtility.class, arguments);
				if (tda!=null) {
					boolean a = tda.operateOnTaxa(taxa, commandRec);
					if (!tda.pleaseLeaveMeOn())
						fireEmployee(tda);
				}
			}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Hires utility module to alter names of the taxa", "[name of module]", commandName, "doNames")) {
  	 		if (taxa !=null && getModuleWindow() != null && ((TableWindow)getModuleWindow()).getTable()!=null){
	    	 		TaxonNameAlterer tda= (TaxonNameAlterer)hireNamedEmployee(commandRec, TaxonNameAlterer.class, arguments);
				if (tda!=null) {
					boolean a = tda.alterTaxonNames(taxa, ((TableWindow)getModuleWindow()).getTable(), commandRec);
					fireEmployee(tda);
					if (a)
						taxa.notifyListeners(this, new Notification(NAMES_CHANGED, null));
				}
			}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Returns the taxa block shown", null, commandName, "getTaxa")) {
    	 		return taxa;
    	 	}
    	 	else if (checker.compare(this.getClass(), "NOT FUNCTIONING", null, commandName, "saveSelectedRows")) {
    	 		((TaxonListWindow)getModuleWindow()).saveSelectedRows();
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
   	 }
	/*.................................................................................................................*/
    	 public String getName() {
		return "Taxon List";
   	 }
   	 
	/*.................................................................................................................*/
   	 
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Makes windows listing taxa and information about them." ;
   	 }
	/*.................................................................................................................*/
	/** Requests a window to close.  In the process, subclasses of MesquiteWindow might close down their owning MesquiteModules etc.*/
 	public void windowGoAway(MesquiteWindow whichWindow) {
			whichWindow.hide();
	}
   	 
   	 
}

/* ======================================================================== */
class TaxonListWindow extends ListWindow implements MesquiteListener {
	private Taxa taxa = null;
	TaxaSelectCoordinator selectionCoordinator;
	
	public TaxonListWindow (TaxonList ownerModule, CommandRecord commandRec) {
		super(ownerModule);

		taxa = ownerModule.taxa;
		if (taxa == null)
			taxa = ownerModule.getProject().getTaxa(0);
		ownerModule.hireAllEmployees(commandRec, TaxaTableAssistantI.class); 
 		Enumeration enum=ownerModule.getEmployeeVector().elements();
		while (enum.hasMoreElements()){
			Object obj = enum.nextElement();
			if (obj instanceof TaxaTableAssistantI) {
				TaxaTableAssistantI init = (TaxaTableAssistantI)obj;
				init.setTableAndTaxa(getTable(), taxa, true, commandRec);
			}
		}
		if (getTable() != null)
			getTable().setDropDown(-1, -1, true);
		selectionCoordinator = (TaxaSelectCoordinator)ownerModule.hireEmployee(commandRec, TaxaSelectCoordinator.class, null);
		if (selectionCoordinator!=null)
			selectionCoordinator.setTableAndObject(getTable(), taxa, true, commandRec);
	}
	/*.................................................................................................................*/
	/** When called the window will determine its own title.  MesquiteWindows need
	to be self-titling so that when things change (names of files, tree blocks, etc.)
	they can reset their titles properly*/
	public void resetTitle(){
		if (taxa==null || StringUtil.blank(taxa.getName()) || "untitled".equalsIgnoreCase(taxa.getName()))
			setTitle("Taxa"); 
		else
			setTitle("Taxa \"" + taxa.getName() + "\""); 
	}
	/*.................................................................................................................*/
	public Object getCurrentObject(){
		return taxa;
	}
	public void setCurrentObject(Object obj){
		if (obj instanceof Taxa) {
      	 			if (taxa!=null)
    	 				taxa.removeListener(this);
			taxa = (Taxa)obj;
			getTable().setRowAssociable(taxa);
  	 			if (taxa!=null)
    	 				taxa.addListener(this);
	 		Enumeration enum=ownerModule.getEmployeeVector().elements();
			while (enum.hasMoreElements()){
				Object objc = enum.nextElement();
				if (objc instanceof TaxaTableAssistantI) {
					TaxaTableAssistantI init = (TaxaTableAssistantI)objc;
					init.setTableAndTaxa(getTable(), taxa, true, CommandRecord.getRecNSIfNull());
				}
			}
			if (selectionCoordinator!=null)
				selectionCoordinator.setTableAndObject(getTable(), taxa, true, CommandRecord.getRecNSIfNull());
			getTable().synchronizeRowSelection(taxa);
			resetTitle();
		}
	}
	/*.................................................................................................................*/
	public void showSelectionPopup(Container cont, int x, int y) {
		if (selectionCoordinator!=null)
			selectionCoordinator.showPopUp(cont, x+5, y+5);
	}
	public String getItemTypeName(){
		return "Taxon";
	}
	public void setRowName(int row, String name, CommandRecord commandRec){
		if (taxa!=null) {
			String warning = taxa.checkNameLegality(row, name);
			if (warning == null)
				taxa.setTaxonName(row, name);
			else
				ownerModule.discreetAlert(commandRec, warning);
		}
	}
	public String getRowName(int row){
		if (taxa!=null)
			return taxa.getTaxonName(row);
		else
			return null;
	}
	public void saveSelectedRows() {
		if (table.anyRowSelected()) {
			String nameOfTaxonSet = MesquiteString.queryString(this, "Name of taxon set", "Name of taxon set:", "Stored taxon set");
	 		if (StringUtil.blank(nameOfTaxonSet))
	 			return;
	 		TaxaSelectionSet selectionSet= new TaxaSelectionSet(nameOfTaxonSet, taxa.getNumTaxa(), taxa);
	 		
	 		taxa.storeSpecsSet(selectionSet, TaxaSelectionSet.class);
	 		
	 		selectionSet.addToFile(taxa.getFile(), null, null);
	 		selectionSet.setNexusBlockStored("SETS");
			for (int ic= 0; ic< table.getNumRows(); ic++){
				if (table.isRowSelected(ic)) { 
					selectionSet.setSelected(ic, true);
				}
			}
	 		taxa.notifyListeners(this, new Notification(AssociableWithSpecs.SPECSSET_CHANGED));
		}
	}
	/*.................................................................................................................*/
	/** passes which object is being disposed (from MesquiteListener interface)*/
	public void disposing(Object obj){
		if (obj instanceof Taxa &&  (Taxa)obj ==taxa && ownerModule !=null)
			ownerModule.windowGoAway(this);
	}
	/*.................................................................................................................*/
	/** passes which object is being disposed (from MesquiteListener interface)*/
	public boolean okToDispose(Object obj, int queryUser){
		return true;  //TODO: respond
	}
	/*.................................................................................................................*/
	/** passes which object changed*/
	public void changed(Object caller, Object obj, Notification notification, CommandRecord commandRec){
		int code = Notification.getCode(notification);
		if (obj instanceof Taxa && (Taxa)obj ==taxa) {
			if (code==MesquiteListener.NAMES_CHANGED) {
				table.redrawRowNames();
			}
			else if (code==MesquiteListener.SELECTION_CHANGED) {
				table.synchronizeRowSelection(taxa);
				table.repaintAll();
			}
			else if (code==MesquiteListener.PARTS_ADDED || code==MesquiteListener.PARTS_DELETED || code==MesquiteListener.PARTS_CHANGED || code==MesquiteListener.PARTS_MOVED) {
				table.setNumRows(taxa.getNumTaxa());
				table.synchronizeRowSelection(taxa);
				table.repaintAll();
			}
			else {
				table.repaintAll();
			}
		}
		super.changed(caller, obj, notification, commandRec);
	}
	
}

