/* Mesquite (package mesquite.lists).  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lists.lib;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import mesquite.lib.duties.*;
import mesquite.lib.*;
import mesquite.lib.table.*;


/* ======================================================================== */
public class ListTable extends MesquiteTable {
	ListWindow window;
	LTCellAnnotation cellAnnotatable;
	ListModule ownerModule;
	public ListTable (int numRowsTotal, int numColumnsTotal, int totalWidth, int totalHeight, int columnNamesWidth, ListWindow window, ListModule ownerModule) {
		super(numRowsTotal, numColumnsTotal, totalWidth, totalHeight, columnNamesWidth, window.getColorScheme(), true,false);
		this.window = window;
		cellAnnotatable = new LTCellAnnotation(ownerModule, window);
		this.ownerModule = ownerModule;
		showRowGrabbers=true;
		showColumnGrabbers=true;
		cornerIsHeading = true;
		//setAutoEditable(false, true, false, false);
		setEditable(false, true, false, false);
		setSelectable(false, true, true, true, false, false);
		setSelectable(true, true, true, true, true, false);
		setUserMove(true, false);
	}
	
	/*...............................................................................................................*/
	public void selectedRowsDropped(int after){
		if (window.owner.rowsMovable()) {
			if (window.getCurrentObject() !=null && window.getCurrentObject() instanceof Associable){
				Bits b = getRowsSelected();
				
				int i = 0;
				while (i<getNumRows()){
					if (b.isBitOn(i)) {
						deselectRow(i);
						b.clearBit(i);
						((Associable)window.getCurrentObject()).moveParts(i,1,after);
						b.moveParts(i, 1, after);
						if (i>after)
							after++;
						i=0;
					}
					else
						i++;
				}
				
				/*
				for (int i=0; i<getNumRows(); i++)
					if (isRowSelected(i)) {
						((Associable)window.getCurrentObject()).moveParts(i,1,after++);//TODO: not correct! need to adjust and see what new "after" is
					}
				*/
				synchronizeRowSelection(((Associable)window.getCurrentObject()));
				((Associable)window.getCurrentObject()).notifyListeners(this, new Notification(MesquiteListener.PARTS_MOVED));
		 		repaintAll();
	 		}
 		}
	}
	public String getCornerText(){  
		if (getNumRows()==0)
			return "No " + window.owner.getItemTypeNamePlural();
		return window.owner.getItemTypeName();
	}
	/*...............................................................................................................*/
	/** Gets whether the cell is dimmed (true) or not (false).  Column -1 is for row names; row -1 is for column names.*/
	public boolean getCellDimmed(int column, int row){
		if (column == -1 && (row == -1))
				return getNumRows()<=0;
		return super.getCellDimmed(column, row);
	}
	/*...............................................................................................................*/
	/** returns whether or not a cell of table is editable.*/
	public boolean checkCellEditable(int column, int row){
		return super.isCellEditable(column, row);
	}
	/*...............................................................................................................*/
	/** returns whether or not a row name of table is editable.*/
	public boolean checkRowNameEditable(int row){
		return super.isRowNameEditable(row);
	}
	/*...............................................................................................................*/
	/** returns whether or not a column name of table is editable.*/
	public boolean checkColumnNameEditable(int column){
		return super.isColumnNameEditable(column);
	}
	/*...............................................................................................................*/
	/** returns whether or not a cell of table is editable.*/
	public boolean isCellEditable(int column, int row){
		return window.isCellEditable(column, row);
	}
	/*...............................................................................................................*/
	/** returns whether or not a row name of table is editable.*/
	public boolean isRowNameEditable(int row){
		return window.isRowNameEditable(row);
	}
	/*...............................................................................................................*/
	/** returns whether or not a column name of table is editable.*/
	public boolean isColumnNameEditable(int column){
		return window.isColumnNameEditable(column);
	}
	public boolean useString(int column, int row){  
		ListAssistant assistant = window.findAssistant(column);
		if (assistant!=null) {
			return (assistant.useString(row));
		}
		return false;
	}
	public String getMatrixText(int column, int row){  
		ListAssistant assistant = window.findAssistant(column);
		if (assistant!=null) {
			try{
				return assistant.getStringForRow(row);
			}
			catch (NullPointerException e){
			}
		}
		return "?";
	}
	public void drawMatrixCell(Graphics g, int x, int y,  int w, int h, int column, int row, boolean selected){  
		ListAssistant assistant = window.findAssistant(column);
		if (assistant!=null) {
			String s = "";
			try{
				s = assistant.getStringForRow(row); 
			}
			catch (NullPointerException e){
			}
			if (assistant.useString(row) && s!=null) {
				FontMetrics fm = getFontMetrics(getFont());
				int sw = fm.stringWidth(s);
				int sh = fm.getMaxAscent()+ fm.getMaxDescent();
				try{
					Color old = g.getColor();
					Color c = null;
					if (selected)
						c = Color.black;
					else if (isCellEditable(column, row))
						c = Color.cyan;
					else
						c = ColorDistribution.uneditable;
					if (c!=null) g.setColor(c);
					g.fillRect(x, y, w, h);
					if (selected)
						g.setColor(Color.red);
					else
						g.setColor(Color.black);
					g.drawString(assistant.getStringForRow(row), x+(w-sw)/2, y+h-(h-sh)/2);
					g.setColor(old);
					
				}
				catch (NullPointerException e){
					MesquiteMessage.printStackTrace("NPE");
				}
			}
			else
				assistant.drawInCell(row, g, x, y, w, h, selected);
		}
	}
	
	/*public void drawColumnNameCell(Graphics g, int x, int y, int w, int h, int column){
		ListAssistant assistant = window.findAssistant(column);
		if (assistant!=null)
			g.drawString(assistant.getTitle(), x+getNameStartOffset(), y+h-2);
	}
	*/
	public void drawRowNameCell(Graphics g, int x, int y,  int w, int h, int row){
		String s = window.getRowName(row);
		if (s == null)
			return;
		Shape clip = g.getClip();
		g.setClip(x,y,w,h);
		if (window.rowHighlighted(row)){
			Color c = g.getColor();
			g.setColor(ColorDistribution.straw);
			g.fillRect(x,y,w,h);
			if (c!=null) g.setColor(c);
		}

		Object wo = ownerModule.getAnnotation(row); 
		if (wo != null && (wo instanceof String || wo instanceof MesquiteString)){
			if (wo instanceof String && !StringUtil.blank((String)wo))
				s = "*" + s;
			else if (wo instanceof MesquiteString && !StringUtil.blank(((MesquiteString)wo).getValue()))
				s = "*" + s;
		}

		int gnso = x+getNameStartOffset();
		g.drawString(s, gnso, y+h-4);
		g.setClip(clip);
	}
	/*...............................................................................................................*/
	public void cellTouched(int column, int row, int modifiers, int clickCount) {
		window.setAnnotation("", null);
		if (!window.interceptCellTouch(column, row, modifiers)){
			if (window.getCurrentTool()== window.arrowTool)  {
				rowTouched(row, modifiers);
			}
			else
				((TableTool)window.getCurrentTool()).cellTouched(column, row, modifiers);
		}
	}
	/*...............................................................................................................*/
	public void cellDrag(int column, int row, int modifiers) {
		if (window.getCurrentTool()== window.arrowTool) 
			super.cellDrag(column, row, modifiers);
		else
		((TableTool)window.getCurrentTool()).cellDrag(column, row, modifiers);
	}
	/*...............................................................................................................*/
	public void cellDropped(int column, int row, int modifiers) {
		/*
		if (window.getCurrentTool()== window.arrowTool) 
			rowTouched(row, modifiers);
		else  */
		   	((TableTool)window.getCurrentTool()).cellDropped(column, row, modifiers);
	}
	public void rowTouched(int row, int modifiers) {
		super.rowTouched( row,  modifiers);
		showAnnotationAndExplanation(row);
	}
	private void showAnnotationAndExplanation(int row){
		if (row>=0){
			cellAnnotatable.setRow(row);
			window.setAnnotation(cellAnnotatable); 
		}
		else
			window.setAnnotation("", null); 
	}
	public void setFocusedCell(int column, int row){
		showAnnotationAndExplanation(row);
		super.setFocusedCell(column, row);
	}
	/*...............................................................................................................*/
	public void rowNameTouched(int row, int modifiers, int clickCount) {
		showAnnotationAndExplanation(row);
		if (window.getCurrentTool()== window.arrowTool) {
			if (!window.interceptRowNameTouch(row, modifiers)) {
				if (clickCount>1 && isRowNameEditable(row)){
					window.setCurrentTool(window.ibeamTool);
					window.getPalette().setCurrentTool(window.ibeamTool); 
					((TableTool)window.getCurrentTool()).cellTouched(-1, row, modifiers);
				}
				else {
					super.rowNameTouched(row, modifiers,clickCount);
				}
			}
		}
		else
		   ((TableTool)window.getCurrentTool()).cellTouched(-1, row, modifiers);
		 
	}
	public void superRowNameTouched(int row, int modifiers, int clickCount){
		super.rowNameTouched(row, modifiers,clickCount);
	}
	/*...............................................................................................................*/
	public boolean getDropDown(int column, int row){
		if (row != -1)
			return false;
		if (column==-1)
			return super.getDropDown(column, row);
		ListAssistant assistant = window.findAssistant(column);
		if (assistant!=null)
			return assistant.needsMenu();
		else
			return false;
	}
	/*...............................................................................................................*/
	public void columnNameTouched(int column, int modifiers, int clickCount) {
		window.setAnnotation("", null);
		ListAssistant assistant = window.findAssistant(column);

		if (assistant!=null)
			assistant.showPopUp(getColumnNamesPanel(), getLeftOfColumn(column), getColumnNamesHeight());
	}
	/*...............................................................................................................*/
	/** Called if corner panel is touched.  Can be overridden in subclasses to respond to touch.*/
   	public void cornerTouched(int x, int y, int modifiers) {
			window.setAnnotation("", null);
   		super.cornerTouched(x,y,modifiers);
		window.showSelectionPopup(cornerCell, x, y);
   	}
	/*...............................................................................................................*/
	public void mouseInCell(int modifiers, int column,int row, MesquiteTool tool){
		if (row == -1 && column>=0){ //column names
			ListAssistant assistant = window.findAssistant(column);
			if (assistant !=null){
				String ex = assistant.getName() + "\n" + assistant.getExplanation();
				String par = assistant.accumulateParameters("   ");
				if (!StringUtil.blank(par))
					ex += "\nParameters:\n" + par;
				window.setExplanation(ex);
			}
		}
		else if (row >=0 && column>=0){ //internal cell
			ListAssistant assistant = window.findAssistant(column);
			if (assistant !=null){
				String ex = assistant.getExplanationForRow(row);
				if (StringUtil.blank(ex))
					ex = assistant.getName();
				window.setExplanation(ex);
			}
		}
	}
	/*...............................................................................................................*/
	public void mouseExitedCell(int modifiers, int column,int row, MesquiteTool tool){
		if (row == -1 && column>=0){//column names
				window.setExplanation("");
		}
	}
	/*...............................................................................................................*/
	/** Returns text in row name.  */
	public String getColumnNameText(int column){
		ListAssistant assistant = window.findAssistant(column);
		if (assistant!=null)
			return assistant.getTitle();
		return "";
	}
	/*...............................................................................................................*/
	/** Returns text in row name.  */
	public String getRowNameText(int row){
			return window.getRowName(row);
	}
	/*...............................................................................................................*/
	public void returnedRowNameText(int row, String s, CommandRecord commandRec){
		if (s!=null && !s.equals(window.getRowName(row)))
			window.setRowName(row, s, commandRec);
	}
}

/* ======================================================================== */
class LTCellAnnotation implements Annotatable {
	ListModule ownerModule;
	int row;
	ListWindow window;
	public LTCellAnnotation(ListModule ownerModule, ListWindow window) {
		this.ownerModule = ownerModule;
		this.window = window;
	}
	
	void setRow(int r){
		row = r;
	}
	public String getName(){
		if (ownerModule == null)
			return "";
		 return ownerModule.getItemTypeName() + " \"" + window.getRowName(row) + "\"" ;
	}
	public String getAnnotation(){
		if (ownerModule == null)
			return null;
		return ownerModule.getAnnotation(row);
	}
	public void setAnnotation(String s, boolean notify){
		if (ownerModule == null)
			return;
		ownerModule.setAnnotation(row, s, notify);
	}
 }
