/* Mesquite (package mesquite.lists).  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.lists.lib;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import mesquite.lib.duties.*;
import mesquite.lib.*;
import mesquite.lib.table.*;
import java.text.Collator;

/* ======================================================================== */
public abstract class ListWindow extends TableWindow implements KeyListener {
	protected ListTable table;
	private int windowWidth=360;
	private int windowHeight=400;
	ListModule owner;
	TableTool arrowTool, ibeamTool, wandTool, sortTool;
	MesquitePopup selectionPopup;
	MesquiteBoolean equals, greaterThan, lessThan;
	boolean defaultEquals = true;
	boolean defaultGT = false;
	boolean defaultLT = false;
	Collator collator;
	MesquiteCommand deleteCommand;
	
	/** constructor to be used only for accumulating list of commands*/
	public ListWindow () {
	}
	public ListWindow (ListModule ownerModule) {
		super(ownerModule, true); //INFOBAR
		equals = new MesquiteBoolean(defaultEquals);
		greaterThan = new MesquiteBoolean(defaultGT);
		lessThan = new MesquiteBoolean(defaultLT);
		setWindowSize(windowWidth, windowHeight);
		ownerModule.setModuleWindow(this);
		owner = ownerModule;
		int numItems = owner.getNumberOfRows();
		table = new ListTable (numItems, 0, windowWidth, windowHeight, 130, this, ownerModule);
		setCurrentObject(owner.getMainObject());
		collator = Collator.getInstance();
		
		//TODO:  note: strangely enough, the commands of the arrow tools are not used, since it uses default behavior of table
		String selectExplanation = "Select items in table.  If a single cell in the table is touched, and that cell is editable, then you will be given a little box in which you can edit the ";
		selectExplanation += "contents of the cell.  By holding down shift while clicking, the selection will be extended from the first to the last touched cell. ";
		selectExplanation += " A block of cells can be selected either by using shift-click to extend a previous selection, or by clicking on a cell and dragging with the mouse button still down";
		selectExplanation += " Discontinous selections are allowed, and can be obtained by a \"meta\"-click (right mouse button click, or command-click on a MacOS system). ";
		arrowTool = new TableTool(this, "arrow", MesquiteModule.getRootImageDirectoryPath(),"arrow.gif", 4,2,"Select", selectExplanation, null, null, null); //MesquiteModule.makeCommand("arrowTouchCell",  this) , MesquiteModule.makeCommand("arrowDragCell",  this), MesquiteModule.makeCommand("arrowDropCell",  this));
		arrowTool.setIsArrowTool(true);
		addTool(arrowTool);
  	 	setCurrentTool(arrowTool);
		ibeamTool = new TableTool(this, "ibeam", MesquiteModule.getRootImageDirectoryPath(),"ibeam.gif", 7,7,"Edit", "This tool can be used to edit elements in the table.", MesquiteModule.makeCommand("editCell",  (Commandable)table) , null, null);
		ibeamTool.setWorksOnRowNames(true);
		ibeamTool.setWorksOnMatrixPanel(false);
		addTool(ibeamTool);
		String wandExplanation = "Select rows in table according to whether they have the same value as the cell touched in that column.  ";
		wandExplanation += "By holding down shift while clicking, the selection will be added to the previous selection. ";
		wandTool = new TableTool(this, "wand", MesquiteModule.getRootImageDirectoryPath(),"wand.gif", 4,4,"Select rows with same", wandExplanation, MesquiteModule.makeCommand("wandTouch",  this) , null, null);
		wandTool.setOptionsCommand(MesquiteModule.makeCommand("wandOptions", this));
		addTool(wandTool);

		if (owner.rowsMovable()){
			String sortExplanation = "Sorts items (that is, rows) in table from lowest to highest values in the column touched. ";
			sortExplanation += "By holding down option while clicking, items will be sorted from highest to lowest. ";
			sortTool = new TableTool(this, "sort", MesquiteModule.getRootImageDirectoryPath(),"sort.gif", 4,4,"Sort rows", sortExplanation, MesquiteModule.makeCommand("sortTouch",  this) , null, null);
			sortTool.setOptionImageFileName("revSort.gif", 8, 13);
			sortTool.setWorksOnRowNames(true);
			addTool(sortTool);
		}

		//System.out.println("number of chars " + numItems);
		table.setColumnWidthsUniform(100);
		table.setUserAdjust(MesquiteTable.RESIZE, MesquiteTable.RESIZE);
 		addToWindow(table);
 		table.setVisible(true);
		table.setLocation(0,0);
		
 		MesquiteSubmenuSpec mss = ownerModule.addSubmenu(null, "Show Column", ownerModule.makeCommand("newAssistant", this),owner.getAssistantClass());
		mss.setCompatibilityCheck(ownerModule.getAddColumnCompatibility());
		if (columnsRemovable())
 			ownerModule.addMenuItem( "Hide Selected Columns", ownerModule.makeCommand("deleteSelectedColumns", this));
 		//MesquiteSubmenuSpec mss = ownerModule.addSubmenu(null, "Add column", ownerModule.makeCommand("newAssistant", this));
		//mss.setList(owner.getAssistantClass());
 		ownerModule.addMenuItem( "-", null);
  		if (owner.rowsAddable())
 			ownerModule.addMenuItem( "Add " + owner.getItemTypeNamePlural() + "...", ownerModule.makeCommand("addRows", this));
 		if (owner.rowsDeletable()) {
 			ownerModule.addMenuItem( "Delete Selected " + owner.getItemTypeNamePlural(), deleteCommand = ownerModule.makeCommand("deleteSelectedRows", this));
			MesquiteWindow.addKeyListener(this, this);
		}
		setShowExplanation(true);
		setShowAnnotation(true);
		table.requestFocus();
		MesquiteWindow.addKeyListener(this, table);
		resetTitle();
	}
	
	public void keyTyped(KeyEvent e){
		int mod = MesquiteEvent.getModifiers(e);
		if (!MesquiteEvent.commandOrControlKeyDown(mod)) {
			if (getMode() != InfoBar.GRAPHICS)
				return;
			
			if (!owner.rowsDeletable())
				return;
			if ((table == null || (table.anythingSelected() && !table.editingAnything())) && (e.getKeyChar()== '\b' || e.getKeyCode()== KeyEvent.VK_BACK_SPACE || e.getKeyCode()== KeyEvent.VK_DELETE)) {
				//deleteSelectedRows(false, CommandRecord.nonscriptingRecord);
				deleteCommand.doItMainThread(null, "", this);
			}
		}
		
	}
	public void keyPressed(KeyEvent e){
	}
	
	public void keyReleased(KeyEvent e){
	}
	/*.................................................................................................................*/
	/** When called the window will determine its own title.  MesquiteWindows need
	to be self-titling so that when things change (names of files, tree blocks, etc.)
	they can reset their titles properly*/
	public abstract void resetTitle();
	
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) { 
   	 	Snapshot temp = new Snapshot();
		if (equals.getValue()!=defaultEquals)
			temp.addLine("toggleEquals " + equals.toOffOnString());
		if (greaterThan.getValue()!=defaultGT)
			temp.addLine("toggleGT " + greaterThan.toOffOnString());
		if (lessThan.getValue()!=defaultLT)
			temp.addLine("toggleLT " + lessThan.toOffOnString());
		
		for (int i = 0; i<ownerModule.getNumberOfEmployees(); i++) { //if employee is number for character list, then hire indirectly its 
			MesquiteModule e=(MesquiteModule)ownerModule.getEmployeeVector().elementAt(i);
			if (e instanceof ListAssistant) {
				temp.addLine("newAssistant ", e);
			}
		}
		temp.incorporate(super.getSnapshot(file), false);
 	 	return temp;
  	 }
	public abstract void setRowName(int row, String name, CommandRecord commandRec);
	public abstract String getRowName(int row);
	public boolean columnsRemovable(){
		return true;
	}
	/*...............................................................................................................*/
	/** returns whether or not a cell of table is editable.*/
	public boolean isCellEditable(int column, int row){
		return table.checkCellEditable(column, row);
	}
	/*...............................................................................................................*/
	/** returns whether or not a row name of table is editable.*/
	public boolean isRowNameEditable(int row){
		return table.checkRowNameEditable(row);
	}
	/*...............................................................................................................*/
	/** returns whether or not a column name of table is editable.*/
	public boolean isColumnNameEditable(int column){
		return table.checkColumnNameEditable(column);
	}
	/** Returns menu location for item to bring the window to the for (0 = custom or don't show; 1 = system area of Windows menu; 2 = after system area of Windows menu)*/
	public int getShowMenuLocation(){
		return 0;
	}
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Hides the list window", null, commandName, "hideWindow")) {
   	 		hide();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Deletes the selected columns", null, commandName, "deleteSelectedColumns")) {
    	 		deleteSelectedColumns();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Present the popup menu to select options for magic wand", null, commandName, "wandOptions")) {
    			MesquiteButton button = wandTool.getButton();
     			if (button!=null){
	   	 		MesquiteInteger io = new MesquiteInteger(0);
	   			int x= MesquiteInteger.fromString(arguments, io); //getting x and y from arguments
	   			int y= MesquiteInteger.fromString(arguments, io);
	     			MesquitePopup popup = new MesquitePopup(button);
				MesquiteCheckMenuItem equalsItem = new MesquiteCheckMenuItem("Equal", ownerModule, MesquiteModule.makeCommand("toggleEquals", this), null, null);
				equalsItem.set(equals.getValue());
				popup.add(equalsItem);
				MesquiteCheckMenuItem greaterThanItem = new MesquiteCheckMenuItem("Greater than", ownerModule, MesquiteModule.makeCommand("toggleGT", this), null, null);
				greaterThanItem.set(greaterThan.getValue());
				popup.add(greaterThanItem);
				MesquiteCheckMenuItem lessThanItem = new MesquiteCheckMenuItem("Less than", ownerModule, MesquiteModule.makeCommand("toggleLT", this), null, null);
				lessThanItem.set(lessThan.getValue());
				popup.add(lessThanItem);
				popup.showPopup(x,y+6);
   			}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets whether the wand selects taxa with value equal to that in cell touched", "[on = selects equal; off]", commandName, "toggleEquals")) {
    	 		boolean current = equals.getValue();
 	   	 	MesquiteInteger io = new MesquiteInteger(0);
   	 		equals.toggleValue(ParseUtil.getFirstToken(arguments, io));
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets whether the wand selects taxa with value greater than that in cell touched", "[on = selects greater than; off]", commandName, "toggleGT")) {
    	 		boolean current = greaterThan.getValue();
	   	 		MesquiteInteger io = new MesquiteInteger(0);
    	 		greaterThan.toggleValue(ParseUtil.getFirstToken(arguments, io));
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets whether the wand selects taxa with value less than that in cell touched", "[on = selects less than; off]", commandName, "toggleLT")) {
    	 		boolean current = lessThan.getValue();
	   	 		MesquiteInteger io = new MesquiteInteger(0);
    	 		lessThan.toggleValue(ParseUtil.getFirstToken(arguments, io));
    	 	}
    	 	else if (checker.compare(this.getClass(), "Applies the magic wand tool to select like values", "[column touched] [row touched]", commandName, "wandTouch")) {
	   	 		MesquiteInteger io = new MesquiteInteger(0);
	   			int column= MesquiteInteger.fromString(arguments, io);
	   			int row= MesquiteInteger.fromString(arguments, io);
				if (MesquiteInteger.isNonNegative(column)&& (MesquiteInteger.isNonNegative(row))) {
					if (getCurrentObject() instanceof Associable){
						Associable assoc = (Associable)getCurrentObject();
			   			if (arguments.indexOf("shift")<0)
			   				assoc.deselectAll();
			   			table.offAllEdits();
						String text = table.getMatrixText(column, row);
						for (int i=0; i<table.getNumRows(); i++){
							if (satisfiesCriteria(text, table.getMatrixText(column, i))) {
								assoc.setSelected(i, true);
							}
						}
						assoc.notifyListeners(this, new Notification(MesquiteListener.SELECTION_CHANGED), commandRec);
					}
					else {
			   			if (arguments.indexOf("shift")<0)
			   				table.deselectAll();
			   			table.offAllEdits();
						String text = table.getMatrixText(column, row);
						for (int i=0; i<table.getNumRows(); i++){
							if (satisfiesCriteria(text, table.getMatrixText(column, i))) {
								table.selectRow(i);
							}
						}
						table.repaintAll();
					}
				}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sorts rows", "[column touched] [row touched]", commandName, "sortTouch")) {
				if (!(getCurrentObject() instanceof Associable))
					return null;
				Associable assoc = (Associable)getCurrentObject();
	   	 		MesquiteInteger io = new MesquiteInteger(0);
	   			boolean gT = true;
    				if (arguments.indexOf("option")>=0)
    					gT = false;
	   			int column= MesquiteInteger.fromString(arguments, io);
	   			int row= MesquiteInteger.fromString(arguments, io);
	   			if (!MesquiteInteger.isCombinable(column) && !MesquiteInteger.isCombinable(row))
	   				return null;
				if (column>=0 && row >=0) {
					String[] text = new String[assoc.getNumberOfParts()];
					for (int i=0; i<assoc.getNumberOfParts(); i++) {
						text[i] = table.getMatrixText(column, i);
					}
					for (int i=1; i<assoc.getNumberOfParts(); i++) {
						for (int j= i-1; j>=0 && compare(gT, text[j], text[j+1]); j--) {
							swapParts(assoc, j, j+1, text);
						}
					}
					assoc.notifyListeners(this, new Notification(MesquiteListener.PARTS_MOVED), commandRec);
				}
				else if (column == -1 && row >=0) { //row names selected; sort by taxon name
					String[] text = new String[assoc.getNumberOfParts()];
					for (int i=0; i<assoc.getNumberOfParts(); i++) {
						text[i] = getRowName(i);
					}
					for (int i=1; i<assoc.getNumberOfParts(); i++) {
						for (int j= i-1; j>=0 && compare(gT, text[j], text[j+1]); j--) {
							swapParts(assoc, j, j+1, text);
						}
					}
					assoc.notifyListeners(this, new Notification(MesquiteListener.PARTS_MOVED), commandRec);
				}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Deletes the selected rows", null, commandName, "deleteSelectedRows")) {
    	 			deleteSelectedRows(true, commandRec);
    	 	}
    	 	else if (checker.compare(this.getClass(), "Adds rows (and their corresponding objects)", "[number of rows to add]", commandName, "addRows")) {
	   	 		MesquiteInteger io = new MesquiteInteger(0);
	   			int num= MesquiteInteger.fromString(arguments, io);
				if (!MesquiteInteger.isCombinable(num))
					num = MesquiteInteger.queryInteger(this, "Add", "Add how many?", 1);
				if (MesquiteInteger.isCombinable(num))
					addRows(num, commandRec);
    	 	}
    	 	else if (checker.compare(this.getClass(), "Selects a row", "[number of row]", commandName, "selectRow")) { //TODO: should this use internal or external character numbers???
    	 		selectRow(MesquiteInteger.fromString(arguments));
    	 	}
    	 	else if (checker.compare(this.getClass(), "Selects a column", "[number of column]", commandName, "selectColumn")) {
    	 		selectColumn(MesquiteInteger.fromString(arguments));
    	 	}
    	 	else if (checker.compare(this.getClass(), "Hires a list assistant module", "[name of module]", commandName, "newAssistant")) {
    	 		ListAssistant assistant= (ListAssistant)ownerModule.hireNamedEmployee(commandRec, owner.getAssistantClass(), arguments);
    	 		if (assistant!=null) {
    	 			assistant.setUseMenubar(false);
 				addListAssistant(assistant, commandRec);
 				repaintAll();
 				ownerModule.resetContainingMenuBar();
 			}
 			else {
				for (int i = 0; i<ownerModule.getNumberOfEmployees(); i++) { //if already hired then pass along commands
					MesquiteModule e=(MesquiteModule)ownerModule.getEmployeeVector().elementAt(i);
					if (e instanceof ListAssistant && owner.getAssistantClass().isAssignableFrom(e.getClass())) {
						if (e.nameMatches(ParseUtil.getToken(arguments, new MesquiteInteger(0))) && !e.canHireMoreThanOnce())
							return assistant;
					}
				}
 			}
 			return assistant;
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
   	 }
  	 boolean compare(boolean greaterThan, String one, String two){
  	 	if (one == null || two == null)
  	 		return false;
		double dOne = MesquiteDouble.fromString(one);
		double dTwo = MesquiteDouble.fromString(two);
		if (MesquiteDouble.isCombinable(dOne) && MesquiteDouble.isCombinable(dTwo)) {
			if (greaterThan && (dTwo<dOne))
				return true;
			if (!greaterThan && (dTwo>dOne))
				return true;
			return false;
		}
		int order = collator.compare(one, two);
		if ((order == 1 && greaterThan) ||(order == -1 && !greaterThan))
			return true;
		return false;
  	 }
	/*.................................................................................................................*/
 	/** Swaps parts first and second*/
	void swapParts(Associable assoc, int first, int second, String[] text){  
  		String temp = text[first];
  		text[first] = text[second];
  		text[second] = temp;
		assoc.swapParts(first, second); 
	}
  	 boolean satisfiesCriteria(String one, String two){
		if (equals.getValue() && one.equals(two))
			return true;
		double dOne = MesquiteDouble.fromString(one);
		double dTwo = MesquiteDouble.fromString(two);
		if (MesquiteDouble.isCombinable(dOne) && MesquiteDouble.isCombinable(dTwo)) {
			if (greaterThan.getValue() && (dTwo>dOne))
				return true;
			if (lessThan.getValue() && (dTwo<dOne))
				return true;
			return false;
		}
		int order = collator.compare(one, two);
		if (greaterThan.getValue() && (order == -1))
			return true;
		if (lessThan.getValue() && (order == 1))
			return true;
		if (equals.getValue() && (order == 0))
			return true;
		return false;
  	 }
	public void addListAssistant(ListAssistant assistant, CommandRecord commandRec) {
		if (assistant !=null) {
			assistant.setTableAndObject(table, getCurrentObject(), commandRec);
			table.setNumColumns(table.getNumColumns()+1);
			if (!table.columnAdjusted(table.getNumColumns()-1)) {
				int w = assistant.getColumnWidth();
				if (w==0) {
					FontMetrics fm = table.getFontMetrics(table.getFont());
					if (fm!=null)
						w = fm.stringWidth(assistant.getWidestString());
					else
						w = 100;  // should something else be used as default?
				}
				table.setColumnWidth(table.getNumColumns()-1, w);
			}
		}
	}
	
	public void reviewColumnWidths() {
			ListAssistant assistant=null;
			int column = 0;
			for (int i=0; i< ownerModule.getNumberOfEmployees(); i++) {
				Object obj =  ownerModule.getEmployeeVector().elementAt(i);
				if (obj instanceof ListAssistant) {
					if (!table.columnAdjusted(column)) {
						assistant = (ListAssistant)obj;
						int w = assistant.getColumnWidth();
						if (w==0) {
							FontMetrics fm = table.getFontMetrics(table.getFont());
							w = fm.stringWidth(assistant.getWidestString()+table.getNameStartOffset()*2);
						}
						if (w!=table.getColumnWidth(column))
							table.setColumnWidth(column, w);
					}
					column++;
				}
			}	
	}
	public ListAssistant findAssistant(int index) {
			ListAssistant assistant=null;
			int num=0;
			if (ownerModule == null || ownerModule.getEmployeeVector() == null)
				return null;
			for (int i=0; i< ownerModule.getNumberOfEmployees(); i++) {
				Object obj =  ownerModule.getEmployeeVector().elementAt(i);
				if (obj instanceof ListAssistant) {
					if (num== index)
						return (ListAssistant)obj;
					num++;
				}
			}	
		return null;
	}
	
	public int findAssistant(ListAssistant mb) {
		int num=0;
		for (int i=0; i< ownerModule.getNumberOfEmployees(); i++) {
			Object obj =  ownerModule.getEmployeeVector().elementAt(i);
			if (obj instanceof ListAssistant) {
				if (mb == obj)
					return num;
				num++;
			}
		}	
		return -1;
	}
	public void blankColumn(MesquiteModule mb){
		if (mb instanceof ListAssistant) {
			int which = findAssistant((ListAssistant)mb);
			if (which>=0)
				table.blankColumn(which);
		}
	}
	public boolean rowHighlighted(int row) {
		return false;
	}
	public void showSelectionPopup(Container cont, int x, int y) {
	}
	public void selectRow(int row) {
		if (getCurrentObject() instanceof Associable){
			((Associable)getCurrentObject()).setSelected(row, true);
			((Associable)getCurrentObject()).notifyListeners(this, new Notification(MesquiteListener.SELECTION_CHANGED));
		}
		else {
			table.selectRow(row);
		}
	}
	
	public boolean isRowSelected(int row){
		return table.isRowSelected(row);
	}
	
	public void selectColumn(int column) {
		table.selectColumn(column);
	}
	/*.................................................................................................................*/
	public void setCurrentTool(MesquiteTool tool){
		super.setCurrentTool(tool);
		if ((getCurrentTool()!=ibeamTool) && (table!=null))
			table.offAllEdits();
   	 }
	/*.................................................................................................................*/
	public boolean interceptCellTouch(int  column, int row, int modifiers){
		return false;
	}
	public boolean interceptRowNameTouch(int row, int modifiers){
		return false;
	}
	public void deleteSelectedColumns() {
		int numSelected = 0;
		for (int ic = table.getNumColumns()-1; ic>=0; ic--){
			if (table.isColumnSelected(ic)) { 
				numSelected++;
			}
		}
		if (numSelected>0) {

			int[] which = new int[numSelected];
			int whichIs = 0;
			for (int ic = table.getNumColumns()-1; ic>=0; ic--){
				if (table.isColumnSelected(ic)) { //columns selected get deleted!
					which[whichIs]=ic;
					whichIs++;
				}
			}
			for (int ic = 0; ic<numSelected; ic++){
				table.deleteColumn(which[ic]);
				ListAssistant assistant = findAssistant(which[ic]);
				ownerModule.fireEmployee(assistant);
			}
			table.repaintAll();
		}
		else
			ownerModule.alert("Columns must be selected before \"Hide\" command is given");
	}
	
	public void addRows(int num, CommandRecord commandRec) {
		MenuOwner.incrementMenuResetSuppression();
		int count =0;
		int currentNumRows = owner.getNumberOfRows();
		for (int ic = 0; ic<num; ic++){
	 		if (owner.addRow(commandRec))
				count++;
		}
		table.setNumRows(currentNumRows+count);
		Object obj = getCurrentObject();
		if (obj instanceof Listenable)
			((Listenable)obj).notifyListeners(this, new Notification(MesquiteListener.PARTS_ADDED), commandRec);
		table.repaintAll();
		
		MenuOwner.decrementMenuResetSuppression();
		
	}
	public void deleteSelectedRows(boolean byCommand, CommandRecord commandRec) {
		int numSelected = 0;
		for (int ic = table.getNumRows()-1; ic>=0; ic--){
			if (table.isRowSelected(ic)) { 
				numSelected++;
			}
		}
		if (!byCommand && (table.editingAnything() || numSelected == 0))
			return;
		if (numSelected>0) {
			String message;
			if  (numSelected > 1)
				message = "Are you sure you want to permanently delete " +  numSelected + " " + owner.getItemTypeNamePlural() + "?";
			else
				message = "Are you sure you want to permanently delete the selected " + owner.getItemTypeName() + "?";
			if (!AlertDialog.query(this, "Delete?",message, "Yes", "No"))
				return;
			MenuOwner.incrementMenuResetSuppression();
			int[] which = new int[numSelected];
			int howMany = 0;
			for (int ic = table.getNumRows()-1; ic>=0; ic--){
				if (table.isRowSelected(ic)) {
		 			if (!owner.rowDeletable(ic)){
			 			if (!commandRec.scripting())
			 				owner.alert("Sorry, you can't delete that");
			 		}
			 		else {
						which[howMany]=ic;
						howMany++;
					}
				}
			}
			boolean notifyInBlocks = false;
			if (howMany>0) {
				int count =0;
				int currentNumRows = owner.getNumberOfRows();
				Object obj = getCurrentObject();
				
				//NOTE: this code allows reporting of what contiguous blocks were deleted, but causes full recalculations for each discontiguity
				if (notifyInBlocks &&obj instanceof Associable)
					((Associable)obj).incrementNotifySuppress();
				
				int firstInBlockDeleted = -1;
				for (int ic = 0; ic<howMany; ic++){
					
					//NOTE: this code allows reporting of what contiguous blocks were deleted, but caused  full recalculations for each discontiguity
					if (notifyInBlocks && obj instanceof Associable && firstInBlockDeleted < 0) {
						firstInBlockDeleted = which[ic];
					}
				
			 		if (owner.deleteRow(which[ic]))
						count++;
					
					//NOTE: this code allows reporting of what contiguous blocks were deleted, but caused  full recalculations for each discontiguity
					if (notifyInBlocks && obj instanceof Associable && (ic+1 >= howMany || which[ic+1] != which[ic]-1)){ //next one is not adjacent; notify listeners

						((Listenable)obj).notifyListeners(this, new Notification(MesquiteListener.PARTS_DELETED, new int[] {which[ic], firstInBlockDeleted - which[ic]+1}), commandRec);
						((Listenable)obj).decrementNotifySuppress();
						((Listenable)obj).incrementNotifySuppress();
						firstInBlockDeleted = -1;
					}
					
						
				}
				table.setNumRows(currentNumRows-count);
				//NOTE: this code allows reporting of what contiguous blocks were deleted, but causes full recalculations for each discontiguity
				if (notifyInBlocks){
					if (obj instanceof Listenable && !(obj instanceof Associable)) {
						((Listenable)obj).notifyListeners(this, new Notification(MesquiteListener.PARTS_DELETED), commandRec);
					}
					if (obj instanceof Associable)
						((Associable)obj).decrementNotifySuppress();
				}
				else if (obj instanceof Listenable) 
					((Listenable)obj).notifyListeners(this, new Notification(MesquiteListener.PARTS_DELETED), commandRec);
				table.repaintAll();
				if (count>=0)
					ownerModule.resetAllMenuBars();
			}
			MenuOwner.decrementMenuResetSuppression();
		}
		else
			ownerModule.alert("Rows must be selected before \"delete\" command is given");
	}
	public void setObject(Object object, CommandRecord commandRec) {
		setCurrentObject(object);
		if (object instanceof Annotatable)
			setDefaultAnnotatable((Annotatable)object);
		ListAssistant assistant;
		Enumeration enum=ownerModule.getEmployeeVector().elements();
		while (enum.hasMoreElements()){
			Object obj = enum.nextElement();
			if (obj instanceof ListAssistant) {
				assistant =  (ListAssistant)obj;
				assistant.setTableAndObject(table, getCurrentObject(), commandRec);
			}
		}	
		table.setNumRows(owner.getNumberOfRows());
		table.repaintAll();
	}
	
	/*.................................................................................................................*/
	public void windowResized() {
	   	if (MesquiteWindow.checkDoomed(this))
	   		return;
   	 	if ((getHeight()!=windowHeight) || (getWidth()!=windowWidth)) {
   	 		windowHeight =getHeight();
   	 		windowWidth = getWidth();
   	 		if (table !=null)
   	 			table.setTableSize(windowWidth, windowHeight);
   	 	}
		MesquiteWindow.uncheckDoomed(this);
	}
	/*.................................................................................................................*/
    	 public MesquiteTable getTable() {
		return table; 
   	 }
}


