/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.minimal.BasicFileCoordinator;
/*~~  */

import java.util.*;
import java.awt.*;
import java.net.*;
import java.io.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;

/** The "vice president" of the Mesquite system (the MesquiteTrunk module "Mesquite" being the president).  One of these is hired to coordinate each project.
Should actually be named ProjectCoordinator.*/
public class BasicFileCoordinator extends FileCoordinator implements PackageIntroInterface {
	MesquiteMenuSpec elementsMenu;
	
	boolean ready = false;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
 	//	elementsMenu = makeMenu("Elements");

		loadPreferences();
		
		MesquiteModule e;
		hireAllEmployees(commandRec, FileInterpreter.class);
		hireInit(commandRec, FileElementManager.class, "Manage TREES blocks");
		hireInit(commandRec, FileElementManager.class, "Manage TAXA blocks"); 
		hireInit(commandRec, FileElementManager.class, "Manage CHARACTERS blocks");
		hireInit(commandRec, FileInit.class, "Manage Character Models"); 
		hireInit(commandRec, FileInit.class, "Manage parsimony model sets");
		hireInit(commandRec, FileInit.class, "Manage probability model sets");
		hireInit(commandRec, FileInit.class, "Manage character sets");
		hireInit(commandRec, FileInit.class, "Manage SETS blocks");
		hireInit(commandRec, FileInit.class, "Manage ASSUMPTIONS blocks");
		hireInit(commandRec, FileInit.class, "Manage NOTES blocks");
		hireAllOtherEmployees(commandRec, FileElementManager.class);
		return true;
  	 }
	/*.................................................................................................................*/
	public void processPreferencesFromFile (String[] prefs) {
		if (prefs!=null && prefs.length>0) {
			MesquiteTrunk.suggestedDirectory = prefs[0];
		}
	}
	/*.................................................................................................................*/
	public String[] preparePreferencesForFile () {
		String[] prefs= new String[1];
		prefs[0] = MesquiteTrunk.suggestedDirectory;
		return prefs;
	}
  	 private void hireInit(CommandRecord commandRec, Class dutyClass, String name){
  	 	if (mesquiteTrunk.mesquiteModulesInfoVector.findModule(dutyClass, name) == null)
  	 		return;
		MesquiteModule e = hireNamedEmployee(commandRec, dutyClass, StringUtil.tokenize(name));
		if (e!=null)
			e.setPriorityFixed(true);
  	 }
  	 public void endJob() {
  	 	doomed = true;
		incrementMenuResetSuppression();
  	 	disposeMenuSpecifications();
		MesquiteProject p=getProject();
		if (p!=null) {	
			p.isDoomed = true;
			int count = 0;
			boolean done = false;
			while (p.developing && !done) {
				count++;
				if (count % 10000 == 0)
					if (AlertDialog.query(containerOfModule(), "Force Close?", "The project is not responding to the request to close it.  Force it to close?", "Force Close", "Continue", 1))
						done = true;
			}
			
			doomEmployees(this);
			//closeWindows(this);
			p.dispose();
			MesquiteTrunk.mesquiteTrunk.removeProject(p);
		}
 		decrementMenuResetSuppression();
 		setProject(null);
		super.endJob();
  	 }
	/*.................................................................................................................*/
  	 public void employeeQuit(MesquiteModule m){
  	 	if (m instanceof FileAssistantC) {
	  	 	resetAllMenuBars();
  	 	}
  	 	super.employeeQuit(m);
  	 }
  	 private void doomEmployees(MesquiteModule mb){
 		if (mb.getEmployeeVector()==null)
 			return;
 		Enumeration e = mb.getEmployeeVector().elements();
 		while (e.hasMoreElements()) {
			Object obj = e.nextElement();
			MesquiteModule mbe = (MesquiteModule)obj;
	 		if (mbe!=null) {
	 			doomEmployees(mbe);
	 			mbe.doom();
	 		}
		}
  	 }
  	 private void closeWindows(MesquiteModule mb){
 		if (mb.getEmployeeVector()==null)
 			return;
 		Enumeration e = mb.getEmployeeVector().elements();
 		while (e.hasMoreElements()) {
			Object obj = e.nextElement();
			MesquiteModule mbe = (MesquiteModule)obj;
	 		if (mbe!=null) {
	 			closeWindows(mbe);
	 			if (mbe.getModuleWindow() !=null)
	 				mbe.getModuleWindow().hide();
	 		}
		}
  	 }
	/*.................................................................................................................*/
 	public MesquiteProject initiateProject(String pathName, MesquiteFile homeFile) {
		MesquiteProject p = new MesquiteProject((FileCoordinator)this);
		setProject(p);
		setModuleWindow( new ProjectWindow(this));
		getModuleWindow().setLocation(8,8); //TODO: should set staggered positions
		MesquiteTrunk.mesquiteTrunk.addProject(getProject());
	 	p.addFile(homeFile);
	 	homeFile.setProject(p);
	 	p.setHomeFile(homeFile);
		hireAllOtherEmployees(CommandRecord.getRecNSIfNull(), FileInit.class);  //TODO: NOTSCRIPTING???
		if (getProject()==null)
			alert("FILE NULL");
		///THESE SHOULD BE ONLY AFTER Project made
 		addModuleMenuItems(MesquiteTrunk.windowsMenu, makeCommand("newAssistant",  this), FileAssistantN.class);
		MesquiteMenuItemSpec mi;
		if (numCompatibleModulesAvailable(FileAssistantCH.class, null, this)>0) {
 			mi = addSubmenu(MesquiteTrunk.analysisMenu, "New Histogram for", makeCommand("newAssistant",  this), FileAssistantCH.class);
 			mi.setZone(0);
		}
		if (numCompatibleModulesAvailable(FileAssistantCS.class, null, this)>0) {
 			mi = addSubmenu(MesquiteTrunk.analysisMenu, "New Scattergram for", makeCommand("newAssistant",  this), FileAssistantCS.class);
 			mi.setZone(0);
		}
		if (numCompatibleModulesAvailable(FileAssistantCP.class, null, this)>0) {
 			mi = addSubmenu(MesquiteTrunk.analysisMenu, "New Pairwise Chart for", makeCommand("newAssistant",  this), FileAssistantCP.class);
 			mi.setZone(0);
		}
 		MesquiteSubmenuSpec mss = addSubmenu(MesquiteTrunk.analysisMenu, "Current Charts", makeCommand("showChart", this));
 		mss.setZone(1);
 		mi = addMenuItem(MesquiteTrunk.analysisMenu, "-", null);
 		mi.setZone(1);
 		Enumeration e = MesquiteTrunk.windowVector.elements();
		mss.setList(MesquiteTrunk.windowVector);
		mss.setListableFilter(ChartWindow.class);

		broadcastProjectEstablished(this);
		return p;
   	 }
	/** Dumps list of available modules to System.out.*/
	public void dumpModuleList () {
		int num = MesquiteTrunk.mesquiteModulesInfoVector.size();
		logln("\n\n=====All Modules Loaded=====");
		MesquiteModuleInfo mbi;
		for (int i=0; i<num; i++){
			mbi = (MesquiteModuleInfo)MesquiteTrunk.mesquiteModulesInfoVector.elementAt(i);
			logln(mbi.getClassName() + "              duty class " + mbi.getDutyClass());
		}
		logln("=================\n\n");
	}
	/*.................................................................................................................*/
 	public MesquiteFile createProject(String pathName, boolean createTaxaBlockIfNew) { 
 	//TODO: ALLOW THIS ONLY ONCE FOR A FILE COORDINATOR
		MainThread.setShowWaitWindow(false);
 		incrementMenuResetSuppression();
 		String dirName;
 		String fileName;
 		boolean brandNew = false;
 		if (StringUtil.blank(pathName)) {
			FileDialog fdlg= new FileDialog(containerOfModule(), "New file:", FileDialog.SAVE);
			fdlg.setBackground(ColorDistribution.light[0]);
			fdlg.setFile("untitled.nex");
			fdlg.setVisible(true);
			dirName= fdlg.getDirectory();
			fileName= fdlg.getFile();
			if (StringUtil.blank(dirName) || StringUtil.blank(fileName)) {
 				decrementMenuResetSuppression();
				MainThread.setShowWaitWindow(true);
				if (StringUtil.blank(dirName) &&  StringUtil.blank(fileName))
					logln("New Project cancelled by user");
				else if ( StringUtil.blank(fileName))
					logln("New Project not created because file name not supplied.");
				else if ( StringUtil.blank(dirName))
					logln("New Project not created because directory name not supplied.");
				return null;
			}
			brandNew = true;
 		}
 		else {
 			dirName = StringUtil.getAllButLastItem(pathName, MesquiteFile.fileSeparator, "/") + MesquiteFile.fileSeparator;
 			fileName = StringUtil.getLastItem(pathName, MesquiteFile.fileSeparator, "/");
 		}
    	 	MesquiteFile thisFile =MesquiteFile.newFile(dirName, fileName);

    	 	MesquiteProject p=null;
 		if (thisFile!=null) {
			p = initiateProject(thisFile.getFileName(), thisFile);
			thisFile.setProject(p);
			//add MesquiteBlock
			afterProjectRead();
			if (brandNew && createTaxaBlockIfNew) {
				
				MesquiteInteger buttonPressed = new MesquiteInteger(1);
				ExtensibleDialog makeFileDialog = new ExtensibleDialog(containerOfModule(), "New File Options",buttonPressed);
				makeFileDialog.addLargeOrSmallTextLabel ("Do you want your new file to include taxa?");
				Checkbox tx = makeFileDialog.addCheckBox("Make Taxa Block", true);
				SingleLineTextField nameTaxa = makeFileDialog.addTextField("Name", "Untitled Taxa Block", 30);
				IntegerField numberTaxa = makeFileDialog.addIntegerField("Number of Taxa", 3, 6);
				makeFileDialog.addHorizontalLine(1);
				Checkbox showTree = makeFileDialog.addCheckBox("Show Tree Window", true);
				makeFileDialog.addHorizontalLine(1);
				makeFileDialog.addLargeOrSmallTextLabel ("If your file includes taxa, do you want it to include a character matrix?");
				Checkbox mcm = makeFileDialog.addCheckBox("Make Character Matrix", false);
				makeFileDialog.completeAndShowDialog("OK",null,null,"OK");
				
				if (buttonPressed.getValue()==0) {
					if (tx.getState() && numberTaxa.isValidInteger()){
						makeFileDialog.dispose();
						TaxaManager mTaxa = (TaxaManager)findEmployeeWithDuty(TaxaManager.class);
						Object t = null;
						if (mTaxa!=null)
							t = mTaxa.doCommand("newTaxa",Integer.toString(numberTaxa.getValue()) + " " + ParseUtil.tokenize(nameTaxa.getText()), CommandRecord.getRecNSIfNull(), CommandChecker.defaultChecker); //treat as scripting so as not to complain if something not found 19Jan02
						if (t!=null){
							if (mcm.getState()){
								CharactersManager mChars = (CharactersManager)findEmployeeWithDuty(CharactersManager.class);
								if (mChars!=null)
									mChars.doCommand("newMatrix",null, CommandRecord.getRecNSIfNull(), CommandChecker.defaultChecker); 
							}
							if (showTree.getState()){
								MesquiteModule btwc = findEmployeeWithName("#BasicTreeWindowCoord", true);
								if (btwc !=null) {
									Commandable c = (Commandable)btwc.doCommand("makeTreeWindow",null, CommandRecord.scriptingRecord, CommandChecker.defaultChecker); 
									if (c!=null)
										c.doCommand("showWindow",null, CommandRecord.scriptingRecord, CommandChecker.defaultChecker);
								}
							}
						}
						else {
							makeFileDialog.dispose();
							doCommand("showWindow", null, CommandRecord.scriptingRecord, CommandChecker.defaultChecker); //treat as scripting so as not to complain if something not found 19Jan02
						}
					}
					else {
						makeFileDialog.dispose();
						doCommand("showWindow", null, CommandRecord.scriptingRecord, CommandChecker.defaultChecker); //treat as scripting so as not to complain if something not found 19Jan02
					}
				}
				else {
					makeFileDialog.dispose();
					doCommand("showWindow", null, CommandRecord.scriptingRecord, CommandChecker.defaultChecker); //treat as scripting so as not to complain if something not found 19Jan02
				}
			}
				
		}
 		resetAllMenuBars();
 		if (p!=null)
 			p.developing = false;
 		decrementMenuResetSuppression();
		MainThread.setShowWaitWindow(true);
 		return thisFile;
   	 }
	/*.................................................................................................................*/
   	public MesquiteFile createFile(String pathName){
		MainThread.setShowWaitWindow(false);
 		String dirName;
 		String fileName;
 		if (StringUtil.blank(pathName)) {
			MainThread.setShowWaitWindow(false);
			FileDialog fdlg= new FileDialog(containerOfModule(), "New linked file:", FileDialog.SAVE);
			fdlg.setBackground(ColorDistribution.light[0]);
			fdlg.setFile("linked.nex");
			fdlg.setVisible(true);
			dirName= fdlg.getDirectory();
			fileName= fdlg.getFile();
			MainThread.setShowWaitWindow(true);
			if (StringUtil.blank(dirName) || StringUtil.blank(fileName)) {
 				decrementMenuResetSuppression();
				MainThread.setShowWaitWindow(true);
				if (StringUtil.blank(dirName) &&  StringUtil.blank(fileName))
					logln("New File cancelled by user");
				else if ( StringUtil.blank(fileName))
					logln("New File not created because file name not supplied.");
				else if ( StringUtil.blank(dirName))
					logln("New File not created because directory name not supplied.");
				return null;
			}
 		}
 		else {
 			dirName = StringUtil.getAllButLastItem(pathName, MesquiteFile.fileSeparator, "/") + MesquiteFile.fileSeparator;
 			fileName = StringUtil.getLastItem(pathName, MesquiteFile.fileSeparator, "/");
 		}
			MainThread.setShowWaitWindow(true);
    	 	return MesquiteFile.newFile(dirName, fileName);
   	}
	/*.................................................................................................................*/
 	public MesquiteFile newLinkedFile(String pathName) {  //makes new linked file
 		incrementMenuResetSuppression();
    	 	MesquiteFile thisFile;
    	 	if (StringUtil.blank(pathName)){
			MainThread.setShowWaitWindow(false);
			FileDialog fdlg= new FileDialog(containerOfModule(), "New linked file:", FileDialog.SAVE);
			fdlg.setFile("linked.nex");
			fdlg.setBackground(ColorDistribution.light[0]);
	 		String dirName;
	 		String fileName;

			fdlg.setVisible(true);
			MainThread.setShowWaitWindow(true);
			dirName= fdlg.getDirectory();
			fileName= fdlg.getFile();
			if (StringUtil.blank(dirName) || StringUtil.blank(fileName)) {
 				decrementMenuResetSuppression();
				if (StringUtil.blank(dirName) &&  StringUtil.blank(fileName))
					logln("New File cancelled by user");
				else if ( StringUtil.blank(fileName))
					logln("New File not created because file name not supplied.");
				else if ( StringUtil.blank(dirName))
					logln("New File not created because directory name not supplied.");
				return null;
			}
			thisFile = MesquiteFile.newFile(dirName, fileName);
		}
    	 	else {
	 		if (pathName.indexOf(MesquiteFile.fileSeparator)<0) {
	    	 		thisFile =MesquiteFile.newFile(getProject().getHomeDirectoryName(), pathName);
	 		}
	 		else {
		 		String dirName = StringUtil.getAllButLastItem(pathName, MesquiteFile.fileSeparator, "/") + MesquiteFile.fileSeparator;
		 		String fileName = StringUtil.getLastItem(pathName, MesquiteFile.fileSeparator, "/");
	    	 		thisFile =MesquiteFile.newFile(dirName, fileName);
    	 		}
    	 	}
 		if (thisFile!=null) {
				getProject().addFile(thisFile);
			 	thisFile.setProject(getProject());
				broadcastFileRead(this, thisFile);
 		}
 		resetAllMenuBars();
 		decrementMenuResetSuppression();
		return thisFile;
   	 }
	/*.................................................................................................................*/
 	public MesquiteFile readProject(boolean local, String pathName, String arguments, CommandRecord commandRec) { 
 	//ALLOW THIS ONLY ONCE FOR A FILE COORDINATOR
 		if (getProject()!=null) { 
 			alert("Error: attempt to read second project for single file coordinator");
 			return null;
 		}
 		
 		incrementMenuResetSuppression();
	    	MesquiteProject p=null;
		String message;
		if (local)
			message = "Open file as independent project:";
		else
			message = "Open URL as independent project:";
		
	    	MesquiteFile thisFile =MesquiteFile.open(local, pathName, message, MesquiteTrunk.suggestedDirectory, commandRec);
	    	if (thisFile!=null){
	    		MesquiteTrunk.suggestedDirectory = thisFile.getDirectoryName();
	  	 	storePreferences();
		}
		MesquiteTrunk.mesquiteTrunk.incrementProjectBrowserRefreshSuppression();
 		MesquiteTrunk.mesquiteTrunk.refreshBrowser(MesquiteProject.class);
		if (thisFile!=null && !StringUtil.blank(thisFile.getFileName())) {
			logln("Location: " + thisFile.getDirectoryName() + thisFile.getFileName());
			logln("");
			
			FileInterpreter fileInterp;
			boolean imp = false; //was it imported???
			//first try nexus.  If can't be read, then make list and query user...
			NexusFileInterpreter nfi = (NexusFileInterpreter)findImmediateEmployeeWithDuty(NexusFileInterpreter.class);
			if (nfi!=null && nfi.canReadFile(thisFile))
				fileInterp = nfi;
 			else {
 				imp = true;
 				fileInterp = findImporter(thisFile, 0);
 			}
 			if (fileInterp !=null) {
  				p = initiateProject(thisFile.getFileName(), thisFile);
  				MesquiteFile sf = commandRec.getScriptingFile();
  				if (commandRec.scripting())
  					commandRec.setScriptingFile(thisFile);
				fileInterp.readFile(getProject(), thisFile, arguments, commandRec);
				commandRec.setScriptingFile(sf);
				
				if (thisFile.isClosed()){
 					MesquiteTrunk.mesquiteTrunk.decrementProjectBrowserRefreshSuppression();
		 			if (p !=null)
		 				p.developing = false;
			 		resetAllMenuBars();
			 		decrementMenuResetSuppression();
					iQuit();
			 		return null;
				}
		 		else {
		 			p.fileSaved(thisFile);
		 			if (imp && local) {//was imported; change name
		 				thisFile.changeLocation(thisFile.getDirectoryName(), thisFile.getFileName()+".nex");
		 				if (commandRec.scripting() || thisFile.changeLocation("Save imported file as NEXUS file"))
		 					writeFile(thisFile); 
		 				else {
 							MesquiteTrunk.mesquiteTrunk.decrementProjectBrowserRefreshSuppression();
				 			if (p !=null)
				 				p.developing = false;
					 		resetAllMenuBars();
					 		decrementMenuResetSuppression();
							iQuit();
					 		return null;
		 				}
		 			}
		 		}
				afterProjectRead();
			}
			else {
				alert("Sorry, an interpreter was not found for this file");
			}
 		}
 		resetAllMenuBars();
		MesquiteTrunk.mesquiteTrunk.decrementProjectBrowserRefreshSuppression();
 		if (p!=null)
 			p.developing = false;
 		decrementMenuResetSuppression();
 		if (noWindowsShowing())
					doCommand("showWindow", null, CommandRecord.nonscriptingRecord, CommandChecker.defaultChecker); //TODO: will this always be non-scripting???
 		
		if (thisFile != null && thisFile.getCloseAfterReading()){
			closeFile(thisFile, commandRec);
			return null;
		}
 		return thisFile;
   	 }
	/*.................................................................................................................*/
 	public MesquiteFile readProjectGeneral(String arguments, CommandRecord commandRec) { 
 		if (getProject()!=null) { 
 			alert("Error: attempt to read second project for single file coordinator");
 			return null;
 		}
 		
 		incrementMenuResetSuppression();
		MesquiteTrunk.mesquiteTrunk.incrementProjectBrowserRefreshSuppression();
 		MesquiteTrunk.mesquiteTrunk.refreshBrowser(MesquiteProject.class);
		
		//hire reader
		GeneralFileMaker e = (GeneralFileMaker)hireNamedEmployee(commandRec, GeneralFileMaker.class, arguments);
		MesquiteProject p = null;
  		if (e != null) {
	  			
	  		p = e.establishProject(parser.getNextToken(), commandRec);
	 		if (p!=null){
	 			p.developing = false;
				afterProjectRead();
	 		}
	 		resetAllMenuBars();
 		}
  		MesquiteTrunk.mesquiteTrunk.decrementProjectBrowserRefreshSuppression();
 		decrementMenuResetSuppression();
		if (p==null){
			iQuit();
	 		return null;
 		}
   		MesquiteFile thisFile = p.getHomeFile();
 		if (noWindowsShowing())
			doCommand("showWindow", null, CommandRecord.nonscriptingRecord, CommandChecker.defaultChecker); //TODO: will this always be non-scripting???
 		
		if (thisFile != null && thisFile.getCloseAfterReading()){
			closeFile(thisFile, commandRec);
			return null;
		}
 		return thisFile;
   	 }
	/*.................................................................................................................*/
	private boolean noWindowsShowing(){
	 		Enumeration e = MesquiteTrunk.windowVector.elements();
	 		while (e.hasMoreElements()) {
				Object obj = e.nextElement();
				MesquiteWindow mw = (MesquiteWindow)obj;
		 		if (mw !=null && !mw.getMinimalMenus() && mw.isVisible() && mw.ownerModule!=null && mw.ownerModule.getProject()==getProject())
		 			return false;
	 		}
	 		return true;
	}
	/*.................................................................................................................*/
   	 public MesquiteFile readLinkedFile(String pathName, String importer, String arguments, int fileType, CommandRecord commandRec){ //make new/read new linked file
		if (commandRec.scripting()) {
			ObjectContainer f = new ObjectContainer();
			FileRead pt = new FileRead(pathName, importer, arguments, fileType, commandRec,  this, 0, f);
			pt.run();
			MesquiteFile mf = (MesquiteFile)f.getObject();
			f.setObject(null);
			return mf;
		}
		else {
			FileRead fr = new FileRead(pathName, importer, arguments, fileType, commandRec,  this, 0, null);
			MesquiteThread pt = new MesquiteThread(fr);
			pt.start();
			return null;
 		}
   	 }
	/*.................................................................................................................*/
   	 public void includeFile(String pathName, String importer, String arguments, int fileType, CommandRecord commandRec){ //make new/read new linked file
		FileRead pt = new FileRead(pathName, importer, arguments, fileType, commandRec,  this, 1, null);
		if (commandRec.scripting()) {
			pt.run();
		}
		else {
			MesquiteThread mt = new MesquiteThread(pt);
			mt.start();
 		}
   	 }
	/*.................................................................................................................*/
   	 public MesquiteFile getNEXUSFileForReading(String path, CommandRecord commandRec){ 
    	 	MesquiteFile file = null;
    	 	if (StringUtil.blank(path)) {
    	 		String message = "Choose file to open";
    	 		file =MesquiteFile.open(true, (String)null, message, getProject().getHomeDirectoryName());
 	 	}
 	 	else if (path.indexOf(MesquiteFile.fileSeparator)<0) {
	    	 	file =MesquiteFile.open(getProject().getHomeDirectoryName(), path);
	 	}
   	 	else {
	 		String dirName = StringUtil.getAllButLastItem(path, MesquiteFile.fileSeparator, "/") + MesquiteFile.fileSeparator;
	 		String fileName = StringUtil.getLastItem(path, MesquiteFile.fileSeparator, "/");
    	 		file =MesquiteFile.open(dirName, fileName);
    	 	}
		if (file == null)
			return null;
		NexusFileInterpreter nfi = (NexusFileInterpreter)findImmediateEmployeeWithDuty(NexusFileInterpreter.class);
		if (!nfi.canReadFile(file)) {
			return null; //not a nexus file
 		}
 		return file;
   	 }
	/*.................................................................................................................*/
   	 public FileBlock getNEXUSBlock(String blockType, String path, CommandRecord commandRec){ 
    	 	MesquiteFile file = null;
    	 	if (StringUtil.blank(path)) {
    	 		String message = "Choose file from which to obtain " + blockType + " block";
    	 		file =MesquiteFile.open(true, (String)null, message, getProject().getHomeDirectoryName());
 	 	}
 	 	else if (path.indexOf(MesquiteFile.fileSeparator)<0) {
	    	 	file =MesquiteFile.open(getProject().getHomeDirectoryName(), path);
	 	}
   	 	else {
	 		String dirName = StringUtil.getAllButLastItem(path, MesquiteFile.fileSeparator, "/") + MesquiteFile.fileSeparator;
	 		String fileName = StringUtil.getLastItem(path, MesquiteFile.fileSeparator, "/");
    	 		file =MesquiteFile.open(dirName, fileName);
    	 	}
		if (file == null)
			return null;
		NexusFileInterpreter nfi = (NexusFileInterpreter)findImmediateEmployeeWithDuty(NexusFileInterpreter.class);
		if (!nfi.canReadFile(file)) {
			return null;
 		}
 		FileBlock block = nfi.readOneBlock(getProject(), file, blockType, 0, commandRec);
 		file.dispose();
 		return block;
   	 }
	/*.................................................................................................................*/
 	public TextDisplayer displayText(String text, String title, CommandRecord commandRec) {
		TextDisplayer displayer = (TextDisplayer)hireEmployee(commandRec, TextDisplayer.class, null);
		if (displayer!=null)
			displayer.showText(text, title, true); 
		return displayer;
 	}
	/*.................................................................................................................*/
 	public TextDisplayer displayFile(MesquiteFile file, int maxCharacters, CommandRecord commandRec) {
		TextDisplayer displayer = (TextDisplayer)hireEmployee(commandRec, TextDisplayer.class, null);
		if (displayer!=null) {
			displayer.showFile(file, maxCharacters, true,9, true); 
		}
		return displayer;
 	}
	/*.................................................................................................................*/
 	public TextDisplayer displayFile(String pathName, int maxCharacters, CommandRecord commandRec) {
		TextDisplayer displayer = (TextDisplayer)hireEmployee(commandRec, TextDisplayer.class, null);
		if (displayer!=null) {
			displayer.showFile(pathName, maxCharacters, true,9, true); 
		}
		return displayer;
 	}
	/*.................................................................................................................*/
 	public void afterProjectRead() {
	 	if (getModuleWindow()!=null) 
			getModuleWindow().resetTitle(); //WHAT IF MULTIPLE LINKED FILES??
		FileInit employee;
		Enumeration enum=getEmployeeVector().elements();
		
		while (enum.hasMoreElements()){
			Object obj = enum.nextElement();
			((MesquiteModule)obj).setProject(getProject());
		}
		broadcastFileRead(this, getProject().getHomeFile());
		ready = true;
		
		resetAllMenuBars();
   	 }
	/*.................................................................................................................*/
 	/** A method called immediately after the file has been established but not yet read in.*/
 	 void broadcastProjectEstablished(MesquiteModule module) {
 	 	if (module==null)
 	 		return;

		EmployeeVector e = module.getEmployeeVector();
		if (e==null)
			return;
		for (int i = 0; i<e.size(); i++) {
			MesquiteModule employee = (MesquiteModule)e.elementAt(i);
			if (employee!=null){
				employee.projectEstablished();
				broadcastProjectEstablished(employee);
			}
		}
 	}
	/*.................................................................................................................*/
 	/** A method called immediately after a file has been read in.*/
 	 void broadcastFileRead(MesquiteModule module, MesquiteFile f) {
 	 	if (module==null)
 	 		return;
		EmployeeVector e = module.getEmployeeVector();
		if (e==null)
			return;
		for (int i = 0; i<e.size(); i++) {
			MesquiteModule employee = (MesquiteModule)e.elementAt(i);
			if (employee!=null){
				employee.fileReadIn(f);
				broadcastFileRead(employee, f);
			}
		}
 	}
	/*.................................................................................................................*/
 	/** A method called immediately before a file is to be written.*/
 	 void broadcastFileAboutToBeSaved(MesquiteModule module, MesquiteFile f) {
 	 	if (module==null)
 	 		return;
		EmployeeVector e = module.getEmployeeVector();
		if (e==null)
			return;
		for (int i = 0; i<e.size(); i++) {
			MesquiteModule employee = (MesquiteModule)e.elementAt(i);
			if (employee!=null){
				employee.fileAboutToBeWritten(f);
				broadcastFileAboutToBeSaved(employee, f);
			}
		}
 	}
 	/*.................................................................................................................*/
 	/** THIS APPEARS NO LONGER USED*/
	public String userPutFile(){
		MainThread.setShowWaitWindow(false);
		FileDialog fdlg= new FileDialog(containerOfModule(), "Save file", FileDialog.SAVE);
			fdlg.setBackground(ColorDistribution.light[ColorDistribution.getColorScheme(this)]);
		fdlg.setVisible(true);
		MainThread.setShowWaitWindow(true);
		return fdlg.getDirectory() + fdlg.getFile();
	}
 	/*.................................................................................................................*/
	/*  */
	public void saveLinkagesAs(String pathName){
		MesquiteMessage.warnProgrammer("Oops: saving linkages not yet working");
		//writes Nexus file with Mesquite script requesting all linked files to be opened
		// needs home file and other files in order.
		//(NEED vector of files)
	}
 	/*.................................................................................................................*/
	/*  */
	public void saveAllFiles(){
		if (getProject() ==null)
			return;
		Enumeration enum=getProject().getFiles().elements();
		MesquiteFile fi;
		while (enum.hasMoreElements()){
			Object obj = enum.nextElement();
			if (obj instanceof MesquiteFile) {
				fi = (MesquiteFile)obj;
				if (fi.isLocal())
					writeFile(fi);
				else
					MesquiteMessage.notifyUser("File \"" + fi.getName() + "\" cannot be written because it was accessed as a URL");
			}
		}
	}
 	/*.................................................................................................................*/
	/*  */
	public boolean closeFile(MesquiteFile fi, CommandRecord commandRec){
		if (getProject() ==null)
			return false;
		MesquiteTrunk.mesquiteTrunk.incrementProjectBrowserRefreshSuppression();
		if (fi!=null) {
			if (getProject().getHomeFile() == fi) { //should ask for all files
				if (commandRec.scripting() || !getProject().isDirty()) {
					waitWriting(null);
					logln("Closing " + getProject().getName());
					iQuit();
				}
				else {
					ListableVector files = getProject().getFiles();
					Enumeration enum=files.elements();
					MesquiteFile fiP;
					while (enum.hasMoreElements()){
						Object obj = enum.nextElement();
						if (obj instanceof MesquiteFile) {
							fiP = (MesquiteFile)obj;
							
							if (fiP!=null && fiP.isDirty() && fiP.isLocal()) {
								String message = "Do you want to save changes to \"" + fiP.getName() + "\" before closing?";
								int q = AlertDialog.query(containerOfModule(), "Save changes?",  message, "Save", "Cancel", "Don't Save");
								if (q==0) {
									logln("Writing " + fiP.getName());
									writeFile(fiP);
								}
								else if (q==1) {
									logln("File close cancelled by user");
									MesquiteTrunk.mesquiteTrunk.decrementProjectBrowserRefreshSuppression();
									return false;
								}
							}
						}
					}
					logln("Closing " + getProject().getName());
					iQuit();
				}
			}
			else {
				if (commandRec.scripting() || !fi.isDirty()) {
					waitWriting(fi);
					fi.close();
				}
				else if (fi.isLocal()){
					String message = "Do you want to save changes to \"" + fi.getName() + "\" before closing?";
					int q = AlertDialog.query(containerOfModule(), "Save changes?",  message, "Save", "Cancel", "Don't Save");
					if (q==0) 
						writeFile(fi);
					else if (q==1) {
						logln("File close cancelled by user");
						MesquiteTrunk.mesquiteTrunk.decrementProjectBrowserRefreshSuppression();
						return false;
					}
					logln("Closing file " + fi.getName());
					fi.close();
				}
			}
		}
		MesquiteTrunk.mesquiteTrunk.decrementProjectBrowserRefreshSuppression();
		return true;
	}
	
	private void waitWriting(MesquiteFile f){
		try {
			boolean done = false;
			while (!done){
				if (f == null) {
					done = true;
					ListableVector files = getProject().getFiles();
					for (int i=0; i<files.size(); i++){
						if (((MesquiteFile)files.elementAt(i)).isWriting())
							done = false;
					}
				}
				else
					done = !f.isWriting();
				if (!done)
					Thread.sleep(50);
			}
		}
		catch (InterruptedException e){
		}
	}
 	/*.................................................................................................................*/
	/*  */
	public void closeFile(int id, CommandRecord commandRec){
		MesquiteFile fi = getProject().getFileByID(id);
		closeFile(fi, commandRec);
	}
	public void closeFile(String id, CommandRecord commandRec){
		if (getProject() == null)
			return;
		MesquiteFile fi = getProject().getFile(id);
		closeFile(fi, commandRec);
	}
 	/*.................................................................................................................*/
	/*  */
	public void export(FileInterpreterI exporter, MesquiteFile file, String arguments, CommandRecord commandRec){
		if (exporter!=null) 
			exporter.exportFile(file, arguments, commandRec); 
	}
 	/*.................................................................................................................*/
	/*  */
	public void saveFile(int id){
		if (getProject() ==null)
			return;
		MainThread.setShowWaitWindow(false);
		MesquiteFile fi;
		if (!MesquiteInteger.isCombinable(id) && getProject().getNumberLinkedFiles()==1)
			fi = getProject().getHomeFile();
		else
			fi = getProject().getFileByID(id);
		if (fi != null) { 
			if (fi.isLocal())
				writeFile(fi);
			else
				MesquiteMessage.notifyUser("File \"" + fi.getName() + "\" cannot be written because it was accessed as a URL");
		}
		else 
			MesquiteMessage.notifyUser("Request to save file id#" + MesquiteInteger.toString(id)+ "; file not found");
		MainThread.setShowWaitWindow(true);
	}
 	/*.................................................................................................................*/
	/*  */
	public void saveFileAs(int id){
		MainThread.setShowWaitWindow(false);
		MesquiteFile fi;
		if (!MesquiteInteger.isCombinable(id) && getProject().getNumberLinkedFiles()==1)
			fi = getProject().getHomeFile();
		else
			fi = getProject().getFileByID(id);
		if (fi !=null){
			/*if (!fi.isLocal())
				MesquiteMessage.notifyUser("File \"" + fi.getName() + "\" cannot be written because it was accessed as a URL");
			else */
			if (fi.changeLocation("Save file")) {
				fi.setWriteProtected(false);
				writeFile(fi);
			}
		}
		//TODO: change titles of windows and menu items!!!
		MainThread.setShowWaitWindow(true);
	}
	public void renameAndSaveFile(int id, String name){
		if (StringUtil.blank(name))
			return;
		MainThread.setShowWaitWindow(false);
		MesquiteFile fi;
		if (!MesquiteInteger.isCombinable(id) && getProject().getNumberLinkedFiles()==1)
			fi = getProject().getHomeFile();
		else
			fi = getProject().getFileByID(id);
		if (fi !=null){
			fi.changeLocation(fi.getDirectoryName(), name);
			fi.setWriteProtected(false);
			writeFile(fi);
		}
		MainThread.setShowWaitWindow(true);
	}
 	/*.................................................................................................................*/
	/*  */
	public void writeFile(MesquiteFile nMF){
		broadcastFileAboutToBeSaved(this, nMF);
		Runtime rt = Runtime.getRuntime();
		rt.gc();
		
		NexusFileInterpreter nfi = (NexusFileInterpreter)findImmediateEmployeeWithDuty(NexusFileInterpreter.class);
		if (nfi!=null) {
			nfi.writeFile(getProject(), nMF);
		}
	}
	/*.................................................................................................................*/
 	/** Finds the first employee in the heirarchy that manages a particular subclass of FileElement */
	public ElementManager findManager(MesquiteModule m, Class fileElementClass) {
		if (fileElementClass == null)
			return null;
		ElementManager r = findExactManager(m, fileElementClass);
		if (r!=null)
			return r;
		else
			return findCapableManager(m, fileElementClass);
	}
	/*.................................................................................................................*/
 	/** Finds the first employee in the heirarchy that manages a particular subclass of FileElement */
	private ElementManager findExactManager(MesquiteModule m, Class fileElementClass) {
		if (m instanceof ElementManager) {
			ElementManager fEM = (ElementManager)m;
			if (fEM!=null){
				Class eC = fEM.getElementClass();
				if (eC != null && eC ==fileElementClass) {  //manager must handle fileElementClass or its superclasses
					return fEM;
				}
			}
		}
		EmployeeVector ev = m.getEmployeeVector();
		if (ev==null)
			return null;
		Enumeration enum=ev.elements();
		while (enum.hasMoreElements()){
			MesquiteModule mb = (MesquiteModule)enum.nextElement();
			ElementManager fEM = findExactManager(mb, fileElementClass);
			if (fEM != null)
				return fEM;
		}
		return null;
	}
	/*.................................................................................................................*/
 	/** Finds the first employee in the heirarchy that manages a particular subclass of FileElement */
	public ElementManager findCapableManager(MesquiteModule m, Class fileElementClass) {
		if (m instanceof ElementManager) {
			ElementManager fEM = (ElementManager)m;
			if (fEM != null) {
				Class eC = fEM.getElementClass();
				if (eC != null && eC.isAssignableFrom(fileElementClass)) {  //manager must handle fileElementClass or its superclasses
					return fEM;
				}
			}
		}
		EmployeeVector ev =m.getEmployeeVector();
		if (ev == null)
			return null;
		Enumeration enum=ev.elements();
		while (enum.hasMoreElements()){
			MesquiteModule mb = (MesquiteModule)enum.nextElement();
			ElementManager fEM = findCapableManager(mb, fileElementClass);
			if (fEM != null)
				return fEM;
		}
		return null;
	}
	/*.................................................................................................................*/
	public FileInterpreter findImporter(MesquiteFile f, int fileType) {
		TextDisplayer fd = displayFile(f, 4000, CommandRecord.getRecNSIfNull());  //TODO: should say if scripting
		
		MesquiteModule[] fInterpreters = null;
		if (fileType == 0)
			fInterpreters = getImmediateEmployeesWithDuty(FileInterpreterI.class);
		else
			fInterpreters = getImmediateEmployeesWithDuty(FileInterpreterITree.class);
		if (fInterpreters==null)
			return null;
		int count = 0;
		for (int i=0; i<fInterpreters.length; i++){
			if (((FileInterpreterI)fInterpreters[i]).canImport())
				count++;
		}
		MesquiteModule[] fInterpretersCanImport = new MesquiteModule[count];
		count = 0;
		for (int i=0; i<fInterpreters.length; i++){
			if (((FileInterpreterI)fInterpreters[i]).canImport())
				fInterpretersCanImport[count++] = fInterpreters[i];
		}
		
		Listable fInt = ListDialog.queryList(containerOfModule(), "Import File", "This file is not a NEXUS file.  Please choose an interpreter for this file", MesquiteString.helpString,fInterpretersCanImport, 0);
		if (fd!=null)
			fireEmployee(fd);
		return (FileInterpreter)fInt;
	}
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) {
   	 	Snapshot temp = new Snapshot();
   	 	ListableVector files = getProject().getFiles();
		for (int i = 0; i<files.size(); i++) {
			MesquiteFile e=(MesquiteFile)files.elementAt(i);
			if (e!=getProject().getHomeFile())
				temp.addLine("linkFile " + StringUtil.tokenize(e.getName()));  //quote //todo: should parse name relative to path to home file!!!!!
			
		}
		for (int i = 0; i<getNumberOfEmployees(); i++) {
			Object e=getEmployeeVector().elementAt(i);
			if (!(e instanceof FileAssistant)) {
				temp.addLineIfAnythingToTell("getEmployee " , ((MesquiteModule)e));  //addLineIfAnythingToTell
			}
		}
		for (int i = 0; i<getNumberOfEmployees(); i++) {
			Object e=getEmployeeVector().elementAt(i);
			if (e instanceof FileAssistant) {
				temp.addLine("newAssistant " , ((MesquiteModule)e)); 
			}
		}
  	 	return temp;
  	 }
  	 /** these are the commands that shouldn't dirty the file */
	public void fileDirtiedByCommand(MesquiteCommand command){
		if (command !=null && command.getOwner() == this) {
			if ("saveFiles".equalsIgnoreCase(command.getName()))
				return;
			else if ("saveFile".equalsIgnoreCase(command.getName()))
				return;
			else if ("saveFileAs".equalsIgnoreCase(command.getName()))
				return;
			else if ("saveLinkagesAs".equalsIgnoreCase(command.getName()))
				return;
			else if ("closeFile".equalsIgnoreCase(command.getName()))
				return;
			else if ("closeProject".equalsIgnoreCase(command.getName()))
				return;
		}
		super.fileDirtiedByCommand(command);
	}
	/*.................................................................................................................*/
  	 MesquiteInteger pos = new MesquiteInteger(0);
  	 private String filterIfMarkedArgument(String s){
    	 		if (s == null || s.length() <1 || s.charAt(0) == StringUtil.argumentMarker)
    	 			return null;
    	 		return s;
  	 }
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	 if (checker.compare(this.getClass(), "Returns the total number of character data matrices stored in the project", null, commandName, "getNumberOfDataSets")) {
    	 		return (new MesquiteInteger(getProject().getNumberCharMatrices()));
		}
    	 	else if (checker.compare(this.getClass(), "Returns a vector of the character data matrices stored in the project", null, commandName, "getDataSets")) {
    	 		return (getProject().getCharacterMatrices());
		}
    	 	else if (checker.compare(this.getClass(), "Sets the home file of the project so it is write protected", null, commandName, "writeProtect")) { //TODO: should use arguments to toggle
    	 		getProject().getHomeFile().setWriteProtected(true);
		}
    	 	else if (checker.compare(this.getClass(), "Returns the number of sets of taxa stored in the project", null, commandName, "getNumberOfTaxas")) {
    	 		MesquiteInteger mi = new MesquiteInteger(getProject().getNumberTaxas());
    	 		return (mi);
		}
    	 	else if (checker.compare(this.getClass(),  "Returns a vector of the sets of taxa stored in the project", null, commandName, "getTaxas")) {
    	 		return (getProject().getTaxas());
		}
		//TODO: getTaxa(id), getDataSet(id)
    	 	else if (checker.compare(this.getClass(), "Returns a character model stored in the project", "[name of character model", commandName, "getCharacterModel")) {
    	 		return getProject().getCharacterModel(parser.getFirstToken(arguments));
		}
    	 	else if (checker.compare(this.getClass(), "Reads a file as linked to a current file, so that they can share information", "[path to file]", commandName, "linkFile")) {
    	 		String path = filterIfMarkedArgument(parser.getFirstToken(arguments));
    	 		String importer = filterIfMarkedArgument(parser.getNextToken());
    	 		return (readLinkedFile(path, importer, arguments, 0, commandRec));
		}
    	 	else if (checker.compare(this.getClass(), "Reads a file at a URL as linked to a current file, so that they can share information", "[URL]", commandName, "linkURL")) {
    	 		MesquiteMessage.notifyUser("Sorry, can't yet read a linked URL");
    	 	//	return (readLinkedURL(arguments));
		}
    	 	else if (checker.compare(this.getClass(), "Reads a file and incorporates all of its information into the home file of the current project", "[path to file]", commandName, "includeFile")) {
    	 		String path = filterIfMarkedArgument(parser.getFirstToken(arguments));
    	 		String importer = filterIfMarkedArgument(parser.getNextToken());
    	 		includeFile(path, importer, arguments, 0, commandRec);
		}
    	 	else if (checker.compare(this.getClass(), "Opens a NEXUS file and returns it for reading", "[path to file]", commandName, "getNEXUSFileForReading")) {
    	 		String path = parser.getFirstToken(arguments); //optional argument
    	 		
    	 		return getNEXUSFileForReading(path, commandRec);
		}
    	 	else if (checker.compare(this.getClass(), "Opens a NEXUS file and returns the first block of a given type", "[block type][path to file]", commandName, "getNEXUSFileBlock")) {
    	 		String blockType = parser.getFirstToken(arguments); //required argument
    	 		String path = parser.getNextToken();//optional argument
    	 		
    	 		return getNEXUSBlock(blockType, path, commandRec);
		}
    	 	else if (checker.compare(this.getClass(), "Reads a file at a URL and incorporates all of its information into the home file of the current project", "[URL]", commandName, "includeURL")) {
    	 		MesquiteMessage.notifyUser("Sorry, can't yet include a URL");
    	 	//	return (readLinkedURL(arguments));
		}
    	 	else if (checker.compare(this.getClass(), "Reads a tree file as linked to a current file, so that they can share information", "[path to file]", commandName, "linkTreeFile")) {
    	 		String path = filterIfMarkedArgument(parser.getFirstToken(arguments));
    	 		String importer = filterIfMarkedArgument(parser.getNextToken());
    	 		return (readLinkedFile(path, importer, arguments, 1, commandRec));
		}
    	 	else if (checker.compare(this.getClass(), "Reads a tree file and incorporates all of its information into the home file of the current project", "[path to file]", commandName, "includeTreeFile")) {
    	 		String path = filterIfMarkedArgument(parser.getFirstToken(arguments));
    	 		String importer = filterIfMarkedArgument(parser.getNextToken());
    	 		includeFile(path, importer, arguments, 1, commandRec);
		}
    	 	else if (checker.compare(this.getClass(), "Create a new linked file", "[path to file]", commandName, "newLinkedFile")) {
    	 		MesquiteFile file = (newLinkedFile(parser.getFirstToken(arguments)));
    	 		return file;
		}
    	 	else if (checker.compare(this.getClass(), "Brings a current chart window to the fore", "[number of chart window]", commandName, "showChart")) {
    	 		pos.setValue(0);
    	 		int which = MesquiteInteger.fromString(arguments, pos);
    	 		if ((which == 0 || MesquiteInteger.isPositive(which))) {
	 			ListableVector ws = MesquiteTrunk.windowVector;
		 		int count = 0;
		 		for (int i=0; i<ws.size(); i++) {
					Object obj = ws.elementAt(i);
					if (obj instanceof ChartWindow){
						if (which == count) {
							((ChartWindow)obj).show();
							return obj;
						}
						count++;
					}
		 		}
	 		}
	 		return null;
    	 		
		}
    	 	else if (checker.compare(this.getClass(), "Returns a reference to the file assistant module", "[name of module]", commandName, "getAssistant")) {
    	 		MesquiteModule mb = findEmployeeWithName(parser.getFirstToken(arguments), true);
    	 		return mb;
		}
    	 	else if (checker.compare(this.getClass(), "Hires new file assistant module", "[name of module]", commandName, "newAssistant")) {
    	 		return (FileAssistant)hireNamedEmployee(commandRec, FileAssistant.class, arguments);
		}
       	 	else if (checker.compare(this.getClass(), "Save all files in project", null, commandName, "saveFiles")) {  //all files in project
			saveAllFiles();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Saves file", "[number of file]", commandName, "saveFile")) {  //Single file only
			saveFile(MesquiteInteger.fromString(arguments));
    	 	}
    	 	else if (checker.compare(this.getClass(), "Saves home file", "[number of file]", commandName, "saveHomeFile")) {  //Single file only
			saveFile((int)getProject().getHomeFile().getID());
    	 	}
    	 	else if (checker.compare(this.getClass(), "Renames file", "[number of file]", commandName, "renameAndSaveFile")) {
    	 		MesquiteInteger pos = new MesquiteInteger(0);
    	 		int i = MesquiteInteger.fromString(arguments, pos);
    	 		String name = ParseUtil.getToken(arguments, pos);
			renameAndSaveFile(i, name);
    	 	}
    	 	else if (checker.compare(this.getClass(), "Saves file as... to create a separate copy", "[number of file]", commandName, "saveFileAs")) {
			saveFileAs(MesquiteInteger.fromString(arguments));
    	 	}
    	 	else if (checker.compare(this.getClass(), "Gives information about the project", null, commandName, "getInfo")) {
				String st = "This represents a project, which currently represents information from ";
				if (proj.getNumberLinkedFiles() == 1) {
					st = "The current file is " + proj.getHomeFileName();
					if (!StringUtil.blank(proj.getHomeDirectoryName()))
						st += ", located at " + proj.getHomeDirectoryName() + ".";
				}
				else {
					st = "The current project includes " + proj.getNumberLinkedFiles() + " files.  The home file of the project is " + proj.getHomeFileName();
					if (!StringUtil.blank(proj.getHomeDirectoryName()))
						st += ", located at " + proj.getHomeDirectoryName() + ".";
					String[] s = proj.getFiles().getStrings();
					
					
					st += "\n\nThe other files linked in the project are:\n";
					for (int i=1; i<s.length; i++)
						st += "     " + s[i] + "\n";
				}
				alert(st);
    	 	}
    	 	else if (checker.compare(this.getClass(), "Exports information to another format", null, commandName, "export")) {
			//arguments = "#mesquite.io.InterpretNonaHennig.InterpretNonaHennig 1 file= 'exported.ss' directory = '/Users/wmaddisn/Desktop/'";
			String exporterName = parser.getFirstToken(arguments);
			//which file?
			MesquiteFile file =null;
			int id = MesquiteInteger.fromString(parser.getNextToken());
			if (MesquiteInteger.isCombinable(id) && getProject().getNumberLinkedFiles()>1)
				file = getProject().getFileByID(id);
			FileInterpreterI exporter = (FileInterpreterI)findEmployeeWithName(exporterName);
			if (exporter == null){
				MesquiteModule[] fInterpreters = getImmediateEmployeesWithDuty(FileInterpreterI.class);
				if (fInterpreters==null)
					return null;
				int count = 0;
				for (int i=0; i<fInterpreters.length; i++){
					if (((FileInterpreterI)fInterpreters[i]).canExport())
						count++;
				}
				MesquiteModule[] fInterpretersCanExport = new MesquiteModule[count];
				count = 0;
				for (int i=0; i<fInterpreters.length; i++){
					if (((FileInterpreterI)fInterpreters[i]).canExport())
						fInterpretersCanExport[count++] = fInterpreters[i];
				}
				
				exporter = (FileInterpreterI)ListDialog.queryList(containerOfModule(), "Export format", "Export part or all of the information as a file of the following format",MesquiteString.helpString, fInterpretersCanExport, 0);
			}
			export(exporter, file, arguments, commandRec);
    	 	}
    	 	else if (checker.compare(this.getClass(), "NOT FUNCTIONING", null, commandName, "saveLinkagesAs")) {
			saveLinkagesAs(parser.getFirstToken(arguments));
			//writeFile(false, null);
    	 	}
   	 	else if (checker.compare(this.getClass(), "Closes file", "[number of file]", commandName, "closeFile")) {
   	 	//query...
    			closeFile(MesquiteInteger.fromString(arguments), commandRec);
    	 	}
   	 	else if (checker.compare(this.getClass(), "Closes file", "[name of file]", commandName, "closeFileByName")) {
   	 	//query...
    			closeFile(arguments, commandRec);
    	 	}
    	 	else if (checker.compare(this.getClass(), "Closes this project (which may consist of a single open file)", null, commandName, "closeProject")) {
			if (commandRec.scripting()){
				MesquiteProject p=getProject();
				if (p != null)
					p.developing = false; //so that project closes!
			}
			closeFile(getProject().getHomeFile(), commandRec);
	 		if (!commandRec.scripting() && MesquiteTrunk.mesquiteTrunk.getProjectList().getNumProjects()==0){
		 		MesquiteWindow w = MesquiteTrunk.mesquiteTrunk.getModuleWindow();
		 		if (w!=null)
		 			w.setVisible(true);
	 		}
    	 	}
      	 	else if (checker.compare(this.getClass(), "Returns path to this project", null, commandName, "getPathToProject")) {
			return getProject().getHomeDirectoryName();
    	 	}
      	 	else if (checker.compare(this.getClass(), "Returns name of home file of this project", null, commandName, "getHomeFileName")) {
			return getProject().getHomeFileName();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Returns this project", null, commandName, "getProject")) {
			return getProject();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Returns identification number of this project", null, commandName, "getProjectID")) {
			return new MesquiteInteger((int)getProject().getID());
    	 	}
    	 	else if (checker.compare(this.getClass(), "Brings all of the windows belonging to the project to the front.", null, commandName, "allToFront")) {
	 		Enumeration e = MesquiteTrunk.mesquiteTrunk.windowVector.elements();
	 		while (e.hasMoreElements()) {
				Object obj = e.nextElement();
				MesquiteWindow mw = (MesquiteWindow)obj;
		 		if (mw.isVisible() && mw.getOwnerModule()!=null && mw.getOwnerModule().getProject()== getProject())
		 			mw.setVisible(true);
	 		}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Not used; placed here to suppress warning that command \"end\" from Mesquite block not handled", null, commandName, "end")) { //this is end of MesquiteBlock; placed here to suppress warning that command not handled
			return null;
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
   	 }
   
	/*.................................................................................................................*/
	/**Returns command to hire employee if clonable*/
	public String getClonableEmployeeCommand(MesquiteModule employee){
		if (employee!=null && employee.getEmployer()==this) {
			if (employee.getHiredAs()==FileAssistant.class)
				return ("newAssistant " + StringUtil.tokenize(employee.getName()) + ";"); //quote
		}
		return null;
	}
	/*.................................................................................................................*/
    	 public String getName() {
		return "Basic File Coordinator";
   	 }
	/*.................................................................................................................*/
   	 
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Coordinates the reading, maintenance and linkages of files, which include Taxa, Trees, CharacterData, and other objects." 
 		+ " This module is hired directly by the Mesquite trunk module.";
   	 }
	/*.................................................................................................................*/
 	/** returns current parameters, for logging etc..*/
 	public String getParameters() {
  		String s =""; 
		if (getProject()!=null) 
 		 	s += "Project name: " + getProject().getName();
 		return s;
   	 }
	/*.................................................................................................................*/
	/** Returns the URL of document shown when splash screen icon touched. By default, returns path to module's manual*/
	public String getSplashURL(){
 		return null; //return getPath() + "splash.html"; 
	}
	/** Returns whether there is a splash banner*/
	public boolean hasSplash(){
 		return true; 
	}
	
	/*.................................................................................................................*/
	/** Returns the name of the package of modules (e.g., "Basic Mesquite Package", "Rhetenor")*/
 	public String getPackageName(){
 		return "Basic Mesquite Package";
 	}
	/*.................................................................................................................*/
	/*.................................................................................................................*/
  	 public String getPackageVersion() {
		return null;//this causes defult Mesquite info to be used
   	 }
	/*.................................................................................................................*/
  	 public String getPackageAuthors() {
		return null; //this causes defult Mesquite info to be used
   	 }
	/*.................................................................................................................*/
  	 public String getPackageDateReleased() {
		return null; //this causes defult Mesquite info to be used
   	 }
	/*.................................................................................................................*/
	/** Returns citation for a package of modules*/
 	public String getPackageCitation(){
 		return null;//this causes defult Mesquite info to be used
 	}
	/*.................................................................................................................*/
	/** Requests a window to close.  In the process, subclasses of MesquiteWindow might close down their owning MesquiteModules etc.*/
 	public void windowGoAway(MesquiteWindow whichWindow) {
 			//Debug.println("disposing of window");
			whichWindow.hide();
			//whichWindow.dispose();
	}
}

/*======================================================================== */
/* an object of this class represents a single data file, with its associated data matrices, data editors, tree blocks, tree windows*/
class ProjectWindow extends MesquiteWindow {
	BasicPanel panel;
	/*.................................................................................................................*/
	public ProjectWindow(FileCoordinator  ownerModule){
		super(ownerModule, false);
		setResizable(false);
		setWindowSize(300, 32);
		panel = new BasicPanel();
		addToWindow(panel);
		panel.setVisible(true);
		panel.setLocation(0,0);
		panel.setSize(300, 32);
		panel.setBackground(ColorDistribution.medium[ColorDistribution.getColorScheme(ownerModule)]);
		resetTitle();
	}
	/*.................................................................................................................*/
	/** When called the window will determine its own title.  MesquiteWindows need
	to be self-titling so that when things change (names of files, tree blocks, etc.)
	they can reset their titles properly*/
	public void resetTitle(){
		if (ownerModule==null)
			setTitle("Project");
		else
			setTitle(ownerModule.getProject().getName());
	}

}
/* ======================================================================== */
class BasicPanel extends Panel{
	Image icon = MesquiteImage.getImage(MesquiteTrunk.mesquiteTrunk.getRootPath() + "images/mesquiteIcon.gif");
	public void paint(Graphics g){
		g.drawImage(icon, 0,0, 32, 32, this);
	}
}

/* ======================================================================== */
class FileRead implements Runnable {
	String arguments, path;
	CommandRecord commandRec;
	BasicFileCoordinator ownerModule;
	ObjectContainer f;
	String importer;
	int category;
	int fileType;
	public FileRead (String path, String importer, String arguments, int fileType, CommandRecord commandRec, BasicFileCoordinator ownerModule, int category, ObjectContainer f) {
		this.f = f;
		this.arguments = arguments;
		this.path = path;
		this.commandRec =  commandRec;
		this.ownerModule = ownerModule;
		this.category = category;
		this.importer = importer;
		this.fileType = fileType;
		//setCurrent(1);
	}
	/*
	public MesquiteFile getFile(){
	 	return linkedFile;
	}
	*/
	public CommandRecord getCommandRecord(){
	
		return commandRec;
	}
	public void setCommandRecord(CommandRecord c){
	
		commandRec = c;
	}
	public String getCurrentCommandName(){
		return "Reading file";
	}
	public String getCurrentCommandExplanation(){
		return null;
	}
	/*.................................................................................................................*/
   	 public MesquiteFile readLinkedFile(String pathName, CommandRecord commandRec){ //make new/read new linked file//TODO: should say if scripting
 		MesquiteFile linkedFile;
 		ownerModule.incrementMenuResetSuppression();
    	 	if (StringUtil.blank(pathName)) {
    	 		String message = "Link file to existing project:";
    	 		if (category == 1)
    	 			message = "Include file into existing file:";
    	 		linkedFile =MesquiteFile.open(true, (String)null, message, ownerModule.getProject().getHomeDirectoryName());
 	 	}
 	 	else if (pathName.indexOf(MesquiteFile.fileSeparator)<0) {
	    	 	linkedFile =MesquiteFile.open(ownerModule.getProject().getHomeDirectoryName(), pathName);
	 	}
   	 	else {
	 		String dirName = StringUtil.getAllButLastItem(pathName, MesquiteFile.fileSeparator, "/") + MesquiteFile.fileSeparator;
	 		String fileName = StringUtil.getLastItem(pathName, MesquiteFile.fileSeparator, "/");
    	 		linkedFile =MesquiteFile.open(dirName, fileName);
    	 	}
	 	boolean imp = false;
		MesquiteTrunk.mesquiteTrunk.incrementProjectBrowserRefreshSuppression();
	 	MesquiteTrunk.mesquiteTrunk.refreshBrowser(MesquiteProject.class);
		if (linkedFile!=null) {
			if (f!=null)
				f.setObject(linkedFile);
			ownerModule.getProject().addFile(linkedFile);
		 	linkedFile.setProject(ownerModule.getProject());
 			FileInterpreter fileInterp =null;
 			if (!StringUtil.blank(importer))
 				fileInterp = (FileInterpreter)ownerModule.findEmployeeWithName(importer);
			NexusFileInterpreter nfi = (NexusFileInterpreter)ownerModule.findImmediateEmployeeWithDuty(NexusFileInterpreter.class);
			if (fileInterp!=null) {
				if (fileInterp != nfi)
					imp = true;
			}
			else if (nfi!=null && nfi.canReadFile(linkedFile)) {
				fileInterp = nfi;
			}
 			else if (!StringUtil.blank(importer)) {
 				MesquiteModule mb = ownerModule.findEmployeeWithName(importer);
 				if (mb instanceof FileInterpreter) {
 					fileInterp = (FileInterpreter)mb;
 					imp = true;
 				}
 			}
 			else {
 				fileInterp = ownerModule.findImporter(linkedFile, fileType);
				imp = true;
 			}
 			if (fileInterp !=null) {
  				MesquiteFile sf = commandRec.getScriptingFile();
  				if (commandRec.scripting())
  					commandRec.setScriptingFile(linkedFile);
 				if (fileType == 0 || fileInterp == nfi)
 					fileInterp.readFile(ownerModule.getProject(), linkedFile, arguments, commandRec);
 				else 
 					((FileInterpreterITree)fileInterp).readTreeFile(ownerModule.getProject(), linkedFile, arguments, commandRec);
 				commandRec.setScriptingFile(sf);
				ownerModule.broadcastFileRead(ownerModule, linkedFile);
			}
			else {
				ownerModule.alert("Sorry, an interpreter was not found for this file");
				ownerModule.getProject().removeFile(linkedFile);
	 			MesquiteTrunk.mesquiteTrunk.decrementProjectBrowserRefreshSuppression();
	 			ownerModule.decrementMenuResetSuppression();
		 		return null;
			}
 		}
 		else {
 			MesquiteTrunk.mesquiteTrunk.decrementProjectBrowserRefreshSuppression();
	 		ownerModule.decrementMenuResetSuppression();
		 	return null;
 		}
 		
 		if (category == 1){
		 	ownerModule.getProject().mergeFile(linkedFile);
 		}
 		else if (!linkedFile.isClosed()) {
		 	ownerModule.getProject().fileSaved(linkedFile);
 			if (imp){ //was imported; change name
		 		linkedFile.changeLocation(linkedFile.getDirectoryName(), linkedFile.getFileName()+".nex");
		 		if (commandRec.scripting() || linkedFile.changeLocation("Save imported linked file as NEXUS file"))
 					ownerModule.writeFile(linkedFile);
 				else {
	 				linkedFile.close();
 				}
 			}
 		}
 		ownerModule.resetAllMenuBars();
		MesquiteTrunk.mesquiteTrunk.decrementProjectBrowserRefreshSuppression();
		
 		ownerModule.decrementMenuResetSuppression();
		if (linkedFile !=null && linkedFile.getCloseAfterReading()){
			ownerModule.closeFile(linkedFile, commandRec);
			return null;
		}
	 	return linkedFile;
   	 }
	/** DOCUMENT */
	public void run() {
		try {
			readLinkedFile(path, commandRec);
		}
		catch (Exception e){
				e.printStackTrace();
				PrintWriter pw = MesquiteFile.getLogWriter();
				if (pw!=null)
					e.printStackTrace(pw);
				MesquiteTrunk.mesquiteTrunk.alert("File reading could not be completed because an exception or error occurred (i.e. a crash; " + e.getClass() + "). If you save any files, you might best use Save As... in case data were lost or file saving doesn't work properly.");
			
		}
		catch (Error e){
			if (!(e instanceof ThreadDeath)){
				e.printStackTrace();
				PrintWriter pw = MesquiteFile.getLogWriter();
				if (pw!=null)
					e.printStackTrace(pw);
				MesquiteTrunk.mesquiteTrunk.alert("File reading could not be completed because an exception or error occurred (i.e. a crash; " + e.getClass() + "). If you save any files, you might best use Save As... in case data were lost or file saving doesn't work properly.");
			}
   	 		throw e;
		}
	}

}


