/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.minimal.ManageTaxa;
/*~~  */

import java.util.*;
import java.awt.*;
import java.io.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;

/** Manages the blocks of TAXA, including reading and writing from a NEXUS file.*/
public class ManageTaxa extends TaxaManager {
	
	MesquiteBoolean alerts;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		setMenuToUse(MesquiteTrunk.treesMenu);
		alerts = new MesquiteBoolean(true);
		
		return true;
  	 }
	/*.................................................................................................................*/
 	/** A method called immediately after the file has been read in.*/
 	public void projectEstablished() {
		MesquiteSubmenuSpec mmis = getFileCoordinator().addSubmenu(MesquiteTrunk.treesMenu, "List of Taxa", makeCommand("showTaxa",  this),  (ListableVector)getProject().taxas);
		//getFileCoordinator().addMenuItem(MesquiteTrunk.treesMenu, "-", null);
		mmis.setBehaviorIfNoChoice(MesquiteSubmenuSpec.ONEMENUITEM_ZERODISABLE);
		getFileCoordinator().addMenuItem(MesquiteTrunk.treesMenu, "New Block of Taxa...", makeCommand("newTaxa",  this));
		getFileCoordinator().addMenuItem(MesquiteTrunk.treesMenu, "List of Taxa Blocks", makeCommand("showTaxasList",  this));
		//getFileCoordinator().addMenuItem(MesquiteTrunk.treesMenu, "Merge taxa blocks...", makeCommand("mergeTaxa",  this));
		getFileCoordinator().addMenuItem(MesquiteTrunk.treesMenu, "-", null);
		super.projectEstablished();
 	}
	/*.................................................................................................................*/
	public MesquiteModule showElement(FileElement e, CommandRecord commandRec){
		if (e instanceof Taxa){
			Taxa t = (Taxa)e;
			return showTaxa(t, commandRec);
		}
		return null;
	}
	public void deleteElement(FileElement e){
		if (e instanceof Taxa){
			for (int i = 0; i<getNumberOfEmployees(); i++) {
				Object ma=getEmployeeVector().elementAt(i);
				if (ma instanceof ManagerAssistant)
					if (((ManagerAssistant)ma).showing(e)) {
						fireEmployee(((ManagerAssistant)ma));
					}
			}
			Taxa taxa = (Taxa)e;
			taxa.doom();
			getProject().removeFileElement(taxa);//must remove first, before disposing
			taxa.dispose();
		}
	}
	/*.................................................................................................................*/
	public Taxa makeNewTaxa(String title, int numTaxa, boolean userQuery){
		Taxa newTaxa=null;
		if (userQuery) {
			title= getProject().getTaxas().getUniqueName("Untitled Block of Taxa");
			MesquiteInteger buttonPressed = new MesquiteInteger(1);
			MesquiteInteger ion = new MesquiteInteger(3);
			MesquiteString ios = new MesquiteString(title);
			String s = "Use this dialog box to create a new, named block of taxa.";
			StringIntegerDialog dialog = new StringIntegerDialog(containerOfModule(), "New Block of Taxa", "Name of new block", "Number of taxa", ios,ion, 1, MesquiteInteger.unassigned, buttonPressed,s);
			
			if (buttonPressed.getValue()!=0)
				return null;
			numTaxa= ion.getValue();
			title= ios.getValue();

			if (getProject().getTaxas().getElementIgnoreCase(title)!=null) {
				title= getProject().getTaxas().getUniqueName("Untitled Block of Taxa");
				message("Sorry, that name is already taken.  The taxa block will be assigned the name " + title);
			}
		}
		if (MesquiteInteger.isCombinable(numTaxa) && numTaxa >= Taxa.MAXNUMTAXA) {
			message("Sorry, the maximum number of taxa in a taxa block is " + Taxa.MAXNUMTAXA);
		}
		else if (numTaxa>=0 && numTaxa< Taxa.MAXNUMTAXA) {
			newTaxa = new Taxa(numTaxa);
			if (title!=null)
				newTaxa.setName(title);
			String taxonName;
			for (int it=0; it<numTaxa; it++) {
				newTaxa.getTaxon(it).setName("taxon " + Taxon.toExternal(it));
			}
		}
		return newTaxa;
	}
	/*.................................................................................................................*/
	public void elementDisposed(FileElement e){
		if (e==null || !(e instanceof Taxa))
			return;
		NexusBlock nb = findNEXUSBlock(e);
		if (nb!=null) {
			removeNEXUSBlock(nb);
		}
		for (int i = 0; i<getNumberOfEmployees(); i++) {
			Object ma=getEmployeeVector().elementAt(i);
			if (ma instanceof ManagerAssistant)
				if (((ManagerAssistant)ma).showing(e)) {
					fireEmployee(((ManagerAssistant)ma));
				}
		}
	}
	/*.................................................................................................................*/
	public NexusBlock elementAdded(FileElement taxa){
		if (taxa==null || !(taxa instanceof Taxa))
			return null;
		NexusBlock nb = findNEXUSBlock(taxa);
		if (nb==null) {
			TaxaBlock t = new TaxaBlock(taxa.getFile(), this);
			t.setTaxa((Taxa)taxa);
			addNEXUSBlock(t);
			resetAllMenuBars();
			return t;
		}
		else return nb;
	}
 	/*.................................................................................................................*/
	public NexusBlockTest getNexusBlockTest(){ return new TaxaBlockTest();}
	/*.................................................................................................................*/
	public NexusCommandTest getNexusCommandTest(){ 
		return new TAXANexusCommandTest();
	}
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) { 
   	 	Snapshot temp = new Snapshot();
 		if (file == null || file == getProject().getHomeFile()){
	 		for (int i = 0; i< getProject().getNumberTaxas(); i++) {
	 			temp.addLine("setID " + i + " " + getProject().getTaxa(i).assignedIDNumber);
  	 			Snapshot fromTaxa = getProject().getTaxa(i).getSnapshot(file);
		  	 	if (fromTaxa != null && fromTaxa.getNumLines() > 0) {
					temp.addLine("tell It");
					temp.incorporate(fromTaxa, true);
					temp.addLine("endTell");
	 			}
	 		}
 		}
   	 	
		for (int i = 0; i<getNumberOfEmployees(); i++) {
			MesquiteModule e=(MesquiteModule)getEmployeeVector().elementAt(i);
			if (e instanceof ManagerAssistant && (e.getModuleWindow()!=null) && e.getModuleWindow().isVisible() && e.getName().equals("Taxa blocks list")) {
  	 				temp.addLine("showTaxasList ", e); 
  	 		}
		}
		for (int i = 0; i<getNumberOfEmployees(); i++) {
			MesquiteModule e=(MesquiteModule)getEmployeeVector().elementAt(i);
			if (e instanceof ManagerAssistant && (e.getModuleWindow()!=null) && e.getModuleWindow().isVisible() && e.getName().equals("Taxon List")) {
				Object o = e.doCommand("getTaxa", null, CommandRecord.getRecNSIfNull(), CommandChecker.defaultChecker);
				
				if (o !=null && o instanceof Taxa) {
					//int wh =getProject().getTaxaReference((Taxa)o);
  	 				temp.addLine("showTaxa " + getProject().getTaxaReference((Taxa)o), e); 
  	 			}
  	 		}
		}
  	 	return temp;
  	 }
	MesquiteInteger pos = new MesquiteInteger();
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Requests that a new taxa block be made", "[number of taxa] [title]", commandName, "newTaxa")) {
    	 		//ask user how many taxa
    	 		//create taxa block and add to file
    	 		//return taxa
    	 		
			incrementMenuResetSuppression();
    	 		Taxa newTaxa=null;
			int numTaxa= 3;
			String title= getProject().getTaxas().getUniqueName("Untitled Block of Taxa");
			MesquiteBoolean answer = new MesquiteBoolean(true);
			MesquiteFile file = getProject().chooseFile(commandRec, "Select file to which to add the new block of taxa"); //added 20 Dec 01
			Object o = null;
			if (StringUtil.blank(arguments)) {
				newTaxa = makeNewTaxa(title, numTaxa, true);
				if (newTaxa==null)
					return null;
				newTaxa.addToFile(file, getProject(), this);
				if (!commandRec.scripting())
					o = doCommand("showTaxa", getProject().getTaxaReference(newTaxa), CommandRecord.scriptingRecord, checker); //changed to scriptingRecord so as not to provoke alert 19Jan02
			}
			else {
	   	 		MesquiteInteger io = new MesquiteInteger(0);
	   			numTaxa= MesquiteInteger.fromString(arguments, io);
	   			title= ParseUtil.getToken(arguments, io);
				newTaxa = makeNewTaxa(title, numTaxa, false);
				if (newTaxa==null)
					return null;
				newTaxa.addToFile(file, getProject(), this);
				if (!commandRec.scripting())
					o = doCommand("showTaxa", getProject().getTaxaReference(newTaxa), CommandRecord.scriptingRecord, checker);//changed to scriptingRecord so as not to provoke alert 19Jan02
			}
			if (!commandRec.scripting() && o!=null && o instanceof MesquiteModule){  
				MesquiteModule mb = (MesquiteModule)o;
				mb.doCommand("showWindow", null, commandRec, checker);
			}
			decrementMenuResetSuppression();
    	 		return newTaxa;
		}
    	 	else if (checker.compare(this.getClass(), "Toggles whether alerts are on or off", "[on or off]", commandName, "alerts")) {
     	 		alerts.toggleValue(parser.getFirstToken(arguments));
    	 	}
    	 	else if (checker.compare(this.getClass(), "Shows list of taxa blocks", null, commandName, "showTaxasList")) {
    	 		//Check to see if already has lister for this
    	 		boolean found = false;
    	 		int numemp = getNumberOfEmployees();
			for (int i = 0; i<numemp; i++) {
				Object e=getEmployeeVector().elementAt(i);
				if (e instanceof ManagerAssistant)
					if (((ManagerAssistant)e).getName().equals("Taxa blocks list")) {
						((ManagerAssistant)e).getModuleWindow().setVisible(true);
						return e;
					}
			}
			ManagerAssistant lister= (ManagerAssistant)hireNamedEmployee(commandRec, ManagerAssistant.class, StringUtil.tokenize("Taxa blocks list"));
 			if (lister==null){
 				message("Sorry, no module was found to list the sets of taxa");
 				return null;
 			}
 			lister.showListWindow(null, commandRec);
 			if (!commandRec.scripting() && lister.getModuleWindow()!=null)
 				lister.getModuleWindow().setVisible(true);
 			return lister;
   	 	}
    	 	else if (checker.compare(this.getClass(), "Shows a taxa list window", "[number of taxa block]", commandName, "showTaxa")) {
    	 		if (StringUtil.blank(arguments)) {
	    	 		for (int i = 0; i< getProject().getNumberTaxas(); i++) {
	    	 			showTaxa(getProject().getTaxa(i), commandRec);
	    	 		}
    	 		}
    	 		else {
	   	 		Taxa t = getProject().getTaxa(checker.getFile(), parser.getFirstToken(arguments));  //first try within file
	   	 		if (t == null)
	   	 			t = getProject().getTaxa(parser.getFirstToken(arguments));
	   	 		
	   	 		if (t!=null) {
		    	 		return showTaxa(t, commandRec);
		    	 	}
    	 		}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets the ID number of a taxa block", "[id number of taxa block]", commandName, "setID")) {
      	 		int t = MesquiteInteger.fromString(parser.getFirstToken(arguments));
    	 		if (MesquiteInteger.isCombinable(t) && t< getProject().getNumberTaxas()) {
    	 			long id  = MesquiteLong.fromString(parser.getNextToken());
	    	 		Taxa tx = getProject().getTaxa(t);
	    	 		if (tx!=null) {
	    	 			tx.assignedIDNumber = id;
	    	 			return tx;
	    	 		}
    	 		}
    	 		
    	 	}

    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
   	 }
   	MesquiteModule showTaxa(Taxa tx, CommandRecord commandRec){
	    	 		boolean found = false;
				for (int i = 0; i<getNumberOfEmployees(); i++) {
					Object e=getEmployeeVector().elementAt(i);
					if (e instanceof ManagerAssistant)
						if (((ManagerAssistant)e).showing(tx)) {
							((ManagerAssistant)e).getModuleWindow().setVisible(true);
							return ((ManagerAssistant)e);
						}
				}
				
				ManagerAssistant lister= (ManagerAssistant)hireNamedEmployee(commandRec, ManagerAssistant.class, StringUtil.tokenize("Taxon List"));
	 			if (lister==null){
	 				if (!commandRec.scripting()) 
	 					message("Sorry, no module was found to list the taxa.");
	 				return null;
	 			}
    	 			lister.showListWindow(tx, commandRec);
    	 			if (!commandRec.scripting() && lister.getModuleWindow()!=null)
    	 				lister.getModuleWindow().setVisible(true);
    	 			return lister;
   	}
	/*.................................................................................................................*/
	public String getNexusCommands(MesquiteFile file, String blockName){ 
		if (blockName.equalsIgnoreCase("NOTES")) {
			String s ="";
			boolean found = false;
			MesquiteProject project = file.getProject();
			for (int i=0; i<project.getNumberTaxas(); i++){
				Taxa taxa = getProject().getTaxa(i);
				for (int it = 0; it<taxa.getNumTaxa(); it++){
					String obj = taxa.getAnnotation(it);
					if (!StringUtil.blank(obj)){
						s += "\tTEXT TAXA = " + StringUtil.tokenize(taxa.getName()) + " TAXON = " + Taxon.toExternal(it) + " TEXT = " + StringUtil.tokenize(obj) + ";" + StringUtil.lineEnding();
						found = true;
					}
				}
			}
			if (found)
				return s;
			else
				return null;
		}
		return null;
	}
	/*...................................................................................................................*/
	public boolean readNexusCommand(MesquiteFile file, String blockName, String command, MesquiteString comment){ 
		if (blockName.equalsIgnoreCase("NOTES")) {
			MesquiteProject project = file.getProject();
			String commandName = parser.getFirstToken(command);
			if (commandName.equalsIgnoreCase("TEXT")) {
				stringPos.setValue(parser.getPosition());
				String[][] subcommands  = ParseUtil.getSubcommands(command, stringPos);
				if (subcommands == null || subcommands.length == 0 || subcommands[0] == null || subcommands[0].length == 0)
					return false;
				int whichTaxon = MesquiteInteger.unassigned;
				String text = null;
				Taxa taxa = getProject().getTaxa(0);
				for (int i=0; i<subcommands[0].length; i++){
					String subC = subcommands[0][i];
					if ("TAXON".equalsIgnoreCase(subC)) {
						String token = subcommands[1][i];
						whichTaxon = MesquiteInteger.fromString(token);
						if (!MesquiteInteger.isCombinable(whichTaxon))
							return false;
						whichTaxon = Taxon.toInternal(whichTaxon);
					}
					else if ("TAXA".equalsIgnoreCase(subC)) {
						String token = subcommands[1][i];
						Taxa t = getProject().getTaxa(token);
						if (t==null){
							int wt = MesquiteInteger.fromString(token);
							if (MesquiteInteger.isCombinable(wt))
								t = getProject().getTaxa(wt);
						}
						if (t!=null)
							taxa = t;
						else
							return false;
					}
					else if ("TEXT".equalsIgnoreCase(subC)) {
						text = subcommands[1][i];
					}
					else if ("CHARACTER".equalsIgnoreCase(subC)) {
						return false;
					}
				}
				if (!MesquiteInteger.isCombinable(whichTaxon))
					return false;
				if (taxa !=null && text !=null) {
					taxa.setAnnotation(whichTaxon, text);
					return true;
				}
				
			}
		}
		return false;
	}
	/*.................................................................................................................*/
	public NexusBlock readNexusBlock(MesquiteFile file, String name, FileBlock block, StringBuffer blockComments, String fileReadingArguments, CommandRecord commandRec){
		Parser commandParser = new Parser();
		commandParser.setString(block.toString());
		MesquiteInteger startCharC = new MesquiteInteger(0);
		Taxa newTaxa=null;
		String title=getProject().getTaxas().getUniqueName("Untitled Block of Taxa");
		String s;
		NexusBlock t=null;
		Vector unrec = new Vector();
		Vector unrecName = new Vector();
		while (!StringUtil.blank(s=commandParser.getNextCommand(startCharC))) {
			String commandName = parser.getFirstToken(s);
			if (commandName.equalsIgnoreCase("DIMENSIONS")) {
				int numTaxa = MesquiteInteger.fromString(parser.getTokenNumber(4));
				newTaxa = new Taxa(numTaxa);
				newTaxa.setName(title);
				//t = newTaxa.addToFile(file, getProject(), this);
			}
			else if (commandName.equalsIgnoreCase("TITLE")) {
				title = parser.getTokenNumber(2);
			}
			else if (commandName.equalsIgnoreCase("TAXLABELS") && newTaxa!=null) {
				String taxonName;
				for (int it=0; it<newTaxa.getNumTaxa() && !(taxonName=parser.getNextToken()).equals(";"); it++) {
					newTaxa.setTaxonName(it, taxonName);
				}
			}
			else if (!(commandName.equalsIgnoreCase("BEGIN") || commandName.equalsIgnoreCase("END")  || commandName.equalsIgnoreCase("ENDBLOCK")))  {
				unrec.addElement(s); //store unrecognized commands because they can't be stored until issue of duplicate taxa blocks is resolved
				unrecName.addElement(commandName);
			}
		}
		if (newTaxa!=null) {
			Taxa eT = existsInOtherFile(newTaxa);
			if (eT !=null){
				String ftn = "";
				if (newTaxa.getTaxon(0)!=null)
					ftn = "; name of first taxon: " + newTaxa.getTaxon(0).getName();
				String w = ("There is apparently a duplicate taxa block in the file.  First block: \"" + eT.getName() + "\"; second block: \"" + newTaxa.getName() + "\"" + ftn + "."); 
				if (alerts.getValue() && !CommandRecord.threadIsScripting()) {
					if (AlertDialog.query(containerOfModule(), "Duplicate taxa block?", w + " Do you want to delete the second block?", "Delete", "Keep", 1)){
						newTaxa.dispose();
						return new TaxaBlock(null, null);
					}
				}
				else {
					logln(w + " Only the first block will be kept.");
					newTaxa.dispose();
					return new TaxaBlock(null, null);
				}
			}

			if (t==null)
				t = newTaxa.addToFile(file, getProject(), this);
			if (unrec.size()>0){
				for (int i = 0; i<unrec.size(); i++){
					String commandName = (String)unrecName.elementAt(i);
					s = (String)unrec.elementAt(i);
					readUnrecognizedCommand(file, t, name, block, commandName, s, blockComments, null);
				}
			}
			if (almostExistsInOtherFile(newTaxa)) {
				String ftn = "";
				if (newTaxa.getTaxon(0)!=null)
					ftn = "; name of first taxon: " + newTaxa.getTaxon(0).getName();
				message("A taxa block (\"" + newTaxa.getName() + "\"" + ftn + ") has been found that has more than 80% overlap in taxon names with another taxa block.  If you had intended them to be the same, review them to see that they contain the same number of taxa, and that their taxon names are identical.");
				
			}
			
			newTaxa.setName(title);
		}
		if (newTaxa!=null){
			String d = hasDuplicateNames(newTaxa);
			if (d !=null){
				message("A taxa block has duplicated taxon names.  You should fix this.  (duplicated names: " + d + ")");
			}
		}
		if (newTaxa != null && blockComments!=null && blockComments.length()>0)
			newTaxa.setAnnotation(blockComments.toString(), false);
		return t;
	}
	/*.................................................................................................................*/
	Taxa existsInOtherFile (Taxa taxa){
		
		for (int i =0; i<getProject().getNumberTaxas(); i++){
			Taxa t = getProject().getTaxa(i);
			if (t.getFile() != taxa.getFile() && t.equals(taxa, true))
				return t;
		}
		return null;
	}
	/*.................................................................................................................*/
	boolean almostExistsInOtherFile (Taxa taxa){
		if (taxa == null || taxa.getNumTaxa()==0)
			return false;
		for (int i =0; i<getProject().getNumberTaxas(); i++){
			Taxa t = getProject().getTaxa(i);
			if (t!=taxa && t.getFile()!=taxa.getFile()){
				if (t == null || t.getNumTaxa()==0)
					return false;
				double matches = 0.0;
				for (int it=0; it<t.getNumTaxa(); it++)
					if (taxa.getTaxon(t.getTaxonName(it))!=null)
						matches+= 1.0;
				double avg = matches/t.getNumTaxa();
				double matches2 = 0.0;
				for (int it=0; it<taxa.getNumTaxa(); it++)
					if (t.getTaxon(taxa.getTaxonName(it))!=null)
						matches2+= 1.0;
				if (MesquiteDouble.minimum( matches2/t.getNumTaxa(), avg) > 0.8)
					return true;
			}
		}
		return false;
	}
	/*.................................................................................................................*/
	String hasDuplicateNames (Taxa taxa){
		String list = "";
		for (int i=0; i<taxa.getNumTaxa(); i++){
			String name = taxa.getTaxonName(i);
 			for (int j=i+1; j<taxa.getNumTaxa(); j++){
 				String name2 = taxa.getTaxonName(j);
 				if (name!=null && name.equals(name2)) {
 					list += " " + name;
 				}
 			}
 		}
 		if (list.length() == 0)
 			return null;
 		return list;
	}
	/*.................................................................................................................*/
	String fixDuplicateNames (Taxa taxa){
		String list = "";
		for (int i=0; i<taxa.getNumTaxa(); i++){
			String name = taxa.getTaxonName(i);
 			for (int j=i+1; j<taxa.getNumTaxa(); j++){
 				String name2 = taxa.getTaxonName(j);
 				if (name!=null && name.equals(name2)) {
					 String s = taxa.getUniqueName(name2);
 					taxa.setTaxonName(j, s);
 					list += "\"" + name2 + "\" changed to \"" + s + "\"\n";
				}
 			}
 		}
 		return list;
	}
	void message(String s){
		if (alerts.getValue() && !CommandRecord.threadIsScripting())
			alert(s);
		else
			logln(s);
	}
	/*.................................................................................................................*/
	public String getTaxaBlock(Taxa taxa, TaxaBlock tB){
				//check first for file ready to write.  In future should have general call to modules, but for now just check for duplicate taxon names
		String d = hasDuplicateNames(taxa);
		if (d !=null){
			if (AlertDialog.query(containerOfModule(), "Duplicate names!", "The block of taxa being saved (" + taxa.getName() + ") has duplicate taxon names.  This may cause problems in saving and reading trees and other functions.  Do you want Mesquite to fix this by generating unique names by suffixing a number to duplicate names?   (Duplicates: " +d + ")", "Fix", "Don't Fix"))
				alert("Summary of name changes:\n" + fixDuplicateNames(taxa));
		}

		MesquiteFile file = taxa.getFile();
		if (file!=null && file.useDataBlock) //removed 19Jan02 so as not to make minimal dependent on categorical: &&  file.getFileElements().size(mesquite.categ.lib.CategoricalData.class)>0
			return null;
		String block ="";
		String end = StringUtil.lineEnding();
		block+=end+"BEGIN TAXA;" + end;
		
		if (getProject().getNumberTaxas()>1 || !NexusBlock.suppressTITLE)
			block+="\tTITLE " + StringUtil.tokenize(taxa.getName()) + ";" + end;
		if (taxa.getAnnotation()!=null) {
			block += "[!" + taxa.getAnnotation() + "]" + StringUtil.lineEnding();
		}
		block+="\tDIMENSIONS NTAX=" + taxa.getNumTaxa() + ";" + end + "\tTAXLABELS" + end + "\t\t";
		for (int it=0; it<taxa.getNumTaxa(); it++) {
			if (taxa.getTaxon(it).getName()!=null)
				block += StringUtil.tokenize(taxa.getTaxon(it).getName()) + " ";
			else
				block +=StringUtil.tokenize(" ");
		}
		block += end + "\t;" + end + tB.getUnrecognizedCommands()+ end+  "END;" + end+ end;
		return block;
	}
	/*.................................................................................................................*/
    	 public String getName() {
		return "Manage TAXA blocks";
   	 }
	/*.................................................................................................................*/
    	 public String getNameForMenuItem() {
		return "Taxon Manager";
   	 }
   	 
	/*.................................................................................................................*/
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Manages sets of taxa (including read/write TAXA block in NEXUS file)." ;
   	 }
}
	
	
/* ======================================================================== */
class TaxaBlockTest extends NexusBlockTest  {
	public TaxaBlockTest () {
	}
	public  boolean readsWritesBlock(String blockName, FileBlock block){ //returns whether or not can deal with block
		return blockName.equalsIgnoreCase("TAXA");
	}
}

/** An object of this kind can be returned by getNexusCommandTest that will be stored in the modulesinfo vector and used
to search for modules that can read a particular command in a particular block.  (Much as the NexusBlockObject.)*/
class TAXANexusCommandTest extends NexusCommandTest  {
	MesquiteInteger pos = new MesquiteInteger();
	/**returns whether or not the module can deal with command*/
	public boolean readsWritesCommand(String blockName, String commandName, String command){
		boolean b = (blockName.equalsIgnoreCase("NOTES")  && commandName.equalsIgnoreCase("TEXT"));
		if (b){
			pos.setValue(0);
			String firstToken = ParseUtil.getFirstToken(command,  pos);
			
			String[][] subcommands  = ParseUtil.getSubcommands(command, pos);
			if (subcommands == null)
				return false;
			if (StringArray.indexOfIgnoreCase(subcommands, 0, "TAXON")<0)
				return false;
			if (StringArray.indexOfIgnoreCase(subcommands, 0, "CHARACTER")>=0)
				return false;
			return true;
		}
		return false;
	} 
}
