/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.minimal.NEXUSDefaults;

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;

/* ======================================================================== */
public class NEXUSDefaults extends DefaultsAssistant {
	MesquiteBoolean suppressTitleLink;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		suppressTitleLink = new MesquiteBoolean(false);
		loadPreferences();
		addCheckMenuItem( null, "Suppress TITLE in NEXUS files", makeCommand("toggleSuppress",  this), suppressTitleLink);
		return true;
  	 }
	public void processPreferencesFromFile (String[] prefs) {
		if (prefs!=null && prefs.length>0) {
				if (prefs[0].equals("suppress")) {
					suppressTitleLink.setValue(true);
    	 				NexusBlock.suppressLINK = true;
    	 				NexusBlock.suppressTITLE = true;
    	 			}
				else  {
					suppressTitleLink.setValue(false);
    	 				NexusBlock.suppressLINK = false;
    	 				NexusBlock.suppressTITLE = false;
    	 			}
		}
	}
	/*.................................................................................................................*/
	public String[] preparePreferencesForFile () {
		if (suppressTitleLink.getValue())
			return (new String[] {"suppress"});
		else
			return (new String[] {"nosuppress"});
	}
	MesquiteInteger pos = new MesquiteInteger();
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Sets whether TITLE and LINK commands are to be suppressed where possible in saving NEXUS files", "[on = suppress; off = no]", commandName, "toggleSuppress")) {  
    	 		boolean current = suppressTitleLink.getValue();
    	 		suppressTitleLink.toggleValue(parser.getFirstToken(arguments));
    	 		if (current!=suppressTitleLink.getValue()) {
    	 			NexusBlock.suppressTITLE = suppressTitleLink.getValue();
    	 			NexusBlock.suppressLINK = NexusBlock.suppressTITLE;
				storePreferences();
    	 		}
			return suppressTitleLink;
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
   	 }
  	 
	/*.................................................................................................................*/
    	 public String getName() {
		return "Suppress TITLE and LINK";
   	 }
	/*.................................................................................................................*/
  	 public String getExplanation() {
		return "Sets whether TITLE and LINK commands are to be suppressed where possible in saving NEXUS files.";
   	 }
}

