/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.molec.BLASTSearch; 

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import mesquite.lib.table.*;
import mesquite.categ.lib.*;

/* ======================================================================== */
public class BLASTSearch extends CategDataSearcher { 
	MesquiteTable table;
	CharacterData data;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName){
		return true;
	}
	/*.................................................................................................................*/
	/** returns whether this module is requesting to appear as a primary choice */
   	public boolean requestPrimaryChoice(){
   		return true;  
   	}
	/*.................................................................................................................*/
   	 public boolean isPrerelease(){
   	 	return false;
   	 }
   	/** Called to search on the data in selected cells.  Returns true if data searched*/
   	public boolean searchData(CharacterData data, MesquiteTable table, CommandRecord commandRec){
		this.data = data;
		if (!(data instanceof DNAData || data instanceof ProteinData)){
			discreetAlert(commandRec, "Only DNA or protein data can be searched using this module.");
			return false;
		}
		return searchSelectedTaxa(data,table, commandRec);
	}
	/*.................................................................................................................*/
   	public void searchOneTaxon(CharacterData data, int it, int icStart, int icEnd, CommandRecord commandRec){
   		StringBuffer searchBuffer = new StringBuffer(data.getNumChars());
   		for (int ic = icStart; ic<=icEnd; ic++) {
   			data.statesIntoStringBuffer(ic, it, searchBuffer, false, false, false);
   		}
   		String seq = searchBuffer.toString();
		if (!StringUtil.blank(seq) && (seq.length()>2)) {
			String url = "http://www.ncbi.nlm.nih.gov/blast/Blast.cgi?";
			url += "DATABASE=nr&FORMAT_TYPE=HTML";
			if (data instanceof ProteinData) 
				url += "&PROGRAM=blastp";
			else
				url += "&PROGRAM=blastn";
			url += "&CLIENT=web&SERVICE=plain&PAGE=Nucleotides&CMD=Put&QUERY=";
			MesquiteModule.showWebPage(url + seq, true);
		}
		else 
			discreetAlert(commandRec, "Sorry, to use the BLAST search you need to have one or more stretches of sequence selected.");
   	}
	/*.................................................................................................................*/
    	 public String getName() {
		return "BLAST Search";
   	 }
	/*.................................................................................................................*/
  	 public boolean showCitation() {
		return false;
   	 }
   	 
	/*.................................................................................................................*/
  	 public String getExplanation() {
		return "Does a blast search on selected data.";
   	 }
}

/*
Search string into Genbank Entrez:

http://www.ncbi.nlm.nih.gov:80/entrez/query.fcgi?cmd=Search&db=nucleotide&dopt=GenBank&term=Bembidion

http://www.ncbi.nlm.nih.gov/blast/Blast.cgi?DATABASE=nr&FORMAT_TYPE=HTML&PROGRAM=blastn&CLIENT=web&SERVICE=plain&PAGE=Nucleotides&CMD=Put&QUERY= 
http://www.ncbi.nlm.nih.gov/blast/Blast.cgi?DATABASE=nr&HITLIST_SIZE=10&FILTER=L&EXPECT=10&FORMAT_TYPE=HTML&PROGRAM=blastn&CLIENT=web&SERVICE=plain&NCBI_GI=on&PAGE=Nucleotides&CMD=Put&QUERY= 


http://www.ncbi.nlm.nih.gov/blast/Blast.cgi?CMD=put&PAGE=Nucleotides&program=blast&QUERY_FILE=fasta&query="CCAAGTCCTTCTTGAAGGGGGCCATTTACCCATAGAGGGTGCCAGGCCCGTAGTGACCATTTATATATTTGGGTGAGTTTCTCCTTAGAGTCGGGTTGCTTGAGAGTGCAGCTCTAAGTGGGTGGTAAACTCCATCTAAGGCTAAATATGACTGCGAAACCGATAGCGAACAAGTACCGTGAGGGAAAGTTGAAAAGAACTTTGAAGAGAGAGTTCAAGAGTACGTGAAACTGTTCAGGGGTAAACCTGTGGTGCCCGAAAGTTCGAAGGGGGAGATTC"

*/
	

