/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.molec.ColorByNumber; 

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import mesquite.lib.table.*;
import mesquite.cont.lib.*;

/* ======================================================================== */
public class ColorByNumber extends DataWindowAssistantI implements CellColorer {
	MesquiteTable table;
	CharacterData data;
	NumberForCharAndTaxon numberTask;
	MesquiteString numberTaskName;
	MesquiteCommand ntC;
	double[][] values;
	int windowWidth = 1;
	MesquiteSubmenuSpec mss;
	Class stateClass;
	String ntName = null;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName){
		ntC =makeCommand("setNumberTask",  this);
 		numberTaskName = new MesquiteString();
		return true;
	}
	public void setActiveColors(boolean active, CommandRecord commandRec){
 		if (active){
			if (numCompatibleModulesAvailable(NumberForCharAndTaxon.class, stateClass, this)>0) {
				mss = addSubmenu(null, "Values for Colors", ntC, NumberForCharAndTaxon.class);
	 			mss.setSelected(numberTaskName);
	 			mss.setCompatibilityCheck(stateClass);
				mss.setEnabled(false);
			}
			addMenuItem("Moving Window (for colors)...", makeCommand("setWindowWidth", this));
	 		if (!StringUtil.blank(ntName)) //remember last one used
				numberTask = (NumberForCharAndTaxon)hireNamedEmployee(commandRec, NumberForCharAndTaxon.class, ntName, data.getStateClass());//shouldn't ask as this is an init and might not be needed.  "Value to calculate for character state in taxon"
			
			if (numberTask == null)
				numberTask = (NumberForCharAndTaxon)hireCompatibleEmployee(commandRec, NumberForCharAndTaxon.class, data.getStateClass(), "Number for nodes");//shouldn't ask as this is an init and might not be needed.  "Value to calculate for character state in taxon"
			
	 		if (numberTask != null){
		 		ntName = "#"+numberTask.getShortClassName();
		 		numberTask.setHiringCommand(ntC);
		 		numberTaskName.setValue(numberTask.getName());
 			}
 			mss.setCompatibilityCheck(data.getStateClass());
	 		
			mss.setEnabled(active);
		}
		else {
			fireEmployee(numberTask);
			numberTask = null;
			deleteMenuItem(mss);
		}
		resetContainingMenuBar();
		
	}
	public void endJob(){
		if (data!=null)
			data.removeListener(this);
		super.endJob();
	}
	double minValue = 0;
	double maxValue = 0;
	/*.................................................................................................................*/
	void calculateNums(CommandRecord commandRec){
   	 	if (data == null)
   	 		return;
    	 	if (values == null || data.getNumChars() != values.length || (values.length < 1 || data.getNumTaxa() != values[0].length))
   	 		values = new double[data.getNumChars()][data.getNumTaxa()];
   	 	if (numberTask ==null)
   	 		return;
   	 	MesquiteNumber result = new MesquiteNumber();
   	 	MesquiteNumber min = new MesquiteNumber();
   	 	MesquiteNumber max = new MesquiteNumber();
   	 	
   	 	for (int ic= 0; ic < data.getNumChars(); ic++)
   	 		for (int it= 0; it < data.getNumTaxa(); it++) {
   	 			result.setToUnassigned();
   	 			numberTask.calculateNumber(data, ic, it, result, null, commandRec);
   	 			values[ic][it] = result.getDoubleValue();
   	 			min.setMeIfIAmMoreThan(result);
   	 			max.setMeIfIAmLessThan(result);
   	 		}
   	 	if (windowWidth>1){
   	 		double[][] win = new double[data.getNumChars()][data.getNumTaxa()];
   	 		for (int it= 0; it < data.getNumTaxa(); it++) {
		   	 	for (int ic= 0; ic < data.getNumChars(); ic++){
		   	 		int left = windowWidth/2;
		   	 		int n = 0;
		   	 		//first go left, skipping gaps!
		   	 		int iw = 0;
		   	 		int is = ic; //start at current character
		   	 		while (iw<left && is>0){
		   	 			is--;
	   	 				if (!data.isInapplicable(is, it)){
	   	 					if (MesquiteDouble.isCombinable(values[is][it])){
			   	 				win[ic][it] += values[is][it];
			   	 				n++;
			   	 				iw++;
		   	 				}
		   	 			}
	   	 			}
	   	 			is = ic-1;
		   	 		while (iw<windowWidth && is<data.getNumChars()){
		   	 			is++;
	   	 				if (!data.isInapplicable(is, it)){
	   	 					if (MesquiteDouble.isCombinable(values[is][it])){
			   	 				win[ic][it] += values[is][it];
			   	 				n++;
			   	 				iw++;
		   	 				}
		   	 			}
	   	 			}
		   	 		/* old; has problems at edges of indels
		   	 		for (int iw = 0; iw<windowWidth; iw++){
	   	 				int is = ic -left + iw;
	   	 				if (is >=0 && is<data.getNumChars() && MesquiteDouble.isCombinable(values[is][it])){
		   	 				win[ic][it] += values[is][it];
		   	 				n++;
		   	 			}
	   	 			}
	   	 			*/
	   	 			if (n!=0)
	   	 				win[ic][it] /= n;
	   	 		}
	   	 	}
   	 		values = win;
 			min.setToUnassigned();
 			max.setToUnassigned();
   	 		for (int it= 0; it < data.getNumTaxa(); it++) {
		   	 	for (int ic= 0; ic < data.getNumChars(); ic++){
	   	 			result.setValue(values[ic][it]);
	   	 			min.setMeIfIAmMoreThan(result);
	   	 			max.setMeIfIAmLessThan(result);
	   	 		}
	   	 	}
 	 	}
   	 	minValue = min.getDoubleValue();
   	 	maxValue = max.getDoubleValue();
   	 }
	/*.................................................................................................................*/
   	 public boolean isSubstantive(){
   	 	return false;
   	 }
	/*.................................................................................................................*/
	public void setTableAndData(MesquiteTable table, CharacterData data, CommandRecord commandRec){
		this.table = table;
		if (this.data!=data && this.data!=null)
			this.data.removeListener(this);
		this.data = data;
		data.addListener(this);
		stateClass = data.getStateClass();
		if (mss !=null && mss.getCompatibilityCheck() == null) {
 			mss.setCompatibilityCheck(data.getStateClass());
 			resetContainingMenuBar();
 		}
		calculateNums(commandRec);
	}
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) { 
 	 	if (numberTask ==null)
 	 		return null;
   	 	Snapshot temp = new Snapshot();
 	 	temp.addLine("setNumberTask ", numberTask);  
  	 	temp.addLine("setWindowWidth " + windowWidth);
  	 	return temp;
  	 }
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
   	 	if (checker.compare(this.getClass(), "Sets the module that calculates numbers by which to color the cells", "[name of module]", commandName, "setNumberTask")) {
    	 		NumberForCharAndTaxon temp =  (NumberForCharAndTaxon)replaceCompatibleEmployee(commandRec, NumberForCharAndTaxon.class, arguments, numberTask, stateClass);
 			if (temp!=null) {
 				numberTask = temp;
		 		ntName = "#"+numberTask.getShortClassName();
		 		numberTask.setHiringCommand(ntC);
		 		numberTaskName.setValue(numberTask.getName());
				calculateNums(commandRec);
				table.repaintAll();
	 			return numberTask;
	 		}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets the width of the moving window", "[width]", commandName, "setWindowWidth")) {
    	 		int w = MesquiteInteger.fromString(parser.getFirstToken(arguments));
    	 		if (!MesquiteInteger.isCombinable(w) && !commandRec.scripting()){
    	 			w = MesquiteInteger.queryInteger(containerOfModule(), "Moving Window", "Width of Moving Window", windowWidth, 1, 1000);
    	 		}
    	 		if (!MesquiteInteger.isCombinable(w))
    	 			return null;
    	 		windowWidth = w;
			calculateNums(commandRec);
			if (table!=null)
				table.repaintAll();
    	 	}
      	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
    	 	return null;
    	 }
	/*.................................................................................................................*/
   	 public void employeeParametersChanged(MesquiteModule employee, MesquiteModule source, Notification notification, CommandRecord commandRec) {
			calculateNums(commandRec);
			if (table !=null)
				table.repaintAll();
   	 }
	/*.................................................................................................................*/
    	 public String getName() {
		return "Color By Value";
   	 }
	/*.................................................................................................................*/
  	 public String getExplanation() {
		return "Colors the cells of a character matrix according to a value for that cell or a moving window of cells.";
   	 }
	public Color getCellColor(int ic, int it){
		//data version has changed!?
		if (data == null || values == null)
			return Color.white;
		else if (data.isInapplicable(ic, it)){
			return ColorDistribution.straw;
		}
		else {
			
			return MesquiteColorTable.getBlueScale(values[ic][it], minValue, maxValue, false);
		}
	}
	/** passes which object changed, along with optional code number (type of change) and integers (e.g. which character)*/
	public void changed(Object caller, Object obj, Notification notification, CommandRecord commandRec){
		if (obj instanceof CharacterData){
			if (Notification.appearsCosmetic(notification))
				return;
			calculateNums(commandRec);
			if (table!=null)
				table.repaintAll();
		}
	}
	
	/*.................................................................................................................*/
    	 public boolean isPrerelease() {
		return false;
   	 }
	public CompatibilityTest getCompatibilityTest(){
		return new CbNCompatibilityTest();
	}
}

class CbNCompatibilityTest extends CharacterStateTest{
	public  boolean isCompatible(Object obj, MesquiteProject project, EmployerEmployee prospectiveEmployer){
		if (obj == null)
			return true;
		if (!(super.isCompatible(obj, project, prospectiveEmployer)))
			return false;
		return (((MesquiteModule)prospectiveEmployer).numCompatibleModulesAvailable(NumberForCharAndTaxon.class, obj, (MesquiteModule)prospectiveEmployer)>0); 
	}
}



	

