/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.molec.ReverseSequence;
/*~~  */

import java.util.*;
import java.awt.*;
import java.awt.image.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import mesquite.lib.table.*;

/* ======================================================================== */
public class ReverseSequence extends DataAlterer {
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		return true;
	}
		
   	/** Called to alter data in those cells selected in table*/
   	public boolean alterData(CharacterData data, MesquiteTable table, CommandRecord commandRec){
   			boolean did=false;

   	 		if (table==null && data!=null){
					for (int i=0; i<data.getNumTaxa(); i++)
						reverseCells(data, 0, data.getNumChars()-1, i);
					return true;
   	 		}
   	 		else if (table!=null && data !=null){
	   			if (table.anythingSelected()) {
					for (int j=0; j<table.getNumRows(); j++) {
		   				int minChar = table.getNumColumns();
		   				int maxChar = -1;
						boolean someSelected = false;
						for (int i=0; i<table.getNumColumns(); i++)
							if (table.isCellSelectedAnyWay(i,j)) {
								if (i<minChar)
									minChar=i;
								else if (i>maxChar)
									maxChar = i;
								someSelected = true;
							}
						if (someSelected){
							for (int i=minChar; i<=maxChar; i++)
								if (!table.isCellSelectedAnyWay(i,j)) {
									alert("Sorry, can't reverse the sequence when selection of characters is discontinuous within any taxa");
									return false;
								}
						}
					}
					for (int j=0; j<table.getNumRows(); j++) {
		   				int minChar = table.getNumColumns();
		   				int maxChar = -1;
						boolean someSelected = false;
						for (int i=0; i<table.getNumColumns(); i++)
							if (table.isCellSelectedAnyWay(i,j)) {
								if (i<minChar)
									minChar=i;
								else if (i>maxChar)
									maxChar = i;
								someSelected = true;
							}
						if (someSelected){
							reverseCells(data, minChar, maxChar, j);
							did = true;
						}
					}
				}
				else {
					for (int i=0; i<data.getNumTaxa(); i++)
						reverseCells(data, 0, data.getNumChars()-1, i);
				}
				return did;
			}
			return false;
   	}

   	
	void reverseCells(CharacterData data, int ic, int ic2, int it){

   		for (int i=0; i <= (ic2-ic)/2 && ic+i< ic2-i; i++) {
   			data.tradeStatesBetweenCharacters(ic+i, ic2-i, it);
   		}
	}
	
	/*.................................................................................................................*/
    	 public String getName() {
		return "Reverse Sequence";
   	 }
	/*.................................................................................................................*/
    	 public boolean isPrerelease() {
		return false;
   	 }
	/*.................................................................................................................*/
   	 public boolean showCitation(){
   	 	return true;
   	 }
	/*.................................................................................................................*/
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Alters data by reversing sequence of states." ;
   	 }
   	 
}

