/* Mesquite (package mesquite.ornamental).  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.ornamental.ManagePictures;
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;

/* ======================================================================== */
public class ManagePictures extends FileInit /*implements ElementManager*/ {
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
 		return true;
	}
	
	
	public boolean isSubstantive(){
		return false;
	}   	 
	public NexusBlock elementAdded(FileElement e){
		return null;
	}
	public void elementDisposed(FileElement e){
		//nothing needs doing since separate reference not stored locally
	}
	public Class getElementClass(){
		return null;
	}
	/*.................................................................................................................*/
	public String getNexusCommands(MesquiteFile file, String blockName){ 
		if (blockName.equalsIgnoreCase("NOTES")) {
			String s ="";
			boolean found = false;
			MesquiteProject project = file.getProject();
			for (int i=0; i<project.getNumberTaxas(); i++){
				Taxa taxa = getProject().getTaxa(i);
				for (int it = 0; it<taxa.getNumTaxa(); it++){
					String obj = taxa.getTaxon(it).getIllustrationPath();
					if (obj!=null){
						s += "\tPICTURE TAXA = " + StringUtil.tokenize(taxa.getName()) + " TAXON = " + (it+1) + " PICTURE = " + StringUtil.tokenize(MesquiteFile.decomposePath(getProject().getHomeDirectoryName(), obj)) + ";" + StringUtil.lineEnding();
						found = true;
					}
				}
			}
			if (found)
				return s;
			else
				return null;
		}
		return null;
	}
	/*.................................................................................................................*/
	public boolean readNexusCommand(MesquiteFile file, String blockName, String command, MesquiteString comment){ 
		if (blockName.equalsIgnoreCase("NOTES")) {
			MesquiteProject project = file.getProject();
			MesquiteInteger startCharT = new MesquiteInteger(0);
			String commandName = ParseUtil.getToken(command, startCharT);
			Taxon taxon = null;
			if (commandName.equalsIgnoreCase("PICTURE")) {
				String token = ParseUtil.getToken(command, startCharT); 
				String dummy;
				String pathName = "";
				int taxonNumber=MesquiteInteger.unassigned;
				Taxa taxa = getProject().getTaxa(0);
				while (!StringUtil.blank(token) && !token.equals(";")) {
					if (token.equalsIgnoreCase("TAXON")) {
						dummy =ParseUtil.getToken(command, startCharT); // =
						String whichItem = (ParseUtil.getToken(command, startCharT)); // name of taxon/etc
						taxonNumber = MesquiteInteger.fromString(whichItem);
						if (MesquiteInteger.isCombinable(taxonNumber))
							taxonNumber--; //to convert to internal
					}
					else if (token.equalsIgnoreCase("TAXA")) {
						dummy =ParseUtil.getToken(command, startCharT); // =
						String taxaTitle = (ParseUtil.getToken(command, startCharT));
						taxa = getProject().getTaxa(taxaTitle);
						if (taxa == null) {
							taxa = getProject().getTaxa(0);
						}
					}
					else if (token.equalsIgnoreCase("SOURCE")) {
						dummy =ParseUtil.getToken(command, startCharT); // =
						String source = (ParseUtil.getToken(command, startCharT));
						if (!("file".equalsIgnoreCase(source))) {
							file.setOpenAsUntitled("A picture source (\"" + source + "\", in NOTES block) was not recognized.  Mesquite may be unable to read and use the picture.");
							if (taxon != null)
								taxon.setIllustration(null, pathName);
							return true; //returns true without saving object so that the note is deleted from the file
						}
					}
					else if (token.equalsIgnoreCase("PICTURE")) {
						if (taxa!=null){
							dummy =ParseUtil.getToken(command, startCharT); // =
							pathName = ParseUtil.getToken(command, startCharT);
							if (MesquiteInteger.isCombinable(taxonNumber)) {
								taxon = taxa.getTaxon(taxonNumber);
								if (taxon!=null)
									taxon.setIllustration(MesquiteImage.getImage(pathName, file.getProject()), pathName);
							}
						}
					}
					token = ParseUtil.getToken(command, startCharT); 
				}
				return true;
			}
		}
		return false;
	}
	/*.................................................................................................................*/
	public NexusCommandTest getNexusCommandTest(){ 
		return new HyperlinkNexusCommandTest();
	}
	/*.................................................................................................................*/
    	 public String getName() {
		return "Manage pictures";
   	 }
   	 
	/*.................................................................................................................*/
   	 
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Manages (including NEXUS read/write) pictures." ;
   	 }
	/*.................................................................................................................*/
   	 
}

class HyperlinkNexusCommandTest  extends NexusCommandTest{
	public boolean readsWritesCommand(String blockName, String commandName, String command){  //returns whether or not can deal with command
		return (blockName.equalsIgnoreCase("NOTES") && commandName.equalsIgnoreCase("PICTURE"));
	}
}

