/* Mesquite (package mesquite.parsimony).  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.parsimony.StepMatrixCurator;
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import mesquite.categ.lib.*;
import mesquite.parsimony.lib.*;

/* ======================================================================== */
public class StepMatrixCurator extends WholeCharModelCurator implements EditingCurator {
	
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		return true;
  	 }
	/*.................................................................................................................*/
    	 public String getName() {
		return "Stepmatrix";
   	 }
   	 
	/*.................................................................................................................*/
  	 public String getExplanation() {
		return "Supplies editor for and manages stepmatrices (cost matrices).";
   	 }
	/*.................................................................................................................*/
	/** passes which object changed, along with optional code number (type of change) and integers (e.g. which character)*/
	public void changed(Object caller, Object obj, Notification notification, CommandRecord commandRec){
		if (obj instanceof CharacterModel){
	 		int i = getModelNumber((CharacterModel)obj);
	 		if (i>=0) {
	 			CategTModelEditWindow window = (CategTModelEditWindow)getWindow(i);
				if (window!=null)
					window.setModel((CategTModel)obj);
	 		}
		}
		super.changed(caller, obj, notification, commandRec);
	}
	/*.................................................................................................................*/
   	public MesquiteModule editModelNonModal(CharacterModel model, ObjectContainer w, CommandRecord commandRec){
		if (model!=null && model instanceof StepMatrixModel) {
			StepMatrixModel modelToEdit = (StepMatrixModel)model;
			MesquiteModule windowServer = hireNamedEmployee (commandRec, WindowHolder.class, "#WindowBabysitter");
			if (windowServer == null)
				return null;
			
			CategTModelEditWindow window = new CategTModelEditWindow(this, windowServer);
			windowServer.makeMenu("Step_Matrix");
			windowServer.addMenuItem("Set maximum state...", makeCommand("setMaxState", modelToEdit));
			
			window.setDefaultAnnotatable(model);
			window.setModel((StepMatrixModel)model);
			if (w!=null)
				w.setObject(window);
			return windowServer;
		}
		return this;
	}
	public boolean curatesModelClass(Class modelClass){
		return StepMatrixModel.class.isAssignableFrom(modelClass);
	}
	/*.................................................................................................................*/
	public String getNameOfModelClass() {
		return "Stepmatrix";
	}
	/*.................................................................................................................*/
	public String getNEXUSNameOfModelClass() {
		return "Stepmatrix";
	}
	/*.................................................................................................................*/
	public Class getModelClass() {
		return StepMatrixModel.class;
	}
	public boolean isPrerelease(){
		return false;
	}
	/*.................................................................................................................*/
   	public CharacterModel makeNewModel(String name) {
   		StepMatrixModel model = new StepMatrixModel(name, getProject());
   		return model;
   	}
	/*.................................................................................................................*/
   	public CharacterModel readCharacterModel(String name, MesquiteInteger stringPos, String description, int format) {
   		StepMatrixModel model = new StepMatrixModel(name, getProject());
   		model.fromString(description, stringPos, format);
  		return model;
   	}
	/*.................................................................................................................*/

	public String getNexusCommands(MesquiteFile file, String blockName){ return "";}
}

	/*=======================================================*/
class StepmatrixNexusCmdTest  extends NexusCommandTest{
	public boolean readsWritesCommand(String blockName, String commandName, String command){  //returns whether or not can deal with command
		return (blockName.equalsIgnoreCase("ASSUMPTIONS") && commandName.equalsIgnoreCase("stepmatrix"));
	}
}

/* ======================================================================== */
/** A character model for categorical characters to be used in parsimony calculations*/
class StepMatrixModel  extends CostMatrixModel implements CategTModel {
	int[] states;
	NumberArray costs;
	int numStates;
	MesquiteNumber utilityNumber;
	MesquiteProject project;
	int maxNumStates = CategoricalState.maxCategoricalState+1;
	
	public StepMatrixModel (String name, MesquiteProject project) {
		super(name, CategoricalState.class);
		this.project = project;
		costs = new NumberArray(maxNumStates*maxNumStates);
		states = new int[maxNumStates];
		for (int i=0; i< maxNumStates*maxNumStates; i++) {
			costs.setValue(i, 1);
		}
		for (int i=0; i< maxNumStates; i++) {
			costs.setValue(i*maxNumStates + i, 0);
		}
		numStates = 10;
		for (int i=0; i< numStates; i++) {
			states[i]=i;
		}
		utilityNumber = new MesquiteNumber();
	}
	
	/** returns whether or not size can be changed externally*/
	public boolean canChangeSize() {
		return true;
	}
	/** returns whether or not model is default (unordered, initial Jukes-Cantor; if so, then doesn't need to be written to file*/
	public boolean isBuiltIn() {
		return false;
	}
	/** returns name of model class (e.g. "stepmatrix")*/
	public String getNEXUSClassName() {
		return "Stepmatrix";
	}
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Sets the maximum state value handled by the step matrix", "[state number]", commandName, "setMaxState")) {
   	 		MesquiteInteger io = new MesquiteInteger(0);
   			int maxState = MesquiteInteger.fromString(arguments, io);
   			
   			if (!MesquiteInteger.isCombinable(maxState) || maxState<0 || maxState>= CategoricalState.maxCategoricalState){
   				maxState = MesquiteInteger.queryInteger(MesquiteTrunk.mesquiteTrunk.containerOfModule(), "Set Maximum state", "Maximum state value handled by the step matrix", 9);
   			}
   			if (maxState !=getMaxStateDefined() && !(!MesquiteInteger.isCombinable(maxState) || maxState<0 || maxState> CategoricalState.maxCategoricalState)){
   				setMaxStateDefined(maxState);
   			}
   			return null;
		}
    	 	else
 			return super.doCommand(commandName, arguments, commandRec, checker);
   	 }
	public void setMaxState(int maxState){
		if (maxState>0 && maxState<=CategoricalState.maxCategoricalState) {
			numStates = maxState+1;
			notifyListeners(this, new Notification(MesquiteListener.UNKNOWN),CharacterModel.class, true, CommandRecord.getRecNSIfNull());
			notifyListeners(this, new Notification(MesquiteListener.UNKNOWN),CharacterModel.class, false, CommandRecord.getRecNSIfNull());
		}
	}
	public void setMaxStateDefined(int maxState){
		setMaxState(maxState);
	}
	public int getMaxStateDefined(){
		return numStates -1;
	}
	public void fromString (String description, MesquiteInteger stringPos, int format) {
   		costs.deassignArray();
   		for (int i=0; i<states.length; i++)
   			states[i]=-1;
   		int matrixSize = MesquiteInteger.fromString(description, stringPos);
   		int maxS = matrixSize-1;
   		if (project != null && project.getNumberCharMatrices(CategoricalState.class) ==1) {
   			CategoricalData data = (CategoricalData)project.getCharacterMatrix(0, CategoricalState.class);
	    		for (int i=0; i<matrixSize; i++) {
	    			String s = ParseUtil.getToken(description, stringPos);
	    			int state = -1;
	    			if (s !=null && s.length()==1)
	    				state = CategoricalState.minimum(data.fromChar(s.charAt(0)));
		   		else
		   			state = MesquiteInteger.fromString(s);
		   		if (MesquiteInteger.isCombinable(state) && state>=0)
		   			states[i] = state;
		   		else
		   			states[i] = i; //todo: warning???
		   		if (states[i]>maxS)
		   			maxS = states[i];
	    		}
   		}
    		else {
	   		CategoricalState cs = new CategoricalState();
	    		for (int i=0; i<matrixSize; i++) {
		   		String s = ParseUtil.getToken(description, stringPos);
		   		int state = -1;
		   		if (s !=null && s.length() == 1)
		   			state = cs.whichState(s.charAt(0));
		   		else
		   			state = MesquiteInteger.fromString(s);
				    /*Note: stepmatrix descriptions in NEXUS can use symbols of given matrix, but this causes
				    problems if there is more than one matrix.  Currently, Mesquite does not handle non-standard symbols well,
				    because there is no reference to a specific CharacterData
				    */
		   		if (MesquiteInteger.isCombinable(state) && state>=0)
		   			states[i] = state;
		   		else
		   			states[i] = i; //todo: warning???
		   		if (states[i] >maxS)
		   			maxS = states[i] ;
	    		}
    		}
    		for (int i=0; i<matrixSize; i++) {
    			if (states[i]>=0){
		    		for (int j=0; j<matrixSize; j++) {
		    			if (states[j]>=0){
			    			String costString = ParseUtil.getToken(description, stringPos);
			    			if (costString.equals("."))
					   		costs.setValue(states[i]*maxNumStates + states[j], 0);
				   		else {
				   			utilityNumber.setValue(costString);
					   		costs.setValue(states[i]*maxNumStates + states[j], utilityNumber);
				   		}
			   		}
			   	}
		   	}
    		}
    		numStates = maxS+1;
	}
	
	public String getNexusSpecification () {
   		String s= " ";
 		if (!StringUtil.blank(getAnnotation()))
 			s +=" [!" + getAnnotation() + "] ";
   		
   		s += numStates + StringUtil.lineEnding();
   		if (project != null && project.getNumberCharMatrices(CategoricalState.class) ==1) {
   			CategoricalData data = (CategoricalData)project.getCharacterMatrix(0, CategoricalState.class);
	    		for (int i=0; i<numStates; i++) 
	    			s += "\t" + data.getSymbol(states[i]);
   		}
    		else for (int i=0; i<numStates; i++) {
	   		s += "\t" + states[i];
    		}
	    	s+= StringUtil.lineEnding();
    		for (int i=0; i<numStates; i++) {
	    		for (int j=0; j<numStates; j++) {
	    			int loc = i*maxNumStates + j;
	    			if (costs.isInfinite(loc))
	    				s+= "\ti";
	    			else
	    				s+= "\t" + costs.toString(loc);
		   	}
	    		s+=  StringUtil.lineEnding();
    		}
    		return s;
	}
	/** Returns cost to change from beginning to ending state.  Returned value may be undefined
	if either the begin or end states are not in list of states for which matrix defined*/
	public MesquiteNumber getTransitionValue (int beginState, int endState, MesquiteNumber result){
		if (result ==null)
			result = new MesquiteNumber();
		if (beginState>=numStates || endState>=numStates)
			result.setToInfinite();
		else
			costs.placeValue(beginState*maxNumStates+endState, result);
		return result;
	}
	
	public boolean isSymmetrical(){
		for (int i=0; i< numStates; i++){
			for (int j=i+1; j< numStates; j++)
				if (!costs.equal(i*maxNumStates+j, j*maxNumStates+i))
					return false;
		}
		return true;
	}
	
	/** Returns cost to change from beginning to ending state.  Returned value may be undefined
	if either the begin or end states are not in list of states for which matrix defined*/
	public void setTransitionValue (int beginState, int endState, MesquiteNumber result, boolean notify){
		if (beginState>=numStates || endState>=numStates)
			return;
		costs.setValue(beginState*maxNumStates+endState, result);
		if (notify){
			notifyListeners(this, new Notification(MesquiteListener.UNKNOWN),CharacterModel.class, true, CommandRecord.getRecNSIfNull());
			notifyListeners(this, new Notification(MesquiteListener.UNKNOWN),CharacterModel.class, false, CommandRecord.getRecNSIfNull());
		}
	}
	/** To set what are the states available for transitions*/
	public long getStatesDefined () {
		long sd = CategoricalState.compressFromList(states);
		return sd & (CategoricalState.span(0, numStates-1));
	}
	
	/** returns the highest state for which the values are defined*/
	public int getMaxState (){
		return CategoricalState.maximum(getStatesDefined());
	}
	
	public String getExplanation(){	
		return "Cost of change from state to state is given by a matrix";
	}
}
