/* Mesquite source code (Rhetenor package).  Copyright 1997-2004 E. Dyreson and W. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.rhetenor.ShowCharLoadings;
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;
import mesquite.rhetenor.lib.*;

/* ======================================================================== */
public class ShowCharLoadings extends OrdinationAssistant {
	ChartWindow cWindow;
	ItemsCharter chartWindowTask;
	CharacterLoadings loadingsTask;
	CharsFromMatrixSource charSourceConverter;
	MiniScroll axisScroll;
	boolean firstTime = true;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
 		makeMenu("Loadings");
		chartWindowTask = (ItemsCharter)hireEmployee(commandRec, ItemsCharter.class, null);
		if (chartWindowTask == null) {
			return sorry(commandRec, getName() + " couldn't start because no charting module obtained.");
		}
		cWindow = chartWindowTask.makeChartWindow(this, commandRec);
		cWindow.doCommand("valuesByItems", null, commandRec, CommandChecker.defaultChecker);

		setModuleWindow( cWindow);
		
 		//needs to be same set of characters!!!!
 		loadingsTask =  (CharacterLoadings)hireNamedEmployee(commandRec, CharacterLoadings.class, StringUtil.tokenize("Character Loadings"));
 		if (loadingsTask==null) {
			return sorry(commandRec, getName() + " couldn't start because no character loadings module obtained.");
		}
		charSourceConverter = (CharsFromMatrixSource)hireNamedEmployee(commandRec, CharsFromMatrixSource.class, StringUtil.tokenize("Characters from Matrix Source"));
		if (charSourceConverter==null) {
			return sorry(commandRec, getName() + " couldn't start because no character source module obtained.");
		}
		addMenuItem(null, "Next Axis", makeCommand("nextAxis",  this));
		addMenuItem(null, "Previous Axis", makeCommand("prevAxis",  this));
 		return true;
  	 }
  	 
  	 public void employeeQuit(MesquiteModule m){
  	 	iQuit();
  	 }
  	 public boolean canHireMoreThanOnce(){
  	 	return false;
  	 }
	/*.................................................................................................................*/
 	public void setOrdination(Ordination ordination, Taxa taxa, MatrixSourceCoord matrixSourceTask, CommandRecord commandRec){
		if (ordination!=null) {
			chartWindowTask.setTaxa(taxa);
			charSourceConverter.setMatrixSource(matrixSourceTask);
			chartWindowTask.setItemsSource(charSourceConverter);
			loadingsTask.setOrdination(ordination, taxa);
			loadingsTask.setCurrentAxis(0);
			chartWindowTask.setNumberTask(loadingsTask);
			if (firstTime){
				cWindow.setScroller(axisScroll = new MiniScroll(makeCommand("setAxis",  this), true, true, 1, 1, loadingsTask.getNumberOfAxes(),"axis"));
				cWindow.setChartTitle("Character Loadings");
				cWindow.resetTitle();
				if (!commandRec.scripting()) {
					cWindow.setChartVisible();
					if (!cWindow.isVisible())
						cWindow.setVisible(true);
				}
				firstTime = false;
			}
			else {
				if (axisScroll != null && loadingsTask != null)
					axisScroll.setMaximumValue(loadingsTask.getNumberOfAxes());
			}
			chartWindowTask.doCounts(commandRec);
	 		resetContainingMenuBar();
			resetAllWindowsMenus();
		}
		
	}
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) { 
   	 	Snapshot temp = new Snapshot();
  	 	temp.addLine("getCharter", chartWindowTask); 
 	 	temp.addLine("setChartVisible"); 
 	 	temp.addLine("doCounts"); 
 	 	temp.addLine("showWindow"); 
 	 	return temp;
  	 }
	MesquiteInteger pos = new MesquiteInteger(0);
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Goes to next axis from ordination", null, commandName, "nextAxis")) {
			int curr = loadingsTask.getCurrentAxis();
			int max = loadingsTask.getNumberOfAxes();
			if (curr+1<max) {
				loadingsTask.setCurrentAxis(curr+1);
				chartWindowTask.doCounts(commandRec);
			}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Goes to previous axis from ordination", null, commandName, "prevAxis")) {
			int curr = loadingsTask.getCurrentAxis();
			if (curr>0) {
				loadingsTask.setCurrentAxis(curr-1);
				chartWindowTask.doCounts(commandRec);
			}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets which axis from ordination is shown", "[axis number]", commandName, "setAxis")) {
  	 		int i = MesquiteInteger.fromFirstToken(arguments, pos) -1;
			int max = loadingsTask.getNumberOfAxes();
			if (MesquiteInteger.isCombinable(i) && i<max) {
				loadingsTask.setCurrentAxis(i);
				chartWindowTask.doCounts(commandRec);
			}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Returns chart drawing module", null, commandName, "getCharter")) {
  	 		return chartWindowTask;
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets the chart to visible", null, commandName, "setChartVisible")) {
  	 		if (cWindow!=null)
  	 			cWindow.setChartVisible();
    	 	}
    	 	else if (checker.compare(this.getClass(), "Requests counts of chart", null, commandName, "doCounts")) {
  	 		if (chartWindowTask!=null)
				chartWindowTask.doCounts(commandRec);
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
   	 }
	/*.................................................................................................................*/
 	public void windowGoAway(MesquiteWindow whichWindow) {
		whichWindow.hide();
		whichWindow.dispose();
		iQuit();
	}
	/*.................................................................................................................*/
    	 public boolean isPrerelease() {
		return false;
   	 }
	/*.................................................................................................................*/
    	 public String getName() {
		return "Display Character Loadings";
   	 }
	/*.................................................................................................................*/
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Shows character loadings in ordination." ;
   	 }
   	 
   	 
}
	
