/* Mesquite (package mesquite.search).  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.search.SPRRearranger;
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;

/* ======================================================================== */
public class SPRRearranger extends TreeSwapper {
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
  		return true;
  	 }
  	 
	/*.................................................................................................................*/
	private void findAttach(AdjustableTree tree, int node, int whichNode, int cutNode, MesquiteInteger total, MesquiteInteger attach){
		if (node == cutNode)
			return;
		if (tree.motherOfNode(cutNode)!=node && !tree.nodesAreSisters(cutNode, node))
			total.increment();
		if (total.getValue()==whichNode) {
			attach.setValue(node);
			return;
		}
		for (int d = tree.firstDaughterOfNode(node); tree.nodeExists(d) && attach.isUnassigned(); d = tree.nextSisterOfNode(d)) {
			findAttach(tree, d, whichNode, cutNode, total, attach);
		}
	}
	/*.................................................................................................................*/
	private void findBranchesOfRearrangement(AdjustableTree tree, int node,  MesquiteInteger total, int target, MesquiteInteger cut, MesquiteInteger attach, int numNodesTotal){
		if (!cut.isUnassigned())
			return;
		int current = total.getValue();
		if (tree.nodeIsTerminal(node)){
			total.add(numNodesTotal-3); //TODO: -2 if node is polytomous
			if (total.getValue()>=target) {
				cut.setValue(node);
				MesquiteInteger counter = new MesquiteInteger(0);
				findAttach(tree, tree.getRoot(), target-current, node, counter, attach);
			}
		}
		else {
			for (int d = tree.firstDaughterOfNode(node); tree.nodeExists(d) && cut.isUnassigned(); d = tree.nextSisterOfNode(d)) {
				findBranchesOfRearrangement(tree, d, total, target, cut, attach, numNodesTotal);
			}
			if (!cut.isUnassigned())
				return;
			total.add(numNodesTotal-tree.numberOfNodesInClade(node)-2);//TODO: -1 if node is polytomous
			if (total.getValue()>=target) {
				cut.setValue(node);
				MesquiteInteger counter = new MesquiteInteger(-1);
				findAttach(tree, tree.getRoot(), target-current, node, counter, attach);
			}
		}
	}
	/*.................................................................................................................*/
	/** finding the cut node for rearrangement # target, by checking total number of moves a branch can make*/
	private void findCutNode(AdjustableTree tree, int node,MesquiteInteger cut, MesquiteInteger whichMove, MesquiteLong total, long target){
		if (!cut.isUnassigned())
			return;
		long currentCount = total.getValue();
		total.add(numberOfMovesNodeCanMake(tree, node));
		if (currentCount<=target && total.getValue()>target) {
			cut.setValue(node);
			whichMove.setValue((int)(target-currentCount));
			return;
		}
		for (int d = tree.firstDaughterOfNode(node); tree.nodeExists(d) && cut.isUnassigned(); d = tree.nextSisterOfNode(d)) {
			findCutNode(tree, d, cut, whichMove, total, target);
		}
	}
	/*.................................................................................................................*/
	/** finding the cut node for rearrangement # target, by checking total number of moves a branch can make*/
	private void findAttachNode(AdjustableTree tree, int node,int cutNode,  MesquiteInteger attach, MesquiteLong total, long target){
		if (!attach.isUnassigned())
			return;
		//seeing if valid attach node for given cutNode
		if (node==cutNode) //don't even go into that clade
			return;
		int motherOfCut = tree.motherOfNode(cutNode);
		if ((node!=motherOfCut && motherOfCut!=tree.motherOfNode(node)) || tree.nodeIsPolytomous(motherOfCut)) {
			//can move only if not mother or sister, unless polytomous
			//is a legitimate target; see if it's the index wanted
			if (total.getValue() == target) {
				attach.setValue(node);
				return;
			}
			total.add(1);
		}
		for (int d = tree.firstDaughterOfNode(node); tree.nodeExists(d) && attach.isUnassigned(); d = tree.nextSisterOfNode(d)) {
			findAttachNode(tree, d, cutNode, attach, total, target);
		}
	}
	/*.................................................................................................................*/
	private void upCountTotal(AdjustableTree tree, int node,MesquiteLong total, int numNodesTotal){
		total.add(numberOfMovesNodeCanMake(tree, node));
		for (int d = tree.firstDaughterOfNode(node); tree.nodeExists(d); d = tree.nextSisterOfNode(d)) {
			upCountTotal(tree, d, total, numNodesTotal);
		}
	}
	
	/*.................................................................................................................*/
	/** Returns the number of moves a node can make.  Is number of nodes in the tree outside of its own clade (since node can't move into its own
	clade) and, if its mother is dichotomous, minus 2 (if dichtomous, it can't move onto its own ancestor since it's already there, and 
	it can move onto a sister only if it has more than one sister).*/
	private long numberOfMovesNodeCanMake(AdjustableTree tree, int node){
		int root = tree.getRoot();
		if (!tree.nodeExists(node) || root==node)
			return 0;
		int numInClade = tree.numberOfNodesInClade(node);
		int numInTree = tree.numberOfNodesInClade(root);
		int mother = tree.motherOfNode(node);
		if (!tree.nodeIsPolytomous(mother))
			return numInTree-numInClade-2;
		else
			return numInTree-numInClade;
	}
	/*.................................................................................................................*/
	/** Returns the number of rearrangments.*/
	public long numberOfRearrangements(AdjustableTree tree){
		MesquiteLong counter = new MesquiteLong(0); 
		upCountTotal(tree, tree.getRoot(), counter, tree.numberOfNodesInClade(tree.getRoot()));
		return counter.getValue();
	}
	int count = 0;
	int rearrangement=0;
	/*.................................................................................................................*/
	/** Rearranges the tree to the i'th rearrangment. */
	public void rearrange(AdjustableTree tree, long i){
		if (tree==null) // || i<0 || i> 2*tree.numberOfInternalsInClade(tree.getRoot())-1  //TODO: check to see that not out of bounds
			return;
		count = 0;
		MesquiteLong total = new MesquiteLong(0);
		MesquiteInteger cut = new MesquiteInteger();
		MesquiteInteger whichMove = new MesquiteInteger(0);
		cut.setToUnassigned();
		findCutNode(tree, tree.getRoot(), cut, whichMove,  total, i);
		
		MesquiteInteger attach = new MesquiteInteger();
		attach.setToUnassigned();
		total.setValue(0);
		findAttachNode(tree, tree.getRoot(),cut.getValue(),  attach, total, whichMove.getValue());
		if (!cut.isUnassigned() && !attach.isUnassigned()) {
			tree.moveBranch(cut.getValue(), attach.getValue(), false);
		}
	}
	/*.................................................................................................................*/
    	 public String getName() {
		return "SPR Rearranger";
   	 }
	/*.................................................................................................................*/
  	 public String getExplanation() {
		return "Rearranges a tree by subtree pruning and regrafting (SPR).";
   	 }
   	 public boolean requestPrimaryChoice(){
   	 	return true;
   	 }
   	 public boolean isPrerelease(){
   	 	return false;
   	 }
	/*.................................................................................................................*/
   	 public boolean showCitation(){
   	 	return true;
   	 }
}
