/* Mesquite (package mesquite.search).  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.search.TreeSearch;
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;

/* ======================================================================== */
public class TreeSearch extends TreeBlockFiller implements Incrementable {
	TreeSearcher searchTask;
 
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
 		if (arguments !=null) {
			searchTask = (TreeSearcher)hireNamedEmployee(commandRec, TreeSearcher.class,arguments);
			if (searchTask==null) {
				return sorry(commandRec, getName() + " couldn't start because the requested tree searching module not obtained");
			}
		}
    	 	else {
    	 		searchTask= (TreeSearcher)hireEmployee(commandRec, TreeSearcher.class, "Tree Searcher");
			if (searchTask==null) return sorry(commandRec, getName() + " couldn't start because tree searching module not obtained.");
  		}
  		return true;
  	 }


	/*.................................................................................................................*/
 	public void setCurrent(long i, CommandRecord commandRec){  //SHOULD NOT notify (e.g., parametersChanged)
 		if (searchTask instanceof Incrementable)
 			((Incrementable)searchTask).setCurrent(i, commandRec);
 	}
 	public long getCurrent(CommandRecord commandRec){
 		if (searchTask instanceof Incrementable)
 			return ((Incrementable)searchTask).getCurrent(commandRec);
 		return 0;
 	}
 	public String getItemTypeName(){
 		if (searchTask instanceof Incrementable)
 			return ((Incrementable)searchTask).getItemTypeName();
 		return "";
 	}
 	public long getMin(CommandRecord commandRec){
 		if (searchTask instanceof Incrementable)
 			return ((Incrementable)searchTask).getMin(commandRec);
		return 0;
 	}
 	public long getMax(CommandRecord commandRec){
 		if (searchTask instanceof Incrementable)
 			return ((Incrementable)searchTask).getMax(commandRec);
		return 0;
 	}
 	public long toInternal(long i){
 		if (searchTask instanceof Incrementable)
 			return ((Incrementable)searchTask).toInternal(i);
 		return i-1;
 	}
 	public long toExternal(long i){ //return whether 0 based or 1 based counting
 		if (searchTask instanceof Incrementable)
 			return ((Incrementable)searchTask).toExternal(i);
 		return i+1;
 	}
 	
	/*.................................................................................................................*/
  	 public void employeeQuit(MesquiteModule m){
  	 	iQuit();
  	 }
	/*.................................................................................................................*/
  	 public Class getHireSubchoice(){
  	 	return TreeSearcher.class;  
  	 }
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) {
   	 	Snapshot temp = new Snapshot();
  	 	temp.addLine("setSearcher " , searchTask);
  	 	return temp;
  	 }
  	 MesquiteInteger pos = new MesquiteInteger();
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Sets the module performing the tree searching", "[name of module]", commandName, "setSearcher")) {
	    	 	TreeSearcher temp= (TreeSearcher)replaceEmployee(commandRec, TreeSearcher.class, arguments, "Tree Searcher", searchTask);
			if (temp!=null){
	    	 		searchTask=  temp;
	 			parametersChanged(null, commandRec); //?
 			}
 			return searchTask;
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
   	 }
   	 
   	/** Called to provoke any necessary initialization.  This helps prevent the module's intialization queries to the user from
   	happening at inopportune times (e.g., while a long chart calculation is in mid-progress)*/
   	public void initialize(Taxa taxa, CommandRecord commandRec){
   		searchTask.initialize(taxa, commandRec);
   		
   	}
	/*.................................................................................................................*/
  	public void fillTreeBlock(TreeVector treeList, int numberIfUnlimited, CommandRecord commandRec){
  		searchTask.fillTreeBlock(treeList, commandRec);
  	}
	/*.................................................................................................................*/
   	public String getParameters() {
		if (searchTask==null)
			return("");
		else
			return "Searcher: " + searchTask.getName();
   	}
	/*.................................................................................................................*/
   	 public boolean hasLimitedTrees(Taxa taxa, CommandRecord commandRec){
   	 	return true;
   	 }
	/*.................................................................................................................*/
    	 public String getName() {
		return "Tree Search";
   	 }
	/*.................................................................................................................*/
  	 public String getExplanation() {
		return "Supplies trees resulting from a search to optimize some value.";
   	 }
   	 public boolean isPrerelease(){
   	 	return false;
   	 }
   	 public boolean requestPrimaryChoice(){
   	 	return true;
   	 }
	/*.................................................................................................................*/
   	 public boolean showCitation(){
   	 	return true;
   	 }
   	 
}
