/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.treefarm.CladeInTree;

import mesquite.lib.*;
import mesquite.lib.duties.*;

/** this is a little module that deterimines whether or not a set of taxa is a clade in a tree */
public class CladeInTree extends BooleanForTree implements MesquiteListener {
	Taxa currentTaxa = null;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
 		return true;
  	 }
	/*.................................................................................................................*/
   	 public boolean isSubstantive(){
   	 	return true;
   	 }
	/*.................................................................................................................*/
   	 public boolean isPrerelease(){
   	 	return false;
   	 }
	/*.................................................................................................................*/
	/** returns whether this module is requesting to appear as a primary choice */
   	public boolean requestPrimaryChoice(){
   		return true;  
   	}
  	 public void endJob(){
  	 	if (currentTaxa != null)
  	 		currentTaxa.removeListener(this);
  	 	super.endJob();
  	 }
	/** passes which object changed, along with optional code number (type of change) and integers (e.g. which character)*/
	public void changed(Object caller, Object obj, Notification notification, CommandRecord commandRec){
		int code = Notification.getCode(notification);
		if (obj == currentTaxa && code == MesquiteListener.SELECTION_CHANGED){
			parametersChanged(notification, commandRec);
		}
	}
	/*.................................................................................................................*/
	public void calculateBoolean(Tree tree, MesquiteBoolean result, MesquiteString resultString, CommandRecord commandRec) {
    	 	if (result==null || tree==null)
    	 		return;
		Taxa taxa = tree.getTaxa();
		if (currentTaxa != taxa){
			if (currentTaxa != null)
				currentTaxa.removeListener(this);
			currentTaxa = taxa;
			currentTaxa.addListener(this);
		}
		
		if (!taxa.anySelected()) {
 			return;
		}
		boolean bt = tree.isClade(taxa.getSelectedBits());
		result.setValue(bt);
		if (resultString!=null)
			if (bt)
				resultString.setValue("Clade present");
			else
				resultString.setValue("Clade absent");
	}
	/*.................................................................................................................*/
	/**Returns true if the module is to appear in menus and other places in which users can choose, and if can be selected in any way other than by direct request*/
	public boolean getUserChooseable(){
		return true;
	}
	/*.................................................................................................................*/
	/*.................................................................................................................*/
    	 public String getName() {
		return "Selected Taxa Form Clade";
   	 }
	/*.................................................................................................................*/
    	 public String getVeryShortName() {
		return "Clade?";
   	 }
	/*.................................................................................................................*/
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Determines if the selected taxa form a clade in a tree." ;
   	 }
   	 
}
