/* Mesquite source code, Treefarm package.  Copyright 1997-2004 W. Maddison, D. Maddison and P. Midford. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.treefarm.DetermModifTree;
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;
import mesquite.treefarm.lib.*;

/* ======================================================================== */
public class DetermModifTree extends SourceModifiedTree {
	int currentTree=0;
	DetTreeModifier modifierTask;
	MesquiteString modifierName;
	MesquiteCommand stC;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
 		if (!super.startJob(arguments, condition, commandRec, hiredByName))
 			return false;
 		if (arguments ==null)
 			modifierTask = (DetTreeModifier)hireEmployee(commandRec, DetTreeModifier.class, "Modifier of tree");
	 	else {
	 		modifierTask = (DetTreeModifier)hireNamedEmployee(commandRec, DetTreeModifier.class, arguments);
 			if (modifierTask == null)
 				modifierTask = (DetTreeModifier)hireEmployee(commandRec, DetTreeModifier.class, "Modifier of tree");
 		}
 		if (modifierTask == null) {
 			return sorry(commandRec, getName() + " couldn't start because no tree modifier was obtained.");
 		}
 	 	stC = makeCommand("setModifier",  this);
 	 	modifierTask.setHiringCommand(stC);
 		modifierName = new MesquiteString();
	    	modifierName.setValue(modifierTask.getName());
		if (numModulesAvailable(DetTreeModifier.class)>1){
			MesquiteSubmenuSpec mss = addSubmenu(null, "Modifier of Tree", stC, DetTreeModifier.class);
 			mss.setSelected(modifierName);
  		}
  		return true;
  	 }
	/*.................................................................................................................*/
  	public void setPreferredTaxa(Taxa taxa){
  	}
	/*.................................................................................................................*/
   	public boolean isPrerelease(){
   		return false;
   	}
	/*.................................................................................................................*/
	/** returns whether this module is requesting to appear as a primary choice */
   	public boolean requestPrimaryChoice(){
   		return true;  
   	}
	/*.................................................................................................................*/
	public  Class getHireSubchoice(){
		return DetTreeModifier.class;
	}
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) { 
   	 	Snapshot temp = super.getSnapshot(file);
  	 	temp.addLine("setModifier ", modifierTask); 
  	 	return temp;
  	 }
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	 if (checker.compare(this.getClass(), "Sets the tree modifier", "[name of module]", commandName, "setModifier")) {
				DetTreeModifier temp = (DetTreeModifier)replaceEmployee(commandRec, DetTreeModifier.class, arguments, "Modifier of tree", modifierTask);
				if (temp !=null){
					modifierTask = temp;
	    	 		modifierName.setValue(modifierTask.getName());
			 	 	modifierTask.setHiringCommand(stC);
					parametersChanged(null, commandRec);
	    	 			return modifierTask;
	    	 		}
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
    	 	return null;
    	 }
	/*.................................................................................................................*/
   	public Tree getTree(Taxa taxa, int ic, CommandRecord commandRec) {  
   		Tree tree = getBasisTree(taxa, commandRec);
   		currentTree = ic;
   		if (tree == null)
   			return null;
   		MesquiteTree modified =  tree.cloneTree();
   		modifierTask.modifyTree(tree, modified, currentTree, commandRec);
   		modified.setName("Modified from " + tree.getName() + " (#" + (currentTree + 1) + ")");
   		return modified;
   	}
	/*.................................................................................................................*/
   	public int getNumberOfTrees(Taxa taxa, CommandRecord commandRec) {
   		Tree tree = getBasisTree(taxa, commandRec);
		if (tree ==null)
			return 0;
		else
			return modifierTask.getNumberOfTrees(tree, commandRec);
   	}
	/*.................................................................................................................*/
   	public Tree getFirstTree(Taxa taxa, CommandRecord commandRec) {
   		currentTree = 0;
   		return getTree(taxa, currentTree, commandRec);
   	}
	/*.................................................................................................................*/
   	public Tree getNextTree(Taxa taxa, CommandRecord commandRec) {
   		currentTree++;
   		return getTree(taxa, currentTree, commandRec);
   	}
	/*.................................................................................................................*/
   	public String getTreeNameString(Taxa taxa, int itree, CommandRecord commandRec) {
   		return "Modification #" + (itree +1);
   	}
	/*.................................................................................................................*/
   	public String getParameters() {
   		return"Modifying trees from: " + getBasisTreeSource().getParameters();
   	}
	/*.................................................................................................................*/
    	 public String getName() {
		return "Modify Current Tree";
   	 }
	/*.................................................................................................................*/
  	 public String getExplanation() {
		return "Modifies current tree.";
   	 }
   	 
}
