/* Mesquite source code, Treefarm package.  Copyright 1997-2004 W. Maddison, D. Maddison and P. Midford. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.treefarm.SharedPartitions2Trees;
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;



/* ======================================================================== */
public class SharedPartitions2Trees extends NumberFor2Trees {
	StringArray terminalsAbove, terminalsBelow, otherTerminalsAbove, otherTerminalsBelow;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
 		return true;
  	 }
  	 public void employeeQuit(MesquiteModule m){
  	 	iQuit();
  	 }
	/*.................................................................................................................*/
  	 private boolean arraysMatch(StringArray one, StringArray two){
   	 	return arrayIsSubset(one, two) && arrayIsSubset(two, one);
 	 }
	/*.................................................................................................................*/
  	 private boolean arrayIsSubset(StringArray one, StringArray two){
  	 	if (one==null || two == null)
  	 		return false;
  	 	for (int i=0; i<one.getSize(); i++){
  	 		if (one.getValue(i)!=null && two.indexOf(one.getValue(i))<0)
  	 			return false;
  	 	}
  	 	return true;
  	 }
	/*.................................................................................................................*/
  	 private void accumulateTerminals(Tree tree,int node, int target, boolean aboveTarget, StringArray terminalsAbove, StringArray terminalsBelow){
		if (tree.nodeIsTerminal(node)){
			if (aboveTarget)
				terminalsAbove.fillNextUnassigned(tree.getTaxa().getTaxonName(tree.taxonNumberOfNode(node)));
			else
				terminalsBelow.fillNextUnassigned(tree.getTaxa().getTaxonName(tree.taxonNumberOfNode(node)));
			
		}
		else for (int daughter = tree.firstDaughterOfNode(node); tree.nodeExists(daughter); daughter = tree.nextSisterOfNode(daughter))
				accumulateTerminals(tree, daughter, target, (node==target || aboveTarget), terminalsAbove, terminalsBelow);
  	 }
	/*.................................................................................................................*/
  	 /* Returns true if partitions are consistent (same taxa in each subset in each tree, or group in tree1 is subset of
  	 group in tree 2 AND other group in 1 is subset of other group in 2*/
  	 private boolean partitionsConsistent(Tree tree, int node, Tree otherTree, int otherNode){
		terminalsAbove.deassignArray();
		terminalsBelow.deassignArray();
		otherTerminalsAbove.deassignArray();
		otherTerminalsBelow.deassignArray();
		// first, load arrays with node numbers in taxa on either side of partition defined by above and below node
		accumulateTerminals(tree, tree.getRoot(), node, false, terminalsAbove, terminalsBelow);
		accumulateTerminals(otherTree, otherTree.getRoot(), otherNode, false, otherTerminalsAbove, otherTerminalsBelow);
		
		// Next, compare arrays to see that same
		if (arraysMatch(terminalsAbove, otherTerminalsAbove)){
			return (arraysMatch(terminalsBelow, otherTerminalsBelow)|| arrayIsSubset(terminalsBelow, otherTerminalsBelow) || arrayIsSubset(otherTerminalsBelow, terminalsBelow));
		}
		else if (arraysMatch(terminalsBelow, otherTerminalsBelow)){
			return (arrayIsSubset(terminalsAbove, otherTerminalsAbove) || arrayIsSubset(otherTerminalsAbove, terminalsAbove));
		}
		else if (arraysMatch(terminalsAbove, otherTerminalsBelow)){
			return (arraysMatch(terminalsBelow, otherTerminalsAbove)|| arrayIsSubset(terminalsBelow, otherTerminalsAbove) || arrayIsSubset(otherTerminalsAbove, terminalsBelow));
		}
		else if (arraysMatch(terminalsBelow, otherTerminalsAbove)){
			return (arraysMatch(terminalsAbove, otherTerminalsBelow)|| arrayIsSubset(terminalsAbove, otherTerminalsBelow) || arrayIsSubset(otherTerminalsBelow, terminalsAbove));
		}
		else if (arrayIsSubset(terminalsAbove, otherTerminalsAbove)){
			return (arrayIsSubset(terminalsBelow, otherTerminalsBelow));
		}
		else if (arrayIsSubset(otherTerminalsAbove, terminalsAbove)){
			return (arrayIsSubset(otherTerminalsBelow, terminalsBelow));
		}
		else
			return false;
  	 }
  	 
  	 
	/*.................................................................................................................*/
  	 private boolean consistentPartitionFound(Tree tree,int node,  Tree otherTree, int otherNode){
		if (otherTree.nodeIsInternal(otherNode)){
			if (otherNode != otherTree.getRoot() && partitionsConsistent(tree, node, otherTree, otherNode)) {
				return true;
			}
			
			boolean found = false;
			for (int daughter = otherTree.firstDaughterOfNode(otherNode); otherTree.nodeExists(daughter) && !found; daughter = otherTree.nextSisterOfNode(daughter)) {
				found = consistentPartitionFound(tree, node, otherTree, daughter);
				if (found)
					return true;
			}
			
		}
		return false;
  	 }
  	 private void visitOriginal(Tree tree,int node,  Tree otherTree, MesquiteInteger numConsistent){
		if (tree.nodeIsInternal(node)){
			boolean count = (node != tree.getRoot() && tree.numberOfDaughtersOfNode(tree.motherOfNode(node))!=1);
			boolean dichotDescOfRoot =tree.motherOfNode(node)==tree.getRoot() &&  !tree.nodeIsPolytomous(tree.getRoot());
			if (dichotDescOfRoot) {
				count = count && !(tree.nodeIsFirstDaughter(node));
				int sis = tree.nextSisterOfNode(node);
				if (!tree.nodeExists(sis))
					sis = tree.previousSisterOfNode(node);
				count = count && !tree.nodeIsTerminal(sis);
			}
			if (count && consistentPartitionFound(tree, node, otherTree, otherTree.getRoot()))
				numConsistent.increment();
			for (int daughter = tree.firstDaughterOfNode(node); tree.nodeExists(daughter); daughter = tree.nextSisterOfNode(daughter))
				visitOriginal(tree, daughter, otherTree, numConsistent);
			
		}
  	 }
  	 
   	/** Called to provoke any necessary initialization.  This helps prevent the module's intialization queries to the user from
   	happening at inopportune times (e.g., while a long chart calculation is in mid-progress)*/
	public void initialize(Tree t1, Tree t2, CommandRecord commandRec) {
	
	}
	/*.................................................................................................................*/
	public void calculateNumber(Tree tree1, Tree tree2, MesquiteNumber result, MesquiteString resultString, CommandRecord commandRec) {
    	 	if (result==null)
    	 		return;
		result.setToUnassigned();
		if (tree1 == null)
			return;
		if (tree2 == null)
			return;
		int numTerms = MesquiteInteger.maximum(tree1.getTaxa().getNumTaxa(), tree2.getTaxa().getNumTaxa());
		if (terminalsAbove==null || numTerms > terminalsAbove.getSize()) {
			terminalsAbove = new StringArray(numTerms);
			terminalsBelow = new StringArray(numTerms);
			otherTerminalsAbove = new StringArray(numTerms);
			otherTerminalsBelow = new StringArray(numTerms);
		}
		MesquiteInteger numCon = new MesquiteInteger(0);
		visitOriginal(tree1, tree1.getRoot(), tree2, numCon);
		
		result.setValue(numCon.getValue());
		if (resultString!=null)
			resultString.setValue("Shared Partitions: "+ result.toString());
	}
	/*.................................................................................................................*/
   	public boolean isPrerelease(){
   		return false;
   	}
	 
	/*.................................................................................................................*/
    	 public String getParameters() {
		return "";
   	 }
	/*.................................................................................................................*/
    	 public String getName() {
		return "Shared Partitions";
   	 }
	/*.................................................................................................................*/
  	 public String getAuthors() {
		return "W. P. Maddison (slightly adjusted by J. Klingner)";
   	 }
	/*.................................................................................................................*/
  	 public String getExplanation() {
		return "Calculates the number of shared partitions between two trees.";
   	 }
	/*.................................................................................................................*/
   	 public boolean showCitation(){
   	 	return true;
   	 }
}
