/* Mesquite source code, Treefarm package.  Copyright 1997-2004 W. Maddison, D. Maddison and P. Midford. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.trees.AlterTrees;
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;
import mesquite.lists.lib.*;

/* ======================================================================== */
public class AlterTrees extends TreeListUtility { 
	TreeAltererMult alterTask;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName){
 		if (arguments ==null)
			alterTask = (TreeAltererMult)hireEmployee(commandRec, TreeAltererMult.class, "Module to alter trees");
	 	else {
			alterTask = (TreeAltererMult)hireNamedEmployee(commandRec, TreeAltererMult.class, arguments);
 			if (alterTask == null)
				alterTask = (TreeAltererMult)hireEmployee(commandRec, TreeAltererMult.class, "Module to alter trees");
 		}
 		if (alterTask == null) {
 			return sorry(commandRec, getName() + " couldn't start because no tree alterer was obtained.");
 		}
		return true;
	}
	
	/*.................................................................................................................*/
	public  Class getHireSubchoice(){
		return TreeAltererMult.class;
	}
	
   	/** if returns true, then requests to remain on even after operateData is called.  Default is false*/
   	public boolean pleaseLeaveMeOn(){
   		return false;
   	}
	/*.................................................................................................................*/
	/** returns whether this module is requesting to appear as a primary choice */
   	public boolean isPrerelease(){
   		return false;
   	}
   	public boolean requestPrimaryChoice(){
   		return true;  
   	}
   	/** Called to operate on the data in all cells.  Returns true if data altered*/
   	public boolean operateOnTrees(TreeVector trees, CommandRecord commandRec){
   		if (trees == null)
   			return false;
   		Taxa taxa = trees.getTaxa();
   		int numTrees = trees.size();
   		boolean doAll = !trees.anySelected();


		for (int j=0; j<numTrees; j++){
			if (doAll || trees.getSelected(j)){
				Tree tree = trees.getTree(j);
				if (tree!=null && tree instanceof MesquiteTree)  {
 					boolean success = alterTask.transformTree((MesquiteTree)tree, null, false, commandRec);
 				}
			}
		}
		trees.notifyListeners(this, new Notification(MesquiteListener.UNKNOWN), commandRec);
		return true;
		
	}
	/*.................................................................................................................*/
	/*.................................................................................................................*/
    	 public String getName() {
		return "Alter Trees";
   	 }
   	 
	/*.................................................................................................................*/
  	 public String getExplanation() {
		return "Alters selected trees.";
   	 }
}


	

