/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.trees.BasicTreeWindowCoord;
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;

/** Coordinates the display of the basic Tree Windows (BasicTreeWindowMaker actually makes the window) */
public class BasicTreeWindowCoord extends FileInit {
	ListableVector treeWindows;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
 		getFileCoordinator().addMenuItem(MesquiteTrunk.treesMenu, "New Tree Window", makeCommand("makeTreeWindow",  this));
 		treeWindows = new ListableVector();
 		MesquiteSubmenuSpec mms = getFileCoordinator().addSubmenu(MesquiteTrunk.treesMenu, "Current Tree Windows", makeCommand("showTreeWindow",  this));
 		mms.setList(treeWindows);
 		
 		return true;
  	 }
  	 
  	 String taxaRef(Taxa d){
  	 	return getProject().getTaxaReference(d);
  	 }
	/*.................................................................................................................*/
   	 public boolean isSubstantive(){
   	 	return false;
   	 }
	/*.................................................................................................................*/
   	 public boolean isPrerelease(){
   	 	return false;
   	 }
	/*.................................................................................................................*/
  	 public void employeeQuit(MesquiteModule m){
  	 	if (m instanceof TreeWindowMaker) {
  	 		for (int i=0; i<treeWindows.size(); i++){
  	 			MesquiteWindow w = (MesquiteWindow)treeWindows.elementAt(i);
  	 			if (w.getOwnerModule() == m || w.getOwnerModule() == null) {
  	 				treeWindows.removeElement(w, false);
  	 				resetAllMenuBars();
  	 				return;
  	 			}
  	 		}
  	 		resetAllMenuBars();
  	 	}
  	 }
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) {
   	 	Snapshot temp = new Snapshot();
		for (int i = 0; i<getNumberOfEmployees(); i++) {
			Object e=getEmployeeVector().elementAt(i);
			if (e instanceof TreeWindowMaker) {
				TreeWindowMaker dwm = (TreeWindowMaker)e;
				Taxa d = (Taxa)dwm.doCommand("getTaxa", null, CommandRecord.getRecNSIfNull(), CommandChecker.defaultChecker);
				if (d != null) {
					temp.addLine("makeTreeWindow " + taxaRef(d) + " ", dwm ); //TODO: must pass treeWindowTask
				}
			}
		}
  	 	return temp;
  	 }
  	 MesquiteInteger pos = new MesquiteInteger();
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Requests that a tree window be made", "[number of taxa block]", commandName, "makeTreeWindow")) {
	   	 	Taxa taxa = null;
		   	if (!StringUtil.blank(arguments)) //rearranged to attempt to get some taxa to be used
		   		taxa =  getProject().getTaxa(checker.getFile(), parser.getFirstToken(arguments));
	   	 	if (taxa == null){
	   	 		int numTaxas = getProject().getNumberTaxas(checker.getFile());
	   	 		//if only one taxa block, use it
	   	 		if (numTaxas<=0){
	   	 			return null;
	   	 		}
	   	 		else if (numTaxas==1){
		   	 		taxa =  getProject().getTaxa(checker.getFile(), 0);
	   	 		}
	   	 		else {
		   	 		taxa =  getProject().chooseTaxa(containerOfModule(), "For which block of taxa do you want to show trees in the tree window?", commandRec);
	   	 			//else, query user
	   	 		}
	   	 	}
   	 		if (taxa==null)
   	 			return null;
			TreeWindowMaker treeWindowTask= (TreeWindowMaker)hireCompatibleEmployee(commandRec, TreeWindowMaker.class, taxa, null);
			if (treeWindowTask !=null){
	   	 		treeWindowTask.doCommand("makeTreeWindow", getProject().getTaxaReference(taxa), commandRec, checker);
 				treeWindows.addElement(treeWindowTask.getModuleWindow(), false);
 				resetAllMenuBars();
				return treeWindowTask;
   	 		}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Shows an existing tree window", "[number of tree window as employee of coordinator]", commandName, "showTreeWindow")) {
    	 		pos.setValue(0);
    	 		int which = MesquiteInteger.fromString(arguments, pos);
    	 		if ((which == 0 || MesquiteInteger.isPositive(which)) && which<treeWindows.size()) {
    	 			MesquiteWindow win = (MesquiteWindow)treeWindows.elementAt(which);
    	 			win.show();
    	 		}
    	 			
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
   	 }
	/*.................................................................................................................*/
	/**Returns command to hire employee if clonable*/
	public String getClonableEmployeeCommand(MesquiteModule employee){
		if (employee!=null && employee.getEmployer()==this) {
			if (employee.getHiredAs()==TreeWindowMaker.class) {
				Taxa d = (Taxa)employee.doCommand("getTaxa", null, CommandRecord.getRecNSIfNull(), CommandChecker.defaultChecker);
				if (d != null) {
					return ("makeTreeWindow " + taxaRef(d) + "  " + StringUtil.tokenize(employee.getName()) + ";");//quote
				}
			}
		}
		return null;
	}
	/*.................................................................................................................*/
    	 public String getName() {
		return "Tree Window Coordinator";
   	 }
   	 
	/*.................................................................................................................*/
   	 
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Coordinates the creation of basic tree windows." ;
   	 }
   	 
}
	
