/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.trees.NumForMatCurrentTree;

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;

/* ======================================================================== */
public class NumForMatCurrentTree extends NumberForMatrix { // implements NumForCharTreeDep {
	NumberForMatrixAndTree numberTask;
	OneTreeSource treeTask;
	Taxa taxa;
	Tree tree;
	MesquiteString numberTaskName;
	MesquiteCommand ntC;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		treeTask = (OneTreeSource)hireEmployee(commandRec, OneTreeSource.class, "Source of current tree");
		if (treeTask == null)
			return sorry(commandRec, getName() + " couldn't start because no source of trees obtained.");
		numberTask = (NumberForMatrixAndTree)hireEmployee(commandRec, NumberForMatrixAndTree.class, "Value to calculate for character matrix");
		if (numberTask == null)
			return sorry(commandRec, getName() + " couldn't start because no calculator module obtained.");

		ntC =makeCommand("setNumberTask",  this);
 		numberTask.setHiringCommand(ntC);
 		numberTaskName = new MesquiteString();
 		numberTaskName.setValue(numberTask.getName());
		if (numModulesAvailable(NumberForMatrixAndTree.class)>1) {
			MesquiteSubmenuSpec mss = addSubmenu(null, "Values with current tree", ntC, NumberForMatrixAndTree.class);
 			mss.setSelected(numberTaskName);
		}
		
		return true;
  	 }
  	 public CompatibilityTest getCompatibilityTest(){
  	 	if (numberTask !=null)
  	 		return numberTask.getCompatibilityTest();
  	 	return null;
  	 }
	/*.................................................................................................................*/
	/** returns whether this module is requesting to appear as a primary choice */
   	public boolean requestPrimaryChoice(){
   		return true;  
   	}
	/*.................................................................................................................*/
	/*.................................................................................................................*/
   	 public boolean isPrerelease(){
   	 	return false;
   	 }
  	 public void employeeQuit(MesquiteModule m){
  	 	iQuit();
  	 }
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) { 
   	 	Snapshot temp = new Snapshot();
 	 	temp.addLine("setNumberTask ", numberTask);  //note: not snapshotting current tree task
  	 	return temp;
  	 }
	/*.................................................................................................................*/
   	 public void employeeParametersChanged(MesquiteModule employee, MesquiteModule source, Notification notification, CommandRecord commandRec) {
   	 	if (employee==treeTask) {
			parametersChanged(notification, commandRec);
   	 	}
   	 	else if (employee==numberTask) {
			parametersChanged(notification, commandRec);
   	 	}
   	 }
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
   	 	if (checker.compare(this.getClass(), "Sets the module that calculates numbers for matrices with the current tree", "[name of module]", commandName, "setNumberTask")) {
    	 		NumberForMatrixAndTree temp =  (NumberForMatrixAndTree)replaceEmployee(commandRec, NumberForMatrixAndTree.class, arguments, "Number for matrix and tree", numberTask);
 			if (temp!=null) {
 				numberTask = temp;
		 		numberTask.setHiringCommand(ntC);
		 		numberTaskName.setValue(numberTask.getName());
				parametersChanged(null, commandRec);
	 			return numberTask;
	 		}
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
    	 	return null;
    	 }
  	 
   	/** Called to provoke any necessary initialization.  This helps prevent the module's intialization queries to the user from
   	happening at inopportune times (e.g., while a long chart calculation is in mid-progress)*/
   	public void initialize(MCharactersDistribution data, CommandRecord commandRec){
   		if (data!=null) {
   			taxa = data.getTaxa();
	   		treeTask.initialize(taxa, commandRec);
	   		numberTask.initialize(treeTask.getTree(taxa, commandRec), data, commandRec);
   		}
   		
   		if (taxa==null)
   			taxa = getProject().chooseTaxa(containerOfModule(), "Taxa",commandRec); 
   	}
	public void endJob(){
			if (taxa!=null)
				taxa.removeListener(this);
			super.endJob();
	}
    	 MesquiteString rs = new MesquiteString();
	/*.................................................................................................................*/
	public  void calculateNumber(MCharactersDistribution data, MesquiteNumber result, MesquiteString resultString, CommandRecord commandRec) {
    	 	if (result==null)
    	 		return;
		result.setToUnassigned();
	   	if (taxa==null){
	   		initialize(data, commandRec);
		}
		tree = treeTask.getTree(taxa, commandRec);
		rs.setValue("");
		numberTask.calculateNumber(tree, data, result, rs, commandRec);
		if (resultString!=null) {
			resultString.setValue("For current tree, ");
			resultString.append(rs.toString());
		}
	}
	/*.................................................................................................................*/
	public String getParameters(){
		if (tree ==null)
			return "Calculator: " + numberTask.getName(); //of which tree??
		else
			return "Calculator: " + numberTask.getName() + " with tree \"" + tree.getName() + "\""; 
	}
	/*.................................................................................................................*/
    	 public String getName() {
		return "Matrix value with current tree";
   	 }
	/*.................................................................................................................*/
    	 public String getVeryShortName() {
    	 	if (numberTask ==null)
			return "Matrix value with current tree";
		else
			return numberTask.getVeryShortName() + " (current tree)";
   	 }
	/*.................................................................................................................*/
    	 public String getNameForMenuItem() {
		return "Matrix value with current tree...";
   	 }
	/*.................................................................................................................*/
 	/** returns an explanation of what the module does.*/
 	public String getExplanation() {
 		return "Coordinates the calculation of a number for a character matrix based on a current tree." ;
   	 }
   	 
   	 
}
