/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.trees.OtherTreeBlocks;
/*~~  */

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;

/** Supplies tree blocks from other sources.*/
public class OtherTreeBlocks extends TreeBlockSource {
	int currentTreeBlockIndex=MesquiteInteger.unassigned;
	TreeVector currentTreeBlock = null;
	TreeVector lastUsedTreeBlock = null;
	TreeBlockFiller fillerTask = null;
	Taxa preferredTaxa =null;
	
	int numTrees = 100;
	MesquiteMenuItemSpec ntreesItem = null;
	boolean nTreesSet = false;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		fillerTask = (TreeBlockFiller)hireEmployee(commandRec, TreeBlockFiller.class, "Source of trees for tree blocks");
		if (fillerTask == null)
			return sorry(commandRec, getName() + " couldn't start because no source of trees obtained.");
		ntreesItem = addMenuItem( "Number of Trees...", makeCommand("setNumberTrees",  this));
		return true;
  	 }
	/*.................................................................................................................*/
   	 public boolean isSubstantive(){
   	 	return true;
   	 }
	/*.................................................................................................................*/
   	 public boolean isPrerelease(){
   	 	return false;
   	 }
	/*.................................................................................................................*/
  	public void setPreferredTaxa(Taxa taxa) {
  		preferredTaxa = taxa;
  	}
   	/** Called to provoke any necessary initialization.  This helps prevent the module's intialization queries to the user from
   	happening at inopportune times (e.g., while a long chart calculation is in mid-progress)*/
   	public void initialize(Taxa taxa, CommandRecord commandRec){
   		setPreferredTaxa(taxa);
   		fillerTask.initialize(taxa, commandRec);
		if (!fillerTask.hasLimitedTrees(taxa, commandRec)){
			if (!commandRec.scripting()  && !nTreesSet){
				int n = MesquiteInteger.queryInteger(containerOfModule(), "Trees per block?", "How many trees to include per tree block?", numTrees);
				if (MesquiteInteger.isCombinable(n) && n>0)
					numTrees = n;
				nTreesSet = true;
			}
		}
		ntreesItem.setEnabled(!fillerTask.hasLimitedTrees(taxa, commandRec));
		MesquiteTrunk.resetMenuItemEnabling();
   	}
	/*.................................................................................................................*/
  	 public Snapshot getSnapshot(MesquiteFile file) {
   	 	Snapshot temp = new Snapshot();
  	 	temp.addLine("setTreeFiller ", fillerTask);
   	 	temp.addLine("setNumberTrees " + numTrees);
 	 	return temp;
  	 }
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Sets the number of trees included in each tree block", "[number of trees]", commandName, "setNumberTrees")) {
			MesquiteInteger pos = new MesquiteInteger(0);
			int newNum= MesquiteInteger.fromFirstToken(arguments, pos);
			if (!MesquiteInteger.isCombinable(newNum))
				newNum = MesquiteInteger.queryInteger(containerOfModule(), "Set Number of Trees", "Number of Trees:", numTrees, 0, MesquiteInteger.infinite);
    	 		if (newNum>0  && newNum!=numTrees) {
    	 			numTrees = newNum;
				parametersChanged(null, commandRec);
    	 		}
    	 	}
    	 	else if (checker.compare(this.getClass(), "Sets the module supplying trees", "[name of module]", commandName, "setTreeFiller")) {
    	 		TreeBlockFiller temp=  (TreeBlockFiller)replaceEmployee(commandRec, TreeBlockFiller.class, arguments, "Source of trees for tree blocks", fillerTask);
	    	 	if (temp!=null) {
	    	 		fillerTask = temp;
	    	 		initialize(preferredTaxa, commandRec);
 				parametersChanged(null, commandRec); //?
 			}
 			return temp;
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
		return null;
   	 }
	/*.................................................................................................................*/
   	public TreeVector getFirstBlock(Taxa taxa, CommandRecord commandRec) {
   		currentTreeBlockIndex=0;
   		return getCurrentBlock(taxa, commandRec);
   	}
	/*.................................................................................................................*/
   	public TreeVector getBlock(Taxa taxa, int ic, CommandRecord commandRec) {
   		currentTreeBlockIndex=ic;
   		return getCurrentBlock(taxa, commandRec);
   	}
	/*.................................................................................................................*/
   	public TreeVector getCurrentBlock(Taxa taxa, CommandRecord commandRec) {
   		if (fillerTask instanceof Incrementable)
   			((Incrementable)fillerTask).setCurrent(currentTreeBlockIndex, commandRec);
   		setPreferredTaxa(taxa);
   		TreeVector treeList = new TreeVector(taxa);
   		fillerTask.fillTreeBlock(treeList, numTrees, commandRec); 
   		return treeList;
   	}
	/*.................................................................................................................*/
   	public TreeVector getNextBlock(Taxa taxa, CommandRecord commandRec) {
   		currentTreeBlockIndex++;
   		return getCurrentBlock(taxa, commandRec);
   	}
	/*.................................................................................................................*/
   	public int getNumberOfTreeBlocks(Taxa taxa, CommandRecord commandRec) {
   		if (fillerTask instanceof Incrementable)
   			return (int)( ((Incrementable)fillerTask).getMax(commandRec) -  ((Incrementable)fillerTask).getMin(commandRec) + 1);
		return 1;
   	}
   
	/*.................................................................................................................*/
   	public String getTreeBlockNameString(Taxa taxa, int index, CommandRecord commandRec) {
		return fillerTask.getName() + "  " + currentTreeBlockIndex;//todo!  not right
   	}
	/*.................................................................................................................*/
   	public String getCurrentTreeBlockNameString(Taxa taxa, CommandRecord commandRec) {
		return fillerTask.getName() + "  " + currentTreeBlockIndex; //todo!  not right
  	}
	/*.................................................................................................................*/
    	 public String getName() {
		return "Other Tree Blocks";
   	 }
   	 
	/*.................................................................................................................*/
  	 public String getExplanation() {
		return "Supplies blocks of trees from various sources";
   	 }
}
