/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.trees.SetPolytomyDefault;

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.duties.*;

/* ======================================================================== */
public class SetPolytomyDefault extends DefaultsAssistant {
	MesquiteBoolean polytomiesHard;
	static boolean changed = false;
	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
		polytomiesHard = new MesquiteBoolean(true);
		loadPreferences();
		addCheckMenuItem( null, "Polytomies Hard by Default", makeCommand("polytomiesHard",  this), polytomiesHard);
		return true;
  	 }
	public void processPreferencesFromFile (String[] prefs) {
		if (prefs!=null && prefs.length>0) {
			if (!changed) {
				if (prefs[0].equals("soft") && polytomiesHard.getValue()) {
					polytomiesHard.setValue(false);
    	 				MesquiteTree.polytomyDefaultHard = false;
    	 				MesquiteModule.mesquiteTrunk.classFieldChanged(Tree.class, "polytomyDefaultHard", CommandRecord.getRecNSIfNull());
    	 			}
					
			}
		}
	}
	/*.................................................................................................................*/
	public String[] preparePreferencesForFile () {
		if (polytomiesHard.getValue())
			return (new String[] {"hard"});
		else
			return (new String[] {"soft"});
	}
	MesquiteInteger pos = new MesquiteInteger();
	/*.................................................................................................................*/
    	 public Object doCommand(String commandName, String arguments, CommandRecord commandRec, CommandChecker checker) {
    	 	if (checker.compare(this.getClass(), "Sets universal, Mesquite-wide setting whether polytomies are hard or soft by default", "[on = polytomies hard; off = soft]", commandName, "polytomiesHard")) {  
    	 		boolean current = polytomiesHard.getValue();
    	 		polytomiesHard.toggleValue(parser.getFirstToken(arguments));
    	 		if (current!=polytomiesHard.getValue()) {
    	 			MesquiteTree.polytomyDefaultHard = polytomiesHard.getValue();
    	 			//tell all modules that default has changed  classFieldChanged(Class class, String fieldName)
    	 			MesquiteModule.mesquiteTrunk.classFieldChanged(Tree.class, "polytomyDefaultHard", commandRec);
    	 			storePreferences();
    	 			changed = true;
    	 			
    	 		}
			return polytomiesHard;
    	 	}
    	 	else
    	 		return super.doCommand(commandName, arguments, commandRec, checker);
   	 }
  	 
	/*.................................................................................................................*/
    	 public String getName() {
		return "Set Polytomy Default";
   	 }
	/*.................................................................................................................*/
  	 public String getExplanation() {
		return "Sets the default state for polytomies.";
   	 }
}

