/* Mesquite source code.  Copyright 1997-2004 W. Maddison and D. Maddison. 
Version 1.01, January 2004.
Disclaimer:  The Mesquite source code is lengthy and we are few.  There are no doubt inefficiencies and goofs in this code. 
The commenting leaves much to be desired. Please approach this source code with the spirit of helping out.
Perhaps with your help we can be more than a few, and make Mesquite better.

Mesquite is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.
Mesquite's web site is http://mesquiteproject.org

This source code and its compiled class files are free and modifiable under the terms of 
GNU Lesser General Public License.  (http://www.gnu.org/copyleft/lesser.html)
*/
package mesquite.trees.TreeValueUsingMatrix;

import java.util.*;
import java.awt.*;
import mesquite.lib.*;
import mesquite.lib.characters.*;
import mesquite.lib.duties.*;


/* ======================================================================== */
public class TreeValueUsingMatrix extends NumberForTree {
	MatrixSourceCoord characterSourceTask;
	NumberForMatrixAndTree numberTask;
	Taxa oldTaxa = null;
    	 MCharactersDistribution matrix;

	/*.................................................................................................................*/
	public boolean startJob(String arguments, Object condition, CommandRecord commandRec, boolean hiredByName) {
 		numberTask = (NumberForMatrixAndTree)hireEmployee(commandRec, NumberForMatrixAndTree.class, "Value to calculate for trees");
 		if (numberTask == null)
 			return sorry(commandRec, getName() + " couldn't start because no steps counting module was obtained.");
 		characterSourceTask = (MatrixSourceCoord)hireCompatibleEmployee(commandRec, MatrixSourceCoord.class, numberTask.getCompatibilityTest(), "Source of characters (for " + numberTask.getName() + ")");
 		if (characterSourceTask == null)
 			return sorry(commandRec, getName() + " couldn't start because no source of characters was obtained.");
  		return true;
  	 }
  	 public void employeeQuit(MesquiteModule m){
  	 	if (m==numberTask)
  	 		iQuit();
  	 }
  	 
	/*.................................................................................................................*/
	/** returns whether this module is requesting to appear as a primary choice */
   	public boolean requestPrimaryChoice(){
   		return true;  
   	}
	/*.................................................................................................................*/
   	 public boolean isSubstantive(){
   	 	return false;
   	 }
   	/** Called to provoke any necessary initialization.  This helps prevent the module's intialization queries to the user from
   	happening at inopportune times (e.g., while a long chart calculation is in mid-progress)*/
   	public void initialize(Tree tree, CommandRecord commandRec){
   		characterSourceTask.initialize(tree.getTaxa(), commandRec);
   	}
	MesquiteString cs = new MesquiteString();
	/*.................................................................................................................*/
	public void calculateNumber(Tree tree, MesquiteNumber result, MesquiteString resultString, CommandRecord commandRec) {
    	 	if (result==null || tree == null)
    	 		return;
		result.setToUnassigned();
    	 	int count=0;
    	 	Taxa taxa = tree.getTaxa();
 		matrix = characterSourceTask.getCurrentMatrix(tree, commandRec);
		
		if (matrix == null) {
			if (resultString!=null)
				resultString.setValue("Treelength not calculated; no matrix supplied");
			return;
		}
		cs.setValue("");
		numberTask.calculateNumber(tree, matrix, result, cs, commandRec);
		if (resultString!=null)
			resultString.setValue(cs.getValue() + " (for matrix " + characterSourceTask.getCurrentMatrixName(tree.getTaxa(), commandRec) + ")");
	}
	public boolean biggerIsBetter() {
		return false;
	}
	/*.................................................................................................................*/

	/*.................................................................................................................*/
   	 public void employeeParametersChanged(MesquiteModule employee, MesquiteModule source, Notification notification, CommandRecord commandRec) {
   	 	if (employee==characterSourceTask) {
			parametersChanged(notification, commandRec);
   	 	}
   	 	else if (employee==numberTask) {
			parametersChanged(notification, commandRec);
   	 	}
   	 }
	/*.................................................................................................................*/
    	 public String getParameters() {
		return "Value calculated: " + numberTask.getName() + "(Source of matrices: " + characterSourceTask.getNameAndParameters() + ")";
   	 }
	/*.................................................................................................................*/
    	 public String getNameAndParameters() {
		return numberTask.getName() + "(Source of matrices: " + characterSourceTask.getNameAndParameters() + ")";
   	 }
	/*.................................................................................................................*/
    	 public String getName() {
		return "Tree value using character matrix";
   	 }
	/*.................................................................................................................*/
  	 public String getExplanation() {
		return "Calculates a value for the tree using a character data matrix.";
   	 }
}
